Package: trackViewer
Type: Package
Title: A R/Bioconductor package with web interface for drawing elegant
        interactive tracks or lollipop plot to facilitate integrated
        analysis of multi-omics data
Version: 1.49.0
Authors@R: c(person(given="Jianhong", family="Ou", email="jou@morgridge.org", 
        role=c("aut", "cre"), comment=c(ORCID="0000-0002-8652-2488")), 
        person(given="Julie Lihua", family="Zhu", 
        role="aut", email="Julie.Zhu@umassmed.edu"))
Maintainer: Jianhong Ou <jou@morgridge.org>
Description: Visualize mapped reads along with annotation as track
        layers for NGS dataset such as ChIP-seq, RNA-seq, miRNA-seq,
        DNA-seq, SNPs and methylation data.
License: GPL (>= 2)
Depends: R (>= 3.5.0), grDevices, methods, GenomicRanges, grid
Imports: Seqinfo, GenomeInfoDb, GenomicAlignments, GenomicFeatures,
        Gviz, Rsamtools, S4Vectors, rtracklayer, BiocGenerics, scales,
        tools, IRanges, AnnotationDbi, grImport, htmlwidgets,
        InteractionSet, utils, rhdf5, strawr, txdbmaker
Suggests: biomaRt, TxDb.Hsapiens.UCSC.hg19.knownGene, RUnit,
        org.Hs.eg.db, BiocStyle, knitr, VariantAnnotation, httr,
        htmltools, rmarkdown, motifStack
biocViews: Visualization
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/pak/sysreqs: cmake make libbz2-dev libicu-dev libjpeg-dev
        liblzma-dev libpng-dev libuv1-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:38:35 UTC
RemoteUrl: https://github.com/bioc/trackViewer
RemoteRef: HEAD
RemoteSha: ea7da89d27c3c123f98fcd043e958c269a1a997b
NeedsCompilation: no
Packaged: 2026-05-30 09:52:54 UTC; root
Author: Jianhong Ou [aut, cre] (ORCID: <https://orcid.org/0000-0002-8652-2488>),
  Julie Lihua Zhu [aut]
Built: R 4.6.0; ; 2026-05-30 15:16:04 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ARA)
export(GIoperator)
export(GRoperator)
export(addArrowMark)
export(addGuideLine)
export(addInteractionAnnotation)
export(browseTracks)
export(browseTracksOutput)
export(coverageGR)
export(dandelion.plot)
export(geneModelFromTxdb)
export(geneTrack)
export(getCurTrackViewport)
export(getGeneIDsFromTxDb)
export(getLocation)
export(gi2track)
export(gieStain)
export(ideogramPlot)
export(importBam)
export(importData)
export(importGInteractions)
export(importScore)
export(listChromosomes)
export(listNormalizations)
export(listResolutions)
export(loadIdeogram)
export(lolliplot)
export(optimizeStyle)
export(parse2GRanges)
export(parseWIG)
export(plotGRanges)
export(renderbrowseTracks)
export(trackList)
export(trackViewerStyle)
export(viewGene)
export(viewTracks)
exportClasses(pos)
exportClasses(track)
exportClasses(trackList)
exportClasses(trackStyle)
exportClasses(trackViewerStyle)
exportClasses(xscale)
exportClasses(yaxisStyle)
exportMethods("$")
exportMethods("$<-")
exportMethods("seqlevelsStyle<-")
exportMethods(reduce)
exportMethods(seqlevels)
exportMethods(seqlevelsStyle)
exportMethods(setTrackStyleParam)
exportMethods(setTrackViewerStyleParam)
exportMethods(setTrackXscaleParam)
exportMethods(setTrackYaxisParam)
exportMethods(show)
import(GenomicAlignments)
import(GenomicRanges)
import(IRanges)
import(Rsamtools)
import(S4Vectors)
import(grid)
import(htmlwidgets)
import(methods)
importClassesFrom(grImport,Picture)
importFrom(AnnotationDbi,columns)
importFrom(AnnotationDbi,mapIds)
importFrom(AnnotationDbi,mget)
importFrom(AnnotationDbi,select)
importFrom(BiocGenerics,basename)
importFrom(BiocGenerics,strand)
importFrom(GenomeInfoDb,"seqlevelsStyle<-")
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomicFeatures,genes)
importFrom(GenomicFeatures,transcripts)
importFrom(Gviz,GeneRegionTrack)
importFrom(IRanges,Views)
importFrom(IRanges,viewMaxs)
importFrom(InteractionSet,GInteractions)
importFrom(InteractionSet,`regions<-`)
importFrom(InteractionSet,anchorIds)
importFrom(InteractionSet,regions)
importFrom(S4Vectors,first)
importFrom(S4Vectors,second)
importFrom(Seqinfo,"genome<-")
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,"seqlevels<-")
importFrom(Seqinfo,Seqinfo)
importFrom(Seqinfo,genome)
importFrom(Seqinfo,isCircular)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(grDevices,as.raster)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,palette)
importFrom(grDevices,rgb)
importFrom(grImport,grid.picture)
importFrom(grImport,readPicture)
importFrom(rhdf5,H5Fopen)
importFrom(rhdf5,H5close)
importFrom(rhdf5,h5closeAll)
importFrom(rhdf5,h5ls)
importFrom(rhdf5,h5read)
importFrom(rtracklayer,GRangesForUCSCGenome)
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,getTable)
importFrom(rtracklayer,import)
importFrom(rtracklayer,ucscTableQuery)
importFrom(scales,rescale)
importFrom(stats,ksmooth)
importFrom(strawr,readHicBpResolutions)
importFrom(strawr,readHicChroms)
importFrom(strawr,readHicNormTypes)
importFrom(strawr,straw)
importFrom(tools,file_ext)
importFrom(txdbmaker,makeTxDb)
importFrom(utils,combn)
importFrom(utils,installed.packages)
importMethodsFrom(GenomicRanges,range)
importMethodsFrom(GenomicRanges,reduce)

# trackViewer

[![DOI](https://img.shields.io/badge/DOI-10.1038%2Fs41592--019--0430--y-brightgreen.svg)](https://doi.org/10.1038/s41592-019-0430-y)
[![platforms](http://bioconductor.org/shields/availability/devel/trackViewer.svg)](http://bioconductor.org/packages/devel/bioc/html/trackViewer.html)
[![build](http://bioconductor.org/shields/build/devel/bioc/trackViewer.svg)](http://bioconductor.org/packages/devel/bioc/html/trackViewer.html)

A R/Bioconductor package for drawing elegant interactive tracks or lollipop plot to facilitate integrated analysis of multi-omics data

Visualize mapped reads along with annotation as track layers for NGS dataset 
  such as ChIP-seq, RNA-seq, miRNA-seq, DNA-seq, SNPs and methylation data.

## Installation

To install this package, start R and enter:

```r
library(BiocManager)
BiocManager::install("trackViewer")
```

## Documentation

To view documentation of trackViewer, start R and enter:
```r
browseVignettes("trackViewer")
```

Or view it online at:
https://bioconductor.org/packages/release/bioc/vignettes/trackViewer/inst/doc/trackViewer.html

## Citation

Please cite the paper below for the trackViewer package.

  Ou J. Zhu L. trackViewer: a Bioconductor package for interactive and
  integrative visualization of multi-omics data. Nature Methods 16, 453–454
  (2019)
  

## ----echo=FALSE, results="hide", warning=FALSE--------------------------------
suppressPackageStartupMessages({
  library(trackViewer)
  library(TxDb.Hsapiens.UCSC.hg19.knownGene)
  library(org.Hs.eg.db)
  library(InteractionSet)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)

## ----plotback2back------------------------------------------------------------
library(trackViewer)
library(InteractionSet)
gi <- readRDS(system.file("extdata", "nij.chr6.51120000.53200000.gi.rds", package="trackViewer"))
head(gi)
## hicexplorer:hicConvertFormat tool can be used to convert other formats into GInteractions
## eg: hicConvertFormat -m mESC_rep.hic --inputFormat hic --outputFormat cool -o mESC_rep.mcool
##     hicConvertFormat -m mESC_rep.mcool::resolutions/10000 --inputFormat cool --outputFormat ginteractions -o mESC_rep.ginteractions --resolutions 10000
## please note that metadata:score is used for plot.
gi$border_color <- NA ## highlight some regions
gi$border_color[sample(seq_along(gi), 20)] <- sample(1:7, 20, replace=TRUE)
## The TADs will be drawn as lines at points start(first), center point, end(second).
tads <- GInteractions(
  GRanges("chr6", 
          IRanges(c(51130001, 51130001, 51450001, 52210001), width = 20000)),
  GRanges("chr6", 
          IRanges(c(51530001, 52170001, 52210001, 53210001), width = 20000)))
range <- GRanges("chr6", IRanges(51120000, 53200000))
heatmap <- gi2track(gi)
ctcf <- readRDS(system.file("extdata", "ctcf.sample.rds", package="trackViewer"))
viewTracks(trackList(ctcf, heatmap, heightDist = c(1, 3)), 
           gr=range, autoOptimizeStyle = TRUE)
## add TAD information
addInteractionAnnotation(tads, "heatmap", grid.lines, gp=gpar(col="#E69F00", lwd=3, lty=3))
## add highlight interested regions
gi_sub <- gi[order(gi$score, decreasing = TRUE)]
gi_sub <- head(gi_sub[distance(first(gi_sub), second(gi_sub))>200000], n=5)
start(regions(gi_sub)) <- start(regions(gi_sub))-40000
end(regions(gi_sub)) <- end(regions(gi_sub))+40000
addInteractionAnnotation(gi_sub, "heatmap", grid.polygon, gp=gpar(col="red", lwd=2, lty=2, fill=NA))
## add interesting anchor at giving coordinate.
addInteractionAnnotation(52900000, "heatmap", gp=gpar(col="blue", lwd=3))
addInteractionAnnotation(-52900000, "heatmap", gp=gpar(col="cyan", lwd=3, lty=4))
## view the interaction data back to back.
## Please make sure the data are normalized.
gi2 <- gi
set.seed(123)
gi2$score <- gi$score + rnorm(length(gi), sd = sd(gi$score))
back2back <- gi2track(gi, gi2)
## change the color
setTrackStyleParam(back2back, "breaks", 
                   c(seq(from=0, to=50, by=10), 200))
setTrackStyleParam(back2back, "color",
                   c("lightblue", "yellow", "red"))
## chang the lim of y-axis (by default, [0, 1])
setTrackStyleParam(back2back, "ylim", c(0, .5))
viewTracks(trackList(ctcf, back2back, heightDist=c(1, 5)),
           gr=range, autoOptimizeStyle = TRUE)
addInteractionAnnotation(tads, "back2back", grid.lines,
                         gp=gpar(col="cyan", lwd=3, lty=2))
addInteractionAnnotation(-52208000, "back2back", gp=gpar(col="blue", lwd=3),
                         panel="top")
addInteractionAnnotation(51508000, "back2back", gp=gpar(col="gray", lwd=3, lty=2),
                         panel="bottom")

## ----plotLinks, fig.width=6, fig.height=3-------------------------------------
setTrackStyleParam(heatmap, "tracktype", "link")
setTrackStyleParam(heatmap, "breaks", 
                   c(seq(from=0, to=50, by=10), 200))
setTrackStyleParam(heatmap, "color",
                   c("lightblue", "yellow", "red"))
## filter the links to simulate the real data
keep <- distance(heatmap$dat, heatmap$dat2) > 5e5 & heatmap$dat$score>20
heatmap$dat <- heatmap$dat[keep]
heatmap$dat2 <- heatmap$dat2[keep]
viewTracks(trackList(heatmap), gr=range, autoOptimizeStyle = TRUE)

## ----plotHeatmapLink, fig.width=6, fig.height=4-------------------------------
heatmapLinks <- gi2track(gi, gi2[keep])
## change the color
setTrackStyleParam(heatmapLinks, "breaks", 
                   c(seq(from=0, to=50, by=10), 200))
setTrackStyleParam(heatmapLinks, "color",
                   c("lightblue", "yellow", "red"))
## chang the lim of y-axis (by default, [0, 1])
setTrackStyleParam(heatmapLinks, "ylim", c(0, .5))
setTrackStyleParam(heatmapLinks, "tracktype", c("heatmap", "link"))
setTrackStyleParam(heatmapLinks, "ysplit", 0.75) # heatmap space 75% of height
viewTracks(trackList(heatmapLinks), gr=range, autoOptimizeStyle = TRUE)
addInteractionAnnotation(tads, "heatmapLinks", grid.lines,
                         gp=gpar(col="cyan", lwd=3, lty=2), panel='top')
addInteractionAnnotation(-52208000, "heatmapLinks", gp=gpar(col="blue", lwd=3),
                         panel="top")

## ----inportHic----------------------------------------------------------------
hic <- system.file("extdata", "test_chr22.hic", package = "trackViewer",
                    mustWork=TRUE)
if(.Platform$OS.type!="windows"){
importGInteractions(file=hic, format="hic",
                    ranges=GRanges("22", IRanges(50000000, 100000000)),
                    out = "GInteractions")
}

## ----importCool, eval=FALSE---------------------------------------------------
# cool <- system.file("extdata", "test.mcool", package = "trackViewer",
#                      mustWork=TRUE)
# importGInteractions(file=cool, format="cool",
#                     resolution = 2,
#                     ranges=GRanges("chr1", IRanges(10, 28)),
#                     out = "GInteractions")

## ----sessionInfo, results='asis'----------------------------------------------
sessionInfo()

## ----echo=FALSE, results="hide", warning=FALSE--------------------------------
suppressPackageStartupMessages({
    library(trackViewer)
    library(rtracklayer)
    library(Gviz)
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    library(org.Hs.eg.db)
    library(VariantAnnotation)
  library(httr)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)

## ----plotComp,echo=TRUE,fig.keep='none'---------------------------------------
library(Gviz)
library(rtracklayer)
library(trackViewer)
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED",
                    ranges=gr)
fox2$dat <- coverageGR(fox2$dat)

viewTracks(trackList(fox2), gr=gr, autoOptimizeStyle=TRUE, newpage=FALSE)

dt <- DataTrack(range=fox2$dat[strand(fox2$dat)=="-"] , 
                genome="hg19", type="hist", name="fox2", 
                window=-1, chromosome="chr11", 
                fill.histogram="black", col.histogram="NA",
                background.title="white",
                col.frame="white", col.axis="black",
                col="black", col.title="black")
plotTracks(dt, from=122929275, to=122930122, strand="-")

## ----Gviz,echo=FALSE,fig.cap='Plot data with **Gviz** and **trackViewer**. Please note that **trackViewer** can generate similar figure as **Gviz** with several lines of simple codes.',fig.width=8,fig.height=3----
viewerStyle <- trackViewerStyle()
setTrackViewerStyleParam(viewerStyle, "margin", c(.01, .13, .02, .02))
empty <- DataTrack(showAxis=FALSE, showTitle=FALSE, background.title="white")
plotTracks(list(empty, dt), from=122929275, to=122930122, strand="-")
pushViewport(viewport(0, .5, 1, .5, just=c(0, 0)))
viewTracks(trackList(fox2), viewerStyle=viewerStyle, 
                 gr=gr, autoOptimizeStyle=TRUE, newpage=FALSE)
popViewport()
grid.text(label="Gviz track", x=.3, y=.4)
grid.text(label="trackViewer track", x=.3, y=.9)

## ----lostcode,echo=TRUE,fig.keep='none'---------------------------------------
gr <- GRanges("chr1", IRanges(c(1, 6, 10), c(3, 6, 12)), score=c(3, 4, 1))
dt <- DataTrack(range=gr, data="score", type="hist")
plotTracks(dt, from=2, to=11)
tr <- new("track", dat=gr, type="data", format="BED")
viewTracks(trackList(tr), chromosome="chr1", start=2, end=11)

## ----GvizLost,echo=FALSE,fig.cap='Plot data with **Gviz** and **trackViewer**. Note that **trackViewer** is not only including more details but also showing all the data involved in the given range.',fig.width=8,fig.height=3----
plotTracks(list(empty, dt), from=2, to=11)
pushViewport(viewport(0, .5, 1, .5, just=c(0, 0)))
viewTracks(trackList(tr), viewerStyle=viewerStyle, 
           chromosome="chr1", start=2, end=11, 
           autoOptimizeStyle=TRUE, newpage=FALSE)
popViewport()
grid.text(label="Gviz track", x=.3, y=.4)
grid.text(label="trackViewer track", x=.3, y=.9)

## ----importData---------------------------------------------------------------
library(trackViewer)
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
repA <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                    file.path(extdata, "cpsf160.repA_+.wig"),
                    format="WIG")
## Because the wig file does not contain any strand info, 
## we need to set it manually.
strand(repA$dat) <- "-"
strand(repA$dat2) <- "+"

## ----coverage-----------------------------------------------------------------
fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED",
                    ranges=GRanges("chr11", IRanges(122830799, 123116707)))
dat <- coverageGR(fox2$dat)
## We can split the data by strand into two different track channels
## Here, we set the dat2 slot to save the negative strand info. 
 
fox2$dat <- dat[strand(dat)=="+"]
fox2$dat2 <- dat[strand(dat)=="-"]

## ----geneModel----------------------------------------------------------------
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)

## ----geneTrack----------------------------------------------------------------
entrezIDforFMR1 <- get("FMR1", org.Hs.egSYMBOL2EG)
theTrack <- geneTrack(entrezIDforFMR1,TxDb.Hsapiens.UCSC.hg19.knownGene)[[1]]

## ----viewTracks,fig.cap='plot data and annotation information along genomic coordinates',fig.width=8,fig.height=3----
viewerStyle <- trackViewerStyle()
setTrackViewerStyleParam(viewerStyle, "margin", c(.1, .05, .02, .02))
trackList <- trackList(repA, fox2, trs)
vp <- viewTracks(trackList, 
                 gr=gr, viewerStyle=viewerStyle, 
                 autoOptimizeStyle=TRUE)
addGuideLine(c(122929767, 122929969), vp=vp)
addArrowMark(list(x=122929650, 
                  y=2), # 2 means track 2 from the bottom.
             label="label",
             col="blue",
             vp=vp)

## ----browseTrack,fig.cap='interactive tracks',fig.width=6,fig.height=4--------
browseTracks(trackList, gr=gr)

## ----plotgeneTrack,fig.cap='Plot multiple genes in one track',fig.width=6,fig.height=4----
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
grW <- parse2GRanges("chr11:122,830,799-123,116,707")
ids <- getGeneIDsFromTxDb(grW, TxDb.Hsapiens.UCSC.hg19.knownGene)
symbols <- mget(ids, org.Hs.egSYMBOL)
genes <- geneTrack(ids, TxDb.Hsapiens.UCSC.hg19.knownGene, 
                   symbols, asList=FALSE)
# set color to the gene
genes$dat$color <- as.numeric(factor(genes$dat$featureID))
optSty <- optimizeStyle(trackList(repA, fox2, genes), theme="safe")
trackListW <- optSty$tracks
viewerStyleW <- optSty$style
viewTracks(trackListW, gr=grW, viewerStyle=viewerStyleW)

## ----viewTracksOperator1,fig.cap='show data with operator "+"',fig.width=8,fig.height=4----
newtrack <- repA
## Must keep the same format for dat and dat2
newtrack <- parseWIG(newtrack, "chr11", 122929275, 122930122)
newtrack$dat2 <- newtrack$dat
newtrack$dat <- fox2$dat2
setTrackStyleParam(newtrack, "color", c("#4A95E0", "#CF5D6D"))
viewTracks(trackList(newtrack, trs), 
           gr=gr, viewerStyle=viewerStyle, operator="+")

## ----viewTracksOperator2,fig.cap='show data with operator "-"',fig.width=8,fig.height=4----
viewTracks(trackList(newtrack, trs), gr=gr, viewerStyle=viewerStyle, operator="-")

## ----viewTracksOperator3,fig.cap='show data with multiple operators', fig.width=8,fig.height=5----
viewTracks(trackList(newtrack, newtrack, newtrack, trs), gr=gr, viewerStyle=viewerStyle, operator=c(NA, "-", "+"))

## ----viewTracksOperator4,fig.cap='show data with operator "-"',fig.width=8,fig.height=4----
newtrack$dat <- GRoperator(newtrack$dat, newtrack$dat2, col="score", operator="-")
newtrack$dat2 <- GRanges()
viewTracks(trackList(newtrack, trs), gr=gr, viewerStyle=viewerStyle)

## -----------------------------------------------------------------------------
library(trackViewer)
features <- GRanges("chr1", IRanges(c(1, 501, 1001), 
                                    width=c(120, 400, 405),
                                    names=paste0("block", 1:3)),
                    fill = c("#FF8833", "#51C6E6", "#DFA32D"),
                    height = c(0.02, 0.05, 0.08))
SNP <- c(10, 100, 105, 108, 400, 410, 420, 600, 700, 805, 840, 1400, 1402)
sample.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)),
                     color = sample.int(6, length(SNP), replace=TRUE),
                     score = sample.int(5, length(SNP), replace = TRUE))
lolliplot(sample.gr, features)


## ----fig.width=4.5,fig.height=3-----------------------------------------------
library(trackViewer)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
methy <- import(system.file("extdata", "methy.bed", package="trackViewer"), "BED")
gr <- GRanges("chr22", IRanges(50968014, 50970514, names="TYMP"))
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)
features <- c(range(trs[[1]]$dat), range(trs[[5]]$dat))
names(features) <- c(trs[[1]]$name, trs[[5]]$name)
features$fill <- c("lightblue", "mistyrose")
features$height <- c(.02, .04)
dandelion.plot(methy, features, ranges=gr, type="pin")

## -----------------------------------------------------------------------------
library(InteractionSet)
gi <- readRDS(system.file("extdata", "nij.chr6.51120000.53200000.gi.rds", package="trackViewer"))
head(gi)
range <- GRanges("chr6", IRanges(51120000, 53200000))
tr <- gi2track(gi)
ctcf <- readRDS(system.file("extdata", "ctcf.sample.rds",
                            package="trackViewer"))
## change the color
setTrackStyleParam(tr, "breaks", 
                   c(seq(from=0, to=50, by=10), 200))
setTrackStyleParam(tr, "color",
                   c("lightblue", "yellow", "red"))
viewTracks(trackList(ctcf, tr, heightDist=c(1, 3)), gr=range,
           autoOptimizeStyle = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# ideo <- loadIdeogram("hg38", chrom=c("chr1", "chr3", "chr4", 'chr21', "chr22"))

## ----echo=FALSE, results="hide"-----------------------------------------------
path <- system.file("extdata", "ideo.hg38.rds", package = "trackViewer")
ideo <- readRDS(path)

## ----eval=FALSE---------------------------------------------------------------
# dataList <- trim(ideo)
# dataList$score <- as.numeric(as.factor(dataList$gieStain))
# dataList <- dataList[dataList$gieStain!="gneg"]
# dataList <- GRangesList(dataList)
# ideogramPlot(ideo, dataList,
#             layout=list("chr1", c("chr3", "chr22"),
#                         c("chr4", "chr21")))

## ----sessionInfo, results='asis'----------------------------------------------
sessionInfo()

## ----echo=FALSE, results="hide", warning=FALSE--------------------------------
suppressPackageStartupMessages({
    library(trackViewer)
    library(rtracklayer)
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    library(org.Hs.eg.db)
    library(VariantAnnotation)
  library(httr)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)

## ----lolliplot1, fig.width=6, fig.height=3------------------------------------
library(trackViewer)
SNP <- c(10, 12, 1400, 1402)
sample.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)))
features <- GRanges("chr1", IRanges(c(1, 501, 1001), 
                                    width=c(120, 400, 405),
                                    names=paste0("block", 1:3)))
lolliplot(sample.gr, features)
## More SNPs
SNP <- c(10, 100, 105, 108, 400, 410, 420, 600, 700, 805, 840, 1400, 1402)
sample.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)))
lolliplot(sample.gr, features)
## Define the range
lolliplot(sample.gr, unname(features), ranges = GRanges("chr1", IRanges(104, 109)))

## ----lolliplot2, fig.width=6, fig.height=3------------------------------------
features$fill <- c("#FF8833", "#51C6E6", "#DFA32D")
lolliplot(sample.gr, features)

## ----lolliplot3, fig.width=6, fig.height=3------------------------------------
sample.gr$color <- sample.int(6, length(SNP), replace=TRUE)
sample.gr$border <- sample(c("gray80", "gray30"), length(SNP), replace=TRUE)
sample.gr$alpha <- sample(100:255, length(SNP), replace = TRUE)/255
lolliplot(sample.gr, features)

## ----lolliplot.index, fig.width=6, fig.height=3-------------------------------
sample.gr$node.label <- as.character(seq_along(sample.gr))
sample.gr$node.label.col <- 
  ifelse(sample.gr$alpha>0.5 | sample.gr$color==1, "white", "black")
sample.gr$node.label.cex <- sample.int(3, length(sample.gr), replace = TRUE)/2
lolliplot(sample.gr, features)
sample.gr$node.label.cex <- 1 ## change it back for pretty showcase.

## ----lolliplot4, fig.width=6, fig.height=3------------------------------------
features$height <- c(0.02, 0.05, 0.08)
lolliplot(sample.gr, features)
## Specifying the height and its unit
features$height <- list(unit(1/16, "inches"),
                        unit(3, "mm"),
                        unit(12, "points"))
lolliplot(sample.gr, features)

## ----lolliplot.mul.features, fig.width=6, fig.height=3------------------------
features.mul <- rep(features, 2)
features.mul$height[4:6] <- list(unit(1/8, "inches"),
                                 unit(0.5, "lines"),
                                 unit(.2, "char"))
features.mul$fill <- c("#FF8833", "#F9712A", "#DFA32D", 
                       "#51C6E6", "#009DDA", "#4B9CDF")
end(features.mul)[5] <- end(features.mul[5])+50
features.mul$featureLayerID <- 
    paste("tx", rep(1:2, each=length(features)), sep="_")
names(features.mul) <- 
    paste(features.mul$featureLayerID, 
          rep(1:length(features), 2), sep="_")
lolliplot(sample.gr, features.mul)
## One name per transcript
names(features.mul) <- features.mul$featureLayerID
lolliplot(sample.gr, features.mul)

## ----lolliplot4.1, fig.width=6, fig.height=3.5--------------------------------
#Note: the score value is an integer less than 10
sample.gr$score <- sample.int(5, length(sample.gr), replace = TRUE)
lolliplot(sample.gr, features)
##Remove y-axis
lolliplot(sample.gr, features, yaxis=FALSE)

## ----lolliplot4.20, fig.width=6, fig.height=4.5-------------------------------
#Try a score value greater than 10
sample.gr$score <- sample.int(15, length(sample.gr), replace=TRUE)
sample.gr$node.label <- as.character(sample.gr$score)
lolliplot(sample.gr, features)

## ----lolliplot4.21, fig.width=6, fig.height=5---------------------------------
#increase the cutoff value of style switch.
lolliplot(sample.gr, features, lollipop_style_switch_limit=15)

## ----lolliplot4.22, fig.width=6, fig.height=4.5-------------------------------
#Try a float numeric score
sample.gr$score <- runif(length(sample.gr))*10
sample.gr$node.label <- as.character(round(sample.gr$score, digits = 1))
lolliplot(sample.gr, features)
# Score should not be smaller than 1
# remove the alpha for following samples
sample.gr$alpha <- NULL

## ----lolliplot.xaxis, fig.width=6, fig.height=4.5-----------------------------
xaxis <- c(1, 200, 400, 701, 1000, 1200, 1402) ## define the position
lolliplot(sample.gr, features, xaxis=xaxis)
names(xaxis) <- xaxis # define the labels
names(xaxis)[4] <- "center" 
lolliplot(sample.gr, features, xaxis=xaxis)

## ----lolliplot.yaxis, fig.width=6, fig.height=4.5-----------------------------
#yaxis <- c(0, 5) ## define the position
#lolliplot(sample.gr, features, yaxis=yaxis)
yaxis <- c(0, 5, 10, 15) ## define the position
names(yaxis) <- yaxis # define the labels
names(yaxis)[3] <- "y-axis" 
lolliplot(sample.gr, features, yaxis=yaxis)

## ----lolliplot.jitter, fig.width=6, fig.height=4.5----------------------------
sample.gr$dashline.col <- sample.gr$color
lolliplot(sample.gr, features, jitter="label")

## ----lolliplot.legend, fig.width=6, fig.height=4.5----------------------------
legend <- 1:6 ## legend fill color
names(legend) <- paste0("legend", letters[1:6]) ## legend labels
lolliplot(sample.gr, features, legend=legend)
## use list to define more attributes. see ?grid::gpar to get more details.
legend <- list(labels=paste0("legend", LETTERS[1:6]), 
               col=palette()[6:1], 
               fill=palette()[legend])
lolliplot(sample.gr, features, legend=legend)
## if you have multiple tracks, please try to set the legend by list.
## see more examples in the section [Plot multiple samples](#plot-multiple-samples)
legendList <- list(legend)
lolliplot(sample.gr, features, legend=legendList)

# from version 1.21.8, users can also try to set legend 
# as a column name in the metadata of GRanges.
sample.gr.newlegend <- sample.gr
sample.gr.newlegend$legend <- LETTERS[sample.gr$color]
lolliplot(sample.gr.newlegend, features, legend="legend")
# from version 1.41.6, users can set the legend position to right
lolliplot(sample.gr, features, legend=legend,
          legendPosition = list(position='right',
                                width=unit(1, 'inch')))
## use ncol or nrow to control the legend layout.
legendList[[1]]$ncol <- 2 # if legend is not a list of list, use legend$ncol <- 2
lolliplot(sample.gr, features, legend=legendList, legendPosition = 'right')

## ----lolliplot.labels.control, fig.width=6, fig.height=4----------------------
sample.gr.rot <- sample.gr
sample.gr.rot$label.parameter.rot <- 45
lolliplot(sample.gr.rot, features, legend=legend)
sample.gr.rot$label.parameter.rot <- 60
sample.gr.rot$label.parameter.col <- "brown"
## change the label text into user-defined names other than names of the sample.gr
sample.gr.rot$label.parameter.label <- names(sample.gr)
random_ids <- sample(seq_along(sample.gr), 5)
sample.gr.rot$label.parameter.label[random_ids] <- 
  paste("new label", random_ids)
random_ids <- sample(seq_along(sample.gr), 2)
sample.gr.rot$label.parameter.label[random_ids] <- NA ## remove some labels
lolliplot(sample.gr.rot, features, legend=legend)
## try different colors
sample.gr.rot$label.parameter.col <- sample.int(7,
                                                length(sample.gr),
                                                replace = TRUE)
sample.gr.rot$label.parameter.draw <- TRUE
sample.gr.rot$label.parameter.draw[[1]] <- FALSE ## another method to remove the first label
lolliplot(sample.gr.rot, features, legend=legend)

## ----lolliplot.labels.ctl.one.by.one, fig.width=6, fig.height=4---------------
label.parameter.gp.brown <- gpar(col="brown")
label.parameter.gp.blue <- gpar(col="blue")
label.parameter.gp.red <- gpar(col="red")
sample.gr$label.parameter.gp <- sample(list(label.parameter.gp.blue,
                                            label.parameter.gp.brown,
                                            label.parameter.gp.red),
                                       length(sample.gr), replace = TRUE)
lolliplot(sample.gr, features)

## ----lolliplot.labels.on.feature, fig.width=6, fig.height=4-------------------
lolliplot(sample.gr, features, label_on_feature=TRUE)

## ----lolliplot.xlab.ylab.title, fig.width=6, fig.height=5.2-------------------
lolliplot(sample.gr.rot, features, legend=legend, ylab="y label here")
grid.text("label of x-axis here", x=.5, y=.01, just="bottom")
grid.text("title here", x=.5, y=.98, just="top", 
          gp=gpar(cex=1.5, fontface="bold"))

## ----lolliplot.motiflogo, fig.width=6, fig.height=5.2-------------------------
library(motifStack)
pcms<-readPCM(file.path(find.package("motifStack"), "extdata"),"pcm$")
sample.gr.rot$label.parameter.pfm <- pcms[sample(seq_along(pcms),
                                                 length(sample.gr.rot),
                                                 replace = TRUE)]
lolliplot(sample.gr.rot, features, legend=legend)

## ----lolliplotShape, fig.width=6, fig.height=4.5------------------------------
## shape must be "circle", "square", "diamond", "triangle_point_up", or "triangle_point_down"
available.shapes <- c("circle", "square", "diamond", 
                      "triangle_point_up", "triangle_point_down")
sample.gr$shape <- sample(available.shapes, size = length(sample.gr), replace = TRUE)
sample.gr$legend <- paste0("legend", as.numeric(factor(sample.gr$shape)))
lolliplot(sample.gr, features, type="circle", legend = "legend")
sample.gr.mul.shape <- sample.gr
sample.gr.mul.shape$score <- ceiling(sample.gr.mul.shape$score)
sample.gr.mul.shape$shape <- lapply(sample.gr.mul.shape$score, function(s){
  sample(available.shapes, size = s, replace = TRUE)
})
sample.gr.mul.shape$color <- lapply(sample.gr.mul.shape$score, function(s){
  sample.int(7, size = s, replace = TRUE)
})
lolliplot(sample.gr.mul.shape, features, type="circle",
          lollipop_style_switch_limit = max(sample.gr.mul.shape$score))

## ----lolliplot5, fig.width=6, fig.height=4.5----------------------------------
lolliplot(sample.gr, features, type="pin")
sample.gr$color <- lapply(sample.gr$color, function(.ele) c(.ele, sample.int(6, 1)))
sample.gr$border <- sample.int(6, length(SNP), replace=TRUE)
lolliplot(sample.gr, features, type="pin")

## ----lolliplotFlag, fig.width=6, fig.height=4---------------------------------
sample.gr.flag <- sample.gr
sample.gr.flag$label <- names(sample.gr) ## move the names to metadata:label
names(sample.gr.flag) <- NULL
#lolliplot(sample.gr.flag, features, 
#          ranges=GRanges("chr1", IRanges(0, 1600)), ## use ranges to leave more space on the right margin.
#          type="flag")
## change the flag rotation angle
sample.gr.flag$node.label.rot <- 15
sample.gr.flag$node.label.rot[c(2, 5)] <- c(60, -15)
sample.gr.flag$label[7] <- "I have a long name"
sample.gr.flag$node.label.cex <- 1
sample.gr.flag$node.label.cex[4] <- 2
lolliplot(sample.gr.flag, features, 
          ranges=GRanges("chr1", IRanges(0, 1600)),## use ranges to leave more space on the right margin. 
          type="flag")

## ----lolliplot6, fig.width=6, fig.height=3------------------------------------
sample.gr$score <- NULL ## must be removed, because pie will consider all the numeric columns except column "color", "fill", "alpha", "shape", "lwd", "id" and "id.col".
sample.gr$label <- NULL
sample.gr$node.label.col <- NULL
x <- sample.int(100, length(SNP))
sample.gr$value1 <- x
sample.gr$value2 <- 100 - x # for pie plot, 2 value columns are required.
## the length of the color should be no less than that of value1 or value2
sample.gr$color <- rep(list(c("#87CEFA", '#98CE31')), length(SNP))
sample.gr$border <- "gray30"
lolliplot(sample.gr, features, type="pie")

## ----lolliplot7, fig.width=6, fig.height=5.5----------------------------------
SNP2 <- sample(4000:8000, 30)
x2 <- sample.int(100, length(SNP2), replace=TRUE)
sample2.gr <- GRanges("chr3", IRanges(SNP2, width=1, names=paste0("snp", SNP2)), 
             value1=x2, value2=100-x2)
sample2.gr$color <- rep(list(c('#DB7575', '#FFD700')), length(SNP2))
sample2.gr$border <- "gray30"

features2 <- GRanges("chr3", IRanges(c(5001, 5801, 7001), 
                                    width=c(500, 500, 405),
                                    names=paste0("block", 4:6)),
                    fill=c("orange", "gray30", "lightblue"),
                    height=unit(c(0.5, 0.3, 0.8), "cm"))
legends <- list(list(labels=c("WT", "MUT"), fill=c("#87CEFA", '#98CE31')), 
                list(labels=c("WT", "MUT"), fill=c('#DB7575', '#FFD700')))
lolliplot(list(A=sample.gr, B=sample2.gr), 
          list(x=features, y=features2), 
          type="pie", legend=legends)

## ----lolliplot.multiple.type, fig.width=6, fig.height=7.5---------------------
sample2.gr$score <- sample2.gr$value1 ## The circle layout needs the score column 
lolliplot(list(A=sample.gr, B=sample2.gr), 
          list(x=features, y=features2), 
          type=c("pie", "circle"), legend=legends)

## ----lolliplot.pie.stack, fig.width=6, fig.height=5---------------------------
rand.id <- sample.int(length(sample.gr), 3*length(sample.gr), replace=TRUE)
rand.id <- sort(rand.id)
sample.gr.mul.patient <- sample.gr[rand.id]
## pie.stack require metadata "stack.factor", and the metadata can not be 
## stack.factor.order or stack.factor.first
len.max <- max(table(rand.id))
stack.factors <- paste0("patient", formatC(1:len.max, 
                                           width=nchar(as.character(len.max)), 
                                           flag="0"))
sample.gr.mul.patient$stack.factor <- 
    unlist(lapply(table(rand.id), sample, x=stack.factors))
sample.gr.mul.patient$value1 <- 
    sample.int(100, length(sample.gr.mul.patient), replace=TRUE)
sample.gr.mul.patient$value2 <- 100 - sample.gr.mul.patient$value1
patient.color.set <- as.list(as.data.frame(rbind(rainbow(length(stack.factors)), 
                                                 "#FFFFFFFF"), 
                                           stringsAsFactors=FALSE))
names(patient.color.set) <- stack.factors
sample.gr.mul.patient$color <- 
    patient.color.set[sample.gr.mul.patient$stack.factor]
legend <- list(labels=stack.factors, col="gray80", 
               fill=sapply(patient.color.set, `[`, 1))
## remove one mutation label
sample.gr.mul.patient$label.parameter.draw <- TRUE
sample.gr.mul.patient$label.parameter.draw[
  names(sample.gr.mul.patient)==
    sample(unique(names(sample.gr.mul.patient)), 1)] <-
  FALSE
lolliplot(sample.gr.mul.patient, features, type="pie.stack", 
          legend=legend, dashline.col="gray")

## ----lolliplot.caterpillar, fig.width=6, fig.height=4-------------------------
sample.gr$SNPsideID <- sample(c("top", "bottom"), 
                               length(sample.gr),
                               replace=TRUE)
lolliplot(sample.gr, features, type="pie", 
          legend=legends[[1]])

## ----lolliplot.caterpillar2, fig.width=6, fig.height=12-----------------------
## Two layers
sample2.gr$SNPsideID <- "top"
idx <- sample.int(length(sample2.gr), 15)
sample2.gr$SNPsideID[idx] <- "bottom"
sample2.gr$color[idx] <- '#FFD700'
lolliplot(list(A=sample.gr, B=sample2.gr), 
          list(x=features.mul, y=features2), 
          type=c("pie", "circle"), legend=legends)

## ----ProteinsAPI, fig.width=6, fig.height=3, eval=FALSE, echo=TRUE------------
# library(httr) # load library to get data from REST API
# APIurl <- "https://www.ebi.ac.uk/proteins/api/" # base URL of the API
# taxid <- "9606" # human tax ID
# gene <- "TP53" # target gene
# orgDB <- "org.Hs.eg.db" # org database to get the uniprot accession id
# eid <- mget("TP53", get(sub(".db", "SYMBOL2EG", orgDB)))[[1]]
# chr <- mget(eid, get(sub(".db", "CHR", orgDB)))[[1]]
# accession <- unlist(lapply(eid, function(.ele){
#   mget(.ele, get(sub(".db", "UNIPROT", orgDB)))
# }))
# stopifnot(length(accession)<=20) # max number of accession is 20
# 
# tryCatch({ ## in case the internet connection does not work
#   featureURL <- paste0(APIurl,
#                        "features?offset=0&size=-1&reviewed=true",
#                        "&types=DNA_BIND%2CMOTIF%2CDOMAIN",
#                        "&taxid=", taxid,
#                        "&accession=", paste(accession, collapse = "%2C")
#   )
#   response <- GET(featureURL)
#   if(!http_error(response)){
#     content <- httr::content(response)
#     content <- content[[1]]
#     acc <- content$accession
#     sequence <- content$sequence
#     gr <- GRanges(chr, IRanges(1, nchar(sequence)))
#     domains <- do.call(rbind, content$features)
#     domains <- GRanges(chr, IRanges(as.numeric(domains[, "begin"]),
#                                      as.numeric(domains[, "end"]),
#                                      names = domains[, "description"]))
#     names(domains)[1] <- "DNA_BIND" ## this is hard coding.
#     domains$fill <- 1+seq_along(domains)
#     domains$height <- 0.04
#     ## GET variations. This part can be replaced by user-defined data.
#     variationURL <- paste0(APIurl,
#                            "variation?offset=0&size=-1",
#                            "&sourcetype=uniprot&dbtype=dbSNP",
#                            "&taxid=", taxid,
#                            "&accession=", acc)
#     response <- GET(variationURL)
#     if(!http_error(response)){
#       content <- httr::content(response)
#       content <- content[[1]]
#       keep <- sapply(content$features, function(.ele) length(.ele$evidences)>2 && # filter the data by at least 2 evidences
#                        !grepl("Unclassified", .ele$clinicalSignificances)) # filter the data by classified clinical significances.
#       nkeep <- c("wildType", "alternativeSequence", "begin", "end",
#                  "somaticStatus", "consequenceType", "score")
#       content$features <- lapply(content$features[keep], function(.ele){
#         .ele$score <- length(.ele$evidences)
#         unlist(.ele[nkeep])
#       })
#       variation <- do.call(rbind, content$features)
#       variation <-
#         GRanges(chr,
#                 IRanges(as.numeric(variation[, "begin"]),
#                         width = 1,
#                         names = paste0(variation[, "wildType"],
#                                        variation[, "begin"],
#                                        variation[, "alternativeSequence"])),
#                 score = as.numeric(variation[, "score"]),
#                 color = as.numeric(factor(variation[, "consequenceType"]))+1)
#       variation$label.parameter.gp <- gpar(cex=.5)
#       lolliplot(variation, domains, ranges = gr, ylab = "# evidences", yaxis = FALSE)
#     }else{
#       message("Can not get variations. http error")
#     }
#   }else{
#     message("Can not get features. http error")
#   }
# },error=function(e){
#   message(e)
# },warning=function(w){
#   message(w)
# },interrupt=function(i){
#   message(i)
# })

## ----VCF, fig.width=6, fig.height=5-------------------------------------------
library(VariantAnnotation)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
fl <- system.file("extdata", "chr22.vcf.gz", package="VariantAnnotation")
gr <- GRanges("22", IRanges(50968014, 50970514, names="TYMP"))
if(.Platform$OS.type!="windows"){# This line is for avoiding error from VariantAnnotation in the windows platform, which will be removed when VariantAnnotation's issue gets fixed.
tab <- TabixFile(fl)
vcf <- readVcf(fl, "hg19", param=gr)
mutation.frequency <- rowRanges(vcf)
mcols(mutation.frequency) <- cbind(mcols(mutation.frequency), 
                                   VariantAnnotation::info(vcf))
mutation.frequency$border <- "gray30"
mutation.frequency$color <- 
    ifelse(grepl("^rs", names(mutation.frequency)), "lightcyan", "lavender")
## Plot Global Allele Frequency based on AC/AN
mutation.frequency$score <- mutation.frequency$AF*100
seqlevelsStyle(mutation.frequency) <- "UCSC"
if(!grepl("chr", seqlevels(mutation.frequency)[1])){
  seqlevels(mutation.frequency) <- 
    paste0("chr", seqlevels(mutation.frequency))
}
}
seqlevelsStyle(gr) <- "UCSC" 
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)
features <- c(range(trs[[1]]$dat), range(trs[[5]]$dat))
names(features) <- c(trs[[1]]$name, trs[[5]]$name)
features$fill <- c("lightblue", "mistyrose")
features$height <- c(.02, .04)
if(.Platform$OS.type!="windows"){
lolliplot(mutation.frequency, features, ranges=gr)
}

## ----methylation, eval=FALSE, echo=TRUE---------------------------------------
# library(rtracklayer)
# session <- browserSession()
# query <- ucscTableQuery(session,
#                         table="wgEncodeHaibMethylRrbs",
#                         range=GRangesForUCSCGenome("hg19",
#                                                    seqnames(gr),
#                                                    ranges(gr)))
# tableName(query) <- tableNames(query)[1]
# methy <- track(query)
# methy <- GRanges(methy)

## ----methylation.hide, echo=FALSE---------------------------------------------
methy <- import(system.file("extdata", "methy.bed", package="trackViewer"), "BED")

## ----fig.width=6,fig.height=4-------------------------------------------------
lolliplot(methy, features, ranges=gr, type="pin")

## ----fig.width=6,fig.height=2.5-----------------------------------------------
methy$lwd <- .5
lolliplot(methy, features, ranges=gr, type="pin", cex=.5)
#lolliplot(methy, features, ranges=gr, type="circle", cex=.5)
methy$score2 <- max(methy$score) - methy$score
lolliplot(methy, features, ranges=gr, type="pie", cex=.5)
## We can change it one by one
methy$cex <- runif(length(methy))
lolliplot(methy, features, ranges=gr, type="pin")
#lolliplot(methy, features, ranges=gr, type="circle")

## ----fig.width=6,fig.height=2.5-----------------------------------------------
methy$cex <- 1
lolliplot(methy, features, ranges=gr, rescale = TRUE)
## by set percentage for features and non-features segments
xaxis <- c(50968014, 50968514, 50968710, 50968838, 50970514)
rescale <- c(.3, .4, .3)
lolliplot(methy, features, ranges=gr, type="pin",
          rescale = rescale, xaxis = xaxis)
## by set data.frame to rescale
rescale <- data.frame(
  from.start = c(50968014, 50968515, 50968838), 
  from.end   = c(50968514, 50968837, 50970514),
  to.start   = c(50968014, 50968838, 50969501), 
  to.end     = c(50968837, 50969500, 50970514)
)
lolliplot(methy, features, ranges=gr, type="pin",
          rescale = rescale, xaxis = xaxis)

## ----fig.width=6,fig.height=2.5-----------------------------------------------
lolliplot(methy, features, ranges=gr, rescale = "exon")
# exon region occupy 99% of the plot region.
lolliplot(methy, features, ranges=gr, rescale = "exon_99")
lolliplot(methy, features, ranges=gr, rescale = "intron")

## ----fig.width=9,fig.height=8-------------------------------------------------
grSplited <- tile(gr, n=2)
lolliplot(methy, features, ranges=grSplited, type="pin")

## ----fig.width=8,fig.height=4-------------------------------------------------
gene <- geneTrack(get("HSPA8", org.Hs.egSYMBOL2EG), TxDb.Hsapiens.UCSC.hg19.knownGene)[[1]]
SNPs <- GRanges("chr11", IRanges(sample(122929275:122930122, size = 20), width = 1), strand="-")
SNPs$score <- sample.int(5, length(SNPs), replace = TRUE)
SNPs$color <- sample.int(6, length(SNPs), replace=TRUE)
SNPs$border <- "gray80"
SNPs$feature.height = .1
gene$dat2 <- SNPs
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
repA <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                    file.path(extdata, "cpsf160.repA_+.wig"),
                    format="WIG")
fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED",
                    ranges=GRanges("chr11", IRanges(122830799, 123116707)))
optSty <- optimizeStyle(trackList(repA, fox2, gene), theme="col")
trackList <- optSty$tracks
viewerStyle <- optSty$style
gr <- GRanges("chr11", IRanges(122929275, 122930122))
setTrackStyleParam(trackList[[3]], "ylabgp", list(cex=.8))
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
## lollipopData track
SNPs2 <- GRanges("chr11", IRanges(sample(122929275:122930122, size = 30), width = 1), strand="-")
SNPs2 <- c(SNPs2, promoters(gene$dat, upstream = 0, downstream = 1))
SNPs2$score <- sample.int(3, length(SNPs2), replace = TRUE)
SNPs2$color <- sample.int(6, length(SNPs2), replace=TRUE)
SNPs2$border <- "gray30"
SNPs2$feature.height = .1
lollipopData <- new("track", dat=SNPs, dat2=SNPs2, type="lollipopData")
gene <- geneTrack(get("HSPA8", org.Hs.egSYMBOL2EG), TxDb.Hsapiens.UCSC.hg19.knownGene)[[1]]
optSty <- optimizeStyle(trackList(repA, lollipopData, gene, heightDist = c(3, 3, 1)), theme="col")
trackList <- optSty$tracks
viewerStyle <- optSty$style
gr <- GRanges("chr11", IRanges(122929275, 122930122))
setTrackStyleParam(trackList[[2]], "ylabgp", list(cex=.8))
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
addGuideLine(122929538, vp=vp)
## plot with customized geneTrack
dat <- gene$dat
mcols(dat) <- NULL
dat <- subsetByOverlaps(dat, gr)
dat$feature <- 'exon' # feature is required
dat$featureID <- paste0('name', seq_along(dat)) # treat each as single exon gene
dat$color <- sample(seq.int(7), length(dat), replace = TRUE) # set the color
dat$height <- sample(c(0.5, 1, 2), length(dat), replace = TRUE) # set the height
dat$hide_label <- TRUE # do not add labels to the block
gene <- new('track', dat=dat, dat2=SNPs, type='gene', name='a name')
optSty <- optimizeStyle(trackList(fox2, gene), theme="col")
trackList <- optSty$tracks
viewerStyle <- optSty$style
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----sessionInfo, results='asis'----------------------------------------------
sessionInfo()

test_cool<-function(){
  cool <- system.file("extdata", "test.mcool", package = "trackViewer",
                      mustWork=TRUE)
  r1 <- listResolutions(cool, format='cool')
  n1 <- listNormalizations(cool, format='cool')
  c1 <- listChromosomes(cool, format='cool')
  one <- importGInteractions(file=cool, format="cool",
                      resolution = 2,
                      ranges=GRanges("chr1", IRanges(10, 28)),
                      out = "GInteractions")
  cool <- system.file("extdata", "test.cool", package = "trackViewer",
                      mustWork=TRUE)
  r2 <- listResolutions(cool, format='cool')
  n2 <- listNormalizations(cool, format='cool')
  c2 <- listChromosomes(cool, format='cool')
  two <- importGInteractions(file=cool, format="cool",
                      ranges=GRanges("chr1", IRanges(10, 28)),
                      out = "GInteractions")
  checkIdentical(one, two)
}

#' convert GInteractions to track object
#' @description Convert GInteractions object to track object
#' @param gi an object of GInteractions
#' @param gi2 an object of GInteractions
#' @return an track object
#' @importFrom S4Vectors first second
#' @export
#' @examples 
#' gi <- readRDS(system.file("extdata", "nij.chr6.51120000.53200000.gi.rds", package="trackViewer"))
#' gi2track(gi)
gi2track <- function(gi, gi2){
  stopifnot(is(gi, "GInteractions"))
  .oneGI2track <- function(.gi){
    a <- first(.gi)
    if(length(.gi$score)==length(a)){
      a$score <- .gi$score
    }else{
      a$score <- 1
    }
    if(length(.gi$border_color)==length(a)){
      a$border_color <- .gi$border_color
    }
    if(length(.gi$tads)==length(a)){
      a$tads <- .gi$tads
    }
    return(new("track", dat=a, dat2=second(.gi),
               type="interactionData", format="BED"))
  }
  .gi2gr <- function(.gi){
    a <- first(.gi)
    a$target <- second(.gi)
    if(length(.gi$score)==length(a)){
      a$score <- .gi$score
    }else{
      a$score <- 1
    }
    if(length(.gi$border_color)==length(a)){
      a$border_color <- .gi$border_color
    }
    if(length(.gi$tads)==length(a)){
      a$tads <- .gi$tads
    }
    return(a)
  }
  if(!missing(gi2)){
    if(length(gi2)>0){
      if(length(gi)==0){
        return(.oneGI2track(gi2))
      }else{
        ## put the second coordinates to target metadata
        return(new("track", dat=.gi2gr(gi),
                   dat2=.gi2gr(gi2),
                   type="interactionData",
                   format="BED"))
      }
    }
  }
  if(length(gi)==0){
    return(new("track", dat=GRanges(), dat2=GRanges(),
               type="interactionData", format="BED"))
  }
  return(.oneGI2track(gi))
}

grid.pie <- function (x=.5, y=.5, 
                      radius=.8,
                      col=NULL,
                      border=NULL,
                      percent=NULL,
                      edges=100,
                      lwd=1,
                      alpha=1) {
    if(length(percent)>0) percent <- unlist(percent[, sapply(percent, is.numeric)])
    if(length(percent)<1){
        percent <- 1
    }
    percent <- c(0, cumsum(percent)/sum(percent))
    if(any(is.na(percent))){
      warning("There are events with NA number after calculating the percentage.",
              "Please make sure all the events must contain at least one values greater than 0")
      percent[is.na(percent)] <- 0
    }
    if(any(percent<0)){
      stop("There are events with number smaller than 0 which is not expected.",
           "Please double check all the input values.")
    }
    dx <- diff(percent)
    nx <- length(dx)
    if (is.null(col)) 
        col <- c("white", "lightblue", "mistyrose", "lightcyan", 
                 "lavender", "cornsilk")
    if (!is.null(col)) 
        col <- rep_len(col, nx)
    if (!is.null(border)) 
        border <- rep_len(border, nx)
    twopi <- 2 * pi
    ratio.yx <- getYXratio()
    t2xy <- function(t) {
        t2p <- twopi * t + pi/2
        list(x = radius * cos(t2p), y = radius * sin(t2p) * ratio.yx)
    }
    for (i in 1L:nx) {
        n <- max(2, floor(edges * dx[i]))
        P <- t2xy(seq.int(percent[i], percent[i + 1], length.out = n))
        grid.polygon(unit(c(P$x, 0)+x,"npc"), unit(c(P$y, 0)+y, "npc"), gp=gpar(col = border[i], fill = col[i], lwd=lwd, alpha=alpha))
    }
    invisible(NULL)
}

rgb2hex <- function(x){
    hex <- function(a) format(as.hexmode(a), width=2, upper.case=TRUE)
    if(length(x==3))
      paste0("#",hex(x[1]),hex(x[2]),hex(x[3]))
    else
      paste0("#",hex(x[1]),hex(x[2]),hex(x[3]),hex(x[4]))
}
grid.lollipop <- function (x1=.5, y1=.5,
                           x2=.5, y2=.75,
                           y3=.04, y4=.02,
                           radius=.8,
                           col=NULL,
                           border=NULL,
                           percent=NULL,
                           edges=100,
                           type=c("circle", "pie", "pin", "pie.stack", "flag"),
                           ratio.yx=1,
                           pin=NULL,
                           scoreMax,
                           scoreType,
                           id=NA,
                           cex=1, lwd=1,
                           dashline.col="gray80",
                           side=c("top", "bottom"),
                           alpha=NULL,
                           shape=shape){
    side <- match.arg(side)
    stopifnot(is.numeric(c(x1, x2, y1, y2, y3, y4, radius, edges)))
    type <- match.arg(type)
    side <- side!="top"
    if(!type %in% c("pie", "pie.stack")){
        this.score <- if(length(percent$score)>0) percent$score else 1
        if(type=="circle"){
            y0 <- c(y1, y2, y2+y3, y2+y3+y4+(this.score-1)*2*radius*ratio.yx)
            if(scoreType) y0[4] <- y2+y3+y4
            if(side) y0 <- 1 - y0
            grid.lines(x=c(x1, x1, x2, x2), y=y0, 
                       gp=gpar(col=border, lwd=lwd))
            y0 <- c(y2+y3+y4+this.score*2*radius*ratio.yx, 
                    y2+y3+y4+scoreMax*ratio.yx)
            if(scoreType) y0[1] <- y2+y3+y4+this.score*2*radius*ratio.yx
            if(side) y0 <- 1 - y0
            grid.lines(x=c(x2, x2), 
                       y=y0, 
                       gp=gpar(col=dashline.col, lty=3, lwd=lwd))
        }else{
            y0 <- c(y1, y2, y2+y3, y2+y3+y4+(this.score-.5)*2*radius*ratio.yx)
            if(side) y0 <- 1 - y0
            grid.lines(x=c(x1, x1, x2, x2), y=y0, 
                       gp=gpar(col=border, lwd=lwd))
        }
        
    }else{
        if(type=="pie.stack"){
            if(percent$stack.factor.first){
                y0 <- c(y1, y2, y2+y3, y2+y3+y4)
                if(side) y0 <- 1 - y0
                grid.lines(x=c(x1, x1, x2, x2), y=y0, 
                           gp=gpar(col=border, lwd=lwd))
                y0 <- c(y2+y3+y4, y2+y3+y4+scoreMax*ratio.yx)
                if(side) y0 <- 1 - y0
                grid.lines(x=c(x2, x2), 
                           y=y0,
                           gp=gpar(col=dashline.col, lty=3, lwd=lwd))
            }
        }else{
            y0 <- c(y1, y2, y2+y3, y2+y3+y4)
            if(side) y0 <- 1 - y0
            grid.lines(x=c(x1, x1, x2, x2), y=y0, 
                       gp=gpar(col=border, lwd=lwd))
        }
    }
    if(length(pin)>0){
        if(length(border)>0) pin@paths[[2]]@rgb <- rgb2hex(col2rgb(border[1]))
        if(length(col)>0) pin@paths[[1]]@rgb <- rgb2hex(col2rgb(col[1]))
        if(length(col)>1) pin@paths[[3]]@rgb <- rgb2hex(col2rgb(col[2]))
    }
    switch(type,
           circle={
               if(length(border)==0) border <- "black"
               if(length(col)==0) col <- "white"
               if(scoreType){
                   for(i in seq_len(this.score)){
                       y0 <- y2+y3+y4+2*radius*ratio.yx*(i-.5)
                       if(side) y0 <- 1 - y0
                       if(length(shape)==this.score){
                         ## multiple shape for each point
                         this_shape <- shape[i]
                       }else{
                         this_shape <- ifelse(length(shape)>0, shape[1], 'circle')
                       }
                       if(length(border)==this.score){
                         ## multiple border for each point
                         this_border <- border[i]
                       }else{
                         this_border <- border[1]
                       }
                       if(length(col)==this.score){
                         ## multiple color for each point
                         this_col <- col[i]
                       }else{
                         this_col <- col[1]
                       }
                       if(length(lwd)==this.score){
                         ## multiple alpha for each point
                         this_lwd <- lwd[i]
                       }else{
                         this_lwd <- lwd[1]
                       }
                       if(length(alpha)==this.score){
                         ## multiple alpha for each point
                         this_alpha <- alpha[i]
                       }else{
                         this_alpha <- alpha[1]
                       }
                       if(length(this_shape)!=1) this_shape <- 'circle'
                       this_gp <- gpar(col=this_border, fill=this_col, lwd=this_lwd, alpha=this_alpha)
                       switch(this_shape, #"circle", "square", "diamond", "triangle_point_up", "star", or "triangle_point_down"
                              circle=grid.circle1(x=x2, y=y0,
                                                  r=radius*ratio.yx*cex, 
                                                  gp=this_gp),
                              square=grid.square(x=x2, y=y0,
                                                  r=radius*ratio.yx*cex, 
                                                  gp=this_gp),
                              diamond=grid.diamond(x=x2, y=y0,
                                                  r=radius*ratio.yx*cex, 
                                                  gp=this_gp),
                              triangle_point_up=grid.triangle_point_up(x=x2, y=y0,
                                                  r=radius*ratio.yx*cex, 
                                                  gp=this_gp),
                              triangle_point_down=grid.triangle_point_down(x=x2, y=y0,
                                                  r=radius*ratio.yx*cex, 
                                                  gp=this_gp),
                              star=grid.star(x=x2, y=y0,
                                             r=radius*ratio.yx*cex, 
                                             gp=this_gp),
                              grid.circle1(x=x2, y=y0,
                                           r=radius*ratio.yx*cex, 
                                           gp=this_gp))
                       
                   }
               }else{
                   y0 <- y2+y3+y4+(this.score-.5)*2*radius*ratio.yx
                   if(side) y0 <- 1 - y0
                   switch(shape,
                          circle=grid.circle1(x=x2, y=y0,
                                              r=radius*ratio.yx*cex, 
                                              gp=gpar(col=border, fill=col, lwd=lwd, alpha=alpha)),
                          square=grid.square(x=x2, y=y0,
                                             r=radius*ratio.yx*cex, 
                                             gp=gpar(col=border, fill=col, lwd=lwd, alpha=alpha)),
                          diamond=grid.diamond(x=x2, y=y0,
                                               r=radius*ratio.yx*cex, 
                                               gp=gpar(col=border, fill=col, lwd=lwd, alpha=alpha)),
                          triangle_point_up=grid.triangle_point_up(x=x2, y=y0,
                                                                   r=radius*ratio.yx*cex, 
                                                                   gp=gpar(col=border, fill=col, lwd=lwd, alpha=alpha)),
                          triangle_point_down=grid.triangle_point_down(x=x2, y=y0,
                                                                       r=radius*ratio.yx*cex, 
                                                                       gp=gpar(col=border, fill=col, lwd=lwd, alpha=alpha)),
                          star=grid.star(x=x2, y=y0,
                                         r=radius*ratio.yx*cex, 
                                         gp=gpar(col=border, fill=col, lwd=lwd, alpha=alpha)),
                          grid.circle1(x=x2, y=y0,
                                       r=radius*ratio.yx*cex, 
                                       gp=gpar(col=border, fill=col, lwd=lwd, alpha=alpha)))
                   if(!is.na(id$label)){
                       y0 <- y2+y3+(this.score-.5)*2*radius*ratio.yx+y4
                       if(side) y0 <- 1 - y0
                       id$gp$cex <- .75*id$gp$cex
                       id$x <- x2
                       id$y <- y0
                       do.call(grid.text, id)
                   }
               }
               },
           pie={
               y0 <- y2+y3+y4+radius*ratio.yx
               if(side) y0 <- 1 - y0
               grid.pie(x=x2, y=y0, 
                        radius = radius*cex, 
                        col = col, 
                        border = border, 
                        percent=percent,
                        edges=edges,
                        lwd=lwd, alpha=alpha)
               },
           pie.stack={
               y0 <- y2+y3+y4+(2*percent$stack.factor.order-1)*radius*ratio.yx
               if(side) y0 <- 1 - y0
               grid.pie(x=x2, 
                        y=y0, 
                        radius = radius*cex, 
                        col = col, 
                        border = border, 
                        percent=percent[, !colnames(percent) %in% 
                                            c("stack.factor.order", 
                                              "stack.factor.first")],
                        edges=edges,
                        lwd=lwd, alpha=alpha)
               },
           pin={
               y0 <- y2+y3+(this.score-.5)*2*radius*ratio.yx+y4/2
               if(side) y0 <- 1 - y0
               grid.picture(picture=pin, x=x2, 
                            y=y0,
                            width=2*radius*ratio.yx*cex,
                            height=3*radius*ratio.yx*cex+y4)
               if(!is.na(id$label)){
                   y0 <- y2+y3+(this.score-.25)*2*radius*ratio.yx+2*y4/3
                   id$gp$cex <- .5*id$gp$cex
                   id$x <- x2
                   id$y <- y0
                   do.call(grid.text, id)
               }
               },
           flag={
             if(is.na(id$label)){
               id$label <- " "
             }
             this.cex <- id$gp$cex
             LINEH <- as.numeric(convertY(unit(1, "line"), "npc"))*this.cex
             y0 <- y2+y3+(this.score-.5)*2*radius*ratio.yx+y4/2
             if(side) y0 <- 1 - y0
             LINEW <- as.numeric(convertX(stringWidth(paste0("o", id$label, "u")), "npc"))*this.cex
             LINEW <- LINEW * sign(cos(pi*id$rot/180))
             LINEH0 <- LINEW*ratio.yx*tan(pi*id$rot/180)
             grid.polygon(x=c(x2, x2+LINEW, x2+LINEW, x2),
                          y=c(y0, y0+LINEH0, y0+LINEH0+LINEH*1.25, y0+LINEH*1.25),
                          gp=gpar(fill=col, col=border, alpha=alpha))
             id$x <- x2+LINEW*.5
             id$y <- y0 + LINEH*.625+LINEH0*.5
             do.call(grid.text, id)
           },
           grid.pie(x=x2, y=y2+y3+y4+radius*ratio.yx, 
                    radius = radius*cex, 
                    col = col, 
                    border = border, 
                    percent=percent,
                    edges=edges,
                    lwd=lwd, alpha=alpha))
}

jitterLables <- function(coor, xscale, lineW, weight=1.2){
    if(weight==1.2) {
      stopifnot("Please sort your inputs by start position"= 
                  order(coor)==1:length(coor))
    }
    if(weight<0.5) return(coor)
    stopifnot(length(xscale)==2)
    pos <- convertX(unit(coor, "native"), "npc", valueOnly=TRUE)
    pos.diff <- diff(c(0, pos, 1))
    idx <- which(pos.diff < weight*lineW)
    if(length(idx)<1){
        return(coor)
    }
    if(all(idx %in% c(1, length(pos)+1))){
        return(coor)
    }
    idx.diff <- diff(c(-1, idx))
    idx.grp <- rle(idx.diff)
    idx.grp$values[idx.grp$values==1] <- length(pos) + 1:sum(idx.grp$values==1)
    idx.grp <- inverse.rle(idx.grp)
    idx.grp.w <- which(idx.grp>length(pos))-1
    idx.grp.w <- idx.grp.w[idx.grp.w>0]
    idx.grp[idx.grp.w] <- idx.grp[idx.grp.w+1]
    idx.grp <- split(idx, idx.grp)
    flag <- as.numeric(names(idx.grp))>length(pos)
    idx.grp.mul <- lapply(idx.grp[flag], function(.ele){
        c(.ele[1]-1, .ele)
    })
    idx.grp.sin <- lapply(idx.grp[!flag], function(.ele){
        lapply(as.list(.ele), function(.ele){c(.ele-1, .ele)})
    })
    idx.grp.sin <- unlist(idx.grp.sin, recursive = FALSE)
    idx.grp <- c(idx.grp.mul, idx.grp.sin)
    
    adj.pos <- lapply(idx.grp, function(.ele){
        .ele <- .ele[.ele>0 & .ele<=length(pos)]
        this.pos <- pos[.ele]
        names(this.pos) <- .ele
        if(length(this.pos)%%2==1){
            center <- ceiling(length(this.pos)/2)
        }else{
            center <- length(this.pos)/2 + .5
        }
        if(length(this.pos)>5){ ## too much, how to jitter?
            this.pos <- this.pos + 
                ((1:length(this.pos))-center) * (weight-.1) * 
                lineW/ceiling(log(length(this.pos), 5))
        }else{
            this.pos <- this.pos + 
                ((1:length(this.pos))-center) * (weight-.1) * lineW
        }
        this.pos
    })
    names(adj.pos) <- NULL
    adj.pos <- unlist(adj.pos)
    coor[as.numeric(names(adj.pos))] <- adj.pos*diff(xscale)+xscale[1]
    
    Recall(coor, xscale=xscale, lineW=lineW, weight=weight-0.2)
}

reAdjustLabels <- function(coor, lineW){
  # resort
  coor <- sort(coor)
  bins <- ceiling(1/lineW)
  pos <- convertX(unit(coor, "native"), "npc", valueOnly=TRUE)
  pos.bin <- cut(pos, c(-Inf, (0:bins)*lineW, Inf), labels=0:(bins+1), right=FALSE)
  
  ## split the coors by into clusters
  ## give the clusters with more idx more spaces if there are spaces between clusters
  tbl <- table(pos.bin)
  if(all(tbl<2)) return(coor)
  tbl.len <- length(tbl)
  if(tbl.len<3) return(coor)
  loops <- 1000
  loop <- 1
  while(any(tbl==0) && any(tbl>1) && loop < loops){
    tbl.bk <- tbl
    for(i in order(tbl.bk, decreasing=TRUE)){
      if(tbl[i]>1 && tbl.bk[i]==tbl[i]){
        if(i==1){
          if(tbl[2]<tbl[1]){
            half <- sum(tbl[1:2])/2
            tbl[2] <- ceiling(half)
            tbl[1] <- floor(half)
          }
        }else{
          if(i==tbl.len){
            if(tbl[tbl.len]>tbl[tbl.len-1]){
              half <- sum(tbl[(tbl.len-1):tbl.len])/2
              tbl[tbl.len-1] <- ceiling(half)
              tbl[tbl.len] <- floor(half)
            }
          }else{
            if(tbl[i-1]<tbl[i+1]){
              ## i-1 and i should be balanced
              half <- sum(tbl[(i-1):i])/2
              tbl[i-1] <- floor(half)
              tbl[i] <- ceiling(half)
            }else{
              half <- sum(tbl[i:(i+1)])/2
              tbl[i] <- floor(half)
              tbl[i+1] <- ceiling(half)
            }
          }
        }
      }
    }
    loop <- loop + 1
  }
  coef <- unlist(lapply(tbl, function(.ele){
    if(.ele==0) return(0)
    .ele <- seq(from=0, to=1, length.out=.ele+1)
    (.ele[-length(.ele)] + .ele[-1])/2
  }))
  coef <- coef[coef!=0]
  coor <- (rep(as.numeric(names(tbl)), tbl) - 1 + coef) * lineW
  coor <- convertX(unit(coor, "npc"), "native", valueOnly=TRUE)
  coor
}

## ----echo=FALSE, results="hide", warning=FALSE--------------------------------
suppressPackageStartupMessages({
    library(trackViewer)
    library(rtracklayer)
    library(Gviz)
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    library(org.Hs.eg.db)
    library(VariantAnnotation)
  library(httr)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)

## -----------------------------------------------------------------------------
library(trackViewer)
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
repA <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                    file.path(extdata, "cpsf160.repA_+.wig"),
                    format="WIG")
## Because the wig file does not contain any strand info, 
## we need to set it manually.
strand(repA$dat) <- "-"
strand(repA$dat2) <- "+"

fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED",
                    ranges=GRanges("chr11", IRanges(122830799, 123116707)))
dat <- coverageGR(fox2$dat)
## We can split the data by strand into two different track channels
## Here, we set the dat2 slot to save the negative strand info. 
 
fox2$dat <- dat[strand(dat)=="+"]
fox2$dat2 <- dat[strand(dat)=="-"]

library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)


## ----optSty-------------------------------------------------------------------
optSty <- optimizeStyle(trackList(repA, fox2, trs))
trackList <- optSty$tracks
viewerStyle <- optSty$style

## ----browseTrack, fig.width=6,fig.height=4------------------------------------
browseTracks(trackList, gr=gr)

## ----viewTracksXaxis,fig.cap='plot data with x-scale',fig.width=8,fig.height=3----
setTrackViewerStyleParam(viewerStyle, "xaxis", FALSE)
setTrackViewerStyleParam(viewerStyle, "margin", c(.01, .05, .01, .01))
setTrackXscaleParam(trackList[[1]], "draw", TRUE)
setTrackXscaleParam(trackList[[1]], "gp", list(cex=0.8))
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
setTrackXscaleParam(trackList[[1]], attr="position", 
                    value=list(x=122929700, y=3, label=200))
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----viewTracksYaxis,fig.cap='plot data with y-axis in right side',fig.width=8,fig.height=3----
setTrackViewerStyleParam(viewerStyle, "margin", c(.01, .05, .01, .05))
for(i in 1:2){
    setTrackYaxisParam(trackList[[i]], "main", FALSE)
}
## Adjust the limit of y-axis
setTrackStyleParam(trackList[[1]], "ylim", c(0, 25))
setTrackStyleParam(trackList[[2]], "ylim", c(-25, 0))
## The y-axis tick labels can be turn off/on by setting the 'label' to FALSE/TRUE
#setTrackYaxisParam(trackList[[1]], 'label', TRUE)
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----viewTracksYaxisCol,fig.cap='plot data with adjusted track color',fig.width=8,fig.height=3----
## change the 
setTrackYaxisParam(trackList[[1]], "gp", list(cex=.8, col="green"))
setTrackYaxisParam(trackList[[2]], "gp", list(cex=.8, col="blue"))
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----viewTracksYlab,fig.cap='plot data with adjusted color and size of y-axis label',fig.width=8,fig.height=3----
setTrackStyleParam(trackList[[1]], "ylabgp", list(cex=.8, col="green"))
## set cex to avoid automatic adjust
setTrackStyleParam(trackList[[2]], "ylabgp", list(cex=.8, col="blue"))
setTrackStyleParam(trackList[[2]], "marginBottom", .2)
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----viewTracksYlabTopBottom,fig.cap='plot data with adjusted y-axis label position',fig.width=8,fig.height=3----
setTrackStyleParam(trackList[[1]], "ylabpos", "bottomleft")
setTrackStyleParam(trackList[[2]], "ylabpos", "topright")
setTrackStyleParam(trackList[[2]], "marginTop", .2)
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----viewTracksYlabUpsDown,fig.cap='plot data with adjusted transcripts name position',fig.width=8,fig.height=3----
trackListN <- trackList
setTrackStyleParam(trackListN[[3]], "ylabpos", "upstream")
setTrackStyleParam(trackListN[[4]], "ylabpos", "downstream")
## set cex to avoid automatic adjust
setTrackStyleParam(trackListN[[3]], "ylabgp", list(cex=.6))
setTrackStyleParam(trackListN[[4]], "ylabgp", list(cex=.6))
## remove ylab
setTrackStyleParam(trackListN[[5]], "ylabpos", "none")
names(trackListN)[6] <- ""
gr1 <- range(unname(unlist(GRangesList(sapply(trs, function(.ele) .ele$dat)))))
start(gr1) <- start(gr1) - 2000
end(gr1) <- end(gr1) + 2000
viewTracks(trackListN, gr=gr1, viewerStyle=viewerStyle)

## ----viewTracksCol,fig.cap='plot data with adjusted track color',fig.width=8,fig.height=3----
setTrackStyleParam(trackList[[1]], "color", c("green", "black"))
setTrackStyleParam(trackList[[2]], "color", c("black", "blue"))
for(i in 3:length(trackList)) 
    setTrackStyleParam(trackList[[i]], "color", "black")
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----viewTracksHeight,fig.cap='plot data with adjusted track height',fig.width=8,fig.height=3----
trackListH <- trackList
setTrackStyleParam(trackListH[[1]], "height", .1)
setTrackStyleParam(trackListH[[2]], "height", .44)
for(i in 3:length(trackListH)){
    setTrackStyleParam(trackListH[[i]], "height", 
                       (1-(0.1+0.44))/(length(trackListH)-2))
}
viewTracks(trackListH, gr=gr, viewerStyle=viewerStyle)

## ----viewTracksNames,fig.cap='change the track names',fig.width=8,fig.height=3----
names(trackList) <- c("cpsf160", "fox2", rep("HSPA8", 5))
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----viewTracksPaired,fig.cap='show two data in the same track',fig.width=8,fig.height=2.4----
cpsf160 <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                       file.path(extdata, "cpsf160.repB_-.wig"),
                       format="WIG")
strand(cpsf160$dat) <- strand(cpsf160$dat2) <- "-"
setTrackStyleParam(cpsf160, "color", c("black", "red"))
viewTracks(trackList(trs, cpsf160), gr=gr, viewerStyle=viewerStyle)

## ----viewTracksAnnotation,fig.cap='show called peaks in track', fig.width=8, fig.height=2.4----
cpsf160_cp <- cpsf160
cpsf160$dat2 <- reduce(cpsf160$dat)
cpsf160$dat2$score <- sample.int(length(cpsf160$dat2))
setTrackStyleParam(cpsf160, "tracktype", c("peak", "annotation"))
cpsf160_cp$dat <- reduce(cpsf160_cp$dat2)
cpsf160_cp$dat$score <- sample.int(length(cpsf160_cp$dat))
setTrackStyleParam(cpsf160_cp, "tracktype", c("annotation", "peak"))
setTrackStyleParam(cpsf160_cp, "marginTop", 0.05)
viewTracks(trackList(trs, cpsf160, cpsf160_cp), gr=gr, viewerStyle=viewerStyle)

## ----viewTracksFlipped,fig.cap='show data in the flipped track',fig.width=8,fig.height=4----
viewerStyleF <- viewerStyle
setTrackViewerStyleParam(viewerStyleF, "flip", TRUE)
setTrackViewerStyleParam(viewerStyleF, "xaxis", TRUE)
setTrackViewerStyleParam(viewerStyleF, "margin", c(.1, .05, .01, .01))
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyleF)
addGuideLine(c(122929767, 122929969), vp=vp)
addArrowMark(list(x=122929650,
                  y=2),
             label="label",
             col="blue",
             vp=vp)

## ----themeBW,fig.cap='balck & white theme',fig.width=8,fig.height=4-----------
optSty <- optimizeStyle(trackList(repA, fox2, trs), theme="bw")
trackList <- optSty$tracks
viewerStyle <- optSty$style
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----themeCol,fig.cap='colorful theme',fig.width=8,fig.height=4---------------
optSty <- optimizeStyle(trackList(repA, fox2, trs), theme="col")
trackList <- optSty$tracks
viewerStyle <- optSty$style
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----themeSafe,fig.cap='safe theme',fig.width=8,fig.height=4------------------
optSty <- optimizeStyle(trackList(repA, fox2, trs), theme="safe")
trackList <- optSty$tracks
viewerStyle <- optSty$style
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)

## ----axisBreak,fig.cap='axis with breaks',fig.width=8,fig.height=4------------
gr.breaks <- GRanges("chr11", 
                     IRanges(c(122929275, 122929575, 122929775), 
                             c(122929555, 122929725, 122930122)), 
                     strand="-", percentage=c(.4, .2, .4))
vp <- viewTracks(trackList, gr=gr.breaks, viewerStyle=viewerStyle)

## ----sessionInfo, results='asis'----------------------------------------------
sessionInfo()

#' plot tracks for single cell RNAseq
#' @description Plot single cell RNAseq data as heatmap track for Seurat object.
#' @param object Seurat object.
#' @param files bam file to be scanned.
#' @param samplenames sample names for files.
#' @param ... parameters used by \link[GenomicAlignments:readGAlignments]{readGAlignmentsList} 
#' or \link[GenomicAlignments]{readGAlignments}
#' @param txdb TxDb object for gene model.
#' @param gene Gene name to plot. (row value)
#' @param id The id of gene used in txdb.
#' @param idents indentity class to define the groups to plot. (column value)
#' @param gr GRanges object to define the ploting region.
#' @param color vector of colors used in heatmap.  
#' @param withCoverageTrack plot coverage track or not.
#' @param flag An integer(2) vector used to filter reads based on their 
#' 'flag' entry. 
#' @importFrom Seqinfo Seqinfo isCircular genome
#' @importFrom GenomeInfoDb seqlevelsStyle
#' @examples 
#' \dontrun{
#' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
#' test_file <- "https://github.com/10XGenomics/subset-bam/raw/master/test/test.bam"
#' trs <- importScSeqScore(files=test_file, 
#'                         txdb=TxDb.Hsapiens.UCSC.hg19.knownGene,
#'                         id="653635", gene = "WASH7P")
#' }
#' 

importScSeqScore <- function(object, files, samplenames, ...,
                         txdb, gene, id, idents, gr, color,
                         withCoverageTrack=TRUE,
                         flag=scanBamFlag(isSecondaryAlignment = FALSE,
                                          isUnmappedQuery=FALSE,
                                          isNotPassingQualityControls = FALSE,
                                          isSupplementaryAlignment = FALSE)){
  if(missing(txdb)){
    stop("txdb is required!")
  }
  if(missing(gr)){
    if(missing(id) || missing(gene)){
      stop("id and gene is required.")
    }
    ## get gr from id or gene
    trs <- geneTrack(ids=id, txdb = txdb, symbols = gene)
    gr <- lapply(trs, function(.ele) range(unname(.ele@dat)))
    gr <- unlist(GRangesList(unname(gr)))
  }else{
    if(missing(object)){
      if(missing(id) || missing(gene)){
        ## get id and gene from gr
        trs <- geneModelFromTxdb(txdb=txdb, gr=gr)
        ids <- unlist(lapply(trs, function(.ele) unique(.ele@dat$gene)))
        ids <- unique(ids)
        if(missing(id)) id <- ids
        if(missing(gene)) gene <- id
      }
    }
  }
  if(!missing(object)){
    idents <- as.character(idents)
    object <- subset(object, idents = idents, features=gene)
    if(nrow(object)!=1){
      stop("No feature or more than one feature detected.")
    }
    if(ncol(object)<1){
      stop("Not valid idents.")
    }
    if(missing(id) || missing(gene) || missing(txdb)){
      stop("txdb, id and gene is required.")
    }
    id <- id[gene==rownames(object)]
    gene <- rownames(object)
  }
  tr <- geneTrack(ids = id, txdb = txdb, symbols = gene)[[1]]
  if(!missing(samplenames)){
    stopifnot(length(files)==length(samplenames))
  }else{
    samplenames <- sub(".bam", "", basename(files), ignore.case = TRUE)
  }
  what <- character(0)
  #https://support.10xgenomics.com/single-cell-gene-expression/software/pipelines/latest/output/bam
  # BC: Sample index read
  # CB: Chromium cellular barcode sequence that is error-corrected and confirmed against a list of known-good barcode sequences.
  # xf: Extra alignment flags. The bit flags can be interpreted as follows: 1 - The read is confidently mapped to a feature; 2 - The read maps to a feature that the majority of other reads with this UMI did not; 8 - This read is representative for the molecule and can be treated as a UMI count. Bits 4, 16 and 32 are used internally by 10X.
  tag <- c("BC", "CB", "xf")
  seqlens <- scanBamHeader(files[1])[[1]]$target
  seq_gr <- GRanges(names(seqlens), IRanges(rep(1, length(seqlens)), seqlens))
  if(missing(gr)){
    if(length(tr)){
      gr <- range(tr$dat)
      w10 <- floor(width(gr)/10)
      start(gr) <- max(1, start(gr)-w10)
      end(gr) <- end(gr)+w10
    }else{
      gr <- seq_gr
    }
  }
  which <- gr
  seqlevelsStyle(which) <- seqlevelsStyle(seq_gr)[1]
  param <-
    ScanBamParam(what=what,
                 tag=tag,
                 which=which,
                 flag=flag)
  
  reads <- 
    lapply(files, function(bamFile) 
      readGAlignments(bamFile, ..., param=param))
  names(reads) <- samplenames
  ## filter by xf
  filterByXf <- any(vapply(reads, function(.ele){
    xf <- mcols(.ele)$xf
    if(length(xf)){
      .ele <- .ele[!is.na(xf)]
      xf <- mcols(.ele)$xf
    }else{
      return(FALSE)
    }
    length(xf)==length(.ele) && !all(xf==0)
  }, FUN.VALUE = TRUE))
  if(filterByXf){
    reads <- lapply(reads, function(.ele){
      xf <- mcols(.ele)$xf
      if(length(xf)==length(.ele)){
      .ele <- .ele[bitwAnd(xf, 8)==8]
      }else{
        .ele <- .ele[rep(FALSE, length(.ele))]
      }
      .ele
    })
  }
  
  ## filter by cell barcode
  if(!missing(object)){
    cellNames <- colnames(object)
    reads <- lapply(reads, function(.ele) 
      .ele[sub("-", "_", mcols(.ele)$CB) %in% cellNames])
  }
  
  ## split the reads by cell and get coverage
  cvg <- lapply(reads, function(.ele){
    .e <- .ele
    sep <- "__SEQN__"
    CB <- unique(mcols(.ele)$CB)
    seqn <- paste(rep(CB, each=length(seqlevels(.ele))),
                  rep(seqlevels(.ele), length(CB)),
                  sep=sep)
    seqinf <- Seqinfo(seqnames = c(seqlevels(.ele), seqn),
                      seqlengths = unname(rep(seqlengths(.ele), length(CB)+1)),
                      isCircular = unname(rep(isCircular(.ele), length(CB)+1)),
                      genome= unname(rep(genome(.ele), length(CB)+1)))
    seqname <- paste(mcols(.e)$CB, as.character(seqnames(.e)),
                     sep=sep)
    seqinfo(.e) <- seqinf[c(seqlevels(.ele), seqn[seqn %in% seqname])]
    seqnames(.e) <- factor(seqname, levels = seqlevels(.e))
    .cvg <- coverage(.e)
    .cvg <- split(.cvg, 
                  sub(paste0("^(.*?)", sep, ".*$"), "\\1", names(.cvg)))
    .cvg <- .cvg[names(.cvg) %in% CB]
    .cvg <- lapply(.cvg, function(.e){
      names(.e) <- sub(paste0("^.*?", sep), "", names(.e))
      .e
    })
    .cvg
  })
  if(!missing(object)){
    ## convert coverage to Views
    chr <- as.character(seqnames(which)[1])
    cvg <- lapply(cvg, function(.ele){
      lapply(.ele, function(.e){
        Views(.e[[chr]], ranges(which))
      })
    })
    ## convert Views to dataframe
    cvg <- lapply(cvg, function(.ele){
      .ele <- lapply(.ele, function(.e) 
        viewApply(.e, FUN=as.numeric, simplify = TRUE))
      .ele <- lapply(.ele, as.numeric)
      do.call(rbind, .ele)
    })
  ## recover the data frame for all the cells
    mat <- matrix(0, nrow = ncol(object), ncol = width(which))
    rownames(mat) <- cellNames
    cvg <- lapply(cvg, function(.ele){
      newmat <- mat
      if(!is.null(dim(.ele))){
        newmat[sub("-", "_", rownames(.ele)), ] <- .ele
      }
      newmat
    })
    ## covert it back to GRanges
    cvg <- lapply(cvg, function(.ele){
      .ele <- .ele[order(rowSums(.ele), decreasing = TRUE), ]
      .ele <- apply(.ele, 1, Rle)
      .ele <- lapply(.ele, function(.e){
        l <- runLength(.e)
        l <- cumsum(l)
        shift(IRanges(start = c(1, l[-length(l)]+1), end = l,
                      score=runValue(.e)),
              shift = start(which))
      })
      .ele <- GRanges(chr, unlist(IRangesList(.ele)))
    })
    ## split the cvg by the idents
    cvg <- lapply(cvg, function(.ele){
      .ele$ident <- object@active.ident[names(.ele)] #Idents
      split(.ele, .ele$ident)
    })
  }else{
    ## split the cvg by the idents
    idents <- unique(unlist(lapply(cvg, function(.ele) names(.ele))))
    cvg <- lapply(cvg, function(.ele){
      .ele <- lapply(.ele[idents], function(.e){
        .e <- as(.e, "GRanges")
        .e[.e$score>0]
        })
    })
  }
  
  ## switch list
  cvg <- lapply(idents, function(.ele){
    lapply(cvg, function(.e) .e[[.ele]])
  })
  names(cvg) <- idents
  cvg <- unlist(cvg)
  trs <- lapply(cvg, function(.ele){
    seqlevelsStyle(.ele) <- seqlevelsStyle(tr$dat)[1]
    tr1 <- new("track", dat=.ele, type="scSeq", format="BAM")
    if(withCoverageTrack){
      .cvg <- coverage(.ele, weight = .ele$score)
      tr2 <- new("track", dat=.cvg, type="data", format="BAM")
      list(perCell=tr1, coverage=tr2)
    }else{
      tr1
    }
  })
  trs <- unlist(trs)
  trackList <- trackList(trs, tr, heightDist = c(length(cvg), 1/2))
  return(trackList)
}

#' Reading data from a ginteractions, hic, cool, or validPairs file
#' @description Read a \code{\link{track}} object from a ginteractions, hic, mcool, or validPairs file
#' @param file The path to the file to read.
#' @param format The format of import file. Could be ginteractions, hic, cool or validPairs
#' @param ranges An object of \code{\link[GenomicRanges:GRanges-class]{GRanges}} to indicate
#' the range to be imported. For .hic file, if the length of ranges is 2, 
#' the first range will be used as anchor 1 and the second range will be used
#' as anchor 2.
#' @param ignore.strand ignore the strand or not when do filter. default TRUE
#' @param out output format. Default is track. Possible values: track, GInteractions.
#' @param resolution Resolutions for the interaction data.
#' @param unit BP (base pair) or FRAG (fragment) (.hic file only). 
#' @param normalization Type of normalization, NONE, VC, VC_SORT or KR for .hic and
#' NONE, balanced for .cool.
#' @param matrixType Type of matrix for .hic file. Available choices are "observed", "oe", and "expected".
#' default is "observed".
#' @param ... NOT used.
#' @return a \code{\link{track}} object 
#' @import GenomicRanges
#' @import IRanges
#' @importFrom InteractionSet GInteractions
#' @importFrom rhdf5 H5Fopen h5ls h5read h5closeAll
#' @importFrom strawr straw readHicNormTypes
#' @export
#' @seealso See Also as \code{\link{listResolutions}}, \code{\link{listChromosomes}}, 
#' \code{\link[strawr]{readHicNormTypes}}
#' @examples 
#' #import a ginteractions file
#' #gi <- system.file("extdata", "test.ginteractions.tsv", package="trackViewer",
#' #                       mustWork=TRUE)
#' #dat <- importGInteractions(file=gi, format="ginteractions",
#' #                   ranges=GRanges("chr7", IRanges(127471197, 127474697)))
#' 
#' ##import a hic file
#' if(.Platform$OS.type!="windows"){
#' hic <- system.file("extdata", "test_chr22.hic", package = "trackViewer",
#'                        mustWork=TRUE)
#' dat <- importGInteractions(file=hic, format="hic", 
#'                            ranges=GRanges("22", IRanges(1500000, 100000000)))
#' }
#' 
#' ##import a cool file
#' cool <- system.file("extdata", "test.mcool", package = "trackViewer",
#'                      mustWork=TRUE)
#' dat <- importGInteractions(file=cool, format="cool",
#'                            resolution = 2,
#'                            ranges=GRanges("chr1", IRanges(10, 28)))
#' 
#' ##import a validPairs file
#' #validPairs <- system.file("extdata", "test.validPairs", package = "trackViewer",
#' #                       mustWork=TRUE)
#' #dat <- importGInteractions(file=validPairs, format="validPairs")
#' 

importGInteractions <- function(file, 
                        format=c("ginteractions", "hic", "cool", "validPairs"),
                        ranges=GRanges(), ignore.strand=TRUE,
                        out=c("track", "GInteractions"),
                        resolution = 100000,
                        unit=c("BP", "FRAG"),
                        normalization =c("NONE", "VC", "VC_SQRT", "KR", 
                                         "SCALE", "GW_KR", "GW_SCALE", "GW_VC",
                                         "INTER_KR", "INTER_SCALE", "INTER_VC",
                                         "balanced"),
                        matrixType = c("observed", "oe", "expected"),
                        ...){
    if(missing(file))
        stop("file is required.")
    ##    on.exit(closeAllConnections())
    format <- match.arg(format)
    out <- match.arg(out)
    unit <- match.arg(unit)
    matrixType <- match.arg(matrixType)
    normalization <- match.arg(normalization)
    available_normalization <- listNormalizations(file=file, format = format)
    if(!normalization %in% available_normalization){
      available_normalization <- paste(available_normalization, collapse=', ')
      stop("normalization must be available at file. ",
           "Available normalization are ",
           available_normalization)
    }
    
    #    res <- GRanges(score=numeric(0))
    if(!is(ranges, "GRanges")) stop("ranges must be an object of GRanges.")
    gr <- orderedGR(ranges)
    readginteractions <- function(file){
        localFUN = function(buf){
            buf <- do.call(rbind, strsplit(buf, "\\s"))
            dat <- GRanges(buf[, 1], IRanges(as.numeric(buf[, 2]), 
                                             as.numeric(buf[, 3])))
            dat2 <- GRanges(buf[, 4], IRanges(as.numeric(buf[, 5]), 
                                              as.numeric(buf[, 6])))
            gi <- GInteractions(anchor1=dat, anchor2 = dat2, 
                                score = as.numeric(buf[, 7]))
            subsetByOverlaps(gi, gr)
        }
        readFile(file, localFUN);
    }
    readhic <- function(file){
        if(length(gr)>2){
            stop("length of gr > 2 is not supported for .hic file.")
        }
        if(length(gr)==0){
            return(GInteractions())
        }
        if(length(gr)==1){
            seq1 <- seq2 <- as.character(seqnames(gr))
            start1 <- start2 <- start(gr)
            end1 <- end2 <- end(gr)
        }
        if(length(gr)==2){
            seq1 <- as.character(seqnames(gr)[1])
            start1 <- start(gr)[1]
            end1 <- end(gr)[1]
            seq2 <- as.character(seqnames(gr)[2])
            start2 <- start(gr)[2]
            end2 <- end(gr)[2]
        }
        df <- straw(
          norm = normalization,
          fname = file,
          chr1loc = paste(seq1, start1, end1, sep=":"),
          chr2loc = paste(seq2, start2, end2, sep=":"),
          unit = unit,
          binsize = resolution,
          matrix = matrixType)
        if(length(df)==0){
            return(GInteractions())
        }
        anchor1 <- GRanges(seq1, IRanges(df$x+1, width = resolution))
        anchor2 <- GRanges(seq2, IRanges(df$y+1, width = resolution))
        gi <- GInteractions(anchor1=anchor1, anchor2=anchor2, 
                            score=df$counts)
    }
    readcool <- function(file){
        gi <- cooler_pixels(file, resolution, gr, normalization)
        if(normalization=="NONE"){
            gi$score <- gi$count
        }else{
            gi$score <- gi$balanced
        }
        gi$count <- NULL
        gi$balanced <- NULL
        gi
    }
    readvalidPairs <- function(file){
        localFUN = function(buf){
            buf <- do.call(rbind, strsplit(buf, "\\s"))
            dat <- GRanges(buf[, 2], IRanges(as.numeric(buf[, 3]), 
                                             width = 1),
                           strand = buf[, 4])
            dat2 <- GRanges(buf[, 5], IRanges(as.numeric(buf[, 6]), 
                                              width = 1),
                            strand = buf[, 7])
            gi <- GInteractions(anchor1=dat, anchor2 = dat2, 
                                insert_size = as.numeric(buf[, 8]))
            subsetByOverlaps(gi, gr)
        }
        readFile(file, localFUN);
    }
    readFile <- function(file, FUN){
        s <- file.info(file)$size
        if(s<100000000){
            buf <- readChar(file, s, useBytes=TRUE)
            buf <- strsplit(buf, "\n", fixed=TRUE, useBytes=TRUE)[[1]]
            res <- FUN(buf)
        }else{
            message("file is too huge. Please consider to subset the data before import.")
            res <- NULL
            con <- file(file, open="r")
            while(length(buf <- readLines(con, n=1000000, warn=FALSE))>0){
                buf <- FUN(buf)
                if(length(res)<1) {
                    res <- buf
                }else{
                    suppressWarnings(res <- c(res, buf))
                }
            }
            close(con)
        }
        res <- unique(res)
        return(res)
    }
    readFiles <- function(file, format){
        FUN <- get(paste("read", format, sep=""))
        res <- FUN(file)
        return(res)
    }
    res <- readFiles(file, format)
    if(length(res)==0) return(NULL)
    if(any(duplicated(res))){
        res_uniq <- res[!duplicated(res)]
        res_dup <- res[duplicated(res)]
        ol <- findOverlaps(res_uniq, res_dup, type = "equal")
        qol <- split(res_dup[subjectHits(ol)]$score, queryHits(ol))
        qol <- vapply(qol, FUN=sum, FUN.VALUE = 0)
        res_uniq[as.numeric(names(qol))]$score <- 
            res_uniq[as.numeric(names(qol))]$score + qol
        res <- res_uniq
    }
    if(out=="track"){
        return(gi2track(res))
    }else{
        return(res)
    }
}

#' List the available normalizations
#' @description List the normalizations available in the file.
#' @param file character(1). File name of .hic or .cool/.mcool/.scool
#' @param format character(1). File format, "hic" or "cool".
#' @importFrom rhdf5 H5Fopen h5ls h5read H5close
#' @importFrom strawr readHicNormTypes
#' @export
#' @examples
#' hicfile <- system.file("extdata", "test_chr22.hic", package="trackViewer")
#' listNormalizations(hicfile)
#' coolfile <- system.file("extdata", "test.mcool", package="trackViewer")
#' listNormalizations(coolfile, format="cool")
listNormalizations <- function(file, format=c("hic", "cool")){
  format <- match.arg(format)
  guessFormat(file, format)
  if(format=="hic"){
    readHicNormTypes(fname=file)
  }else{
    coolfile <- checkCoolFile(file)
    if(isMcool(file)){
      coolfileRootName <- coolfileRootName(coolfile)
      resolutions <- listResolutions(file, format = format)
      available_normalization <- lapply(resolutions, function(res){
        mcool_header <- h5ls(coolfile&coolfileRootName&res&"bins",
                             recursive = FALSE)$name
        setdiff(mcool_header, c("chrom", "start", "end"))
      })
      available_normalization <- Reduce(intersect, available_normalization)
    }else{
      mcool_header <- h5ls(coolfile&"bins", recursive = FALSE)$name
      available_normalization <- setdiff(mcool_header,
                                         c("chrom", "start", "end"))
    }
    available_normalization[available_normalization=='weight'] <- 'balanced'
    c('NONE', available_normalization)
  }
}

convertFont <- function(){
        return(36*min(c(convertWidth(unit(1, "snpc"), "inches", valueOnly=TRUE), 
                        convertHeight(unit(1, "snpc"), "inches", valueOnly=TRUE))
                      ))
}

is_color <- function(x) {
  vapply(x, FUN=function(X) {
    tryCatch(is.matrix(col2rgb(X)), 
             error = function(e) FALSE)
  }, FUN.VALUE = logical(1L))
}
mapScoresToColor <- function(scores, color, length.out=100){
  rg <- range(as.numeric(as.matrix(scores)))
  if(length(color)==0){
    color <- colorRampPalette(c("green", "black", "red"))(length.out)
  }else{
    if(length(color)==1){
      color <- if(color=="white") 
        colorRampPalette(c("black", color))(length.out) else
          colorRampPalette(c("white", color))(length.out)
    }else{
      color <- colorRampPalette(color)(length.out)
    }
  }
  v <- seq(rg[1]-1, rg[2]+1, length.out = length.out+1)
  color <- findInterval(scores, v, all.inside = TRUE)
  return(list(color=color, breaks=v))
}


inRange <- function(x, scale){
  x>=scale[1] & x<=scale[2]
}

hrScale <- function(r){
    if(any(r<0))
      stop("'r' must be positive")
    if(length(r)!=2)
        stop("'r' must be a vector with two numbers")
    x <- abs(r[2] - r[1])
    suffix <- c("bp", "K", "M", "G")#NOT USE: "T", "P", "E", "Z", "Y"
    base <- 1000L
    n <- length(suffix)
    for(i in 1:n){
        if(x >= base*10){
            if(i < n)
                x <- x/base
        }else break
    }
    x <- round(x=x/10, digits=0)
    if(i==1) x <- 10^round(log10(x), digits=0) / 2
    return(list(scale=x, unit=suffix[i], range=r))
}

convertNum2HumanNum <- function(x){
  x <- x[1]
  stopifnot(x>0)
  suffix <- c("bp", "K", "M", "G")
  base <- 1000L
  n <- length(suffix)
  for(i in 1:n){
    if(x >= base){
      if(i < n)
        x <- x/base
    }else break
  }
  return(paste(x, suffix[i]))
}

locateScale <- function(x, y, maxY, scale){
    suffix <- c(1, 1000, 1000000, 1000000000)
    names(suffix) <- c("bp", "K", "M", "G")
    rg <- scale[[3]]
    scale <- as.numeric(scale[[1]] * suffix[scale[[2]]])
    scale.10 <- round(x=scale/10, digits=1)
    threshold <- maxY/2
    start <- c()
    end <- c()
    START <- TRUE
    defaultOut <- c(start=mean(rg)-scale, end=mean(rg)+scale)
    if(length(x)==0) return(defaultOut)
    y <- y[order(x)]
    x <- x[order(x)]
    for(i in seq_along(x)){
        if(y[i]>threshold){
            if(START){
                START <- FALSE
                start <- c(start, x[i])
            }
        }else{
            if(!START){
                START <- TRUE
                end <- c(end, x[i])
            }
        }
    }
    if(length(start)==length(end)+1) 
        end <- c(end, x[i])
    if(length(start)==0){
        mid <- sum(rg)/2
        return(c(start=mid-scale, end=mid+scale))
    }
    ir <- IRanges(start=c(x[1],start, x[length(x)])-scale.10, 
                  end=c(x[1], end, x[length(x)])+scale.10)
    ir <- gaps(ir)
    if(length(ir)==0) return(defaultOut)
    ir <- ir[order(width(ir), decreasing=TRUE)]
    mid <- sum(c(start(ir)[1], end(ir)[1]))/2
    return(c(start=mid-scale, end=mid+scale))
}

orderedGR <- function(gr=GRanges()){
    if(length(gr)>0){
        gr[order(as.character(seqnames(gr)), start(gr))]
    }else{
        gr
    }
}

condenceGRs <- function(gr=GRanges(), FUN=sum){
    .gr <- reduce(gr, min.gapwidth=0, with.revmap=TRUE)
    scores <- score(gr)
    .gr$score <- sapply(.gr$revmap, function(.id) FUN(scores[.id]))
    .gr$revmap <- NULL
    .gr
}

disjoinGRs <- function(gr=GRanges(), FUN=sum){
    if(length(gr)<1) return(gr)
    if(length(gr$score)!=length(gr)) gr$score <- rep(1, length(gr))
    .gr <- disjoin(gr)
    ol <- findOverlaps(.gr, gr)
    s <- tapply(score(gr[subjectHits(ol)]), queryHits(ol), FUN=FUN)
    .gr$score <- 0
    .gr$score[as.numeric(names(s))] <- s
    .gr
}

subsetGeneTrack <- function(dat, chrom, from, to, st){
  if(length(dat)){
    if(length(dat$featureID)==length(dat)){
      dat_rg <- split(dat, dat$featureID)
      dat_rg <- GRangesList(dat_rg)
      dat_rg <- range(dat_rg)
      dat_rg <- unlist(dat_rg)
      dat_rg <- subsetByOverlaps(
        dat_rg,
        GRanges(chrom, IRanges(from, to), strand = st))
      dat <- subsetByOverlaps(dat, dat_rg)
    }
  }
  return(dat)
}

filterTracks <- function(tl, chrom, from, to, st){
    for(i in seq_along(tl)){
        if(tl[[i]]@type %in% c("data", "scSeq")){
            if(tl[[i]]@format=="WIG") {
                tl[[i]] <- parseWIG(tl[[i]], chrom, from, to)
            }
            dat <- tl[[i]]@dat
            dat <- disjoinGRs(dat)
            tl[[i]]@dat <- dat[end(dat)>=from &
                                   start(dat)<=to &
                                   seqnames(dat)==chrom]
            if(length(tl[[i]]@dat2)>0){
                dat2 <- tl[[i]]@dat2
                dat2 <- disjoinGRs(dat2)
                tl[[i]]@dat2 <- dat2[end(dat2)>=from &
                                        start(dat2)<=to &
                                        seqnames(dat2)==chrom]
            }
            if(st %in% c("+", "-")) {
                dat <- tl[[i]]@dat
                tl[[i]]@dat <- dat[strand(dat)==st]
                if(length(tl[[i]]@dat2)>0){
                    dat2 <- tl[[i]]@dat2
                    tl[[i]]@dat2 <- dat2[strand(dat2)==st]
                }
            }
        }else{
          if(tl[[i]]@type=="interactionData"){
            ## dat, dat2 are paired, or with target
            getKeep <- function(dat, dat2){
              keep <- ((end(dat)>=from & start(dat)<=to) |
                         (end(dat2)>=from & start(dat2)<=to)) & 
                seqnames(dat)==chrom & seqnames(dat2)==chrom
              ## remove duplicates
              idx1 <- paste(as.character(seqnames(dat)), start(dat), end(dat),
                            as.character(seqnames(dat2)), start(dat2), end(dat2))
              idx2 <- paste(as.character(seqnames(dat2)), start(dat2), end(dat2),
                            as.character(seqnames(dat)), start(dat), end(dat))
              idx <- ifelse(start(dat)<start(dat2), idx1, idx2)
              keep <- keep & (!duplicated(idx))
            }
            .dat <- tl[[i]]@dat
            .dat2 <- tl[[i]]@dat2
            .dat1target <- NULL
            .dat2target <- NULL
            if(length(.dat$target)==length(.dat) && is(.dat$target, "GRanges")){
              .dat1target <- .dat$target
              if(length(.dat2)!=0){
                if(length(.dat2$target)==length(.dat2) && is(.dat2$target, "GRanges")){
                  names(.dat2) <- NULL
                  .dat2target <- .dat2$target
                }
              }
            }else{
              .dat1target <- .dat2
            }
            keep2 <- keep1 <- getKeep(.dat, .dat1target)
            tl[[i]]@dat <- tl[[i]]@dat[keep1]
            if(length(.dat2target)){
              keep2 <- getKeep(.dat2, .dat2target)
            }
            tl[[i]]@dat2 <- tl[[i]]@dat2[keep2]
          }else{
            if(tl[[i]]@type %in% c("transcript", "gene")){
              tl[[i]]@dat <- subsetGeneTrack(tl[[i]]@dat,
                                             chrom, from, to, st)
              tl[[i]]@dat2 <- subsetGeneTrack(tl[[i]]@dat2,
                                              chrom, from, to, st)
            }else{
              if(tl[[i]]@type=="lollipopData"){
                dat <- tl[[i]]@dat
              }else{
                dat <- restrict(tl[[i]]@dat, start = from, end=to)
              }
              dat <- dat[end(dat)>=from &
                           start(dat)<=to &
                           seqnames(dat)==chrom]
              dat2 <- tl[[i]]@dat2
              if(length(dat2)>0){
                dat2 <- dat2[end(dat2)>=from &
                               start(dat2)<=to &
                               seqnames(dat2)==chrom]
              }
              if(tl[[i]]@type=="lollipopData"){
                tl[[i]]@dat <- dat
                tl[[i]]@dat2 <- dat2
              }else{
                tl[[i]]@dat <- dat
                tl[[i]]@dat2 <- dat2
              }
              if(length(dat)==0 && length(dat2)==0)
                tl[[i]]@style@height <- 0
            }
          }
        }
    }
    tl
}

is_null_na <- function(.ele){
  if(is.null(.ele)) return(TRUE)
  is.na(.ele)
}
getYlim <- function(tl, op){
    yscales <- mapply(tl, op, FUN=function(.ele, .op){
        ylim <- .ele@style@ylim
        if(length(ylim)!=2){
            if(.ele@type %in% c("data", "lollipopData", "scSeq")){
                if(length(.ele@dat)>0 && .ele@style@tracktype[1]!='annotation'){
                    ylim <- unique(round(range(.ele@dat$score)))
                }else{
                    ylim <- c(0, 0)
                }
                if(length(.ele@dat2)>0 && is_null_na(.op)[1]){
                  if(length(.ele@style@tracktype)<2){
                    .ele@style@tracktype[2] <- .ele@style@tracktype[1]
                  }
                  if(.ele@style@tracktype[2]!='annotation'){
                    ylim2 <- unique(round(range(.ele@dat2$score)))
                    ylim <- c(ylim, -1*ylim2)
                  }
                }
                ylim <- range(c(0, ylim))
            }else{
              if(.ele@type == "interactionData"){
                ## max interaction height
                ylim <- c(0, 1)
              }else{
                ylim <- c(0, 0)
              }
            }            
        }
        ylim
    }, SIMPLIFY = FALSE)
    
    yscaleR <- range(unlist(yscales))
    if(diff(yscaleR)==0) yscaleR <- c(0, 1)
    yscales <- lapply(yscales, function(.ele){
        if(diff(.ele)==0){
            if(all(.ele>=0)){
                .ele <- c(0, yscaleR[2])
            }else{
                .ele <- c(yscaleR[1], 0)
            }
        }
        if(.ele[1]>.ele[2]) .ele[1] <- 0
        .ele
    })
    names(yscales) <- names(tl)
    yscales
}

getYheight <- function(tl){
    yHeights <- sapply(tl, function(.ele){
        yh <- .ele@style@height
        if(length(yh)==0) yh <- -1
        yh[1]
    })
    noY <- yHeights == -1
    yHeightsT <- sum(yHeights[!noY])
    if(yHeightsT>1.001)
        stop("total heights of data tracks is greater than 1.")
    if(length(yHeights[noY]) > 0){
        yHeights[noY] <- 
            (1 - yHeightsT) / length(yHeights[noY])
    }
    names(yHeights) <- names(tl)
    yHeights
}

drawXaxis <- function(xscale, style){
    scale <- hrScale(xscale)
    suffix <- c(1, 1000, 1000000, 1000000000)
    names(suffix) <- c("bp", "K", "M", "G")
    interval <- scale$scale * suffix[scale$unit]
    start <- ceiling(xscale[1]/interval)
    end <- floor(xscale[2]/interval)
    if(style@flip) xscale <- rev(xscale)
    if(length(style@xat)>0){
      at <- style@xat
      if(length(style@xlabel)==length(style@xat)){
        label <- style@xlabel
      }else{
        label <- at
      }
      at <- rescale(at, from=xscale)
    }else{
      label <- interval * start:end
      at <- rescale(label, from=xscale)
    }
    gp <- style@xgp
    class(gp) <- "gpar"
    rot <- ifelse(style@xlas %in% c(0, 1), 0, 90)
    grid.xaxis(at=at, label=label, gp=gp,
               edits = gEdit(gPath="labels", rot=rot),
               draw=style@xaxis)
}
putGeneYlab <- function(curViewStyle, style, name, height, xscale, rang, withlollipop=FALSE){
    gap <- (xscale[2] - xscale[1])/100
    strand <- unique(as.character(strand(rang)))
    if(length(strand)>1){
      strand <- names(sort(table(strand), decreasing = TRUE))[1]
      strand(rang) <- strand
      rang <- range(rang)
    }
    just <- style@ylabpos=="upstream"
    if(curViewStyle@flip){
        if(strand=="+"){
            x <- ifelse(just, start(rang) + gap, end(rang) - gap)
            just <- ifelse(just, "left", "right")
        }else{
            x <- ifelse(just, end(rang) - gap, start(rang) + gap)
            just <- ifelse(just, "right", "left")
        }
    }else{
        if(strand=="+"){
            x <- ifelse(just, start(rang) - gap, end(rang) + gap)
            just <- ifelse(just, "right", "left")
        }else{
            x <- ifelse(just, end(rang) + gap, start(rang) - gap)
            just <- ifelse(just, "left", "right")
        }
    }
    
    gp <- style@ylabgp
    class(gp) <- "gpar"
    if(is.null(gp$cex)) gp$cex <- optFontSize("z", curViewStyle, 
                                              height=height)
    pushViewport(viewport(x=curViewStyle@margin[2], y=0, 
                          height=1, 
                          width=1-curViewStyle@margin[2]-curViewStyle@margin[4], 
                          clip="off",
                          just=c(0,0), 
                          xscale=xscale))
    grid.text(x=x, y=ifelse(withlollipop, .25, .5), label=name, rot=0, just=just, gp=gp, 
              default.units="native")
    popViewport()
}

putYlab <- function(curViewStyle, style, name, yHeightBottom, yHeightTop, height, yscale){
    ##c("left", "right", "topleft", "bottomleft", "topright", "bottomright", "abovebaseline", "underbaseline")
    vp <- switch(style@ylabpos,
                 left=viewport(x=curViewStyle@margin[2]*.4, 
                               width=curViewStyle@margin[2]*.8,
                               just="center"),
                 right=viewport(x=1 - curViewStyle@margin[4]*.6, 
                                width=curViewStyle@margin[4]*.8,
                                just="center"),
                 topleft=viewport(y=1-yHeightTop, height=yHeightTop, just="bottom"),
                 bottomleft=viewport(y=0, height=yHeightBottom, just="bottom"),
                 topright=viewport(y=1-yHeightTop, height=yHeightTop, just="bottom"),
                 bottomright=viewport(y=0, height=yHeightBottom, just="bottom"),
                 abovebaseline=viewport(y=0.5, height = 1, just="center",yscale=yscale),
                 underbaseline=viewport(y=0.5, height = 1, just="center",yscale=yscale),
                 viewport(x=curViewStyle@margin[2]*.4, 
                          width=curViewStyle@margin[2]*.8))
    just <- switch(style@ylabpos,
                   left="center",
                   right="center",
                   topleft=c(0, 0),
                   topright=c(1, 0),
                   bottomleft=c(0, 1),
                   bottomright=c(1, 1),
                   abovebaseline=c(0, 0),
                   underbaseline=c(0, 1),
                   "center"
        )
    x <- switch(style@ylabpos,
                left=.5,
                right=.5,
                topleft=curViewStyle@margin[2],
                topright=1-curViewStyle@margin[4],
                bottomleft=curViewStyle@margin[2],
                bottomright=1-curViewStyle@margin[4],
                abovebaseline=curViewStyle@margin[2],
                underbaseline=curViewStyle@margin[2],
                .5
        )
    y <- switch(style@ylabpos,
                left=.5,
                right=.5,
                topleft=.1,
                topright=.1,
                bottomleft=.9,
                bottomright=.9,
                abovebaseline=unit(0, "native")+unit(2, "points"),
                underbaseline=unit(0, "native")-unit(2, "points"),
                .5
        )
    pushViewport(vp)
    rot <- ifelse(style@ylablas %in% c(0, 3), 90, 0)
    gp <- style@ylabgp
    class(gp) <- "gpar"
    if(style@ylabpos %in% c("topleft", "topright", "bottomleft", "bottomright", "abovebaseline", "underbaseline")){
        rot <- 0
        curHeight <- ifelse(grepl("top",style@ylabpos), yHeightTop, yHeightBottom)
        if(is.null(gp$cex)) gp$cex <- optFontSize("z", curViewStyle, 
                                                  height=curHeight)
    }else{
        if(curViewStyle@autolas){
            curWidth <- ifelse(style@ylabpos=="left", 
                               curViewStyle@margin[2],
                               curViewStyle@margin[4])
            rot <- ifelse(convertHeight(unit(height, "npc"), 
                                        unitTo="inches", valueOnly=TRUE)
                          > convertWidth(unit(curWidth, "npc"), 
                                         unitTo="inches", valueOnly=TRUE),
                          90, 0)
            if(is.null(gp$cex)) gp$cex <- optFontSize("z", curViewStyle, 
                                                      height=height)
        }
    }
    grid.text(x=x, y=y, label=name, rot=rot, just=just, gp=gp)

    popViewport()
}

drawYaxis <- function(ylim, yaxisStyle, curViewStyle, heatlegends=list()){
    gp <- yaxisStyle@gp
    class(gp) <- "gpar"
    if(length(heatlegends)){
      width <- ifelse(yaxisStyle@main, .3,
                      curViewStyle@margin[4]/4)
      x <- ifelse(yaxisStyle@main, 1.15, 1+curViewStyle@margin[4]/8)
      for(i in seq_along(heatlegends$breaks)){
        if(yaxisStyle@main){
          vp <- viewport(x=curViewStyle@margin[2] * .2, 
                         width=curViewStyle@margin[2] *.4,
                         y=ifelse(i==1, 1-heatlegends$ysplit*.8, 0),
                         height=ifelse(i==1, heatlegends$ysplit, 1-heatlegends$ysplit)*.8,
                         clip="off",
                         just=c(0,0), yscale=ylim)
        }else{
          vp <- viewport(x=curViewStyle@margin[2], 
                         width=1 - curViewStyle@margin[2] - curViewStyle@margin[4],
                         y=ifelse(i==1, 1-heatlegends$ysplit*.8, 0),
                         height=ifelse(i==1, heatlegends$ysplit, 1-heatlegends$ysplit)*.8,
                         clip="off", 
                         just=c(0,0), yscale=ylim)
        }
        pushViewport(vp)
        if(isTRUE(yaxisStyle@draw)){
          grid.raster(rev(heatlegends$crp[[i]]), x=x, width=width, height=1)
        }
        # map breaks to ylim
        if(heatlegends$userdefinedbreaks){
          this_breaks <- heatlegends$breaks[[i]]
          at <- seq(ylim[1], ylim[2],
                    length.out=length(this_breaks))
          label <- this_breaks
        }else{
          at <- ylim
          label <- range(heatlegends$breaks[[i]])
        }
        tryCatch({
          dif <- rle(diff(label))$lengths
          w <- c(1, 1+cumsum(dif))
          at <- at[w]
          label <- label[w]
        }, error=function(.e){}, warning=function(.w){})
        tryCatch({
          if(any(label%%1!=0)){
            label <- formatC(label, format = "fg")
          }
        }, error=function(.e){}, warning=function(.w){})
        grid.yaxis(at=at, label = label,
                   main=FALSE, gp=gp,
                   draw=yaxisStyle@draw)
        popViewport()
      }
    }else{
      if(yaxisStyle@main){
        vp <- viewport(x=curViewStyle@margin[2] * .2, 
                       width=curViewStyle@margin[2] *.4,
                       y=0, height=1, clip="off",
                       just=c(0,0), yscale=ylim)
      }else{
        vp <- viewport(x=curViewStyle@margin[2], 
                       width=1 - curViewStyle@margin[2] - curViewStyle@margin[4],
                       y=0, height=1, clip="off", just=c(0,0), yscale=ylim)
      }
      pushViewport(vp)
      at <- unique(ylim)
      at <- at[order(at)]
      label <- yaxisStyle@label
      if(label){
        label <- at
      }
      grid.yaxis(at=at, label=label, main=FALSE, gp=gp,
                 draw=yaxisStyle@draw)
      popViewport()
    }
}

drawXscale <- function(scale){
    gp <- scale@gp
    class(gp) <- "gpar"
    line1 <- abs(as.numeric(convertY(unit(1, "line"), 
                                 scale@from@unit)))
    if(length(gp$cex)>0){
        if(is.numeric(gp$cex)){
            line1 <- line1 * gp$cex
        }
    }
    if(scale@from@y<0){
        scale@from@y <- scale@from@y - line1
        scale@to@y <- scale@to@y - line1
    }
    grid.segments(x0=scale@from@x, 
                  y0=scale@from@y,
                  x1=scale@to@x,
                  y1=scale@to@y,
                  default.units=scale@from@unit,
                  gp=gp)
    grid.segments(x0=scale@from@x,
                  y0=scale@from@y,
                  x1=scale@from@x,
                  y1=scale@from@y + 0.25 * line1,
                  default.units=scale@from@unit,
                  gp=gp)
    grid.segments(x0=scale@to@x,
                  y0=scale@to@y,
                  x1=scale@to@x,
                  y1=scale@to@y + 0.25 * line1,
                  default.units=scale@to@unit,
                  gp=gp)
    grid.text(label=scale@label, 
              x=(scale@from@x + scale@to@x)/2,
              y=(scale@from@y + scale@to@y)/2 + line1 * 0.2,
              gp=gp,
              just="bottom",
              default.units=scale@from@unit)
}

maxStringWidth <- function(labels, spaces="WW", cex){
    max(as.numeric(convertX(stringWidth(paste0(labels, spaces)), "line"))*cex)
}

getColNum <- function(labels, spaces="WW", cex){
    ncol <- floor(as.numeric(convertX(unit(1, "npc"), "line")) / 
                      maxStringWidth(labels, spaces=spaces, cex) / 
              as.numeric(convertX(stringWidth("W"), "line")))
    nrow <- ceiling(length(labels) / ncol)
    ncol <- ceiling(length(labels) / nrow)
    ncol
}

checkLegendPosition <- function(legendPosition){
  positions <- c('top', 'left', 'right')
  if(!is.list(legendPosition)){
    legendPosition <- list(
      position = match.arg(legendPosition, choices = positions)
    )
  }else{
    legendPosition$position <- 
      match.arg(legendPosition$position,
                choices = positions)
  }
  return(legendPosition)
}
############### handle legend ####################
## set the legend as a list, 
## if all the legend for different tracks is same
## set draw legend for last track later
handleLegend <- function(legend, len, dat){
  if(length(legend)>0){
    if(is.character(legend)){
      if(!missing(dat)){
        if(is.list(dat)){
          if(length(legend)<length(dat)){
            legend <- rep(legend, length(dat))[seq_along(dat)]
          }
        }else{
          dat <- list(dat)
          legend <- legend[1]
        }
        para <- c("shape", "color", "border", "alpha")
        preset <- list("circle", "white", "black", 1)
        shapeMap <- c("circle"=21, "square"=22, "diamond"=23, 
                      "triangle_point_up"=24, "triangle_point_down"=25)
        names(preset) <- para
        legend <- mapply(function(.legend, .dat){
          coln <- colnames(mcols(.dat))
          if(.legend %in% coln){
            labels <- mcols(.dat)[, .legend]
            gp <- lapply(para, function(.ele){
              if(.ele %in% coln){
                mcols(.dat)[, .ele]
              }else{
                rep(preset[[.ele]], length(.dat))
              }
            })
            names(gp) <- para
            names(gp)[names(gp)=="color"] <- "fill"
            if(is.list(gp[["shape"]])){
              warning('multiple shape for one legend is not accept. Using default')
              gp[["shape"]] <- shapeMap[vapply(gp[["shape"]], FUN=function(.ele) .ele[1], FUN.VALUE = character(1L))]
            }else{
              gp[["shape"]] <- shapeMap[gp[["shape"]]]
            }
            gp <- as.data.frame(gp, stringsAsFactors=FALSE)
            gp <- cbind(labels=labels, gp)
            names(gp)[names(gp)=="shape"] <- "pch"
            gp <- gp[!duplicated(gp[, "labels"]), ]
            gp <- gp[order(gp[, "labels"]), ]
            gp <- as.list(gp)
          }
        }, legend, dat, SIMPLIFY = FALSE)
      }
    }
    if(!is.list(legend)){
      tmp <- legend
      legend <- vector(mode = "list", length = len)
      legend[[len]] <- tmp
      rm(tmp)
    }else{
      if(length(legend)==1){
        tmp <- legend[[1]]
        legend <- vector(mode = "list", length = len)
        legend[[len]] <- tmp
        rm(tmp)
      }else{
        if("labels" %in% names(legend)){
          tmp <- legend
          legend <- vector(mode = "list", length = len)
          legend[[len]] <- tmp
          rm(tmp)
        }else{
          if(length(legend)<len){
            length(legend) <- len
          }
        }
      }
    }
  }
  return(legend)
}

legendNchar <- function(legend){
  n <- 0
  if(length(legend)>0){
    if(is.list(legend)){
      thisLabels <- legend[["labels"]]
    }else{
      thisLabels <- names(legend)
    }
    if(length(thisLabels)>0){
      n <- nchar(thisLabels)
    }
  }
  return(n)
}

getMaxYlimNchar <- function(SNP.gr, minVal, types){
  if(!is.list(SNP.gr)){
    SNP.gr <- list(SNP.gr)
  }
  m <- lapply(SNP.gr[types!='pie'], function(.ele){
    .e <- .ele$score
    if(length(.e)>0){
      return(max(.e[!is.infinite(.e)], na.rm=TRUE))
    }else{
      return(0)
    }
  })
  m <- max(c(1, unlist(m, recursive = TRUE)))
  m <- nchar(as.character(round(m))) + 3.5
  if(m<minVal){
    return(minVal)
  }else{
    return(min(10, m))
  }
}
################ handle ranges #####################
## if !missing(ranges) set ranges as feature ranges
handleRanges <- function(ranges, SNP.gr, features, len){
  if(length(ranges)>0){
    stopifnot(inherits(ranges, c("GRanges", "GRangesList", "list")))
    if(is(ranges, "GRanges")){
      if(length(ranges)==1){
        ranges <- split(rep(ranges, len)[seq.int(len)],
                        seq.int(len))
      }else{
        ranges <- split(rep(ranges, len),
                        rep(seq.int(len), each=len))[seq.int(len)]
      }
    }else{## GRangesList
      if(length(ranges)!=len){
        ranges <- rep(ranges, seq.int(len))[seq.int(len)]
      }
    }
    stopifnot(length(ranges)==len)
  }else{
    if(is(features, "GRanges")){
      ranges <- split(range(unname(features), ignore.strand=TRUE)[rep(1, len)],
                      seq.int(len))
    }else{
      if(length(features)!=len){
        stop("if both SNP.gr and features is GRangesList,",
             " the lengthes of them should be identical.")
      }
      ranges <- GRangesList(lapply(features, function(.ele){
        range(unname(.ele), ignore.strand=TRUE)}))
    }
  }
  return(ranges)
}

##cut all SNP.gr by the range
cutSNP <- function(SNP.gr, ranges, len){
  if(is(ranges, "GRanges")){
    for(i in seq.int(len)){
      range <- ranges[i]
      stopifnot(all(width(SNP.gr[[i]])==1))
      SNP.gr[[i]] <- subsetByOverlaps(SNP.gr[[i]], range, ignore.strand=FALSE)
    }
  }else{
    if(inherits(ranges, c("GRangesList", "list"))){
      for(i in seq.int(len)){
        range <- ranges[[i]]
        stopifnot(all(width(SNP.gr[[i]])==1))
        SNP.gr[[i]] <- subsetByOverlaps(SNP.gr[[i]], range, ignore.strand=FALSE)
      }
    }
  }
  return(SNP.gr)
}

## multiple transcripts in one gene could be separated by featureLayerID
setFeatureLayerID <- function(feature, range){
  feature <- feature[end(feature)>=start(range) & 
                       start(feature)<=end(range)]
  if(length(feature$featureLayerID)!=length(feature)){
    feature$featureLayerID <- rep("1", length(feature))
    feature$featureLayerID <- as.character(feature$featureLayerID)
    start(feature)[start(feature)<start(range)] <- start(range)
    end(feature)[end(feature)>end(range)] <- end(range)
  }
  return(feature)
}

## bottomblank, the transcripts legend height
plotFeatureLegend <- function(feature, LINEH, range, xaxis, xaxis.gp, label_on_feature=FALSE){
  if(length(xaxis)>1 || as.logical(xaxis[1])){
    xaxisSpace <- 2
    if(is.numeric(xaxis.gp$cex)) xaxisSpace <- 2*xaxis.gp$cex
  }else{
    xaxisSpace <- 0
  }
  if(length(names(feature))>0 & !label_on_feature ){ ## features legend
    feature.s <- feature[!duplicated(names(feature))]
    cex <- if(length(unlist(feature.s$cex))==length(feature.s)) 
      unlist(feature.s$cex) else 1
    ncol <- getColNum(names(feature.s), cex=cex)
    featureLegendSpace <- max(ceiling(length(names(feature.s)) / ncol) * cex + 1 )
    pushViewport(viewport(x=.5, y=featureLegendSpace*LINEH/2, 
                          width=1,
                          height=featureLegendSpace*LINEH,
                          xscale=c(start(range), end(range))))
    color <- if(length(unlist(feature.s$color))==length(feature.s)) 
      unlist(feature.s$color) else "black"
    fill <- if(length(unlist(feature.s$fill))==length(feature.s)) 
      unlist(feature.s$fill) else "black"
    pch <- if(length(unlist(feature.s$pch))==length(feature.s)) 
      unlist(feature.s$pch) else 22
    grid.legend(label=names(feature.s), ncol=ncol,
                byrow=TRUE, vgap=unit(.2, "lines"),
                hgap=unit(.5, "lines"),
                pch=pch,
                gp=gpar(col=color, fill=fill, cex=cex))
    popViewport()
  }else{
    featureLegendSpace <- 1
  }
  bottomblank <- (xaxisSpace + featureLegendSpace) * LINEH
  return(bottomblank)
}

plot_grid_xaxis <- function(xaxis, gp=gpar(col="black")){
  ## axis, should be in the bottom of transcripts
  if(length(xaxis)==1 && as.logical(xaxis)) {
    grid.xaxis(gp=gp)
  }
  if(length(xaxis)>1 && is.numeric(xaxis)){
    xaxisLabel <- names(xaxis)
    if(length(xaxisLabel)!=length(xaxis)) xaxisLabel <- TRUE
    grid.xaxis(at=xaxis, label=xaxisLabel, gp=gp)
  }
}

getYXratio <- function(){
  as.numeric(convertHeight(unit(1, 'snpc'), 'npc'))/
    as.numeric(convertWidth(unit(1, "snpc"), "npc"))
}
#"circle", "square", "diamond", "triangle_point_up", "star", or "triangle point_down"
grid.circle1 <- function(x = 0.5, y = 0.5, r = 0.5, 
                         default.units = "npc", name = NULL, 
                         gp = gpar(), draw = TRUE, vp = NULL){
  fill <- gp$fill
  col <- gp$col
  lwd <- if(length(gp$lwd)>0) gp$lwd else 1
  alpha <- gp$alpha
  if(is.null(fill)) fill <- "white"
  twopi <- 2 * pi
  ratio.yx <- getYXratio()
  t2xy <- function(t) {
    t2p <- twopi * t + pi/2
    list(x = r * cos(t2p)/ratio.yx, y = r * sin(t2p))
  }
  P <- t2xy(seq.int(0, 1, length.out = 100))
  invisible(grid.polygon(unit(P$x+x,"npc"), unit(P$y+y, "npc"), 
                         gp=gpar(col = col, fill = fill, lwd=lwd, alpha=alpha)))
}

grid.square <- function(x = 0.5, y = 0.5, r = 0.5, 
                        default.units = "npc", name = NULL, 
                        gp = gpar(), draw = TRUE, vp = NULL){
  fill <- gp$fill
  col <- gp$col
  lwd <- if(length(gp$lwd)>0) gp$lwd else 1
  alpha <- gp$alpha
  if(is.null(fill)) fill <- "white"
  ratio.yx <- getYXratio()
  invisible(grid.rect(unit(x,"npc"), unit(y, "npc"), 
                      width = unit(r*2/ratio.yx, "npc"), 
                      height = unit(r*2, "npc"),
                      gp=gpar(col = col, fill = fill, lwd=lwd, alpha=alpha)))
}

grid.diamond <- function(x = 0.5, y = 0.5, r = 0.5, 
                        default.units = "npc", name = NULL, 
                        gp = gpar(), draw = TRUE, vp = NULL){
  fill <- gp$fill
  col <- gp$col
  lwd <- if(length(gp$lwd)>0) gp$lwd else 1
  alpha <- gp$alpha
  if(is.null(fill)) fill <- "white"
  ratio.yx <- getYXratio()
  P <- 
    list(x = c(0, r/ratio.yx, 0, -r/ratio.yx), 
         y = c(-r, 0, r, 0))
  invisible(grid.polygon(unit(P$x+x,"npc"), unit(P$y+y, "npc"), 
                         gp=gpar(col = col, fill = fill, lwd=lwd, alpha=alpha)))
}

grid.triangle_point_up <- function(x = 0.5, y = 0.5, r = 0.5, 
                         default.units = "npc", name = NULL, 
                         gp = gpar(), draw = TRUE, vp = NULL){
  fill <- gp$fill
  col <- gp$col
  lwd <- if(length(gp$lwd)>0) gp$lwd else 1
  alpha <- gp$alpha
  if(is.null(fill)) fill <- "white"
  ratio.yx <- getYXratio()
  P <- 
    list(x = c(-r/ratio.yx, r/ratio.yx, 0, -r/ratio.yx), 
         y = c(-r, -r, r, -r))
  invisible(grid.polygon(unit(P$x+x,"npc"), unit(P$y+y, "npc"), 
                         gp=gpar(col = col, fill = fill, lwd=lwd, alpha=alpha)))
}

grid.triangle_point_down <- function(x = 0.5, y = 0.5, r = 0.5, 
                                   default.units = "npc", name = NULL, 
                                   gp = gpar(), draw = TRUE, vp = NULL){
  fill <- gp$fill
  col <- gp$col
  lwd <- if(length(gp$lwd)>0) gp$lwd else 1
  alpha <- gp$alpha
  if(is.null(fill)) fill <- "white"
  ratio.yx <- getYXratio()
  P <- 
    list(x = c(-r/ratio.yx, r/ratio.yx, 0, -r/ratio.yx), 
         y = c(r, r, -r, r))
  invisible(grid.polygon(unit(P$x+x,"npc"), unit(P$y+y, "npc"), 
                         gp=gpar(col = col, fill = fill, lwd=lwd, alpha=alpha)))
}


grid.star <- function(x = 0.5, y = 0.5, r = 0.5, 
                      default.units = "npc", name = NULL, 
                      gp = gpar(), draw = TRUE, vp = NULL){
  fill <- gp$fill
  col <- gp$col
  lwd <- if(length(gp$lwd)>0) gp$lwd else 1
  alpha <- gp$alpha
  if(is.null(fill)) fill <- "white"
  ratio.yx <- getYXratio()
  i <- 1:11
  angle <- 180
  alpha <- 2*pi / 10
  r <- r * (i %% 2 + 1)/2
  omega <- alpha * i + angle * pi /180
  invisible(grid.polygon(unit(r*sin(omega)/ratio.yx+x,"npc"), 
                         unit(r*cos(omega)+y, "npc"), 
                         gp=gpar(col = col, fill = fill, lwd=lwd, alpha=alpha)))
}

cleanDataMcols <- function(this.dat.mcols, type){
  this.dat.mcols <- 
    this.dat.mcols[, 
                   !colnames(this.dat.mcols) %in% 
                     c("color", "fill", "lwd", "id", 
                       "cex", "dashline.col", 
                       "id.col", "stack.factor", "SNPsideID",
                       "shape", "alpha"), 
                   drop=FALSE]
  if(type!="pie.stack"){
    this.dat.mcols <- 
      this.dat.mcols[, !colnames(this.dat.mcols) %in% 
                       c("stack.factor.order", 
                         "stack.factor.first"), 
                     drop=FALSE]
  }
  this.dat.mcols <- 
    this.dat.mcols[, !grepl("^label.parameter",
                            colnames(this.dat.mcols)), 
                   drop=FALSE]
  this.dat.mcols <- 
    this.dat.mcols[, !grepl("^node.label",
                            colnames(this.dat.mcols)), 
                   drop=FALSE]
  return(this.dat.mcols)
}

#' plot the tracks
#' @description A function to plot the data for given range
#' @param trackList an object of \code{\link{trackList}}
#' @param chromosome chromosome
#' @param start start position
#' @param end end position
#' @param strand strand
#' @param gr an object of \code{\link[GenomicRanges:GRanges-class]{GRanges}}
#' @param ignore.strand ignore the strand or not when do filter. default TRUE
#' @param viewerStyle an object of \code{\link{trackViewerStyle}}
#' @param autoOptimizeStyle should use \code{\link{optimizeStyle}} to optimize style
#' @param newpage should be draw on a new page?
#' @param operator operator, could be +, -, *, /, ^, \%\%, and NA. "-" means dat - dat2, 
#' and so on. NA means do not apply any operator.
#' Note: if multiple operator is supplied, please make sure the length of operator keep
#' same as the length of trackList.
#' @param smooth logical(1) or numeric(). Plot smooth curve or not. If it is numeric, eg n,
#' mean of nearby n points will be used for plot.
#' If it is numeric, the second number will be the color. Default coloer is 2 (red).
#' @param lollipop_style_switch_limit The cutoff value for lollipop style for the 'circle' type.
#' If the max score is greater than this cutoff value, trackViewer will only plot one shape at
#' the highest score. Otherwise trackViewer will draw the shapes like `Tanghulu`.
#' @return An object of \code{\link[grid]{viewport}} for \code{\link{addGuideLine}}
#' @import GenomicRanges
#' @import grid
#' @importFrom grDevices as.raster col2rgb colorRampPalette palette
#' @importFrom scales rescale
#' @importFrom stats ksmooth
#' @importFrom IRanges viewMaxs Views
#' @export
#' @seealso See Also as \code{\link{addGuideLine}}, \code{\link{addArrowMark}}
#' @examples 
#' extdata <- system.file("extdata", package="trackViewer",
#'                        mustWork=TRUE)
#' files <- dir(extdata, "-.wig")
#' tracks <- lapply(paste(extdata, files, sep="/"), 
#'                  importScore, format="WIG")
#' tracks <- lapply(tracks, function(.ele) {strand(.ele@dat) <- "-"; .ele})
#' fox2 <- importScore(paste(extdata, "fox2.bed", sep="/"), format="BED")
#' dat <- coverageGR(fox2@dat)
#' fox2@dat <- dat[strand(dat)=="+"]
#' fox2@dat2 <- dat[strand(dat)=="-"]
#' gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
#' viewTracks(trackList(track=tracks, fox2=fox2), gr=gr, autoOptimizeStyle=TRUE)

viewTracks <- function(trackList, chromosome, start, end, strand, gr=GRanges(),
                       ignore.strand=TRUE,
                       viewerStyle=trackViewerStyle(), autoOptimizeStyle=FALSE,
                       newpage=TRUE, operator=NULL, smooth=FALSE,
                       lollipop_style_switch_limit=10){
  if(!is.null(operator)){
    if(!all(operator %in% c("+", "-", "*", "/", "^", "%%", NA))){
      stop('operator must be one of "+", "-", "*", "/", "^", "%%", NA')
    }
  }
  stopifnot(is.numeric(smooth)||is.logical(smooth))
  if(missing(trackList)){
    stop("trackList is required.")
  }
  if(!is(trackList, "trackList") && 
     !(is.list(trackList) && 
        all(vapply(trackList, function(.ele) is(.ele, "track"), 
                   FUN.VALUE = TRUE)))){
    stop("trackList must be an object of \"trackList\"
         (See ?trackList) or a list of track")
  }
  filterTracksFlag <- TRUE
  if(missing(gr)){
    if(missing(chromosome) || missing(start) || missing(end))
      stop("Please input the coordinate.")
    if(missing(strand) || !strand %in% c("+", "-"))
      strand <- "*"
    wavyLine <- FALSE
  }else{
    if(!is(gr, "GRanges")){
      stop("gr must be an object of GRanges.")
    }
    if(length(gr)==0){
      stop("the length of gr must greater than 0.")
    }
    if(length(gr$filterTracks)>0){
      filterTracksFlag <- as.logical(gr$filterTracks[1])
    }
    grs <- range(gr)
    chromosome <- as.character(GenomicRanges::seqnames(grs))[1]
    start <- GenomicRanges::start(grs)[1]
    end <- GenomicRanges::end(grs)[1]
    strand <- as.character(GenomicRanges::strand(grs))[1]
    if(ignore.strand) strand <- "*"
    if(length(gr)>1){ ## more than one region
      if(length(unique(as.character(seqnames(gr))))>1){
        stop("gr has multiple seqnames.")
      }
      if(viewerStyle@flip){
        gr <- sort(gr, decreasing = TRUE)
      }else{
        gr <- sort(gr, decreasing = FALSE)
      }
      if(length(gr$percentage)==0){
        gr$percentage <- 1/length(gr)
      }else{
        gr$percentage <- gr$percentage/sum(gr$percentage, na.rm = TRUE)
        gr$percentage[is.na(gr$percentage)] <- 0
      }
      gr$wavyLine <- TRUE
      gr$wavyLine[length(gr)] <- FALSE
      gr$filterTracks <- FALSE
      if(newpage) grid.newpage()
      
      if(autoOptimizeStyle){
        opt <- optimizeStyle(trackList, viewerStyle)
        trackList <- opt$tracks
        viewerStyle <- opt$style
      }
      trackList <- filterTracks(trackList, chromosome, start, end, strand) ## there is a bug if gr has different strand.
      for(i in seq_along(gr)){
        current.viewerStyle <- viewerStyle
        current.trackList <- trackList
        hgap <- convertWidth(unit(0.25, "lines"), 
                             unitTo = "npc", 
                             valueOnly = TRUE)
        if(i==1){
          setTrackViewerStyleParam(current.viewerStyle, 
                                   "margin", 
                                   c(current.viewerStyle@margin[1],
                                     current.viewerStyle@margin[2]/gr$percentage[i],
                                     current.viewerStyle@margin[3],
                                     hgap/gr$percentage[i]))
          for(j in seq_along(current.trackList)){
            tLjStyle <- current.trackList[[j]]@style
            if(tLjStyle@yaxis@draw && !tLjStyle@yaxis@main){
              setTrackYaxisParam(current.trackList[[j]],
                                 "draw",
                                 FALSE)
            }
            if(!tLjStyle@ylabpos %in% 
               c("upstream", "downstream") &&
               grepl("right", tLjStyle@ylabpos)){
              names(current.trackList)[[j]] <- ""
            }else{
              tLjStyle@yaxis@gp$cex <- tLjStyle@yaxis@gp$cex / gr$percentage[i]
              setTrackYaxisParam(current.trackList[[j]],
                                 "gp", tLjStyle@yaxis@gp)
            }
          }
        }else{
          if(i==length(gr)){
            setTrackViewerStyleParam(
              current.viewerStyle, 
              "margin", 
              c(current.viewerStyle@margin[1],
                hgap/gr$percentage[i],
                current.viewerStyle@margin[3],
                current.viewerStyle@margin[4]/gr$percentage[i])
            )
            for(j in seq_along(current.trackList)){
              tLjStyle <- current.trackList[[j]]@style
              if(tLjStyle@yaxis@draw && tLjStyle@yaxis@main){
                setTrackYaxisParam(current.trackList[[j]],
                                   "draw",
                                   FALSE)
              }
              if(!tLjStyle@ylabpos %in% 
                 c("upstream", "downstream") &&
                 grepl("left", tLjStyle@ylabpos)){
                names(current.trackList)[[j]] <- ""
              }else{
                tLjStyle@yaxis@gp$cex <- tLjStyle@yaxis@gp$cex / gr$percentage[i]
                setTrackYaxisParam(current.trackList[[j]],
                                   "gp", tLjStyle@yaxis@gp)
              }
            }
          }else{
            setTrackViewerStyleParam(
              current.viewerStyle, 
              "margin", 
              c(current.viewerStyle@margin[1],
                hgap/gr$percentage[i],
                current.viewerStyle@margin[3],
                hgap/gr$percentage[i])
            )
            for(j in seq_along(current.trackList)){
              setTrackYaxisParam(current.trackList[[j]],
                                 "draw",
                                 FALSE)
              if(!current.trackList[[j]]@style@ylabpos %in% 
                 c("upstream", "downstream")){
                names(current.trackList)[[j]] <- ""
              }else{
                tLjStyle@yaxis@gp$cex <- tLjStyle@yaxis@gp$cex / gr$percentage[i]
                setTrackYaxisParam(current.trackList[[j]],
                                   "gp", tLjStyle@yaxis@gp)
              }
            }
          }
        }
        pushViewport(vp=viewport(
          x = unit(ifelse(i==1, 
                          0, 
                          sum(gr$percentage[seq_len(i-1)]))+
                     gr$percentage[i]/2-hgap/2, 
                   "npc"),
          width = unit(gr$percentage[i]-hgap, "npc"),
          name = paste0("vp.track.v.", i)
        ))
        vp <- viewTracks(trackList = current.trackList,
                         gr = gr[i],
                         ignore.strand = ignore.strand,
                         viewerStyle = current.viewerStyle,
                         autoOptimizeStyle = FALSE,
                         newpage = FALSE, operator = operator)
        vp$name <- paste0("panel.", i)
        pushViewport(vp)
        upViewport()
        upViewport()
      }
      return(invisible(current.vpTree()))
    }
    wavyLine <- ifelse(length(gr$wavyLine)>0, as.logical(gr$wavyLine[1]),
                       FALSE)
  }
  if(ignore.strand) strand <- "*"
  if(end < start) stop("end should be greater than start.")
  
  if(!is(viewerStyle, "trackViewerStyle")){
    stop("viewerStyle must be an object of 'trackViewerStyle'.")
  }
  if(!is.logical(newpage)) stop("newpage should be a logical vector.")
  
  if(filterTracksFlag) {
    trackList <- filterTracks(trackList, chromosome, start, end, strand)
  }
  
  if(!is.null(operator)){
    ##change dat as operator(dat, dat2)
    ##dat2 no change
    operator <- rep(operator, length(trackList))[seq_along(trackList)]
    trackList <- mapply(trackList, operator, FUN=function(.ele, .operator){
      if(.ele@type=="data" && 
         length(.ele@dat2)>0 &&
         length(.ele@dat)>0 &&
         !is.na(.operator)){
        .ele@dat <- GRoperator(.ele@dat, 
                               .ele@dat2, 
                               col="score", 
                               operator=.operator)
        if(.operator !="+") .ele@dat2 <- GRanges()
      }else{
        warning("operator not supported for the inputs.")
      }
      .ele
    }, SIMPLIFY = FALSE)
  }else{
    operator <- rep(NA, length(trackList))
  }
  
  if(newpage) grid.newpage()
  
  if(autoOptimizeStyle){
    opt <- optimizeStyle(trackList, viewerStyle)
    trackList <- opt$tracks
    viewerStyle <- opt$style
  }
  
  margin <- viewerStyle@margin
  
  ##xscale, yscale
  xscale <- c(start, end)
  
  yscales <- getYlim(trackList, operator)
  yHeights <- getYheight(trackList)
  
  pushViewport(viewport(x=margin[2], y=margin[1], width=1-margin[2]-margin[4],
                        height=1-margin[1]-margin[3], just=c(0,0)))
  if(viewerStyle@xaxis){##draw x axis in bottom
    drawXaxis(xscale, viewerStyle)
  }
  popViewport()
  
  pushViewport(viewport(x=0, y=margin[1], width=1,
                        height=1-margin[1]-margin[3], just=c(0,0)))
  total <- length(trackList)
  #    if(interactive()) pb <- txtProgressBar(min=0, max=total, style=3)
  ht <- 0
  xy <- vector(mode="list", length=total)
  yHeightBottom <- yHeightTop <- rep(0.01, total)
  for(i in 1:total){
    xy[[i]] <- plotTrack(names(trackList)[i], trackList[[i]], 
                         viewerStyle, ht,
                         yscales[[i]], yHeights[i], xscale,
                         chromosome, strand, operator[i], wavyLine,
                         smooth=smooth,
                         lollipop_style_switch_limit=lollipop_style_switch_limit)
    ht <- ht + yHeights[i]
    if(length(trackList[[i]]@style@marginBottom)>0){
      yHeightBottom[i] <- trackList[[i]]@style@marginBottom
    }
    if(length(trackList[[i]]@style@marginTop)>0){
      yHeightTop[i] <- trackList[[i]]@style@marginTop
    }
    #        if(interactive()) setTxtProgressBar(pb, i)
  }
  #    if(interactive()) close(pb)
  popViewport()
  if(viewerStyle@flip) xscale <- rev(xscale)
  options(LastTrackViewer=list(vp=viewport(x=margin[2], y=margin[1], 
                                           height=1 - margin[1]- margin[3], 
                                           width=1 -margin[2] - margin[4],
                                           just=c(0,0), xscale=xscale, yscale=c(0,1)),
                               xy=xy,
                               yHeights=yHeights,
                               yscales=yscales,
                               xscale=xscale,
                               yHeightBottom=yHeightBottom,
                               yHeightTop=yHeightTop,
                               viewerStyle=viewerStyle,
                               chromosome=chromosome,
                               back2back=vapply(trackList, FUN=function(.ele){
                                 c(score1=length(.ele$dat$score),
                                   score2=length(.ele$dat2$score),
                                   target1=length(.ele$dat$target),
                                   target2=length(.ele$dat$target),
                                   ysplit=.ele$style$ysplit)
                               }, FUN.VALUE = numeric(5L))))
  return(invisible(viewport(x=margin[2], y=margin[1], 
                            height=1 - margin[1]- margin[3], 
                            width=1 -margin[2] - margin[4],
                            just=c(0,0), xscale=xscale, yscale=c(0,1))))
  }

#' Add arrow mark to the figure at a given position
#' @description A function to add arrow mark for emphasizing peaks 
#' @param pos A unit object representing the location of arrow mark to be placed 
#' at current viewport. Default is the value of grid.locator, which will get the 
#' location of the mouse click.
#' @param label A character or expression vector.
#' @param angle A parameter passed into grid::arrow function. The angle of arrow 
#' head in degrees (smaller numbers produce narrower, pointier arrows). Essentially 
#' describes the width of the arrow head.
#' @param length A parameter passed into grid::arrow function. Aunit specifying
#' the length of the arrow head.
#' @param col color of the arrow
#' @param cex Multiplier applied to fontsize
#' @param quadrant the direction of arrow, 1: to bottomleft, 2: to bottomright,
#' 3: to topright, 4: to topleft
#' @param type A parameter passed into grid::arrow function. One of "open" or "closed"
#' indicating whether the arrow head should be a closed triangle.
#' @param vp A Grid viewport object. It must be output of \code{\link{viewTracks}}
#' @return invisible x, y position value.
#' @import grid
#' @export
#' @seealso See Also as \code{\link{addGuideLine}}, \code{\link[grid:arrow]{arrow}}
#' @examples 
#' grid.newpage()
#' addArrowMark(list(x=unit(.5, "npc"), 
#'                y=unit(.5, "npc")), 
#'                label="label1",
#'                col="blue")
#' ##  how to get the position by mouse click
#' if(interactive()){
#' pos <- addArrowMark(label="byClick")
#' addArrowMark(pos, label="samePosAsAbove")
#' }
#' 
addArrowMark <- function(pos=grid.locator(), label=NULL, angle=15, 
                         length=unit(.25, "inches"), col="red", cex=1, quadrant=4,
                         type="closed", vp=NULL){
    x <- pos$x
    y <- pos$y
    if(!inherits(x, c("numeric", "integer", "unit")) |
           length(x) < 1)
        stop("x is required as a numeric vector of x-coordinates of genome")
    if(!inherits(y, c("numeric", "integer", "unit")) |
           length(y) < 1)
        stop("y is required as a numeric vector of y-coordinates")
    if(length(x)!=length(y))
        stop("The length of x and y is not identical.")
    if(!quadrant %in% 1:4)
        stop("quadrant must be a integer in 1, 2, 3, 4")
    if(length(vp)>0){
        stopifnot(is(vp, "viewport"))
        if(is(vp, "vpTree")){
           stop("NOT ready for breaked axis.")
        }
    }
    len <- length(x)
    lastTrackViewer <- getOption("LastTrackViewer")
    lastTrackViewerXscale <- c(0, 1)
    if(length(lastTrackViewer)>0) {
        lastTrackViewerXscale <- sort(lastTrackViewer$xscale)
    }
    if(all(inherits(y, c("numeric", "integer")))){
        if(all(floor(y)==y &ceiling(y)==y)){
            xy <- lastTrackViewer$xy
            yHeights <- c(0, lastTrackViewer$yHeights)
            yHeights <- cumsum(yHeights)
            yscales <- lastTrackViewer$yscales
            xscale <- lastTrackViewer$xscale
        }
    }
    
    trimLen <- function(obj, len){
        if(length(obj)<len) 
            obj <- rep(obj, len)[1:len]
        obj
    }
    col <- trimLen(col, len)
    angle <- trimLen(angle, len)
    length <- trimLen(length, len)
    type <- trimLen(type, len)
    cex <- trimLen(cex, len)
    for(i in 1:len){
        if(inherits(x[i], c("numeric", "integer"))){
            if(findInterval(x[i], c(0, 1))==1){
                xi <- unit(x[i], "npc")
            }else{
                #if(!is.null(vp)) {
                    #r <- sort(vp$xscale)
                    #if(findInterval(x[i], r)==1){
                        xi <- unit(x[i], "native")
                    #}
                #}
            }
            if(!is(xi, "unit")) stop("'pos$x' argument must be a unit object.")
        }else{
            xi <- x[i]
        }
        if(inherits(y[i], c("numeric", "integer"))){
            if(findInterval(y[i], c(0, 1))==1){
                yi <- unit(y[i], "npc")
            }else{
                #if(!is.null(vp)) {
                    #r <- vp$yscale
                    #if(y[i]<=r[2] && y[i]>=r[1]){
                        yi <- unit(y[i], "native")
                    #}
                #}
                 if(length(lastTrackViewer)>0 && findInterval(as.numeric(xi), lastTrackViewerXscale)){
                     curViewStyle <- lastTrackViewer$viewerStyle
                     yvp <- list(vp1=viewport(x=0, y=yHeights[y[i]], 
                                              height=diff(yHeights)[y[i]],
                                              width=1, 
                                              just=c(0,0),
                                              clip="off"),
                                 vp2=viewport(x=0, y=lastTrackViewer$yHeightBottom[y[i]], 
                                              height=1 - lastTrackViewer$yHeightTop[y[i]] - 
                                                  lastTrackViewer$yHeightBottom[y[i]],
                                              width=1, 
                                              just=c(0,0),
                                              clip="off", 
                                              xscale=xscale, 
                                              yscale=yscales[[y[i]]]))
                     yi <- xy[[y[i]]]
                     yi <- yi$y[c(0, yi$x[-length(yi$x)])<=x[i] & c(yi$x[-1], .Machine$integer.max)>=x[i]]
                     if(length(yi)>0){
                         yi <- yi[1]
                     }else{
                         yi <- 0
                     }
                     yi <- unit(yi, "native")
                 }
            }
            if(!is(yi, "unit")) stop("'pos$y' argument must be a unit object.")
        }else{
            yi <- y[i]
        }
        ar <- arrow(angle=angle[i], length=length[i], ends="first", type=type[i])
        hjust <- switch(quadrant, 0, 1, 1, 0, 0)
        if(!is.null(vp)){
            pushViewport(vp)
            popv <- FALSE
            if(length(lastTrackViewer)>0 && findInterval(as.numeric(xi), lastTrackViewerXscale) && 
               (inherits(y[i], c("numeric", "integer"))) & findInterval(y[i], c(0, 1))!=1){
                pushViewport(yvp$vp1)
                pushViewport(yvp$vp2)
                popv <- TRUE
            }
            xi <- as.numeric(convertUnit(xi, "native", axisFrom="x"))
            yi <- as.numeric(convertUnit(yi, "native", axisFrom="y"))
            if(popv){
                xi1 <- switch(quadrant,
                              xi + as.numeric(convertUnit(length[i], "native", axisFrom="x")) - xscale[1],
                              xi - as.numeric(convertUnit(length[i], "native", axisFrom="x")) + xscale[1],
                              xi - as.numeric(convertUnit(length[i], "native", axisFrom="x")) + xscale[1],
                              xi + as.numeric(convertUnit(length[i], "native", axisFrom="x")) - xscale[1],
                              xi + as.numeric(convertUnit(length[i], "native", axisFrom="x")) - xscale[1])
                yi1 <- switch(quadrant,
                              yi + as.numeric(convertUnit(length[i], "native", axisFrom="y")) - yscales[[y[i]]][1],
                              yi + as.numeric(convertUnit(length[i], "native", axisFrom="y")) - yscales[[y[i]]][1],
                              yi - as.numeric(convertUnit(length[i], "native", axisFrom="y")) + yscales[[y[i]]][1],
                              yi - as.numeric(convertUnit(length[i], "native", axisFrom="y")) + yscales[[y[i]]][1],
                              yi - as.numeric(convertUnit(length[i], "native", axisFrom="y")) + yscales[[y[i]]][1])
            }else{
                xi1 <- switch(quadrant,
                              xi + as.numeric(convertUnit(length[i], "native", axisFrom="x")) - vp$xscale[1],
                              xi - as.numeric(convertUnit(length[i], "native", axisFrom="x")) + vp$xscale[1],
                              xi - as.numeric(convertUnit(length[i], "native", axisFrom="x")) + vp$xscale[1],
                              xi + as.numeric(convertUnit(length[i], "native", axisFrom="x")) - vp$xscale[1],
                              xi + as.numeric(convertUnit(length[i], "native", axisFrom="x")) - vp$xscale[1])
                yi1 <- switch(quadrant,
                              yi + as.numeric(convertUnit(length[i], "native", axisFrom="y")) - vp$yscale[1],
                              yi + as.numeric(convertUnit(length[i], "native", axisFrom="y")) - vp$yscale[1],
                              yi - as.numeric(convertUnit(length[i], "native", axisFrom="y")) + vp$yscale[1],
                              yi - as.numeric(convertUnit(length[i], "native", axisFrom="y")) + vp$yscale[1],
                              yi - as.numeric(convertUnit(length[i], "native", axisFrom="y")) + vp$yscale[1])
            }
            
            grid.lines(x=c(xi, xi1), y=c(yi, yi1), 
                       gp=gpar(col=col[i], fill=col[i]), arrow=ar,
                       default.units="native")
            if(!is.null(label[i])) grid.text(label=label[i], x = xi1, y = yi1, hjust = hjust,
                                          default.units = "native", gp = gpar(col=col[i], cex=cex[i]))
            if(popv){
                popViewport()
                popViewport()
            }
            popViewport()
        }else{
            xi <- as.numeric(convertUnit(xi, "native", axisFrom="x"))
            yi <- as.numeric(convertUnit(yi, "native", axisFrom="y"))
            l <- min(as.numeric(convertUnit(length[i], "native", axisFrom="x")),
                     as.numeric(convertUnit(length[i], "native", axisFrom="y")))
            xi1 <- switch(quadrant,
                          xi + l,
                          xi - l,
                          xi - l,
                          xi + l,
                          xi + l)
            yi1 <- switch(quadrant,
                          yi - l,
                          yi - l,
                          yi + l,
                          yi + l,
                          yi - l)
            
            grid.lines(x=c(xi, xi1), y=c(yi, yi1), 
                       gp=gpar(col=col[i], fill=col[i]), arrow=ar,
                       default.units="native")
            if(!is.null(label[i])) grid.text(label=label[i], x = xi1, y = yi1, hjust = hjust, 
                                          default.units = "native", 
                                          gp = gpar(col=col[i], cex=cex[i]))
        }
    }
    return(invisible(pos))
}

#' get genomic location by gene symbol
#' @description given a gene name, get the genomic coordinates.
#' @param symbol Gene symbol
#' @param txdb txdb will be used to extract the genes
#' @param org org package name
#' @import GenomicRanges
#' @importFrom AnnotationDbi mget
#' @importFrom Seqinfo seqlevels "seqlevels<-" seqinfo "seqinfo<-"
#' @importFrom GenomeInfoDb "seqlevelsStyle<-"
#' @importFrom GenomicFeatures genes
#' @import IRanges
#' @export
#' @examples 
#' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
#' library(org.Hs.eg.db)
#' getLocation("HSPA8", TxDb.Hsapiens.UCSC.hg19.knownGene, "org.Hs.eg.db")

getLocation <- function(symbol, txdb, org){
  stopifnot(is(symbol, "character"))
  stopifnot(is(txdb, "TxDb"))
  stopifnot(is.character(org))
  stopifnot(length(org)==1)
  if(!grepl("^org.*.db", org)){
    stop("Org package name is required.")
  }
  dbPrefix <- sub(".db", "", org)
  eg <- mget(symbol, get(paste0(dbPrefix, "SYMBOL2EG")), ifnotfound = NA)
  eg <- lapply(eg, `[`, i=1)
  eg <- unlist(eg, use.names = TRUE)
  eg <- eg[!is.na(eg)]
  if(length(eg)<1){
    return(NULL)
  }
  genes <- genes(txdb)
  gr <- genes[eg]
  seqlevelsStyle(gr) <- "UCSC"
  seqlevels(gr) <- seqlevels(gr)[seqlevels(gr) %in% as.character(seqnames(gr))]
  if(length(seqinfo(gr))>0){
    seqinfo(gr) <- seqinfo(gr)[seqlevels(gr)]
  }
  gr
}

plotScSeqTrack <- function(.dat, .dat2, scale, color, yscale, breaks, NAcolor="white"){
  if(length(names(.dat))<1){## no cell barcodes
    return()
  }
  if(length(.dat)<1){
    return()
  }
  ## remove unused mcols
  mcols(.dat) <- mcols(.dat)[, "score"] 
  colnames(mcols(.dat)) <- "score"
  ## split the data by cell barcode
  .dat <- split(.dat, names(.dat))
  ## resort the data by signals
  .dat <- .dat[order(vapply(.dat, FUN=function(.ele){
    mean(.ele$score, na.rm = TRUE)
  }, FUN.VALUE = 0.0), decreasing = TRUE)]
  id <- rep(seq_along(.dat), lengths(.dat))
  .dat <- unlist(GRangesList(.dat), use.names = FALSE)
  .dat$id <- id
  ## convert GRanges to matrix or plot it line by line?
  if(missing(yscale)) yscale <- c(0, 1)
  ## plot rect at position
  ## x = c(start, start, end, end)
  ## heightPerCell <- diff(yscale)/(numberOfCell+1)
  ## y = c(i-.5, i+.5, i+.5, i-.5)*heightPerCell # i is the cell order
  ## color = colorRampPalette(color)(100)
  if(length(breaks)<3){
    rg <- range(.dat$score[!is.na(.dat$score)])
    if(length(rg)!=2){
      return()
    }
    rg <- rg + c(-1, 1)*diff(rg)/1000
    if(rg[1]==rg[2] && rg[1]==0){
      rg[2] <- 1
    }
    breaks <- seq(min(0, rg[1]), rg[2], length.out = 101)
  }
  
  if(length(unique(color))==1){
    if(!tolower(color[1]) %in% c("white", "#ffffff", "#fff")){
      color <- c("white", color[1])
    }else{
      color <- c("black", color[1])
    }
  }
  if(length(color)==0){
    color <- c("white", "red")
  }
  crp <- colorRampPalette(color)(length(breaks)-1)
  mc <- cut(.dat$score, breaks = breaks, labels = crp)
  mc <- as.character(mc)
  mc[is.na(mc)] <- NAcolor
  
  yh <- diff(yscale)/max(.dat$id)
  
  for(i in seq_along(.dat)){
      grid.polygon(x=c(start(.dat)[i], start(.dat)[i], 
                       end(.dat)[i], end(.dat)[i]), 
                   y=c(i-.5, i+.5, i+.5, i-.5)*yh, 
                   default.units="native",
                   gp = gpar(fill=mc[i], col = mc[i]))
  }
  # legend
  vp <- viewport(x = 1 - convertWidth(unit(5, "char"), "npc", valueOnly = TRUE), 
                 y= 1 - convertHeight(unit(1, "char"), "npc", valueOnly = TRUE), 
                 width = convertWidth(unit(5, "char"), "npc", valueOnly = TRUE),
                 height = convertHeight(unit(.5, "char"), "npc", valueOnly = TRUE),
                 default.units = "npc", xscale = range(breaks))
  pushViewport(vp)
  grid.raster(t(crp), width=1, height=1)
  grid.xaxis(at=round(range(breaks)), gp=gpar(fontsize=8))
  popViewport()
}

# codes for cooler
guessFormat <- function(file, format){
  if(is.character(file)){
    if(!grepl(format, file)){
      warning("The input file is:", file, 
              " But the input format is:", format)
    }
  }
}
checkCoolFile <- function(coolfile){
  stopifnot(length(coolfile)==1)
  stopifnot(is.character(coolfile) || is(coolfile, "H5IdComponent"))
  if(is.character(coolfile)){
    coolfile <- H5Fopen(coolfile, flags="H5F_ACC_RDONLY")
  }
  return(coolfile)
}
isMcool <- function(coolfile){
  coolRootName <- coolfileRootName(coolfile)
  return(coolRootName[1]!='')
}

coolfileRootName <- function(coolfile){
  coolfile <- checkCoolFile(coolfile)
  level1 <- h5ls(coolfile, recursive = FALSE)$name
  if(all(c("bins", "chroms", "indexes", "pixels") %in%
         level1)){
    return('')
  }else{
    return(level1)
  }
}

coolfileGetByPath <- function(coolfile, resolution, subPath){
  coolfile <- checkCoolFile(coolfile)
  if(!isMcool(coolfile)){
    res0 <- ""
  }else{
    available_res <- listResolutions(coolfile, "cool")
    if(!resolution %in% available_res){
      stop(paste("The given resolution is not available in file.",
                 "Available resolutions: ",
                 paste(available_res, collapse = ","), "."
      ))
    }
    coolfileRootName <- coolfileRootName(coolfile)
    res0 <- paste(coolfileRootName, resolution, sep='/')
  }
  if(!missing(subPath)){
    subPath <- subPath[1]
    if(subPath!="" && !is.na(subPath)){
      res0 <- paste(res0, subPath, sep='/')
    }
  }
  h5read(coolfile, res0)
}

cooler_indexes <- function(coolfile, resolution){
  coolfileGetByPath(coolfile, resolution, "indexes")
}

coolfilePathName <- function(rootName, resolution, subPath){
  stopifnot(is.character(subPath) && length(subPath)==1)
  ifelse(subPath %in% rootName, subPath, 
         paste(rootName, resolution, subPath, sep="/"))
}

cooler_bins <- function(coolfile, resolution, seqname, normalization){
  seqs <- coolfileGetByPath(coolfile, resolution, "chroms")
  if(!all(seqname %in% seqs$name)){
    stop(paste("The given seqname is not available in file.",
               "Available seqnames: ",
               paste(seqs, collapse = ","), "."
    ))
  }
  rootName <- coolfileRootName(coolfile)
  indexes <- cooler_indexes(coolfile, resolution)
  ir1 <- IRanges(indexes$chrom_offset[-length(indexes$chrom_offset)]+1,
                 indexes$chrom_offset[-1], 
                 names = seqs$name)
  ir0 <- ir1[seqname]
  name <- coolfilePathName(rootName, resolution, 'bins')
  bins <- list()
  on.exit(h5closeAll())
  for(i in seq_along(ir0)){
    h5closeAll()
    bins[[names(ir0)[i]]] <- 
      as.data.frame(h5read(coolfile, 
                           name=name, 
                           start=start(ir0[i]),
                           count = width(ir0[i])))
    bins[[names(ir0)[i]]]$bin <- seq(start(ir0[i]), end(ir0[i]))-1
  }
  bins <- do.call(rbind, bins)
  gr <- GRanges(seqnames = bins$chrom, 
                IRanges(bins$start+1, bins$end, names = bins$bin))
  if(normalization %in% colnames(bins)){
    gr$weight <- bins[, normalization, drop=TRUE]
  }else{
    gr$weight <- 1
  }
  gr
}

cooler_pixels <- function(coolfile, resolution, gr=GRanges(),
                          normalization='balanced'){
  stopifnot(is(gr, "GRanges"))
  stopifnot(length(gr)<3)
  if(length(gr)==0) return(NULL)
  if(normalization=='balanced'){
    normalization <- 'weight'
  }
  if(!isMcool(coolfile)) {
    resolution <- NULL
  } else{
    resolution <- format(resolution, scientific = FALSE)
  }
  seqname <- as.character(seqnames(gr))
  bins <- cooler_bins(coolfile, resolution, seqname, normalization)
  bins1 <- subsetByOverlaps(bins, gr[1])
  if(length(gr)==1){
    bins2 <- bins1
  }else{
    bins2 <- subsetByOverlaps(bins, gr[2])
  }
  if(length(bins)==0) return(NULL)
  indexes <- cooler_indexes(coolfile, resolution)
  bins_index <- IRanges(indexes$bin1_offset[-length(indexes$bin1_offset)]+1,
                        indexes$bin1_offset[-1],
                        names = seq.int(length(indexes$bin1_offset)-1)-1)
  ## read bin1
  bins_index_1 <- bins_index[names(bins1)]
  bins_index_1 <- bins_index_1[width(bins_index_1)>0]
  ir0 <- reduce(bins_index_1)
  if(length(ir0)==0){
    return(NULL)
  }
  rootName <- coolfileRootName(coolfile)
  name <- coolfilePathName(rootName, resolution, "pixels")
  name1 <- coolfilePathName(rootName, resolution, "bins")
  pixels <- list()
  on.exit(h5closeAll())
  for(i in seq_along(ir0)){
    h5closeAll()
    curr_pixels <- 
      as.data.frame(h5read(coolfile, 
                           name=name, 
                           start=start(ir0[i]),
                           count = width(ir0[i])))
    curr_pixels <- 
      curr_pixels[curr_pixels$bin2_id %in% names(bins2),,drop=FALSE]
    pixels[[i]] <- curr_pixels
  }
  pixels <- do.call(rbind, pixels)
  if(nrow(pixels)==0) return(NULL)
  gi <- GInteractions(anchor1 = bins[as.character(pixels$bin1_id)],
                      anchor2 = bins[as.character(pixels$bin2_id)],
                      count = pixels$count)
  gi$balanced <- gi$count*(gi$anchor1.weight*gi$anchor2.weight)
  gi$anchor1.weight <- NULL
  gi$anchor2.weight <- NULL
  gi
}

#' Lolliplots
#' @description Plot variants and somatic mutations
#' @param SNP.gr A object of \link[GenomicRanges:GRanges-class]{GRanges}, 
#' \link[GenomicRanges:GRangesList-class]{GRangesList}
#' or a list of \link[GenomicRanges:GRanges-class]{GRanges}.
#' All the width of GRanges must be 1.
#' @param features A object of \link[GenomicRanges:GRanges-class]{GRanges}, 
#' \link[GenomicRanges:GRangesList-class]{GRangesList}
#' or a list of \link[GenomicRanges:GRanges-class]{GRanges}. 
#' The metadata 'featureLayerID' are used for drawing features in different layers.
#'  See details in vignette.
#' @param ranges A object of \link[GenomicRanges:GRanges-class]{GRanges} or 
#' \link[GenomicRanges:GRangesList-class]{GRangesList}.
#' @param type character. Could be circle, pie, pin, pie.stack or flag.
#' @param newpage Plot in the new page or not.
#' @param ylab Plot ylab or not. If it is a character vector, 
#' the vector will be used as ylab.
#' @param yaxis Plot yaxis or not.
#' @param xaxis Plot xaxis or not. If it is a numeric vector with length greater than 1, 
#' the vector will be used as the points at which tick-marks are to be drawn. 
#' And the names of the vector will be used to as labels to be placed at the tick 
#' points if it has names. 
#' @param ylab.gp,xaxis.gp,yaxis.gp An object of class gpar for ylab, xaxis or yaxis.
#' @param legend If it is a list with named color vectors, a legend will be added.
#' @param legendPosition The position of legend. Possible positions are 'top',
#' 'right', and 'left'.
#' @param cex cex will control the size of circle.
#' @param dashline.col color for the dashed line.
#' @param jitter jitter the position of nodes or labels.
#' @param rescale logical(1), character(1), numeric vector,
#'   or a dataframe with rescale from and to. 
#' Rescalse the x-axis or not.
#' if dataframe is used, colnames must be from.start, from.end,
#'  to.start, to.end. And the from scale must cover the whole plot region.
#' The rescale parameter can be set as "exon" or "intron" to 
#' emphasize "exon" or "intron" region. The "exon" or "intron"
#' can be followed with an integer e.g. "exon_80", or "intron_99".
#' The integer indicates the total percentage of "exon" or "intron" region.
#' Here "exon" indicates all regions in features. 
#' And "intron" indicates all flank regions of the features.
#' @param label_on_feature Labels of the feature directly on them. 
#' Default FALSE.
#' @param lollipop_style_switch_limit The cutoff value for lollipop style for the 'circle' type.
#' If the max score is greater than this cutoff value, trackViewer will only plot one shape at
#' the highest score. Otherwise trackViewer will draw the shapes like `Tanghulu`.
#' @param ... not used.
#' @return NULL
#' @details 
#' In SNP.gr and features, metadata of the GRanges object will be used to control the 
#' color, fill, border, alpha, shape, height, cex, dashline.col, data source of pie if the type is pie. 
#' And also the controls for labels by name the metadata start as 
#' label.parameter.<properties>, and for node labels by name the metadata start as
#' node.label.<properties>,
#' such as label.parameter.rot, label.parameter.gp. The parameter is used for 
#' \link[grid]{grid.text}.or \link[motifStack]{plotMotifLogoA}.
#' The metadata 'featureLayerID' for features are used 
#' for drawing features in different layers. The metadata 'SNPsideID' for SNP.gr
#' are used for determining the side of lollipops. And the 'SNPsideID' could only
#' be 'top' or 'bottom'.
#' @return NULL
#' @import GenomicRanges
#' @import IRanges
#' @import grid
#' @importFrom scales rescale
#' @importClassesFrom grImport Picture
#' @importFrom grImport readPicture grid.picture
#' @importFrom grDevices rgb
#' @export
#' @examples
#' SNP <- c(10, 100, 105, 108, 400, 410, 420, 600, 700, 805, 840, 1400, 1402)
#' x <- sample.int(100, length(SNP))
#' SNP.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)), 
#'                   value1=x, value2=100-x)
#' SNP.gr$color <- rep(list(c("red", 'blue')), length(SNP))
#' SNP.gr$border <- sample.int(7, length(SNP), replace=TRUE)
#' features <- GRanges("chr1", IRanges(c(1, 501, 1001), 
#'                                     width=c(120, 500, 405),
#'                                     names=paste0("block", 1:3)),
#'                     color="black",
#'                     fill=c("#FF8833", "#51C6E6", "#DFA32D"),
#'                     height=c(0.1, 0.05, 0.08),
#'                     label.parameter.rot=45)
#' lolliplot(SNP.gr, features, type="pie") 
#'

lolliplot <- function(SNP.gr, features=NULL, ranges=NULL,
                      type="circle",
                      newpage=TRUE, ylab=TRUE, ylab.gp=gpar(col="black"),
                      yaxis=TRUE, yaxis.gp=gpar(col="black"), 
                      xaxis=TRUE, xaxis.gp=gpar(col="black"), 
                      legend=NULL, legendPosition='top', cex=1, 
                      dashline.col="gray80", 
                      jitter=c("node", "label"), 
                      rescale=FALSE, 
                      label_on_feature=FALSE,
                      lollipop_style_switch_limit=10,
                      ...){
    stopifnot(inherits(SNP.gr, c("GRanges", "GRangesList", "list")))
    stopifnot(inherits(features, c("GRanges", "GRangesList", "list")))
    jitter <- match.arg(jitter)
    legendPosition <- checkLegendPosition(legendPosition)
    rescale.old <- rescale
    xaxis.old <- xaxis
    if(any(type!="circle"&jitter=="label")){
      jitter[which(type!="circle"&jitter=="label")] <- "node"
      warning("if jitter set to label, type must be cirle.")
      message("jitter is set to node.")
    }
    SNP.gr.name <- deparse(substitute(SNP.gr))
    if(is(SNP.gr, "GRanges")){
        SNP.gr <- list(SNP.gr)
        if(length(SNP.gr.name)==length(SNP.gr)) {
          names(SNP.gr) <- SNP.gr.name
        }
    }
    len <- length(SNP.gr)
    for(i in seq.int(len)){
        stopifnot(is(SNP.gr[[i]], "GRanges"))
    }
    
    ## prepare the feature
    if(inherits(features, c("GRangesList", "list"))){
      for(i in seq_along(features)){
        stopifnot("features must be a GRanges or GRangesList object"=
                    is(features[[i]], "GRanges"))
      }
      features <- features[seq.int(len)]
    }else{
      stopifnot("features must be a GRanges or GRangesList object"=
                  is(features, "GRanges"))
      #features.name <- deparse(substitute(features))
      features <- list(features)[seq.int(len)]
    }
    
    
    TYPES <- c("circle", "pie", "pin", "pie.stack", "flag")
    if(any(!type %in% TYPES)){
        stop("Error in match argument: ",
             paste0("'type' should be one of '",  
                    paste(TYPES, collapse="', '"), "'."))
    }
    types <- rep(type, length=len)[seq.int(len)]
    rm(type)
    ############### handle legend ####################
    ## set the legend as a list, 
    ## if all the legend for different tracks is same
    ## set draw legend for last track later
    legend <- handleLegend(legend, len, SNP.gr)
    
    ################ handle ranges #####################
    ## if !missing(ranges) set ranges as feature ranges
    ranges <- handleRanges(ranges, SNP.gr, features, len)
    
    ##cut all SNP.gr by the range
    SNP.gr <- cutSNP(SNP.gr, ranges, len)
    
    dots <- list(...)
    if(length(dots$maxYlimNchar)>0) {
      maxYlimNchar<- dots$maxYlimNchar
    } else {
      if(all(c(TRUE, vapply(yaxis, isTRUE, logical(1L))))){
        maxYlimNchar <- getMaxYlimNchar(SNP.gr, 3.5, types)
      } else {
        if(length(yaxis)>1){
          maxYlimNchar <- max(c(1, nchar(as.character(yaxis))), na.rm = TRUE) +
            4
        }else{
          maxYlimNchar<- 2.5
        }
      }
    }
    
    ################## plot ############################
    ## total height == 1
    height <- 1/sum(lengths(ranges))
    args <- as.list(match.call())
    if(length(args$height0)==0){
      height0 <- 0
    }else{
      height0 <- args$height0
    }
    if(newpage) grid.newpage()
    for(i in seq.int(len)){
      if(length(ranges[[i]])>1){## split into multilayers
        args$newpage <- FALSE
        for(j in rev(seq_along(ranges[[i]]))){
          args$ranges <- ranges[[i]][j]
          args$SNP.gr <- SNP.gr[i]
          args$features <- features[[i]]
          args$type <- types[i]
          args$legend <- legend[[i]]
          args$legendPosition <- legendPosition
          args$height0 <- height0
          args$maxYlimNchar <- maxYlimNchar
          height0 <- do.call(what = lolliplot, args = args)
        }
      }else{## is GRanges with length==1
        type <- match.arg(types[i], TYPES)
        if(type=="pin"){ ## read the pin shape file
          pinpath <- system.file("extdata", "map-pin-red.xml", package="trackViewer")
          pin <- readPicture(pinpath)
        }else{
          pin <- NULL
        }
        ## Here we don't know the real height of each tracks
        vp0 <- viewport(x=.5, y=height0 + height*0.5,
                       width=1, height=height)
        pushViewport(vp0)
        LINEW <- as.numeric(convertX(unit(1, "line"), "npc"))
        LINEH <- as.numeric(convertY(unit(1, "line"), "npc"))
        totalH <- as.numeric(unit(1, "npc"))
        if(LINEH > totalH/20){
          LINEH <- totalH/20
        }
        
        ## GAP the gaps between any elements
        GAP <- .2 * LINEH
        
        SNPs <- SNP.gr[[i]]
        strand(SNPs) <- "*"
        SNPs <- sort(SNPs)
        
        feature <- features[[i]]
        
        ## rescale
        rescale <- rescale.old
        xaxis <- xaxis.old
        if(is.logical(rescale)[1]){
          if(rescale[1]){
            range.tile <- tile(ranges[[i]], n = 5)[[1]]
            if(all(width(range.tile)>2)){
              range.tile.cnt <- countOverlaps(range.tile, SNPs)
              feature.start <- feature.end <- feature
              end(feature.start) <- start(feature.start)
              start(feature.end) <- end(feature.end)
              range.tile.cnt2 <- countOverlaps(range.tile, unique(c(feature.start, feature.end)))
              range.tile.cnt <- range.tile.cnt + range.tile.cnt2
              range.width <- width(ranges[[i]])
              range.tile.width <- log2(range.tile.cnt + 1)
              range.tile.width <- range.tile.width/sum(range.tile.width)
              range.tile.width <- range.width * range.tile.width
              range.tile.width <- cumsum(range.tile.width)
              range.tile.width <- start(ranges[[i]]) + c(0, round(range.tile.width)-1)
              rescale <- data.frame(from.start=start(range.tile), from.end=end(range.tile),
                                    to.start=range.tile.width[-length(range.tile.width)],
                                    to.end=range.tile.width[-1])
              rescale$to.start[-1] <- rescale$to.start[-1] + 1
            }
          }
        }else{
          if(is.numeric(rescale)){## rescale is a percentage, convert it into from.start, from.end, to.start, to.end
            ## reset the scale for exons and introns
            feature.rd <- disjoin(c(feature, ranges[[i]]))
            feature.segment.points <- sort(unique(c(start(feature.rd), end(feature.rd))))
            ## filter the points by viewer port.
            feature.segment.points <- 
              feature.segment.points[feature.segment.points>=start(ranges[[i]]) &
                                       feature.segment.points<=end(ranges[[i]])]
            rescale <- rescale/sum(rescale, na.rm = TRUE)
            rescale[is.na(rescale)] <- 0.01
            if(length(rescale)==length(feature.segment.points)-1){
              rescale.ir <- IRanges(feature.segment.points[-length(feature.segment.points)]+1, 
                                    feature.segment.points[-1])
              start(rescale.ir)[1] <- start(rescale.ir)[1]-1
              rescale.ir.width <- sum(width(rescale.ir))
              rescale.ir.new.width <- cumsum(round(rescale.ir.width*rescale, digits = 0))
              rescale <- data.frame(from.start=start(rescale.ir), 
                                    from.end=end(rescale.ir),
                                    to.start=feature.segment.points[1] + 
                                      c(0, rescale.ir.new.width[-length(rescale.ir.new.width)]),
                                    to.end=feature.segment.points[1] + rescale.ir.new.width)
            }else{
              stop("The length of rescale is not as same as the number of segments (including features and non-features).")
            }
          }else{
            if(is.character(rescale)){
              if(grepl("exon|intron", rescale[1], ignore.case = TRUE)){
                ## reset the scale for exons and introns
                feature.rd <- disjoin(c(reduce(feature), ranges[[i]]), ignore.strand=TRUE)
                feature.rd <- subsetByOverlaps(feature.rd, ranges[[1]], ignore.strand=TRUE)
                feature.segment.exon <- subsetByOverlaps(feature.rd, feature, ignore.strand=TRUE)
                feature.segment.intron <- subsetByOverlaps(feature.rd, feature, invert=TRUE, ignore.strand=TRUE)
                feature.segment.exon$type <- rep("exon", length(feature.segment.exon))
                feature.segment.intron$type <- rep("intron", length(feature.segment.intron))
                feature.rd <- sort(c(feature.segment.exon, feature.segment.intron))
                feature.segment.points <- sort(unique(c(start(feature.segment.exon), end(feature.segment.exon))))
                ## filter the points by viewer port.
                feature.segment.points <- 
                  feature.segment.points[feature.segment.points>=start(ranges[[i]]) &
                                           feature.segment.points<=end(ranges[[i]])]
                ratio.to.full.range <- 9
                if(grepl("exon", rescale[1], ignore.case = TRUE)){#set intron width to 1/10
                  if(grepl("^exon_\\d+$", rescale[1], ignore.case = TRUE)){
                    ratio.to.full.range <- 
                      as.numeric(sub("^exon_(\\d+)$", "\\1", rescale[1], ignore.case = TRUE))
                    ratio.to.full.range <- ratio.to.full.range/(100-ratio.to.full.range)
                  }
                  width.full.range <- sum(width(feature.rd)[feature.rd$type=="exon"])
                  rescale <- width(feature.rd)
                  rescale[feature.rd$type=="intron"] <- 
                    rep(ceiling(width.full.range/ratio.to.full.range/sum(feature.rd$type=="intron")),
                        length(rescale[feature.rd$type=="intron"]))
                }else{#set exon width to 1/10
                  if(grepl("^intron_\\d+$", rescale[1], ignore.case = TRUE)){
                    ratio.to.full.range <- 
                      as.numeric(sub("^intron_(\\d+)$", "\\1", rescale[1], ignore.case = TRUE))
                    ratio.to.full.range <- ratio.to.full.range/(100-ratio.to.full.range)
                  }
                  width.full.range <- sum(width(feature.rd)[feature.rd$type=="intron"])
                  rescale <- width(feature.rd)
                  rescale[feature.rd$type=="exon"] <- 
                    rep(ceiling(width.full.range/ratio.to.full.range/sum(feature.rd$type=="exon")),
                        length(rescale[feature.rd$type=="exon"]))
                }
                rescale <- rescale/sum(rescale)
                if(length(rescale)==length(feature.segment.points)-1){
                  rescale.ir <- IRanges(feature.segment.points[-length(feature.segment.points)]+1, 
                                        feature.segment.points[-1])
                  start(rescale.ir)[1] <- start(rescale.ir)[1]-1
                  rescale.ir.width <- sum(width(rescale.ir))
                  rescale.ir.new.width <- cumsum(round(rescale.ir.width*rescale, digits = 0))
                  rescale <- data.frame(from.start=start(rescale.ir), 
                                        from.end=end(rescale.ir),
                                        to.start=feature.segment.points[1] + 
                                          c(0, rescale.ir.new.width[-length(rescale.ir.new.width)]),
                                        to.end=feature.segment.points[1] + rescale.ir.new.width)
                }else{
                  stop("Something wrong with the auto-scale setting. Please report the bug to https://github.com/jianhong/trackViewer/issues")
                }
              }
            }
          }
        }
        if(is.data.frame(rescale)){
          if(all(c("from.start", "from.end", "to.start", "to.end") %in% colnames(rescale))){
            ## check the from coverage the whole region.
            checkflank <- function(x){
              to <- IRanges(x$to.start, x$to.end)
              xol <- findOverlaps(to, drop.self=TRUE, 
                                  drop.redundant=TRUE,minoverlap=2L)
              if(length(xol)>1){
                stop("There is overlaps of the rescale region for 'to' columns.")
              }
              x <- IRanges(x$from.start, x$from.end)
              xol <- findOverlaps(x, drop.self=TRUE, 
                                  drop.redundant=TRUE,minoverlap=2L)
              if(length(xol)>1){
                stop("There is overlaps of the rescale region for 'from' columns.")
              }
              xgap <- gaps(x, start=min(start(x)), end=max(end(x)))
              if(length(xgap)>0){
                stop("There is gaps of the rescale region for 'from' columns.")
              }
            }
            checkflank(rescale)
            rescale.gr <- function(x){
              if(is(x, "GRanges")){
                x.start <- start(x)
                x.end <- end(x)
                y <- c(x.start, x.end)
                x.cut <- cut(y, breaks=c(rescale$from.start[1], rescale$from.end+1),
                             labels=seq.int(nrow(rescale)), right=FALSE)
                y <- mapply(function(a, b){
                  if(!is.na(b)) {
                    rescale(a, to=c(rescale$to.start[b], rescale$to.end[b]),
                            from=c(rescale$from.start[b], rescale$from.end[b]))
                  }else{
                    a
                  }
                }, y, as.numeric(as.character(x.cut)))
                y <- round(y)
                start(x) <- 1
                end(x) <- y[seq_along(x)+length(x)]
                start(x) <- y[seq_along(x)]
                x
              }else{
                x.cut <- cut(x, breaks=c(rescale$from.start[1], rescale$from.end+1),
                             labels=seq.int(nrow(rescale)), right=FALSE)
                y <- mapply(function(a, b){
                  if(!is.na(b)) {
                    rescale(a, to=c(rescale$to.start[b], rescale$to.end[b]),
                            from=c(rescale$from.start[b], rescale$from.end[b]))
                  }else{
                    a
                  }
                }, x, as.numeric(as.character(x.cut)))
                y <- round(y)
                y
              }
            }
            feature <- rescale.gr(feature)
            SNPs <- rescale.gr(SNPs)
            if(is.logical(xaxis)[1]){
              if(xaxis[1]){
                xaxis <- c(rescale$to.start[1], rescale$to.end)
                names(xaxis) <- c(rescale$from.start[1], rescale$from.end)
              }
            }else{
              xaxis.names <- names(xaxis)
              if(length(xaxis.names)!=length(xaxis)){
                xaxis.names <- as.character(xaxis)
              }
              xaxis <- rescale.gr(xaxis)
              names(xaxis) <- xaxis.names
            }
          }
        }
        
        ## top legend
        if(legendPosition$position=='bottom'){
          ## not supported yet.
          message('not suppport yet.')
        }
        ## convert height to npc number
        feature$height <- convertHeight2NPCnum(feature$height)
        ## multiple transcripts in one gene could be separated by featureLayerID
        feature <- setFeatureLayerID(feature, ranges[[i]])
        feature.splited <- split(feature, feature$featureLayerID)
        
        ## bottomblank, the transcripts legend height
        bottomblank <- plotFeatureLegend(feature, as.numeric(convertY(unit(1, "line"), "npc")),
                                         ranges[[i]], xaxis, xaxis.gp, label_on_feature)
        
        ## get the max score and scoreType
        if(length(SNPs$score)>0){
          SNPs$score <- sapply(SNPs$score, mean) ## fix the bug of score is NumericList
        }
        scoreMax0 <- scoreMax <- 
          if(length(SNPs$score)>0) ceiling(max(c(SNPs$score, 1), na.rm=TRUE)) else 1
        if(type=="pie.stack") scoreMax <- length(unique(SNPs$stack.factor))
        if(!type %in% c("pie", "pie.stack")){
          scoreType <- 
            if(length(SNPs$score)>0) all(floor(SNPs$score)==SNPs$score) else FALSE
          if(length(yaxis)>1 && is.numeric(yaxis)){
            if(length(names(yaxis))!=length(yaxis)){
              names(yaxis) <- yaxis
            }
            scoreMax0 <- max(yaxis, scoreMax0)
            scoreMax <- max(yaxis, scoreMax)
          }
          if(scoreMax>lollipop_style_switch_limit) {
            SNPs$score <- 10*SNPs$score/scoreMax
            scoreMax <- 10*scoreMax0/scoreMax
            scoreType <- FALSE
          }else{
            scoreMax <- scoreMax0
          }
        }else{
          scoreType <- FALSE
        }
        
        ## if the type is caterpillar, there are lollipop in both sides
        ## plot the bottom lollipops first. And push a new viewport
        
        IsCaterpillar <- length(SNPs$SNPsideID) > 0
        if(IsCaterpillar){
          if(any(is.na(SNPs$SNPsideID)) || 
             !all(SNPs$SNPsideID %in% c('top', 'bottom'))){
            warning("Not all SNPsideID is top or bottom")
            IsCaterpillar <- FALSE
          }
        }
        
        if(IsCaterpillar){
          SNPs.top <- SNPs[SNPs$SNPsideID=='top']
          SNPs.bottom <- SNPs[SNPs$SNPsideID=='bottom']
        }else{
          SNPs.top <- SNPs
          SNPs.bottom <- GRanges()
        }
        if(length(SNPs.bottom)<1) IsCaterpillar <- FALSE
        ## viewport of plot region
        if(!IsCaterpillar){
          bottomblank <- bottomblank
        }
        
        ## decide the legend position
        # right side keep 3 lines TODO: figure out the plot region to decide the right side margin
        plotRegionWidth <- unit(1 - (maxYlimNchar+3)*LINEW, 'npc')
        plotRegionX <- unit(maxYlimNchar*LINEW, 'npc')
        if(legendPosition$position %in% c('left', 'right')){
          if(length(legendPosition$width)>0){
            if(is.unit(legendPosition$width)){
              plotRegionWidth <- plotRegionWidth - legendPosition$width
            }else if(is.numeric(legendPosition$width)){
              if(legendPosition$width>0 && legendPosition$width<1){
                legendPosition$width <- unit(legendPosition$width, 'npc')
                plotRegionWidth <- plotRegionWidth - legendPosition$width
              }else{
                stop('legendPosition$width must be a number in the range of 0 ~ 1')
              }
            }else{
              stop('legendPosition$width must be a number of a unit.')
            }
          }else{
            legendPosition$width <- convertX(
              unit(max(legendNchar(legend[[i]]))+3, 'char'),
              'npc')
            plotRegionWidth <- plotRegionWidth - legendPosition$width
          }
          if(legendPosition$position=='left'){
            plotRegionX <- legendPosition$width + unit(2*LINEW, 'npc')
          }else{
            plotRegionX <- unit(maxYlimNchar*LINEW, 'npc')
          }
        }
        vp_track <- viewport(x=plotRegionX, y=bottomblank/2 + .5, 
                             width= plotRegionWidth,
                             height= 1 - bottomblank,
                             xscale=c(start(ranges[[i]]), end(ranges[[i]])),
                             clip="off",
                             just = 'left')
        pushViewport(vp_track)
        ratio.yx <- getYXratio()
        ## plot xaxis
        bottomHeight <- 0
        if(IsCaterpillar){
          ## total height == maxscore + extension + gap + labels
          bottomHeight <- getHeight(SNPs=SNPs.bottom, 
                                    ratio.yx=ratio.yx, 
                                    LINEW=LINEW, 
                                    GAP=GAP, 
                                    cex=cex, 
                                    type=type,
                                    scoreMax=scoreMax,
                                    level="data&labels")
          #bottomHeight <- bottomHeight/len
          vp <- viewport(y=bottomHeight, just="bottom",
                         xscale=c(start(ranges[[i]]), end(ranges[[i]])))
          pushViewport(vp)
          xaxis.gp$col <- "gray"
          plot_grid_xaxis(xaxis, gp=xaxis.gp)
          popViewport()
        }else{
          plot_grid_xaxis(xaxis, gp=xaxis.gp)
        }
        
        ## the baseline, the center of the first transcript
        baseline <- 
          max(c(feature.splited[[1]]$height/2, 
                .0001)) + 0.2 * LINEH
        baselineN <- 
          max(c(feature.splited[[length(feature.splited)]]$height/2, 
                .0001)) + 0.2 * LINEH
        
        ##plot features
        feature.height <- plotFeatures(feature.splited, LINEH, bottomHeight, label_on_feature)
        
        if(length(SNPs.bottom)>0){
          plotLollipops(SNPs.bottom, feature.height, bottomHeight, baselineN, 
                        type, ranges[[i]], yaxis, yaxis.gp, scoreMax, scoreMax0, scoreType, 
                        LINEW, cex, ratio.yx, GAP, pin, dashline.col,
                        side="bottom", jitter=jitter)
        }
        feature.height <- feature.height + 2*GAP
        if(length(SNPs.top)>0){
          plotLollipops(SNPs.top, feature.height, bottomHeight, baseline, 
                        type, ranges[[i]], yaxis, yaxis.gp, scoreMax, scoreMax0, scoreType, 
                        LINEW, cex, ratio.yx, GAP, pin, dashline.col,
                        side="top", jitter=jitter)
        }
        
        ## top legend
        this.height <- getHeight(SNPs.top, 
                                 ratio.yx, LINEW, GAP, cex, type,
                                 scoreMax=scoreMax,
                                 level="data&labels")
        this.height0 <- this.height <- this.height + bottomHeight + feature.height
        if(legendPosition$position=='top'){
          this.height <- plotLegend(legend[[i]], this.height, LINEH)
          if('alpha' %in% names(legend[[i]])){
            legend[[i]]$alpha <- NULL
            if('pch' %in% names(legend[[i]])){
              legend[[i]]$pch <- NA
            }
            plotLegend(legend[[i]], this.height0, LINEH)
          }
        }
        popViewport()## vp_track
        
        this.height <- bottomblank + 
          this.height * (1 - bottomblank)
        
        ## ylab
        vp_ylab <- viewport(x=ifelse(legendPosition$position!='left', 0, 
                                     convertX(legendPosition$width, 'npc',
                                              valueOnly = TRUE)),
                            y=this.height*0.5, 
                            width=plotRegionWidth +
                              unit(2*maxYlimNchar*LINEW, 'npc'),
                            height=this.height,
                            just='left')
        pushViewport(vp_ylab)
        if(is.logical(ylab)){
          if(ylab && length(names(SNP.gr))>0){
            grid.text(names(SNP.gr)[i], x = LINEW, 
                      y = .5, rot = 90, gp=ylab.gp)
          }
        }
        if(is.character(ylab)){
          if(length(ylab)==1) ylab <- rep(ylab, len)
          grid.text(ylab[i], x = LINEW,
                    y = .5, rot = 90, gp=ylab.gp)
        }
        popViewport()#vp_ylab
        
        ## left or right legend
        if(legendPosition$position %in% c('left', 'right')){
          legendRegionWidth <- legendPosition$width
          if(legendPosition$position=='left'){
            legendRegionX <- unit(0, 'npc')
          }else{
            legendRegionX <- unit(1, 'npc') - legendPosition$width
          }
          vp_legend <- viewport(x=legendRegionX, y=.5, 
                                width= legendRegionWidth,
                                height= 1,
                                just = 'left')
          pushViewport(vp_legend)
          plotLegend(legend[[i]], 0, LINEH)
          popViewport()## vp_legend
        }
        
        popViewport()#vp0
        height0 <-  height0 + this.height*height
      }
    }
    return(invisible(height0))
}

## ----echo=FALSE, results="hide", warning=FALSE--------------------------------
suppressPackageStartupMessages({
    library(trackViewer)
    library(rtracklayer)
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    library(org.Hs.eg.db)
    library(VariantAnnotation)
  library(httr)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)

## ----fig.width=4.5,fig.height=3-----------------------------------------------
library(trackViewer)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
library(rtracklayer)
methy <- import(system.file("extdata", "methy.bed", package="trackViewer"), "BED")
gr <- GRanges("chr22", IRanges(50968014, 50970514, names="TYMP"))
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)
features <- c(range(trs[[1]]$dat), range(trs[[5]]$dat))
names(features) <- c(trs[[1]]$name, trs[[5]]$name)
features$fill <- c("lightblue", "mistyrose")
features$height <- c(.02, .04)
dandelion.plot(methy, features, ranges=gr, type="pin")

## ----fig.width=4.5,fig.height=3-----------------------------------------------
methy$color <- 3
methy$border <- "gray"
## Score info is required and the score must be a number in [0, 1]
m <- max(methy$score)
methy$score <- methy$score/m
dandelion.plot(methy, features, ranges=gr, type="fan")

## ----fig.width=4.5,fig.height=4-----------------------------------------------
methy$color <- rep(list(c(3, 5)), length(methy))
methy$score2 <- (max(methy$score) - methy$score)/m
legends <- list(list(labels=c("s1", "s2"), fill=c(3, 5)))
dandelion.plot(methy, features, ranges=gr, type="pie", legend=legends)

## ----fig.width=4.5,fig.height=3.5---------------------------------------------
## Less dandelions
dandelion.plot(methy, features, ranges=gr, type="circle", maxgaps=1/10)

## ----fig.width=4.5,fig.height=4-----------------------------------------------
## More dandelions
dandelion.plot(methy, features, ranges=gr, type="circle", maxgaps=1/100)

## ----fig.width=4,fig.height=4-------------------------------------------------
maxgaps <- tile(gr, n = 10)[[1]]
dandelion.plot(methy, features, ranges=gr, type="circle", maxgaps=maxgaps)

## ----fig.width=4.5,fig.height=3-----------------------------------------------
dandelion.plot(methy, features, ranges=gr, type="pie", 
               maxgaps=1/100, yaxis = TRUE, heightMethod = mean,
               ylab='mean of methy scores')

## ----fig.width=4.5,fig.height=3-----------------------------------------------
yaxis = c(0, 0.5, 1)
dandelion.plot(methy, features, ranges=gr, type="pie", 
               maxgaps=1/100, yaxis = yaxis, heightMethod = mean,
               ylab='mean of methy scores')

## ----sessionInfo, results='asis'----------------------------------------------
sessionInfo()

plotLollipopData <- function(dat, xscale, chr, yaxis, yaxis.gp, ybase=0, side="top", main=TRUE, baselineCol="black", maxHeight, lollipop_style_switch_limit=10){
  if(length(dat)>0){
    dat <- orderedGR(dat)
    xscale.gr <- GRanges(seqnames = chr, ranges=IRanges(min(xscale), max(xscale)))
    dat <- subsetByOverlaps(dat, xscale.gr, ignore.strand=TRUE)
    if(length(dat)<1) return(invisible())
    width(dat) <- 1
    TYPES <- c("circle", "pie", "pin", "pie.stack", "flag")
    type <- if(is.list(dat$type)) dat$type[[1]] else dat$type[1]
    if(length(type)==0) type <- "circle"
    if(!type %in% TYPES) type <- "circle"
    if(type=="pin"){ ## read the pin shape file
      pinpath <- system.file("extdata", "map-pin-red.xml", package="trackViewer")
      pin <- readPicture(pinpath)
    }else{
      pin <- NULL
    }
    cex <- if(is.list(dat$cex)) dat$cex[[1]] else dat$cex[1]
    if(length(cex)==0) cex <- 1
    dashline.col <- if(is.list(dat$dashline.col)) dat$dashline.col[[1]] else dat$dashline.col[1]
    if(length(dashline.col)==0) dashline.col <- "gray80"
    jitter <- if(is.list(dat$jitter)) dat$jitter[[1]] else dat$jitter[1]
    if(length(jitter)==0) jitter <- "node"
    if(!jitter %in% c("node", "label")) jitter <- "node"
    scoreMax0 <- scoreMax <- 
      if(length(dat$score)>0) ceiling(max(c(dat$score, 1), na.rm=TRUE)) else 1
    if(type=="pie.stack") scoreMax <- length(unique(dat$stack.factor))
    if(!type %in% c("pie", "pie.stack")){
      if(scoreMax>lollipop_style_switch_limit) {
        dat$score <- 10*dat$score/scoreMax
        scoreMax <- 10*scoreMax0/scoreMax
      }else{
        scoreMax <- scoreMax0
      }
      scoreType <- 
        if(length(dat$score)>0) all(floor(dat$score)==dat$score) else FALSE
    }else{
      scoreType <- FALSE
    }
    LINEW <- as.numeric(convertX(unit(1, "line"), "npc"))/2
    LINEH <- as.numeric(convertY(unit(1, "line"), "npc"))/2
    ## GAP the gaps between any elements
    GAP <- .2 * LINEH
    ratio.yx <- getYXratio()
    feature.height <- 
      if(is.list(dat$feature.height)) dat$feature.height[[1]] else dat$feature.height[1]
    if(length(feature.height)==0) feature.height <- GAP
    if(maxHeight>1) cex <- cex/maxHeight
    scoreMax0 <- scoreMax0/cex
    guideline <- ifelse(side=="top", ybase, ybase+feature.height)
    grid.lines(y=c(guideline, guideline), gp = gpar(col=baselineCol))
    plotLollipops(dat, feature.height=feature.height, bottomHeight=ybase, baseline=0, 
                  type=type, ranges=xscale.gr, yaxis=yaxis, yaxis.gp=yaxis.gp,
                  scoreMax=scoreMax, scoreMax0=scoreMax0, scoreType=scoreType, 
                  LINEW, cex, ratio.yx, GAP, pin, dashline.col,
                  side=side, jitter=jitter, main=main)
  }
  return(invisible())
}

#' Reading data from a BED or WIG file to RleList
#' @description Read a \code{\link{track}} object from a BED, bedGraph, 
#' WIG or BigWig file to RleList
#' @param files The path to the files to read.
#' @param format The format of import file. Could be BAM, BED, bedGraph, WIG or BigWig
#' @param ranges An object of \code{\link[GenomicRanges:GRanges-class]{GRanges}} to indicate
#' the range to be imported
#' @return a list of \code{\link[IRanges:AtomicList-class]{RleList}}. 
#' @import Rsamtools
#' @importFrom tools file_ext
#' @export
#' @examples 
#' #import a BED file
#' bedfile <- system.file("tests", "test.bed", package="rtracklayer",
#'                        mustWork=TRUE)
#' dat <- importData(files=bedfile, format="BED",
#'                   ranges=GRanges("chr7", IRanges(127471197, 127474697)))
#' 
#' ##import a WIG file
#' wigfile <- system.file("tests", "step.wig", package = "rtracklayer",
#'                        mustWork=TRUE)
#' dat <- importData(files=wigfile, format="WIG", 
#'                   ranges=GRanges("chr19", 
#'                                  IRanges(59104701, 59110920)))
#' 
#' ##import a BigWig file
#' if(.Platform$OS.type!="windows"){
#'   ##this is because we are using rtracklayer::import
#'   bwfile <- system.file("tests", "test.bw", package = "rtracklayer",
#'                         mustWork=TRUE)
#'   dat <- importData(files=bwfile, format="BigWig", 
#'                     ranges=GRanges("chr19", IRanges(1500, 2700)))
#' }

importData <- 
    function(files, 
             format=NA,
             ranges=GRanges()){
    stopifnot(class(ranges)=="GRanges")
    ranges <- reduce(ranges)
    formats <- c("BAM", "BED", "bedGraph", "WIG", "BigWig")
    if(is.na(format[1])){
        format <- vapply(files, file_ext, FUN.VALUE = "BAM") ## need tools package
    }
    format <- rep(format, length(files))[1:length(files)]
    format <- formats[match(tolower(format), tolower(formats))]
    if(any(is.na(format)))
        stop("Can not determine the file format.",
             "Format must be BAM, BED, bedGraph, WIG, or BigWig")
    covBAM <- NULL
    covBW <- NULL
    covBED <- NULL
    covBedGraph <- NULL
    covWIG <- NULL
    bams <- files[format=="BAM"]
    BigWigs <- files[format=="BigWig"]
    BEDs <- files[format=="BED"]
    bedGraphs <- files[format=="bedGraph"]
    WIGs <- files[format=="WIG"]
    if(length(bams)>0){
        ## import bams, need Rsamtools, need GenomicAlignments
        param <- ScanBamParam(which=ranges)
        bam <- lapply(bams, readGAlignments, param=param)
        names(bam) <- bams
        covBAM <- sapply(bam, coverage)
    }
    if(length(BigWigs)>0){
        ## import by rtracklayer
        ### if format == "BigWig"
        ## rtracklayer can't read bigWig files on a Windows computer. 
        ## Type in  ?`BigWigFile-class` to get the help.
        if (.Platform$OS.type != "windows") {
            covBW <- sapply(BigWigs, import, 
                            format="BigWig", 
                            which=ranges, 
                            as="RleList")
        }else{
            stop("Can not import the bigWig files on a Windows computer.")
        }
    }
    if(length(BEDs)>0){
        covBED <- sapply(BEDs, importScore, 
                         format="BED",
                         ranges=ranges,
                         ignore.strand=TRUE)
        covBED <- sapply(covBED, function(.ele){
            coverage(.ele@dat, weight=score(.ele@dat))
        })
    }
    if(length(bedGraphs)>0){
        covBedGraph <- sapply(bedGraphs, importScore, 
                              format="bedGraph",
                              ranges=ranges,
                              ignore.strand=TRUE)
        covBedGraph <- sapply(covBedGraph, function(.ele){
            coverage(.ele@dat, weight=score(.ele@dat))
        })
    }
    if(length(WIGs)>0){
        ranges <- reduce(ranges, ignore.strand=TRUE)
        covWIG <- sapply(WIGs, importScore, 
                              format="WIG",
                              ranges=ranges,
                              ignore.strand=TRUE)
        covWIG <- sapply(covWIG, function(.ele){
            .tracks <- lapply(seq_len(length(ranges)), function(id){
                .gr <- ranges[id]
                .out <- parseWIG(trackScore=.ele,
                                 chrom=as.character(seqnames(.gr)),
                                 from=start(.gr),
                                 to=end(.gr))
                .out@dat
            })
            .tracks <- unlist(GRangesList(.tracks))
            coverage(.tracks, weight=score(.tracks))
        })
    }
    covs <- c(covBAM, covBW, covBED, covBedGraph, covWIG)
    covs[files]
}

#' Prepare gene model from an object of TxDb
#' @description Generate an object of \code{\link{track}} for 
#' \code{\link{viewTracks}} by given parameters.
#' @param txdb An object of \code{\link[GenomicFeatures:TxDb-class]{TxDb}}
#' @param orgDb An object of "OrgDb"
#' @param gr An object of GRanges.
#' @param chrom chromosome name, must be a seqname of txdb
#' @param start start position
#' @param end end position
#' @param strand strand
#' @param txdump output of as.list(txdb), a list of data frames that can be used 
#' to make the db again with no loss of information.
#' @importFrom Gviz GeneRegionTrack
#' @import GenomicRanges
#' @importFrom GenomicFeatures transcripts genes
#' @importFrom txdbmaker makeTxDb
#' @importFrom Seqinfo seqnames
#' @importFrom BiocGenerics strand
#' @importFrom AnnotationDbi mapIds columns
#' @return Generate a list of \code{\link{track}} from a TxDb object.
#' @export
#' @seealso See Also as \code{\link{importScore}}, \code{\link{importBam}}, 
#' \code{\link{viewTracks}}
#' @examples 
#' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
#' library(org.Hs.eg.db)
#' trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
#'                          org.Hs.eg.db,
#'                          chrom="chr20", 
#'                          start=22560000, 
#'                          end=22565000, 
#'                          strand="-")
geneModelFromTxdb <- function(txdb, orgDb, gr, 
                              chrom, start, end, 
                              strand=c("*", "+", "-"), 
                              txdump=NULL){
    if(missing(txdb))
        stop("txdb is required!")
    stopifnot(is(txdb, "TxDb"))
    if(!missing(orgDb)) stopifnot(is(orgDb, "OrgDb"))
    strand <- match.arg(strand)
    if(missing(gr)){
        if(missing(chrom)||missing(start)||missing(end)){
            stop("chromosome location is required.")
        }
        gr <- GRanges(chrom, IRanges(start, end), strand=strand)
    }
    if(length(gr)>1){
        warning("only the first element of gr is used")
        gr <- gr[1]
    }
    stopifnot(is(gr, "GRanges"))
    chrom <- as.character(seqnames(gr))
    strand <- as.character(strand(gr))
    start <- gr@ranges@start
    end <- start + gr@ranges@width - 1
    if(strand=="*"){
        genes <- suppressMessages(transcripts(txdb, columns="exon_id",
                                              filter=list(tx_chrom=chrom)))
    }else{
        genes <- suppressMessages(transcripts(txdb, columns="exon_id", 
                                              filter=list(tx_chrom=chrom, 
                                                          tx_strand=strand)))
    }
    ignore.strand <- strand=="*"
    ol <- findOverlaps(gr, genes, ignore.strand=ignore.strand)
    if(length(ol)>0){
        exon <- genes[subjectHits(ol)]
        r <- range(exon)
        ##subset txdb
        if(is.null(txdump)) txdump <- as.list(txdb)
        txdump$chrominfo <- txdump$chrominfo[txdump$chrominfo$chrom==chrom, , drop=FALSE]
        txdump$transcripts <- txdump$transcripts[txdump$transcripts$tx_chrom==chrom &
                                                 txdump$transcripts$tx_start<end(r) &
                                                 txdump$transcripts$tx_end>start(r), , drop=FALSE]
        txdump$genes <- txdump$genes[txdump$genes$tx_id %in% txdump$transcripts$tx_id, , drop=FALSE]
        txdump$splicings <- txdump$splicings[txdump$splicings$tx_id %in% txdump$transcripts$tx_id, , drop=FALSE]
        txdb <- do.call(makeTxDb, txdump)
        exons <- GeneRegionTrack(txdb, chromosome=chrom,
                                 start=start(r), end=end(r), strand=strand)##time comsuming
        exons <- exons@range
        if(!missing(orgDb)){
            if(all(exons$symbol==exons$transcript) && "SYMBOL" %in% columns(orgDb)){
              suppressMessages(symbol <- tryCatch(mapIds(x=orgDb, keys=exons$gene, 
                                                         column="SYMBOL", keytype="ENTREZID",
                                                         multiVals="first"), 
                                                  error=function(e) NA))
                if(!is.na(symbol[1]) && length(symbol)==length(exons)){
                    exons$symbol <- symbol
                }
            }
        }
        trs <- split(exons, as.character(exons$transcript))
        idx <- sapply(trs, function(.ele){
            .r <- range(.ele)
            if(end(.r)>=start && start(.r)<=end) return(TRUE)
            return(FALSE)
        })
        trs <- trs[idx]
        trs <- lapply(trs, function(.ele){
            new("track", dat=.ele, type="transcript", 
                name=as.character(.ele$symbol)[1],
                style=new("trackStyle", color="lightblue"))
        })
        return(trs)
    }else{
        stop("No transcripts in the given range.")
    }
}

#' browse tracks
#'
#' @description browse tracks by a web browser.
#' @param trackList an object of \code{\link{trackList}}
#' @param gr an object of \code{\link[GenomicRanges:GRanges-class]{GRanges}}
#' @param ignore.strand ignore the strand or not when do filter. default TRUE
#' @param width width of the figure
#' @param height height of the figure
#' @param ... parameters not used
#' @import htmlwidgets
#' @import GenomicRanges
#' @importFrom utils installed.packages
#' @export
#' @return An object of class htmlwidget that will intelligently print itself 
#' into HTML in a variety of contexts including the R console, 
#' within R Markdown documents, and within Shiny output bindings.
#' @examples 
#' extdata <- system.file("extdata", package="trackViewer", mustWork=TRUE)
#' files <- dir(extdata, "-.wig")
#' tracks <- lapply(paste(extdata, files, sep="/"), 
#'                  importScore, format="WIG")
#' tracks <- lapply(tracks, function(.ele) {strand(.ele@dat) <- "-"; .ele})
#' names(tracks) <- c("trackA", "trackB")
#' fox2 <- importScore(paste(extdata, "fox2.bed", sep="/"), format="BED")
#' dat <- coverageGR(fox2@dat)
#' fox2@dat <- dat[strand(dat)=="+"]
#' fox2@dat2 <- dat[strand(dat)=="-"]
#' gr <- GRanges("chr11", IRanges(122929275, 122930122))
#' browseTracks(trackList(tracks, fox2), gr=gr)

browseTracks <- function(trackList, 
                         gr=GRanges(),
                         ignore.strand=TRUE,
                         width=NULL, height=NULL, 
                         ...){
    if(missing(gr)){
      if(interactive()){
        ## ask to input the region
        chr <- readline(prompt = "Enter the chromosome, for example chr1: ")
        start <- as.integer(readline(prompt = "Enter the start coordinate: "))
        end <- as.integer(readline(prompt = "Enter the end coordinate: "))
        gr <- GRanges(chr, IRanges(start, end))
      }else{
        stop("gr is required.")
      }
    }else{
      if(!is(gr, "GRanges")){
        stop("gr must be an object of GRanges.")
      }
      if(length(gr)!=1){
        stop("the length of gr must be 1.")
      }
    }
    if(missing(trackList)){
      if(interactive()){
        ## ask to import files
        trackList <- list()
        n <- 1
        while(n>0){
          message("Please select \n\t0\t stop import.\n\t1\t get gene track. \n\t2\t import BED/bedGraph/WIG/BigWig file.")
          filename <- readline(prompt = "Enter the number: ")
          switch(filename, 
                 "0"={
                   n <- 0
                 },
                 "1"={
                   ip <- as.data.frame(installed.packages()[, c(1, 3:4)])
                   ip <- unique(as.character(ip$Package))
                   txdbs <- ip[grepl("^TxDb", ip)]
                   orgs <- ip[grepl("^org", ip)]
                   txdbFlag <- TRUE
                   message("available txdb\n", paste0(paste0(seq_along(txdbs), ": ", txdbs, "\n")))
                   while(txdbFlag){
                     txdb <- readline(prompt = "select txdb: ")
                     txdb <- as.numeric(txdb)
                     if(!is.na(txdb)){
                       txdb <- txdbs[txdb]
                       txdbFlag <- FALSE
                     }
                   }
                   orgFlag <- TRUE
                   message("available org\n", paste0(paste0(seq_along(orgs), ": ", orgs, "\n")))
                   while(orgFlag){
                     org <- readline(prompt = "select org: ")
                     org <- as.numeric(org)
                     if(!is.na(org)){
                       org <- orgs[org]
                       orgFlag <- FALSE
                     }
                   }
                   require(txdb, character.only = TRUE)
                   require(org, character.only = TRUE)
                   trs <- geneModelFromTxdb(get(txdb), get(org), gr=gr)
                   trackList <- c(trackList, trs)
                 },
                 "2"={
                   filename <- file.choose()
                   if(file.exists(filename)){
                     format <- sub("^.*\\.(.*)$", "\\1", basename(filename))
                     if(tolower(format) %in% tolower(c("BED", "bedGraph", "WIG", "BigWig"))){
                       format <- switch(tolower(format), 
                                        bed="BED",
                                        bedgraph="bedGraph",
                                        wig="WIG", 
                                        bigwig="BigWig")
                       sampleName <- make.names(sub(paste0(".", format), "", basename(filename)))
                       if(sampleName %in% names(trackList)){
                         sampleName <- paste0(sampleName, "_copy")
                       }
                       trackList[[sampleName]] <- importScore(filename, format = format, ranges = gr)
                     }else{
                       message("Can not detect the format of the file. Please rename the file with end of .BED, .bedGraph, .WIG or .BigWig")
                     }
                   }else{
                     message("Can not open the file. Please double check the file path.")
                   }
                 })
        }
        trackList <- optimizeStyle(trackList(trackList), theme="col")$tracks
      }else{
        stop("trackList is required.")
      }
    }
    if(!is(trackList, "trackList") && 
       !((is.list(trackList) && all(sapply(trackList, class)=="track")))){
        stop("trackList must be an object of \"trackList\"
             (See ?trackList) or a list of track")
    }
    if(any(is.na(names(trackList)))){
        stop("trackList must have names")
    }
    if(any(duplicated(names(trackList)))){
        stop("trackList must have unqiue names")
    }
    chromosome <- as.character(GenomicRanges::seqnames(gr))[1]
    start <- GenomicRanges::start(gr)[1]
    end <- GenomicRanges::end(gr)[1]
    if(ignore.strand) GenomicRanges::strand(gr) <- "*"
    strand <- as.character(GenomicRanges::strand(gr))[1]
    if(end < start) stop("end should be greater than start.")
    
    trackList <- filterTracks(trackList, chromosome, 
                              start, end, strand)
    trackList <- rev(trackList)
    viewerStyle=trackViewerStyle()
    ## trackList split into two parts: gene and data
    irl <- IRangesList(ranges(gr)[1])
    names(irl) <- chromosome
    object2list <- function(obj){
        if(!isS4(obj)) return(obj)
        s <- slotNames(obj)
        x <- lapply(s, function(.ele){
            .ele <- slot(obj, .ele)
            if(isS4(.ele)) object2list(.ele) else .ele
        })
        names(x) <- s
        x
    }
    ZERO <- numeric(0)
    getData <- function(.dat){
        if(length(.dat)>0){
            .dat <- .dat[c(1, seq_along(.dat))]
            start(.dat[1]) <- end(.dat[1]) <- end+1
            .dat[1]$score <- 0
            strand(.dat) <- "*"
            co <- coverage(.dat, weight=.dat$score)
            as.numeric(co[irl][[1]])
        }else{
            ZERO
        }
    }
    getInteractionData <- function(.dat, .dat2){
      ## copied from plotInteractionDataTrack.R. TODO: create a sub-function
      names(.dat) <- NULL
      ## if there is target metadata
      .dat1target <- NULL
      .dat2target <- NULL
      if(length(.dat$target)==length(.dat) && is(.dat$target, "GRanges")){
        .dat1target <- .dat$target
        if(length(.dat2)!=0){
          if(length(.dat2$target)==length(.dat2) && is(.dat2$target, "GRanges")){
            names(.dat2) <- NULL
            .dat2target <- .dat2$target
            #mcols(.dat2) <- mcols(.dat2)[, "score"]
            #colnames(mcols(.dat2)) <- "score"
          }
        }
      }else{
        .dat1target <- .dat2
        .dat2 <- NULL
      }
      # convert to the format: startA, endA, startB, endB, value
      createDF <- function(.d, .dt){
        if(length(.d)==0) return(ZERO)
        data.frame(
          startA = start(.d),
          endA = end(.d),
          startB = start(.dt),
          endB = end(.dt),
          value = .d$score
        )
      }
      return(list(
        dat = createDF(.dat, .dat1target),
        dat2 = createDF(.dat2, .dat2target)
      ))
    }
    col2Hex <- function(x){
        if(is(x, "AsIs") || is.list(x)){
          return(lapply(x, col2Hex))
        }
        rgb <- col2rgb(x)
        rgb(rgb[1, ], rgb[2, ], rgb[3, ], maxColorValue = 255)
    }
    LINEW <- as.numeric(convertX(unit(1, "line"), "npc"))
    getLolliplotData <- function(.dat){
      if(length(.dat)==0) return(ZERO);
      .dat <- sort(.dat)
      .dat$textlabel <- names(.dat)
      names(.dat) <- NULL
      if(length(.dat$score)!=length(.dat)){
        .dat$score <- 1
      }
      
      cex <- .5
      if(length(.dat$cex)>0) cex <- cex*sapply(cex, `[`, 1)
      pushViewport(viewport(xscale=c(start(gr), end(gr))))
      
      if(length(.dat$stack.factor)>0){
        stopifnot(is.vector(.dat$stack.factor, mode="character"))
        if(length(.dat$stack.factor.order)>0 || 
           length(.dat$stack.factor.first)>0){
          warning("stack.factor.order and stack.factor.first are used by this function!",
                  "The values in these column will be removed.")
        }
        ## condense the SNPs
        stack.factors <- unique(as.character(.dat$stack.factor))
        stack.factors <- sort(stack.factors)
        stack.factors.order <- 1:length(stack.factors)
        names(stack.factors.order) <- stack.factors
        .dat <- .dat[order(as.character(seqnames(.dat)), start(.dat), 
                           as.character(.dat$stack.factor))]
        .dat$stack.factor.order <- stack.factors.order[.dat$stack.factor]
        .dat$stack.factor.first <- !duplicated(.dat)
        .dat.condense <- .dat
        .dat.condense$oid <- 1:length(.dat)
        .dat.condense$factor <- paste(as.character(seqnames(.dat)), start(.dat), end(.dat))
        .dat.condense <- split(.dat.condense, .dat.condense$factor)
        .dat.condense <- lapply(.dat.condense, function(.ele){
          .oid <- .ele$oid
          .gr <- .ele[1]
          mcols(.gr) <- NULL
          .gr$oid <- NumericList(.oid)
          .gr
        })
        .dat.condense <- unlist(GRangesList(.dat.condense), use.names = FALSE)
        .dat.condense <- sort(.dat.condense)
        lab.pos.condense <- jitterLables(start(.dat.condense), 
                                         xscale=c(start(gr), end(gr)), 
                                         lineW=LINEW*cex)
        lab.pos.condense <- reAdjustLabels(lab.pos.condense, 
                                           lineW=LINEW*cex)
        condense.ids <- .dat.condense$oid
        lab.pos <- rep(lab.pos.condense, elementNROWS(condense.ids))
        lab.pos <- lab.pos[order(unlist(condense.ids))]
      }else{
        lab.pos <- jitterLables(start(.dat), 
                                xscale=c(start(gr), end(gr)), 
                                lineW=LINEW*cex)
        lab.pos <- reAdjustLabels(lab.pos, 
                                  lineW=LINEW*cex)
      }
      popViewport()
      
      dat2 <- as.list(as.data.frame(.dat))
      dat2$seqnames <- chromosome
      dat2$border <- col2Hex(dat2$border)
      dat2$color <- col2Hex(dat2$color)
      dat2$labpos <- lab.pos
      dat2
    }
    trackdat <- function(.ele){
        dat <- .ele$dat
        dat2 <- .ele$dat2
        style <- object2list(.ele$style)
        style$color <- col2Hex(style$color)
        if(.ele$type=="data"){
            dat <- getData(.ele$dat)
            dat2 <- getData(.ele$dat2)
            if(length(.ele$style@ylim)==2) {
                ylim <- .ele$style@ylim
              }else{
                ylim <- range(c(dat, -1*dat2))
              }
            if(length(style$color)==1){
                style$color <- rep(style$color, 2)
            }
        }else{
          if(.ele$type=="lollipopData"){
            dat <- getLolliplotData(.ele$dat)
            if(length(.ele$dat2)==0){
              dat2 <- ZERO
            }else{
              dat2 <- getLolliplotData(.ele$dat2)
            }
            ylim <- c(0, 1)
            if(length(style$color)==1){
              style$color <- rep(style$color, 2)
            }
          }else{
            if(.ele$type=="scSeq"){
              stop("scSeq data not supported.")
            }else{
              if(.ele$type=="interactionData"){
                interactiondata <- getInteractionData(.ele$dat,
                                                      .ele$dat2)
                dat <- interactiondata$dat
                dat2 <- interactiondata$dat2
                if(length(dat2)>0){
                  warning('Current back2back is not supported.')
                }
                ylim <- c(0, 1) ## need to be changed to track@style@ylim
                ## however, javascript is not ready to accept the ylim yet.
              }else{
                ##gene
                dat$textlabel <- names(.ele$dat)
                names(dat) <- NULL
                if(length(dat$featureLayerID)==0){
                  dat$featureLayerID <- 1
                }
                if(length(dat$fill)==0){
                  dat$fill <- style$color[1]
                }else{
                  dat$fill <- col2Hex(dat$fill)
                }
                dat <- as.list(as.data.frame(dat))
                dat$seqnames <- chromosome
                if(length(.ele$dat2)==0){
                  dat2 <- ZERO
                }else{
                  dat2 <- getLolliplotData(.ele$dat2)
                }
                ylim <- c(0, 1)
                
              }
            }
          }
        }
        list(dat=dat,
             dat2=dat2,
             style=style,
             ylim=ylim)
    }
    x <- list(
        tracklist = lapply(trackList, trackdat),
        type = lapply(trackList, function(.ele) .ele$type),
        name = names(trackList),
        chromosome = chromosome,
        start = start,
        end = end,
        strand =strand,
        height = lapply(trackList, function(.ele) .ele$style@height)
    )
    
    htmlwidgets::createWidget(
        name = 'browseTracks',
        x = x,
        width = width,
        height = height,
        package = getPackageName()
    )
}


#' Shiny bindings for browseTracks
#'
#' Output and render functions for using browseTracks within Shiny
#' applications and interactive Rmd documents.
#'
#' @param outputId output variable to read from
#' @param width,height Must be a valid CSS unit (like \code{'100\%'},
#'   \code{'400px'}, \code{'auto'}) or a number, which will be coerced to a
#'   string and have \code{'px'} appended.
#' @param expr An expression that generates a browseTracks
#' @param env The environment in which to evaluate \code{expr}.
#' @param quoted Is \code{expr} a quoted expression (with \code{quote()})? This
#'   is useful if you want to save an expression in a variable.
#'
#' @name browseTracks-shiny
#'
#' @export
browseTracksOutput <- function(outputId, width = '100%', height = '600px'){
  htmlwidgets::shinyWidgetOutput(outputId, 'browseTracks', width, height, 
                                 package = 'trackViewer')
}

#' @rdname browseTracks-shiny
#' @export
renderbrowseTracks <- function(expr, env = parent.frame(), quoted = FALSE) {
  if (!quoted) { expr <- substitute(expr) } # force quoted
  htmlwidgets::shinyRenderWidget(expr, browseTracksOutput, env, quoted = TRUE)
}

#' load ideogram from UCSC
#' @description Download ideogram table from UCSC 
#' @param genome Assembly name  assigned by UCSC, such as hg38, mm10.
#' @param chrom A character vector of chromosome names, or NULL.
#' @param ranges A \link[IRanges:IRanges-class]{Ranges} object with the intervals.
#' @param ... Additional arguments to pass to the 
#' \link[GenomicRanges:GRanges-class]{GRanges} constructor.
#' @importFrom rtracklayer import getTable ucscTableQuery GRangesForUCSCGenome browserSession
#' @importFrom Seqinfo "genome<-"
#' @export
#' @return A \link[GenomicRanges:GRanges-class]{GRanges} object.
#' @seealso See Also as \code{\link{ideogramPlot}}
#' @examples 
#' \dontrun{
#' head(loadIdeogram("hg38", chrom = "chr1"))
#' }
#' 
#' 
loadIdeogram <- function(genome, chrom = NULL, ranges = NULL, ...){
  range <- GRangesForUCSCGenome(genome, chrom, ranges, ...)
  session <- browserSession()
  genome(session) <- genome
  ideo <- getTable(ucscTableQuery(session, 
                          table="cytoBandIdeo", 
                          range = range))
  ideo <- ideo[ideo$name!="", , drop=FALSE]
  ideo$chrom <- as.character(ideo$chrom)
  ideo <- ideo[ideo$chrom %in% seqlevels(range), , drop=FALSE]
  gr <- GenomicRanges::trim(GRanges(ideo))
  suppressWarnings(seqinfo(gr) <- seqinfo(range)[seqlevels(gr)])
  gr
}

Y1pos <- function(SNPs.groups, xscale, lineW, base, cex, ypos, plotYaxis, heightMethod){
  ratio.Yy <- 1/diff(xscale)
  sg <- split(SNPs.groups, SNPs.groups$gps)
  sg <- sg[order(sapply(sg, length), decreasing=FALSE)]
  ratio.yx <- getYXratio()
  t2xy <- function(t, rX, rY, angle) {
    t2p <- angle * t - (angle-pi)/2
    list(x = -rX * cos(t2p), y = rY * sin(t2p))
  }
  
  radius <- sapply(sg, function(.ele) floor(width(range(unname(.ele)))/2) * ratio.Yy *ratio.yx)
  radius <- radius[order(as.numeric(names(radius)))]
  
  stopifnot(length(plotYaxis)==1)
  if(plotYaxis){
    sg.scores <- sapply(sg, function(.ele) heightMethod(.ele$score))
  }else{
    sg.scores <- sapply(sg, function(.ele) heightMethod(.ele$score)) * lineW * ratio.yx
    sg.scores <- sg.scores[order(as.numeric(names(sg.scores)))]
    if(length(sg.scores)>1){
      for(i in 2:length(sg.scores)){
        if(sg.scores[i]>sg.scores[i-1]){
          if(sg.scores[i-1]*1.25+radius[i-1]+lineW>=sg.scores[i]){
            sg.scores[i] <- sg.scores[i-1]*1.25 + radius[i-1]+lineW
          }
        }else{
          if(sg.scores[i]*1.25+radius[i]+lineW>=sg.scores[i-1]){
            sg.scores[i] <- sg.scores[i-1]-radius[i]-sg.scores[i]/4-lineW
            if(sg.scores[i]<lineW) sg.scores[i] <- lineW
          }
        }
      }
    }
  }
  sg.scores <- sg.scores[names(sg)]
  radius <- radius[names(sg)]
  scoreRatio <- scoreMax0 <- max(sg.scores*5/4, na.rm = TRUE)
  yyscaleMax <- 1
  ## reset sg.scores to fit the paper
  if(scoreRatio + ypos + cex*lineW*ratio.yx + base > 1){
    scoreRatio <- (1-ypos-base-cex*lineW*ratio.yx)/scoreRatio
    sg.scores <- sg.scores * scoreRatio
    yyscaleMax <- scoreMax0
  }
  sg <- mapply(function(.ele, .h){
    .ele$Y1 <- .h + base
    if(length(.ele)>1){
      .range <- range(unname(.ele))
      .rX <- floor(width(.range)/2)
      .rY <- .rX * ratio.Yy * ratio.yx
      .center <- start(.range) + .rX
      ## add 1/3 height to radius
      .rY <- .rY + .h / 4
      .rX <- .rX + .h * diff(xscale) / 4
      .percent <- (1:length(.ele)-1)/(length(.ele)-1)
      ## total angle
      perimeter <- 2 * pi * .rY *3/4 # max 3/2 pi
      angle <- pi * 3/2 * min(lineW*length(.ele)*cex/perimeter, 1)
      P <- t2xy(.percent, .rX, .rY, angle)
      .ele$X2 <- .center + P$x
      .ele$Y2 <- .ele$Y1 + P$y
      .ele$alpha <- (.5 - .percent) * angle
    }else{
      .ele$X2 <- start(.ele)
      .ele$Y2 <- .ele$Y1
      .ele$alpha <- 0
    }
    .ele
  }, sg, sg.scores)
  sg <- unlist(GRangesList(unname(sg)))
  sg <- sg[order(sg$idx)]
  sg$yyscaleMax <- yyscaleMax
  sg
}

grid.dandelion <- function(x0, y0, x1, y1, x2, y2, 
                           radius, col=NULL,
                           border=NULL,
                           percent=NULL,
                           edges=100,
                           alpha=0,
                           type=c("fan", "circle", "pie", "pin"),
                           ratio.yx=1,
                           pin=NULL,
                           scoreMax,
                           id=NA, id.col="black",
                           name=NULL, cex=1, lwd=1){
  stopifnot(is.numeric(c(x0, x1, x2, y0, y1, y2, radius, edges)))
  type <- match.arg(type)
  grid.lines(x=c(x0, x1, x2), y=c(y0, y1, y2), 
             gp=gpar(col=border, lwd=lwd))
  if(length(pin)>0){
    if(length(border)>0) pin@paths[[2]]@rgb <- rgb2hex(col2rgb(border[1]))
    if(length(col)>0) pin@paths[[1]]@rgb <- rgb2hex(col2rgb(col[1]))
    if(length(col)>1) pin@paths[[3]]@rgb <- rgb2hex(col2rgb(col[2]))
  }
  switch(type,
         fan={grid.dan(x=x2, y=y2, 
                       radius = radius, 
                       col = col, 
                       border = border, 
                       percent=percent,
                       edges=edges, 
                       alpha=alpha,
                       ratio.yx=ratio.yx, 
                       lwd=lwd)},
         circle={
           if(length(border)==0) border <- "black"
           if(length(col)==0) col <- "white"
           if(length(lwd)==0) lwd <- 1
           grid.circle1(x=x2, y=y2,
                        r=radius*ratio.yx, 
                        gp=gpar(col=border, fill=col, lwd=lwd))
           if(!is.na(id)){
             grid.text(label=id, x=x2, 
                       y=y2+radius*ratio.yx,
                       just="centre", gp=gpar(col=id.col, cex=.75*cex))
           }
         },
         pie=grid.pie(x=x2, y=y2, 
                      radius = radius, 
                      col = col, 
                      border = border, 
                      percent=percent,
                      edges=edges, 
                      lwd=lwd),
         pin={
           grid.picture(picture=pin, x=x2, 
                        y=y2+1.5*radius*ratio.yx,
                        width=2*radius*ratio.yx,
                        height=3*radius*ratio.yx)
           if(!is.na(id)){
             grid.text(label=id, x=x2, 
                       y=y2+1.33*radius*ratio.yx,
                       just="centre", gp=gpar(col=id.col, cex=.5*cex))
           }
         },
         grid.pie(x=x2, y=y2, 
                  radius = radius, 
                  col = col, 
                  border = border, 
                  percent=percent,
                  edges=edges,
                  lwd=lwd))
  if(length(name)>0){
    grid.lines(x=c(x2, x2), 
               y=c(y2+radius*ratio.yx, 
                   y0+scoreMax+radius*ratio.yx), 
               gp=gpar(col="gray80", lty=3, lwd=lwd))
    grid.text(x=x2, y=y0+scoreMax+radius*ratio.yx + radius*max(ratio.yx, 1.2), 
              label = name, rot=90, just="left", gp=gpar(cex=cex))
  }
}

grid.dan <- function (x=.5, y=.5, 
                      radius=.8,
                      col=NULL,
                      border=NULL,
                      percent=NULL,
                      edges=100,
                      alpha=0, ratio.yx,
                      lwd=1) {
  if(length(percent$score)>0){
    if(percent$score>1){
      stop("if type=fan, the score must be a value between [0, 1].")
    }
    percent <- percent$score
  }else{
    percent <- 1
  }
  if(length(percent)<1){
    percent <- 1
  }
  if (is.null(col)) 
    col <- "white"
  twopi <- 2 * pi
  ratio.yx <- getYXratio()
  t2xy <- function(t) {
    t2p <- twopi * t + pi/2 + alpha
    list(x = radius * cos(t2p), y = radius * sin(t2p) * ratio.yx)
  }
  n <- max(2, edges*percent*100)
  P <- t2xy(seq.int(-percent/2, percent/2, length.out = n))
  grid.polygon(unit(c(P$x, 0)+x,"npc"), unit(c(P$y, 0)+y, "npc"), gp=gpar(col = border[1], fill = col[1], lwd=lwd))
  invisible(NULL)
}

#' convert WIG format track to BED format track
#' @description convert WIG format track to BED format track for a given range
#' @param trackScore an object of track with WIG format
#' @param chrom sequence name of the chromosome
#' @param from start coordinate
#' @param to end coordinate
#' @return an object of \code{\link{track}}
#' @import GenomicRanges
#' @export
#' @examples 
#' extdata <- system.file("extdata", package="trackViewer", mustWork=TRUE)
#' repA <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
#'                     file.path(extdata, "cpsf160.repA_+.wig"),
#'                     format="WIG")
#' strand(repA$dat) <- "-"
#' strand(repA$dat2) <- "+"
#' parseWIG(repA, chrom="chr11", from=122929275, to=122930122)

parseWIG <- function(trackScore, chrom, from, to){
    if(!is(trackScore, "track"))
        stop("trackScore must be an object of track")
    if(trackScore@format!="WIG")
        stop("format must be WIG")
    
    parser <- function(data, chrom, from, to){
        data <- data[seqnames(data)==chrom]
        if(length(data)<1) return(GRanges(score=numeric(0)))
        res <- mapply(function(chr, start, data, span, step, 
                               structure, from, to, strand){
            if(length(data)>1000){
                dataList <- split(data, 
                                  rep(1:ceiling(length(data)/1000), each=1000)[length(data)])
            }else{
                dataList <- list(data)
            }
            gr <- GRanges(score=numeric(0))
            for(data in dataList){
                data <- do.call(rbind, strsplit(data, "\\s+"))
                if(structure=="variableStep"){
                    pos1 <- as.numeric(data[, 1])
                    if(is.na(span)) span <- 1
                    pos2 <- pos1 + as.numeric(span) - 1
                    score <- as.numeric(data[, 2])
                }else{##structure=="fixedStep"
                    pos1 <- start + (1:nrow(data) - 1) * step
                    if(!is.na(span)){
                        pos2 <- pos1 + span
                    }else{
                        pos2 <- start + (1:nrow(data)) * step - 1
                    }
                    score <- as.numeric(data[, 1])
                }
                idx <- pos2>=from & pos1<=to
                if(sum(idx)>0){
                    gr <- c(gr, GRanges(seqnames=chr, 
                                        ranges=IRanges(start=pos1[idx],
                                                       end=pos2[idx]),
                                        strand=strand,
                                        score=score[idx]))
                }
            }
            gr
        }, as.character(seqnames(data)), start(data), data$score, 
                      data$span, data$step, data$structure, from, to, 
                      as.character(strand(data)))
        res <- unlist(GRangesList(res))
        res <- split(res, res$score)
        res <- lapply(res, reduce)
        res <- mapply(function(.ele, .sc) {
            .ele$score <- .sc
            .ele}, res, as.numeric(names(res)))
        res <- unlist(GRangesList(res))
        names(res) <- NULL
        if(length(res)<1) res <- GRanges(score=numeric(0))
        return(orderedGR(res))
    }
    if(trackScore@style@tracktype[1]!='annotation'){
      trackScore@dat <- parser(trackScore@dat, chrom, from, to)
      gc(reset=TRUE)
    }
    if(length(trackScore@style@tracktype)<2){
      trackScore@style@tracktype[2] <- trackScore@style@tracktype[1]
    }
    if(trackScore@style@tracktype[2]!='annotation'){
      trackScore@dat2 <- parser(trackScore@dat2, chrom, from, to)
      gc(reset=TRUE)
    }
    ## check the score
    ## if any score of dat smaller than 0, change it to dat2
    if(length(trackScore@dat2)<1 && any(trackScore@dat$score<0)){
      message("data with score smaller than 0 in dat slot moved to dat2 slot.")
      trackScore@dat2 <- trackScore@dat[trackScore@dat$score<0]
      trackScore@dat2$score <- -1 * trackScore@dat2$score
      trackScore@dat <- trackScore@dat[trackScore@dat$score>=0]
    }
    trackScore@format <- "BED"
    return(trackScore)
}

#' Get current track viewport
#' @description Get current track viewport for addGuideLine
#' @param curViewerStyle an object of \code{\link{trackViewerStyle}}
#' @param start start position of current track
#' @param end end position of current track
#' @return an object of \code{\link[grid]{viewport}}
#' @import grid
#' @export
#' @seealso See Also as \code{\link{addGuideLine}}
#' @examples 
#' vp <- getCurTrackViewport(trackViewerStyle(), 10000, 10200)
#' addGuideLine(c(10010, 10025, 10150), vp=vp)

getCurTrackViewport <- function(curViewerStyle, start, end){
  if(!is(curViewerStyle, "trackViewerStyle"))
    stop("curViewerStyle must be an object of trackViewerStyle")
  margin <- curViewerStyle@margin
  xscale <- c(start, end)
  if(curViewerStyle@flip) xscale <- rev(xscale)
  return(viewport(x=margin[2], y=margin[1], 
                  height=1 - margin[1]- margin[3], 
                  width=1 -margin[2] - margin[4],
                  just=c(0,0), 
                  xscale=xscale, 
                  yscale=c(0,1)))
}

#' Reduce method for 'GInteractions'
#' @description
#' Reduce returns an object of the same type as x containing reduced ranges for
#' each distinct (seqname, strand) pairing.
#' @importMethodsFrom GenomicRanges reduce range
#' @importFrom InteractionSet anchorIds regions GInteractions
#' @importFrom S4Vectors first second
#' @exportMethod reduce
#' @aliases reduce,GInteractions
#' @param x GInteractions object.
#' @param min.gapwidth Ranges separated by a gap of at least min.gapwidth
#'  positions are not merged.
#' @param ignore.strand	TRUE or FALSE. Whether the strand of the input ranges
#' should be ignored or not.
#' @param ... Not used.
#' @examples
#' \dontrun{
#' library(InteractionSet) 
#' gi <- readRDS(system.file("extdata", "gi.rds", package="trackViewer"))
#' reduce(head(gi, n=20))
#' }
#' 
setMethod("reduce", "GInteractions",
          function(x, min.gapwidth=1L,
                   ignore.strand=TRUE, ...){
            anchor1 <- first(x)
            anchor2 <- second(x)
            an1 <- reduce(anchor1, with.revmap=TRUE,
                          min.gapwidth=min.gapwidth,
                          ignore.strand=ignore.strand)
            an2 <- reduce(anchor2, with.revmap=TRUE,
                          min.gapwidth=min.gapwidth,
                          ignore.strand=ignore.strand)
            il1 <- an1$revmap[lengths(an1$revmap)>1]
            il2 <- an2$revmap[lengths(an2$revmap)>1]
            tobemerged <- mapply(il1[rep(seq_along(il1), each=length(il2))],
                                 il2[rep(seq_along(il2), length(il1))],
                                 FUN=function(a, b){
              intersect(a, b)
            }, SIMPLIFY = FALSE)
            tobemerged <- tobemerged[lengths(tobemerged)>1]
            anchor1Ids <- anchorIds(x, type="first")
            anchor2Ids <- anchorIds(x, type="second")
            reg <- regions(x)
            if(ignore.strand){
              strand(reg) <- '*'
            }
            tobemerged <- lapply(tobemerged, function(.ele){
              c(range(reg[anchor1Ids[.ele]]),
                range(reg[anchor2Ids[.ele]]))
            })
            tobemerged <- unlist(GRangesList(tobemerged))
            tobemerged <- unique(tobemerged)
            tobemerged <- c(tobemerged, reg)
            tobemerged <- reduce(tobemerged, min.gapwidth=0L)
            tobemerged <- sort(tobemerged)
            ol1 <- findOverlaps(anchor1, tobemerged, type = 'within',
                                minoverlap = 1L)
            ol2 <- findOverlaps(anchor2, tobemerged, type = 'within',
                                minoverlap = 1L)
            anchor1[queryHits(ol1)] <- tobemerged[subjectHits(ol1)]
            anchor2[queryHits(ol2)] <- tobemerged[subjectHits(ol2)]
            newGI <- GInteractions(anchor1, anchor2, score=mcols(x)$score)
            newGI <- unique(newGI)
            newGI[order(seqnames(first(newGI)),
                        start(first(newGI)),
                        seqnames(second(newGI)),
                        start(second(newGI)),
                        decreasing = FALSE)]
          }
)

#' Class \code{"trackViewerStyle"}
#' @description An object of class \code{"trackViewerStyle"} 
#'              represents track viewer style.
#' @aliases trackViewerStyle
#' @rdname trackViewerStyle-class
#' @slot margin \code{"numeric"}, specify the bottom, left, top and right margin.
#' @slot xlas \code{"numeric"}, label direction of x-axis mark. It should 
#' be a integer 0-3. See \code{\link[graphics]{par}:las}
#' @slot xgp A \code{"list"}, object, It will convert to an object of 
#' class \code{\link[grid]{gpar}}. This is basically a list of graphical 
#' parameter settings of x-axis. For y-axis, see \code{\link{yaxisStyle}}
#' @slot xaxis \code{"logical"}, draw x-axis or not
#' @slot xat \code{"numeric"}, the values will be passed to grid.xaxis as
#' 'at' parameter.
#' @slot xlabel \code{"character"}, the values will be passed to grid.xaxis as
#' 'label' parameter.
#' @slot autolas \code{"logical"} automatic determine y label direction
#' @slot flip \code{"logical"} flip the x-axis or not, default FALSE
#' @import methods
#' @exportClass trackViewerStyle
#' @examples
#' tvs <- trackViewerStyle()
#' setTrackViewerStyleParam(tvs, "xaxis", TRUE)
#' 
setClass("trackViewerStyle", 
         representation(
             margin="numeric",
             xlas="numeric",
             xgp="list",
             xaxis="logical",
             xat="numeric",
             xlabel="character",
             autolas="logical",
             flip="logical"),
         prototype(
             margin=c(.01, .05, 0, 0),
             xlas=0,
             xgp=list(),
             xaxis=FALSE,
             xat=numeric(0L),
             xlabel=character(0L),
             autolas=FALSE,
             flip=FALSE
             ),
         validity=function(object){
             if(!(object@xlas %in% 0:3))
                 return("xlas should be numeric in {0,1,2,3}. See ?par")
             if(any(object@margin<0 | object@margin>.8))
                 return("margin could not greater than .8 or smaller than 0")
             return(TRUE)
         }
)

#' @rdname trackViewerStyle-class
#' @param \dots Each argument in \dots becomes an slot in the new trackViewerStyle.
#' @export

trackViewerStyle <- function(...){
    new("trackViewerStyle", ...)
}

#' @rdname trackViewerStyle-class
#' @param tvs An object of \code{trackViewerStyle}.
#' @param attr the name of slot to be changed.
#' @param value values to be assigned.
#' @exportMethod setTrackViewerStyleParam
#' @aliases setTrackViewerStyleParam
#' @aliases setTrackViewerStyleParam,trackViewerStyle,character-method
#' 
setGeneric("setTrackViewerStyleParam", function(tvs, attr, value) 
    standardGeneric("setTrackViewerStyleParam"))
#' @rdname trackViewerStyle-class
#' @aliases setTrackViewerStyleParam,trackViewerStyle,character,ANY-method
setMethod("setTrackViewerStyleParam", 
          signature(tvs="trackViewerStyle", attr="character", value="ANY"),
          function(tvs, attr, value){
              if(!attr %in% c("margin", "xlas", "xgp", "xat", "xlabel", "xaxis", "autolas", "flip"))
                  stop("attr must be a slot name of trackViewerStyle")
              x <- tvs
              slot(x, attr, check = TRUE) <- value
              eval.parent(substitute(tvs <- x))
              return(invisible(x))
          })

#' Class \code{"pos"}
#' @description An object of class \code{"pos"} represents a point location
#' @rdname pos-class
#' @aliases pos
#' @slot x A \code{\link{numeric}} value, indicates the x position
#' @slot y A \code{\link{numeric}} value, indicates the y position
#' @slot unit \code{"character"} apecifying the units for the corresponding
#' numeric values. See \code{\link[grid]{unit}}
#' @exportClass pos
#' 
setClass("pos", representation(x="numeric", y="numeric", unit="character"),
         prototype(x=0.5, y=0.5, unit="npc"),
         validity=function(object){
             if(!object@unit %in% c("npc", "cm", "inches", "mm", "points", "picas",
                                    "bigpts", "dida", "cicero", "scaledpts",
                                    "lines", "char", "native", "snpc",
                                    "strwidth", "strheight", "grobwidth",
                                    "grobheight"))
                 return("unit must be units of grid unit. See ?unit")
             return(TRUE)
         })

#' Class \code{"xscale"}
#' @description An object of class \code{"xscale"} represents x-scale style.
#' @rdname xscale-class
#' @aliases xscale
#' @slot from A \code{\link{pos}} class, indicates the start point 
#' postion of x-scale.
#' @slot to A \code{\link{pos}} class, indicates the end point 
#' postion of x-scale.
#' @slot label \code{"character"} the label of x-scale
#' @slot gp A \code{"list"} object, It will convert to an object of 
#' class \code{\link[grid]{gpar}}. This is basically a list of graphical 
#' parameter settings of x-scale.
#' @slot draw A \code{"logical"} value indicating whether the x-scale
#' should be draw.
#' @exportClass xscale

setClass("xscale",
         representation(from="pos",
                        to="pos",
                        label="character",
                        gp="list",
                        draw="logical"),
         prototype(draw=FALSE, gp=list())
         )

#' Class \code{"yaxisStyle"}
#' @description An object of class \code{"yaxisStyle"} represents y-axis style.
#' @rdname yaxisStyle-class
#' @aliases yaxisStyle
#' @slot at \code{"numeric"} vector of y-value locations for the tick marks
#' @slot label \code{"logical"} value indicating whether to draw the 
#' labels on the tick marks.
#' @slot gp A \code{"list"} object, It will convert to an object of 
#' class \code{\link[grid]{gpar}}. This is basically a list of graphical 
#' parameter settings of y-axis.
#' @slot draw A \code{"logical"} value indicating whether the y-axis
#' should be draw.
#' @slot main A \code{"logical"} value indicating whether the y-axis
#' should be draw in left (TRUE) or right (FALSE).
#' @exportClass yaxisStyle

setClass("yaxisStyle",
         representation(at="numeric",
                        label="logical",
                        gp="list",
                        draw="logical",
                        main="logical"),
         prototype(draw=TRUE, label=FALSE, gp=list(), main=TRUE)
         )

#' Class \code{"trackStyle"}
#' @description An object of class \code{"trackStyle"} represents track style.
#' @rdname trackStyle-class
#' @aliases trackStyle
#' @slot tracktype \code{"character"} track type, could be peak, 
#' line, histogram, or annotation. 
#' Default is "peak". "annotation" is used to mark the peak regions.
#' For interaction data,
#' it could be "heatmap" or "link".
#' @slot color \code{"character"} track color. If the track has dat and dat2 slot,
#' it should have two values.
#' @slot NAcolor \code{"character"} NA color for interactionData.
#' @slot breaks \code{"numeric"} breaks for color keys of interactionData.
#' @slot height \code{"numeric"} track height. It should be a value between 0 and 1
#' @slot marginTop \code{"numeric"} track top margin
#' @slot marginBottom \code{"numeric"} track bottom margin
#' @slot xscale object of \code{\link{xscale}}, describe the details of x-scale
#' @slot yaxis object of \code{\link{yaxisStyle}}, describe the details of y-axis
#' @slot ylim \code{"numeric"} y-axis range
#' @slot ylabpos \code{"character"}, ylable postion, ylabpos should 
#' be 'left', 'right', 'topleft', 'bottomleft', 'topright', 'bottomright',
#' 'abovebaseline', 'underbaseline', or 'none'.
#' For gene type track, it also could be 'upstream' or 'downstream'
#' @slot ylablas \code{"numeric"} y lable direction. It should 
#' be a integer 0-3. See \code{\link[graphics]{par}:las}
#' @slot ylabgp A \code{"list"} object, It will convert to an object of 
#' class \code{\link[grid]{gpar}}. This is basically a list of graphical 
#' parameter settings of y-label.
#' @slot ysplit A \code{"numeric"} to split y plot region for interaction data.
#' Default is 0.5, which will split the region half to half. It will only work
#' for back to back plot.
#' @exportClass trackStyle
#'

setClass("trackStyle",
         representation(tracktype="character",
                        color="character",
                        NAcolor="character",
                        breaks="numeric",
                        height="numeric",
                        marginTop="numeric",
                        marginBottom="numeric",
                        xscale="xscale",
                        yaxis="yaxisStyle",
                        ylim="numeric",
                        ylabpos="character",
                        ylablas="numeric",
                        ylabgp="list",
                        ysplit="numeric"
                        ),
         prototype(
             marginTop=0,
             marginBottom=0.05,
             color=c("black","black"),
             NAcolor="white",
             breaks=0,
             tracktype="peak",
             ylabpos="left",
             ylablas=0,
             ylabgp=list(),
             ysplit=0.5
         ),
         validity=function(object){
             if(!object@ylabpos %in% c("left", "right", "topleft", "bottomleft", 
                                       "topright", "bottomright", "upstream", "downstream",
                                       "abovebaseline", "underbaseline", "none"))
                 return("ylabpos should be 'left', 'right', 'topleft', 'bottomleft', 
                        'topright', 'bottomright', 'upstream', 'downstream', 
                        'abovebaseline', 'underbaseline' or 'none'.")
             if(!(object@ylablas %in% 0:3))
                return("ylas should be numeric in {0,1,2,3}. See ?par")
             if(!all(object@tracktype %in% c("peak", "annotation", "line", "histogram",
                                             "heatmap", "link")))
                 return("tracktype must be on of peak, annotation, line, histogram, heatmap or link")
             return(TRUE)
         }
)

#' Class \code{"track"}
#' @description An object of class \code{"track"} represents scores of a given track.
#' @rdname trackStyle-class
#' @aliases track
#' @slot dat Object of class \code{\link[GenomicRanges:GRanges-class]{GRanges}}
#' the scores of a given track. It should contain score metadata.
#' @slot dat2 Object of class \code{\link[GenomicRanges:GRanges-class]{GRanges}}
#' the scores of a given track. It should contain score metadata. When dat2
#' and dat is paired, dat will be drawn as positive value where dat2 will be 
#' drawn as negative value (-1 * score)
#' @slot type The type of track. It could be 'data', 'gene', 'transcript', 'scSeq', 'lollipopData' or 'interactionData'.
#' @slot format The format of the input. It could be "BED", "bedGraph",
#' "WIG", "BigWig" or "BAM"
#' @slot style Object of class \code{\link{trackStyle}}
#' @slot name unused yet
#' @exportClass track
#' @examples 
#' extdata <- system.file("extdata", package="trackViewer",
#' mustWork=TRUE)
#' fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED")
#' setTrackStyleParam(fox2, "color", c("red","green"))
#' setTrackXscaleParam(fox2, "gp", list(cex=.5))
#' setTrackYaxisParam(fox2, "gp", list(col="blue"))
#' fox2$dat <- GRanges(score=numeric(0))
#' @seealso Please try to use \code{\link{importScore}} and \code{\link{importBam}} to 
#' generate the object.
setClass("track", representation(dat="GRanges",
                                 dat2="GRanges",
                                 type="character",
                                 format="character",
                                 style="trackStyle",
                                 name="character"),
         validity=function(object){
             if(!object@type %in% 
                c("data", "gene", "transcript", "scSeq",
                  "lollipopData", "interactionData"))
                 return("type must be 'data', 'transcript', 'gene', 'scSeq',
                        'lollipopData', 'interactionData'")
             if(object@type %in% c("data", "scSeq")){
                 if(!length(object@format)==1){
                   return("format must be one of \"BED\", 
                            \"bedGraph\", \"WIG\", \"BigWig\"")
                 }
                 if(!object@format %in% 
                    c("BED", "bedGraph", "WIG", "BigWig", "BAM"))
                     return("format must be one of \"BED\", 
                            \"bedGraph\", \"WIG\", \"BigWig\"")
                 if(is.null(object@dat$score))
                     return("dat should contain score metadata.")
                 if(length(object@dat2)>0){
                     if(is.null(object@dat2$score))
                         return("dat2 should contain score metadata.")
                 }
                 if(object@format!="WIG"){
                     if(!inherits(object@dat$score, c("numeric", "integer")))
                         return("class of score metadata should be numeric")
                 }else{
                     if(!is(object@dat$score, "CompressedCharacterList"))
                         return("Please try ?imortScore for WIG files")
                 }
             }else{
               if(object@type=="lollipopData"){
                 if(is.null(object@dat$score))
                   return("dat should contain score metadata.")
                 if(!all(width(object@dat)==1)){
                   return("Width for lollipopData must be 1")
                 }
                 if(length(object@dat2)>0){
                   if(is.null(object@dat2$score))
                     return("dat2 should contain score metadata.")
                   if(!all(width(object@dat2)==1)){
                     return("Width for lollipopData must be 1")
                   }
                 }
               }else{
                 if(object@type=="interactionData"){
                   if(is.null(object@dat$score))
                     return("dat should contain score metadata.")
                   if(length(object@dat2)!=length(object@dat)){
                       if(length(object@dat$target)!=length(object@dat)){
                           return("dat2 should be same length of dat.") 
                       }else{
                           if(length(object@dat2)>0){
                               if(length(object@dat2$target)!=
                                  length(object@dat2)){
                                   return(paste("dat2 does not contain target",
                                                "metadata."))
                               }
                           }
                       }
                   }
                 }else{
                   if(is.null(mcols(object@dat)$feature))
                     return("The metadata of dat must contain colnumn 'feature'") 
                   if(length(object@dat2)>0){
                     if(is.null(object@dat2$score))
                       return("dat2 should contain score metadata.")
                     if(!all(width(object@dat2)==1)){
                       return("Width for lollipop data must be 1")
                     }
                   }
                 }
               }
             }
             return(TRUE)
         })

#' Method seqlevels
#' @rdname trackStyle-class
#' @exportMethod seqlevels
#' @aliases seqlevels,track-method
setMethod("seqlevels", "track", 
          function(x){ seqlevels(x@dat) })
#' Method seqlevelsStyle
#' @rdname trackStyle-class
#' @exportMethod seqlevelsStyle
#' @aliases seqlevelsStyle,track-method
setMethod("seqlevelsStyle", "track", 
          function(x){ seqlevelsStyle(x@dat) })
#' Method seqlevelsStyle<-
#' @rdname trackStyle-class
#' @exportMethod seqlevelsStyle<-
#' @aliases seqlevelsStyle<-,track-method
setReplaceMethod("seqlevelsStyle", "track", 
          function(x, value){ 
              seqlevelsStyle(x@dat) <- value
              seqlevelsStyle(x@dat2) <- value
              return(x)
})

#' @rdname trackStyle-class
#' @param object an object of trackStyle.
#' @exportMethod show
#' 
#' @aliases show,track-method
setMethod("show", "track", function(object){
    cat("This is an object of track\n", "slot name:", object@name, "\n", 
        "slot type:", object@type, "\n", "slot format:", object@format, "\n")
    cat("slot dat:\n")
    show(object@dat)
    cat("slot dat2:\n")
    show(object@dat2)
    cat("slot style: try object$style to see details.\n")
})
#' Method $
#' @rdname trackStyle-class
#' @param x an object of trackStyle or track
#' @param name slot name of trackStyle or track
#' @exportMethod $
#' @aliases $,track-method
#' @aliases $,trackStyle-method
setMethod("$", "track", function(x, name) slot(x, name))
setMethod("$", "trackStyle", function(x, name) slot(x, name))
#' Method $<-
#' @rdname trackStyle-class
#' @exportMethod $<-
#' @aliases $<-,track-method
#' @aliases $<-,trackStyle-method
setReplaceMethod("$", "track", 
                 function(x, name, value){
                     slot(x, name, check = TRUE) <- value
                     x
                 })
setReplaceMethod("$", "trackStyle", 
                 function(x, name, value){
                   slot(x, name, check = TRUE) <- value
                   x
                 })
#' Method setTrackStyleParam
#' @rdname trackStyle-class
#' @param ts An object of \code{track}.
#' @param attr the name of slot of \code{\link{trackStyle}} object to be changed.
#' @param value values to be assigned.
#' @exportMethod setTrackStyleParam
#' @aliases setTrackStyleParam
#' @aliases setTrackStyleParam,track,character-method
setGeneric("setTrackStyleParam", function(ts, attr, value) 
    standardGeneric("setTrackStyleParam"))
#' 
#' @rdname trackStyle-class
#' @aliases setTrackStyleParam,track,character,ANY-method
setMethod("setTrackStyleParam", 
          signature(ts="track", attr="character", value="ANY"),
          function(ts, attr, value){
              if(!attr %in% c("tracktype", "color", "height", "marginTop", 
                              "marginBottom", "ylim", "ylabpos", "ylablas",
                              "ylabgp", "breaks", "NAcolor", 'ysplit'))
                  stop("attr must be a slot name (except xscale and yaxis) of trackStyle.
                       try setTrackXscaleParam for xscale slot and 
                       setTrackYaxisParam for yaxis.")
              x <- ts
              slot(x@style, attr, check = TRUE) <- value
              eval.parent(substitute(ts <- x))
              return(invisible(x))
          })
#' Method setTrackXscaleParam
#' @rdname trackStyle-class
#' @exportMethod setTrackXscaleParam
#' @aliases setTrackXscaleParam
#' @aliases setTrackXscaleParam,track,character-method
setGeneric("setTrackXscaleParam", function(ts, attr, value) 
    standardGeneric("setTrackXscaleParam"))
#' @rdname trackStyle-class
#' @aliases setTrackXscaleParam,track,character,ANY-method
#' @details 
#' The attr of \code{setTrackXscaleParam} could not only be a slot of xscale, but also be position.
#' If the attr is set to position, value must be a list of x, y and label. For example
#' setTrackXscaleParam(track, attr="position", value=list(x=122929675, y=4, label=500))
setMethod("setTrackXscaleParam", 
          signature(ts="track", attr="character", value="ANY"),
          function(ts, attr, value){
              if(!attr %in% c("from", "to", "label", "gp", "draw", "position"))
                  stop("attr must be a slot name of xscale object")
              x <- ts
              if(attr=="position"){
                if(!is.list(value)){
                  stop("if attr is position, value must be a list of x, y, and label.")
                }
                if(any(!c("x", "y", "label") %in% names(value))){
                  stop("if attr is position, value must be a list of x, y, and label. eg: list(x=122929375, y=0.5, label=1000)")
                }
                label0 <- as.numeric(value$label)
                if(is.na(label0)){
                  stop("label can not be converted to number.")
                }
                slot(x@style@xscale, "from", check = TRUE)  <- new("pos", x=value$x-label0/2, y=value$y, unit="native")
                slot(x@style@xscale, "to", check = TRUE)  <- new("pos", x=value$x+label0/2, y=value$y, unit="native")
                slot(x@style@xscale, "label", check = TRUE) <- convertNum2HumanNum(value$label)
              }else{
                slot(x@style@xscale, attr, check = TRUE) <- value
              }
              eval.parent(substitute(ts <- x))
              return(invisible(x))
          })
#' Method setTrackYaxisParam
#' @rdname trackStyle-class
#' @exportMethod setTrackYaxisParam
#' @aliases setTrackYaxisParam
#' @aliases setTrackYaxisParam,track,character-method
setGeneric("setTrackYaxisParam", function(ts, attr, value) 
    standardGeneric("setTrackYaxisParam"))
#' @rdname trackStyle-class
#' @aliases setTrackYaxisParam,track,character,ANY-method
setMethod("setTrackYaxisParam", 
          signature(ts="track", attr="character", value="ANY"),
          function(ts, attr, value){
              if(!attr %in% c("at", "label", "gp", "draw", "main"))
                  stop("attr must be a slot name of xscale object")
              x <- ts
              slot(x@style@yaxis, attr, check = TRUE) <- value
              eval.parent(substitute(ts <- x))
              return(invisible(x))
          })

#' List of tracks
#' @description An extension of List that holds only \code{\link{track}} objects. 
#' @rdname trackList-class
#' @aliases trackList
#' @seealso \code{\link{track}}.
#' @exportClass trackList

setClass("trackList", contains="list", representation(names="vector"),
         validity=function(object){
             re <- sapply(object, class)
             if(any(re!="track"))
                 return("class of elements should be track")
             return(TRUE)
         })

#' Method seqlevelsStyle<-
#' @rdname trackList-class
#' @param x trackList object.
#' @param value values to be assigned.
#' @exportMethod seqlevelsStyle<-
#' @aliases seqlevelsStyle<-,trackList-method
setReplaceMethod("seqlevelsStyle", "trackList", 
                 function(x, value){
                     for(i in seq_along(x)){
                         seqlevelsStyle(x[[i]]) <- value
                     }
                     x
                 })


#' @rdname trackList-class
#' @param \dots Each tracks in ... becomes an element in the new 
#' trackList, in the same order. This is analogous to the list constructor, except
#' every argument in ... must be derived from \code{\link{track}}.
#' @param heightDist A vector or NA to define the height of each track.
#' @export trackList
trackList <- function(..., heightDist=NA){
    listData <- list(...)
    dots <- substitute(list(...))[-1]
    names <- as.character(sapply(dots, deparse))
    if(is.na(heightDist[1])){
        heightDist <- rep(1, length(listData))
    }else{
        if(length(heightDist)!=length(listData)){
            stop("length of heightDist should be same as length of inputs")
        }
    }
    heightDist <- heightDist * 1/sum(heightDist)
    tmpData <- list()
    recursiveList <- function(tmp, tmpData, name, heightDist){
        if(is.list(tmp)){
            if(length(tmp)==0) return(tmpData)
            for(w in 1:length(tmp)) {
                each <- tmp[[w]]
                curHeightDist <- heightDist/length(tmp)
                if(is.list(each)){
                    tmpData <- recursiveList(each, tmpData, 
                                         names(tmp)[w], curHeightDist)
                }else{
                    .name <- names(tmp)[w]
                    each@style@height <- curHeightDist
                    tmpData <- c(tmpData, each)
                    names(tmpData)[length(tmpData)] <- .name
                }
            }
        }else{
            tmp@style@height <- heightDist
            tmpData <- c(tmpData, tmp)
            names(tmpData)[length(tmpData)] <- name
        }
        tmpData
    }
    if(length(listData)>=1){
        for(i in 1:length(listData)){
            tmp <- listData[[i]]
            tmpData <- recursiveList(tmp, tmpData, names[i], heightDist[i])
            rm("tmp")
        }
    }
    listData <- tmpData
    rm("tmpData")
    
    new("trackList", listData)
}

#' Aggregate Region Analysis
#' @description Extract the interaction signal means from given coordinates. 
#' @param gr A `GRanges` object. The center of the object will be used for
#'  alignment for all the given regions.
#' @param upstream,downstream numeric(1L). Upstream and downstream from the 
#' center of given `gr` input will be used to extract the signals.
#' @param resolution numeric(1L). The resolution will be passed to
#'  \link{importGInteractions} function.
#' @param ... The parameters used by \link{importGInteractions} function.
#' Please note that the ranges resolution and out parameter should not be
#' involved.
#' @return A \link[InteractionSet:GInteractions-class]{GInteractions} object
#' with scores which represent the mean values of the interactions.
#' @export
#' @importFrom InteractionSet regions `regions<-`
#' @examples
#' hic <- system.file("extdata", "test_chr22.hic", package = "trackViewer",
#'                    mustWork=TRUE)
#' gr <- GRanges("22", c(seq(20000001, 50000001, by=10000000), width=1))
#' gi <- ARA(gr, file=hic, format="hic")
#' rg <- GRanges("22", IRanges(1, 400000))
#' op <- optimizeStyle(trackList(gi2track(gi)))
#' heatmap <- op$tracks
#' sty <- op$style
#' setTrackViewerStyleParam(sty, "xat", c(1, 200000, 400000))
#' setTrackViewerStyleParam(sty, "xlabel",c("-20K", "center", "20K"))
#' viewTracks(heatmap, viewerStyle=sty, gr=rg)

ARA <- function(gr, upstream=200000, downstream=upstream, resolution=10000,
                ...){
  stopifnot(is(gr, "GRanges"))
  stopifnot(is(upstream, "numeric"))
  stopifnot(is(downstream, "numeric"))
  stopifnot(is(resolution, "numeric"))
  resolution <- resolution[1]
  start(gr) <- floor((start(gr)+end(gr))/2/resolution)*resolution+1
  width(gr) <- resolution
  gr <- promoters(gr, upstream = upstream, downstream = downstream)
  gr <- gr[start(gr)>0]
  gr <- trim(gr)
  gr <- gr[width(gr)==upstream + downstream]
  gi <- lapply(seq_along(gr), function(.ele){
    importGInteractions(ranges=gr[.ele], resolution = resolution,
                        out = "GInteractions", ...)
  })
  seqn <- as.character(seqlevels(gr))[1]
  gi_shift <- mapply(gi, start(gr), FUN = function(.ele, start){
    .ele <- shift(.ele, -1*start)
    seqlevels(regions(.ele)) <- seqn
    .ele
  })
  gi_accum <- GIoperator(gi_shift, operator = "+")
  gi_accum$score <- gi_accum$score/length(gi_accum)
  gi_accum
}

#' calculate coverage
#' @description calculate coverage for \code{\link[GenomicRanges:GRanges-class]{GRanges}},
#'  \code{\link[GenomicAlignments:GAlignments-class]{GAlignments}} or 
#'  \code{\link[GenomicAlignments:GAlignmentPairs-class]{GAlignmentPairs}}
#' @param gr an object of RGanges, GAlignments or GAlignmentPairs
#' @return an object of GRanges
#' @seealso See Also as \code{\link[IRanges:coverage-methods]{coverage}}, 
#' \code{\link[GenomicRanges]{coverage-methods}}
#' @export
#' @import GenomicAlignments
#' @import GenomicRanges
#' @import IRanges
#' @examples 
#' bed <- system.file("extdata", "fox2.bed", package="trackViewer",
#'                    mustWork=TRUE)
#' fox2 <- importScore(bed)
#' fox2$dat <- coverageGR(fox2$dat)
#' 
coverageGR <- function(gr){##todo, check GAlignmentPairs
    if(!inherits(gr, c("GRanges", "GAlignments", "GAlignmentPairs")))
        stop("input is not an object of RGanges, GAlignments or GAlignmentPairs.")
    flag <- TRUE
    if(is(gr, "GRanges")){
        if(is.null(score(gr))) gr$score <- rep(1, length(gr))
        if(length(unique(score(gr)))!=1) flag <- FALSE
    }
    if(flag){
        gr <- split(gr, strand(gr))
        gr <- mapply(function(.ele, strand){
            coverage <- coverage(.ele)
            re <- mapply(function(cov, seqname, strand){
                len <- length(cov)
                if(len>0){
                    width <- cov@lengths
                    start <- cumsum(width)
                    start <- c(1, start[-length(start)]+1)
                    value <- cov@values
                }else{
                    start <- 1
                    width <- 1
                    value <- 0
                }
                idx <- value > 0
                if(all(!idx)) {
                    GRanges(score=numeric(0))
                }else{
                    GRanges(seqnames=seqname, 
                            ranges=IRanges(start=start[idx],
                                           width=width[idx]),
                            strand=strand,
                            score=value[idx])
                }
            }, coverage, names(coverage), strand)
            unlist(GRangesList(re))
            
        }, gr, names(gr))
        gr <- unlist(GRangesList(gr))
        names(gr) <- NULL
    }else{
        disj <- disjoin(gr)
        if(length(disj)>length(gr)){
            ol <- findOverlaps(disj, gr)
            score <- sapply(split(subjectHits(ol), queryHits(ol)), 
                            function(.ids){
                                sum(score(gr)[.ids])
                            })
            disj$score <- rep(0, length(disj))
            disj$score[as.numeric(names(score))] <- score
            ##merge continuous region
            block <- rle(score(disj))
            if(any(block$lengths!=1)){
                block$values <- 1:length(block$values)
                block <- inverse.rle(block)
                dup <- block %in% block[duplicated(block)]
                disj.uni <- disj[!dup]
                disj.dup <- disj[dup]
                block <- rle(score(disj.dup))
                block$values <- 1:length(block$values)
                block <- inverse.rle(block)
                disj.dup <- split(disj.dup, block)
                
                disj.rd <- sapply(disj.dup, reduce)
                disj.rd <- unlist(GRangesList(disj.rd))
                disj.rd$score <- sapply(disj.dup, function(.ele) score(.ele)[1])
                disj <- c(disj.uni, disj.rd)
            }
            gr <- unname(disj)
        }
    }
    return(orderedGR(gr))
}

test_coverageGR<-function(){
	gr <- GRanges("chr1", IRanges(c(1:10), width=3), strand="+", score=rep(1, 10))
    gr <- coverageGR(gr)
	checkEquals(c(1, 2, 3, 2, 1), score(gr))
	checkEquals(c(1, 2, 3, 11, 12), start(gr))
	checkEquals(c(1, 2, 10, 11, 12), end(gr))
}

#' Reading data from a BED or WIG file
#' @description Read a \code{\link{track}} object from a BED, bedGraph, WIG or BigWig file
#' @param file The path to the file to read.
#' @param file2 The path to the second file to read.
#' @param format The format of import file. Could be BED, bedGraph, WIG or BigWig
#' @param ranges An object of \code{\link[GenomicRanges:GRanges-class]{GRanges}} to indicate
#' the range to be imported
#' @param ignore.strand ignore the strand or not when do filter. default TRUE
#' @return a \code{\link{track}} object 
#' @import GenomicRanges
#' @import IRanges
#' @export
#' @seealso See Also as \code{\link{importBam}}, \code{\link{track}}, 
#' \code{\link{viewTracks}}
#' @examples 
#' #import a BED file
#' bedfile <- system.file("tests", "test.bed", package="rtracklayer",
#'                        mustWork=TRUE)
#' dat <- importScore(file=bedfile, format="BED",
#'                    ranges=GRanges("chr7", IRanges(127471197, 127474697)))
#' 
#' ##import a WIG file
#' wigfile <- system.file("tests", "step.wig", package = "rtracklayer",
#'                        mustWork=TRUE)
#' dat <- importScore(file=wigfile, format="WIG")
#' 
#' ##import a BigWig file
#' if(.Platform$OS.type!="windows"){##this is because we are using rtracklayer::import
#'   bwfile <- system.file("tests", "test.bw", package = "rtracklayer",
#'                         mustWork=TRUE)
#'   dat <- importScore(file=bwfile, format="BigWig")
#' }
#' 
#' ##import 2 file
#' wigfile1 <- system.file("extdata", "cpsf160.repA_+.wig", package="trackViewer",
#'                         mustWork=TRUE)
#' wigfile2 <- system.file("extdata", "cpsf160.repA_-.wig", package="trackViewer",
#'                         mustWork=TRUE)
#' dat <- importScore(wigfile1, wigfile2, format="WIG", 
#'                    ranges=GRanges("chr11", IRanges(122817703, 122889073)))

importScore <- function(file, file2, 
                        format=c("BED", "bedGraph", "WIG", "BigWig"),
                        ranges=GRanges(), ignore.strand=TRUE){
    if(missing(file))
        stop("file is required.")
    ##    on.exit(closeAllConnections())
    format <- match.arg(format)
    #    res <- GRanges(score=numeric(0))
    if(!is(ranges, "GRanges")) stop("ranges must be an object of GRanges.")
    gr <- orderedGR(ranges)
    seqn <- unique(as.character(seqnames(gr)))
    filterByRange <- function(r){
        if(length(gr)>0){
            ## find in ranges
            r <- r[r[, 1] %in% seqn, , drop=FALSE]
            nr <- nrow(r)
            if(nr>0){
                ## split r and findOverlaps
                idx <- rep(FALSE, nr)
                l <- floor(nr/1000)
                for(i in 0:l){
                    f <- min(i*1000+1, nr)
                    t <- min((i+1)*1000, nr)
                    x <- r[f:t, , drop=FALSE]
                    xgr <- GRanges(x[,1], IRanges(start=as.numeric(x[,2]),
                                                  end=as.numeric(x[,3])))
                    suppressWarnings(ol <- findOverlaps(xgr, gr, ignore.strand=ignore.strand))
                    if(length(ol)>0) idx[queryHits(ol)+i*1000] <- TRUE
                }
                r <- r[idx, , drop=FALSE]
            }
        }
        r
    }
    
    getWigInfo <- function(firstline){
        firstline <- unlist(strsplit(firstline, "\\s"))
        firstline <- firstline[firstline!=""]
        structure <- firstline[1]
        firstline <- firstline[-1]
        firstline <- do.call(rbind,strsplit(firstline, "=", fixed=TRUE))
        firstline <- firstline[match(c("chrom","span","start","step"), 
                                     firstline[,1]),]
        info <- c(structure, firstline[,2])
        names(info) <- c("structure", "chrom","span","start","step")
        return(info)
    }
    readWIG <- function(buf, lastWigInfo=NULL){
        ##remove browser lines and track lines
        buf <- gsub("^\\s+", "", buf)
        buf <- gsub("\\s+$", "", buf)
        buf <- buf[grepl("^(variableStep|fixedStep|([0-9]+))", buf)]
        infoLine <- grep("Step", buf)
        if(length(infoLine)>0){
            if(infoLine[1]!=1){
                if(is.null(lastWigInfo[1])){
                    ##1 line is the info line
                    stop("WIG file must contain track definition line, 
                     which should start by variableStep or fixedStep.")
                }else{
                    buf <- c(lastWigInfo, buf)
                    infoLine <- grep("Step", buf)
                }
            }
        }else{
            if(is.null(lastWigInfo[1])){
                stop("WIG file must contain track definition line, 
                     which should start by variableStep or fixedStep.")
            }else{
                buf <- c(lastWigInfo, buf)
                infoLine <- grep("Step", buf)
            }
        }
        
        lastWigInfo <- buf[infoLine[length(infoLine)]]
        
        while(infoLine[length(infoLine)]==length(buf)){
            buf <- buf[-length(buf)]
        }
        block <- c(infoLine, length(buf)+1)
        dif <- diff(block)
        block <- rep(infoLine, dif)
        buf <- split(buf, block)
        r <- lapply(buf, function(.ele) {
            wiginfo <- getWigInfo(.ele[1])
            span <- wiginfo["span"]
            step <- as.numeric(wiginfo["step"])
            if(wiginfo["structure"]=="variableStep"){
                start <- as.numeric(strsplit(.ele[2], "\\s+")[[1]][1])
                if(is.na(span)) span <- 1
                lastrow <- strsplit(.ele[length(.ele)], "\\s+")[[1]]
                end <- as.numeric(lastrow)[1] + as.numeric(span)
            }else{##wiginfo["structure"]=="fixedStep"
                start <- as.numeric(wiginfo["start"])
                if(!is.na(span)){
                    end <- start + (length(.ele) - 1) * step + as.numeric(span) - 1
                }else{
                    end <- start + length(.ele) * step - 1
                }
            }
            c(wiginfo["chrom"], start, end, span, step, wiginfo["structure"])
        })
        r <- do.call(rbind, r)
        wiginfo <- getWigInfo(lastWigInfo)
        if(wiginfo["structure"]=="fixedStep"){
            lastWigInfo <- gsub("start=\\d+(\\s)", 
                                paste("start=",
                                      as.numeric(r[nrow(r), 3])+1,
                                      "\\1", sep=""), lastWigInfo)
        }
        buf <- lapply(buf, "[", -1)
        buf <- CharacterList(buf, compress=TRUE)
        ## filter by range
        if(length(gr)>0){
            r <- cbind(r, rid=1:nrow(r))
            r <- filterByRange(r)
            buf <- buf[as.numeric(r[,"rid"])]
        }
        
        list(gr=GRanges(seqnames=r[,1], 
                ranges=IRanges(start=as.numeric(r[,2]),
                               end=as.numeric(r[,3])),
                score=buf,
                span=as.numeric(r[,4]),
                step=as.numeric(r[,5]),
                structure=r[,6]),
             lastWigInfo=lastWigInfo)
    }
    readBED <- function(buf){
        ##c("chrom", "chromStart", "chromEnd", "name", "score", "strand", ...)
        ##remove annotations
        buf <- strsplit(buf, "\t", fixed=TRUE)##
        len <- sapply(buf, length)
        buf <- buf[len>2]
        len <- len[len>2]
        if(length(buf)<1) return(GRanges(score=numeric(0)))
        maxLen <- max(len)
        if(all(len==maxLen)){
            buf <- do.call(rbind, buf)
        }else{
            NAs <- rep("", maxLen)
            buf <- do.call(rbind, 
                           lapply(buf, function(.ele) c(.ele, NAs)[1:maxLen]))
        }
        ##check strand
        if(ncol(buf)==3) buf <- cbind(buf, ".")
        if(ncol(buf)==4) {
            if(all(grepl("^[\\d\\.]+$", buf[,4])) && 
                   length(unique(nchar(buf[,4])))>1){
                buf <- cbind(buf, buf[, 4])
            }else{
                buf <- cbind(buf, 1)
            }
        }
        if(ncol(buf)==5) buf <- cbind(buf, "*")
        buf[!buf[,6] %in% c("+", "-"), 6] <- "*"
        ##filter by range
        buf <- filterByRange(buf)
        if(nrow(buf)>0){
            GRanges(seqnames=buf[,1], 
                    ranges=IRanges(start=as.numeric(buf[,2])+1,##0 based, half open
                                   end=as.numeric(buf[,3])),
                    strand=buf[,6],
                    score=as.numeric(buf[,5]))
        }else{##create a length=0 GRanges
            GRanges(seqnames=buf[,1], 
                    ranges=IRanges(start=as.numeric(buf[,2]),
                                   end=as.numeric(buf[,3])),
                    strand=buf[,6],
                    score=as.numeric(buf[,5]))
        }
    }
    readFourCols <- function(buf){
        buf <- gsub("^\\s+", "", buf)
        buf <- gsub("\\s+$", "", buf)
        buf <- buf[!grepl("^(browser|track|#)", buf)]
        buf <- strsplit(buf, "\t", fixed=TRUE)
        len <- sapply(buf, length)
        buf <- buf[len==4]
        if(length(buf)<1) return(GRanges(score=numeric(0)))
        buf <- do.call(rbind, buf)
        ##filter by range
        buf <- filterByRange(buf) 
        if(nrow(buf)>0){
            GRanges(seqnames=buf[,1], 
                    ranges=IRanges(start=as.numeric(buf[,2])+1,##0 based, half open
                                   end=as.numeric(buf[,3])),
                    score=as.numeric(buf[,4]))
        }else{##create a length=0 GRanges
            GRanges(seqnames=buf[,1], 
                    ranges=IRanges(start=as.numeric(buf[,2]),##0 based, half open
                                   end=as.numeric(buf[,3])),
                    score=as.numeric(buf[,4]))
        }
    }
    readbedGraph <- function(buf){
        readFourCols(buf)
    }
    readBigWig <- function(file){
        if(length(gr)>0){
            import(con=file, format="BigWig", which=gr)
        }else{
            import(con=file, format="BigWig")
        } 
    }
    readFile <- function(file, format, FUN){
        if(format=="WIG"){
            res <- NULL
            con <- file(file, open="r")
            on.exit(close(con))
            lastWigInfo <- NULL
            while(length(buf <- readLines(con, n=1000000, warn=FALSE))>0){
                buf <- FUN(buf, lastWigInfo)
                lastWigInfo <- buf$lastWigInfo
                if(length(res)<1) {
                    res <- buf$gr
                }else{
                    suppressWarnings(res <- c(res, buf$gr))
                }
            }
        }else{
            s <- file.info(file)$size
            if(s<100000000){
                buf <- readChar(file, s, useBytes=TRUE)
                buf <- strsplit(buf, "\n", fixed=TRUE, useBytes=TRUE)[[1]]
                res <- FUN(buf)
            }else{
                message("file is too huge. Please consider to use bedtools or bedops to subset the data.")
                res <- NULL
                con <- file(file, open="r")
                on.exit(close(con))
                while(length(buf <- readLines(con, n=1000000, warn=FALSE))>0){
                    buf <- FUN(buf)
                    if(length(res)<1) {
                        res <- buf
                    }else{
                        suppressWarnings(res <- c(res, buf))
                    }
                }
            }
        }
        res <- unique(res)
        return(res)
    }
    readFiles <- function(file, format){
        FUN <- get(paste("read", format, sep=""))
        if(format=="BigWig"){
            if(.Platform$OS.type=="windows")
                stop("rtracklayer can't read bigWig files on a Windows computer. Type in  ?`BigWigFile-class` to get the help.")
            res <- unique(FUN(file))
        }else{
            res <- readFile(file, format, FUN)
        }
        return(res)
    }
    res <- readFiles(file, format)
    if(!missing(file2)){
        res2 <- readFiles(file2, format)
        return(new("track", dat=orderedGR(res), dat2=orderedGR(res2),
                   type="data", format=format))
    }else{
        return(new("track", dat=orderedGR(res), type="data", format=format))
    }
    }

#' track from TxDb
#' @description Generate a track object from TxDb by given gene ids 
#' @param ids Gene IDs. A vector of character. It should be keys in txdb.
#' @param txdb An object of \code{\link[GenomicFeatures:TxDb-class]{TxDb}}.
#' @param type Output type of track, "gene" or "transcript".
#' @param symbols symbol of genes. 
#' @param asList Output a list of tracks or not. Default TRUE.
#' @return An object of \link{track}
#' @importFrom AnnotationDbi select
#' @import GenomicRanges
#' @import IRanges
#' @export
#' @examples 
#' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
#' library(org.Hs.eg.db)
#' ids <- c("3312", "390259", "341056", "79827")
#' symbols <- mget(ids, org.Hs.egSYMBOL)
#' geneTrack(ids, TxDb.Hsapiens.UCSC.hg19.knownGene, symbols)
#' 
geneTrack <- function(ids, txdb, symbols, 
                      type=c("gene", "transcript"), asList=TRUE){
  stopifnot(is(txdb, "TxDb"))
  stopifnot(is(ids, "character"))
  stopifnot(is(asList, "logical"))
  if(missing(symbols)) symbols <- ids
  if(is.null(names(symbols))){
    names(symbols) <- ids
  }
  symbols <- symbols[ids]
  stopifnot(length(ids)==length(symbols))
  symbols <- sapply(symbols, `[`, i=1)
  type <- match.arg(type)
  getFeatureType <- function(.ele, .cds, .id, .txname, .gene){
    .ele <- disjoin(.ele)
    .cds <- disjoin(.cds)
    .ele <- c(.ele, .cds)
    .ele.disjoin <- disjoin(.ele, with.revmap = TRUE)
    .ele.disjoin$feature <- ifelse(lengths(.ele.disjoin$revmap)==1, "ncRNA", "CDS")
    .ele.disjoin <- sort(.ele.disjoin, 
                         decreasing = as.character(strand(.ele.disjoin)[1])=="-")
    cds <- range(which(.ele.disjoin$feature=="CDS"))
    if(cds[1]>1) .ele.disjoin$feature[seq.int(cds[1]-1)] <- "utr5"
    if(cds[2]<length(.ele.disjoin)) .ele.disjoin$feature[seq(cds[2]+1, length(.ele.disjoin))] <- "utr3"
    .ele.disjoin$id <- .id
    .ele.disjoin$exon <- paste(.id, sapply(.ele.disjoin$revmap, `[`, i=1), sep="_")
    .ele.disjoin$transcript <- .txname
    .ele.disjoin$gene <- .gene
    .ele.disjoin$revmap <- NULL
    .ele.disjoin
  }
  if(type == "gene"){
    suppressMessages({
      exons <- select(txdb, ids, 
                      c("GENEID", "EXONCHROM", "EXONEND", "TXNAME", "EXONSTART", "EXONSTRAND"),
                      "GENEID")
      cds <- select(txdb, unique(exons$TXNAME), 
                    c("TXNAME", "CDSCHROM", "CDSSTRAND", "CDSSTART", "CDSEND"),
                    "TXNAME")
      cds <- merge(unique(exons[, c("GENEID", "TXNAME")]), cds, all.x=TRUE)
    })
    exons <- split(exons, exons$GENEID)
    cds <- split(cds, cds$GENEID)
    exons <- mapply(function(.ele, .cds, .name, .id){
      .ele <- GRanges(seqnames = .ele$EXONCHROM, 
                      ranges = IRanges(.ele$EXONSTART, .ele$EXONEND),
                      strand = .ele$EXONSTRAND)
      .cds <- .cds[!is.na(.cds$CDSCHROM), , drop=FALSE]
      if(nrow(.cds)>0){
        .cds <- GRanges(seqnames = .cds$CDSCHROM,
                        ranges = IRanges(.cds$CDSSTART, .cds$CDSEND),
                        strand = .cds$CDSSTRAND)
        .ele <- getFeatureType(.ele, .cds, .id, "unknown", .name)
      }else{
        .ele$feature <- "ncRNA"
      }
      .ele <- orderedGR(.ele)
      names(.ele) <- rep(.name, length(.ele))
      new("track", dat=.ele, type="gene", 
          name=.name,
          style=new("trackStyle", color="lightblue"))
    }, exons, cds[names(exons)], symbols[names(exons)], names(exons))
    if(!asList){
      exons <- lapply(exons, function(.ele){
        .e <- .ele@dat
        .e$featureID <- .ele@name
        .e
      })
      exons <- unlist(GRangesList(exons))
      exons <- new("track", dat=exons, type="gene",
                   name="genes",
                   style=new("trackStyle", color="black"))
    }
    return(exons)
  }
  ## type == "transcript"
  suppressMessages({
    exons <- select(txdb, ids, 
                  c("GENEID", "EXONCHROM", "EXONSTRAND", "TXNAME", "EXONSTART", "EXONEND"),
                  "GENEID")
  })
  exons <- split(exons, exons$GENEID)
  trs <- mapply(function(exon, .id, .name){
    exon <- GRanges(seqnames = exon$EXONCHROM,
                    ranges = IRanges(exon$EXONSTART, exon$EXONEND),
                    strand = exon$EXONSTRAND, TXNAME=exon$TXNAME)
    txs <- split(exon, exon$TXNAME)
    txs <- lapply(txs, function(.ele){
      suppressMessages({
        cds <- select(txdb, .ele$TXNAME[1], 
                      c("TXNAME", "CDSCHROM", "CDSSTRAND", "CDSSTART", "CDSEND"),
                      "TXNAME")
      })
      cds <- cds[!is.na(cds$CDSCHROM), , drop=FALSE]
      if(nrow(cds)>0){
        cds <- GRanges(seqnames = cds$CDSCHROM,
                       ranges = IRanges(cds$CDSSTART, cds$CDSEND),
                       strand = cds$CDSSTRAND)
        cds$TXNAME <- .ele$TXNAME[1]
        .ele <- getFeatureType(.ele, cds, .ele$TXNAME[1], .ele$TXNAME[1], .name)
      }else{
        .ele$feature <- "ncRNA"
        .ele$id <- .id
        .ele$transcript <- .ele$TXNAME
        .ele$gene <- .name
      }
      .ele <- orderedGR(.ele)
      
      new("track", dat=.ele, type="transcript", 
          name=.ele$transcript[1],
          style=new("trackStyle", color="lightblue"))
    })
  }, exons, names(exons), symbols[names(exons)])
  names(trs) <- symbols[names(exons)]
  trs <- unlist(trs)
  
  if(!asList){
    trs <- lapply(trs, function(.ele){
      .e <- .ele@dat
      .e$featureID <- .ele@name
      .e
    })
    trs <- unlist(GRangesList(trs))
    trs <- new("track", dat=trs, type="transcript",
                 name="transcripts",
                 style=new("trackStyle", color="black"))
  }
  trs
}

plotGeneTrack <- function(track, xscale, chr, yaxis.gp=gpar(), lollipop_style_switch_limit=10){
  if(length(track@dat$featureID)!=length(track@dat)){
    return(plotGeneModel(track, xscale, chr, yaxis.gp, lollipop_style_switch_limit))
  }
  
  if(length(track@dat2)>0){
    feature.height <- 
      if(is.list(track@dat2$feature.height)) track@dat2$feature.height[[1]] else track@dat2$feature.height[1]
    if(length(feature.height)==0) feature.height <- .5
    unit <- feature.height/4
    y <- feature.height/2
  }else{
    unit <- 0.25
    y <- 0.5
  }
  
  pos_vjust <- 1.2
  neg_vjust <- -0.5
  pos_hjust <- 0.2
  neg_hjust <- 0.8
  
  ## split transcripts by featureID, 
  ## get the ranges of each transcripts
  ## assign lines for all transcripts
  ## plot center line for each featureID
  ## plot exons by the feature type
  ## plot direction at TSS
  ## add gene name at TSS
  vp <- viewport(xscale = xscale, y=y, height = unit*2)
  pushViewport(vp)
  trs <- split(track@dat, track@dat$featureID)
  col <- track@style@color
  rgs <- unlist(GRangesList(sapply(trs, function(.ele)range(unname(.ele)))))
  if(length(trs)!=length(rgs)){
    stop("some genes are splited into multiple regions, eg multple strand.")
  }
  strand <- as.character(strand(rgs))
  dj <- disjoin(rgs, with.revmap=TRUE)
  lineId <- data.frame(id=seq_along(rgs), line=0)
  revmap <- dj$revmap
  for(i in seq_along(revmap)){
    revm <- sort(revmap[[i]])
    if(lineId[revm[1], "line"]==0){
      lineId[revm[1], "line"] <- 1
    }
    for(j in seq_along(revm)[-1]){
      lineId[revm[j], "line"] <- lineId[revm[j-1], "line"] + 1
    }
  }
  totalLines <- max(lineId$line)
  ## check height of plot region
  unity <- convertHeight(unit(1, "npc"), unitTo = "lines", valueOnly = TRUE)
  doLabels <- unity>=totalLines
  eachLineHeight <- 1/totalLines
  currLineBottom <- 1
  unitx <- convertWidth(unit(1, "npc"), unitTo = "lines", valueOnly = TRUE)
  arr_size <- min(.5, unitx/length(trs)) #determine arrow size by width/#events
  if(!any(c("fontsize", "cex") %in% names(track@style@ylabgp))){
    if(doLabels){
      track@style@ylabgp$cex <- optFontSize1(.2*y)
    }else{
      track@style@ylabgp$cex <- arr_size
    }
  }
  for(i in seq.int(totalLines)){
    vp <- viewport(y = currLineBottom - eachLineHeight/2, 
                   height = eachLineHeight, xscale = xscale,
                   clip = 'on')
    pushViewport(vp)
    if(doLabels){
      gene_y <- .75
      gene_h <- .5
    }else{
      gene_y <- .5
      gene_h <- 1
    }
    if(length(track@dat2)>0){
      ## add a baseline for lollipop plot
      grid.lines(x=xscale, 
                 y=c(gene_y, gene_y), 
                 gp = gpar(col=track@style@color),
                 default.units = "native")
    }
    
    trs.sub <- trs[which(lineId$line==i)]
    rgs.sub <- rgs[which(lineId$line==i)]
    str.sub <- strand[which(lineId$line==i)]
    oid <- order(start(rgs.sub))
    trs.sub <- trs.sub[oid]
    rgs.sub <- rgs.sub[oid]
    str.sub <- str.sub[oid]
    stringStopPos <- c(0, 0)
    for(j in seq_along(trs.sub)){
      curr_trs <- trs.sub[[j]]
      curr_rg <- rgs.sub[j]
      curr_str <- str.sub[j]
      str_neg <- curr_str=="-"
      this_color <- if(is.list(curr_trs$color)) curr_trs$color[[1]] else if(length(curr_trs$color)) curr_trs$color else col
      this_height <- 
        if(is.list(curr_trs$height)) curr_trs$height[[1]] else if(length(curr_trs$height)) curr_trs$height else gene_h/ifelse(tolower(curr_trs$feature) %in% c('gene', 'transcripts', "cds", "exon"), 2, 4)
      hide_this_label <- FALSE
      if(length(curr_trs$hide_label)>0){
        hide_this_label <- unlist(curr_trs$hide_label)[1]
        if(!is.logical(hide_this_label)){
          hide_this_label <- FALSE
        }
      }
      must_have_label <- FALSE
      if(length(curr_trs$must_have_label)>0){
        must_have_label <- unlist(curr_trs$must_have_label)[1]
        if(!is.logical(must_have_label)){
          must_have_label <- FALSE
        }
      }
      plot_arrow <- TRUE
      curr_width <- unit(width(curr_trs)/(diff(xscale)+1), units = 'npc')
      if(length(curr_trs)==1 &&
         all(tolower(curr_trs$feature) %in% c('gene', 'transcripts'))){
        ## one element per gene/transcripts
        ## no center line
        if(convertUnit(curr_width, unitTo = 'points', valueOnly = TRUE)<6){
          plot_arrow <- TRUE
        }else{
          plot_arrow <- FALSE
        }
      }
      ## plot center line
      grid.lines(x=c(start(curr_rg), end(curr_rg)), 
                 y=c(gene_y, gene_y), 
                 gp = gpar(col=col),
                 default.units = "native")
      if(plot_arrow){
        ## plot exons by the feature type
        grid.rect(x=(start(curr_trs)+end(curr_trs))/2, y=gene_y, 
                  width =width(curr_trs), 
                  height=this_height, 
                  gp=gpar(col=NA, fill=this_color), 
                  default.units = "native")
      }else{
        ## plot exons by the feature type
        curr_width <- width(curr_trs)/(diff(xscale)+1)
        pt1 <- convertWidth(unit(6, units='points'),
                                    'npc', valueOnly = TRUE)*
          (diff(xscale)+1)
        curr_x1 <- ifelse(!str_neg, start(curr_trs), end(curr_trs))
        curr_x2 <- ifelse(!str_neg, end(curr_trs) - pt1,
                          start(curr_trs) + pt1)
        curr_x3 <- ifelse(!str_neg, end(curr_trs), start(curr_trs))
        grid.polygon(x=c(curr_x1, curr_x1,
                         curr_x2, curr_x3,
                         curr_x2, curr_x1), 
                     y=c(gene_y-this_height/2, gene_y+this_height/2,
                         gene_y+this_height/2, gene_y,
                         gene_y-this_height/2, gene_y-this_height/2),
                     gp=gpar(col=NA, fill=this_color), 
                     default.units = "native")
      }
      
      ## plot direction at TSS
      stringW <- convertWidth(stringWidth(names(curr_rg)), unitTo = "native", valueOnly = TRUE)
      pushViewport(viewport(x=ifelse(!str_neg, start(curr_rg), end(curr_rg)),
                            y=gene_y, 
                            width = unit(max(min(gene_h/2, 2*width(curr_rg)/abs(diff(xscale)+1)),
                                             convertWidth(unit(8, "lines"),
                                                          unitTo = "npc",
                                                          valueOnly = TRUE)), "snpc"), 
                            height = unit(gene_h/2, "snpc"),
                            just = c(ifelse(!str_neg, 0, 1), 1),
                            default.units = "native",
                            clip = 'off'))
      if(!hide_this_label){
        if(!str_neg){## positive strand
          if(plot_arrow){
            grid.lines(x=unit(c(0, 0, 1), "npc"),
                       y=unit(c(.5, 0, 0), "npc"),
                       arrow = arrow(type="closed", angle = 15, length = unit(arr_size, "lines")),
                       gp=gpar(col=this_color, fill=this_color))
          }
          ## add gene name at TSS
          if(doLabels || must_have_label){
            if(stringStopPos[1] <= stringStopPos[2]){##lower channel
              grid.text(label = names(curr_rg), x = 0, y = 0, hjust = pos_hjust, vjust=pos_vjust,
                        gp = do.call(gpar, track@style@ylabgp))
              stringStopPos[1] <- start(curr_rg)+stringW
            }else{##higher channel
              grid.text(label = names(curr_rg), x = 0, y = 1, hjust = pos_hjust, vjust=neg_vjust,
                        gp = do.call(gpar, track@style@ylabgp))
              stringStopPos[2] <- start(curr_rg)+stringW
            }
          }else{#dynamic label
            if(i %in% c(1, totalLines)){
              if(start(curr_rg) >= stringStopPos[1] &&
                 end(curr_rg) >= stringStopPos[1] + stringW){
                if(i==1){#top
                  grid.text(label = names(curr_rg), x = 0, y = 1,
                            hjust = pos_hjust, vjust=neg_vjust,
                            gp = do.call(gpar, track@style@ylabgp))
                }else{
                  grid.text(label = names(curr_rg), x = 0, y = 0,
                            hjust = pos_hjust, vjust=pos_vjust,
                            gp = do.call(gpar, track@style@ylabgp))
                }
              }
              stringStopPos[1] <- end(curr_rg)
            }
          }
        }else{## negative strand
          if(plot_arrow){
            grid.lines(x=unit(c(1, 1, 0), "npc"),
                       y=unit(c(.5, 0, 0), "npc"),
                       arrow = arrow(type="closed", angle = 15, length = unit(arr_size, "lines")),
                       gp=gpar(col=this_color, fill=this_color))
          }
          ## add gene name at TSS
          if(doLabels || must_have_label){
            if(stringStopPos[1]<=stringStopPos[2]){##lower channel
              grid.text(label = names(curr_rg), x = 1, y = 0, hjust = neg_hjust, vjust=pos_vjust,
                        gp = do.call(gpar, track@style@ylabgp))
              stringStopPos[1] <- end(curr_rg)
            }else{##higher channel
              grid.text(label = names(curr_rg), x = 1, y = 1, hjust = neg_hjust, vjust=neg_vjust,
                        gp = do.call(gpar, track@style@ylabgp))
              stringStopPos[2] <- end(curr_rg)
            }
          }else{#dynamic label
            if(i %in% c(1, totalLines)){
              if(start(curr_rg) >= stringStopPos[1] &&
                 end(curr_rg) >= stringStopPos[1] + stringW){
                if(i==1){#bottom
                  grid.text(label = names(curr_rg), x = 1, y = 1,
                            hjust = neg_hjust, vjust=neg_vjust,
                            gp = do.call(gpar, track@style@ylabgp))
                }else{
                  grid.text(label = names(curr_rg), x = 1, y = 0,
                            hjust = neg_hjust, vjust=pos_vjust,
                            gp = do.call(gpar, track@style@ylabgp))
                }
              }
              stringStopPos[1] <- end(curr_rg)
            }
          }
        }
      }
      popViewport()
      
    }
    popViewport()
    currLineBottom <- currLineBottom - eachLineHeight
  }
  popViewport()
  
  if(length(track@dat2)>0){
    track@dat2 <- orderedGR(track@dat2)
    xscale.gr <- GRanges(seqnames = chr, ranges=IRanges(min(xscale), max(xscale)))
    track@dat2 <- subsetByOverlaps(track@dat2, xscale.gr, ignore.strand=TRUE)
    if(length(track@dat2)<1) return(invisible())
    width(track@dat2) <- 1
    TYPES <- c("circle", "pie", "pin", "pie.stack", "flag")
    type <- if(is.list(track@dat2$type)) track@dat2$type[[1]] else track@dat2$type[1]
    if(length(type)==0) type <- "circle"
    if(!type %in% TYPES) type <- "circle"
    if(type=="pin"){ ## read the pin shape file
      pinpath <- system.file("extdata", "map-pin-red.xml", package="trackViewer")
      pin <- readPicture(pinpath)
    }else{
      pin <- NULL
    }
    cex <- if(is.list(track@dat2$cex)) track@dat2$cex[[1]] else track@dat2$cex[1]
    if(length(cex)==0) cex <- 1
    dashline.col <- if(is.list(track@dat2$dashline.col)) track@dat2$dashline.col[[1]] else track@dat2$dashline.col[1]
    if(length(dashline.col)==0) dashline.col <- "gray80"
    jitter <- if(is.list(track@dat2$jitter)) track@dat2$jitter[[1]] else track@dat2$jitter[1]
    if(length(jitter)==0) jitter <- "node"
    if(!jitter %in% c("node", "label")) jitter <- "node"
    scoreMax0 <- scoreMax <- 
      if(length(track@dat2$score)>0) ceiling(max(c(track@dat2$score, 1), na.rm=TRUE)) else 1
    if(type=="pie.stack") scoreMax <- length(unique(track@dat2$stack.factor))
    if(!type %in% c("pie", "pie.stack")){
      if(scoreMax>lollipop_style_switch_limit) {
        track@dat2$score <- 10*track@dat2$score/scoreMax
        scoreMax <- 10*scoreMax0/scoreMax
      }else{
        scoreMax <- scoreMax0
      }
      scoreType <- 
        if(length(track@dat2$score)>0) all(floor(track@dat2$score)==track@dat2$score) else FALSE
    }else{
      scoreType <- FALSE
    }
    LINEW <- as.numeric(convertX(unit(1, "line"), "npc"))/2
    LINEH <- as.numeric(convertY(unit(1, "line"), "npc"))/2
    ## GAP the gaps between any elements
    GAP <- .2 * LINEH
    ratio.yx <- getYXratio()
    plotLollipops(track@dat2, feature.height=y+unit, bottomHeight=0, baseline=y, 
                  type=type, ranges=xscale.gr, yaxis=FALSE, yaxis.gp=yaxis.gp,
                  scoreMax=scoreMax, scoreMax0=scoreMax0, scoreType=scoreType, 
                  LINEW, cex, ratio.yx, GAP, pin, dashline.col,
                  side="top", jitter=jitter)
  }
  return(invisible())
}

optFontSize <- function(axis, viewerStyle, height){
  stopifnot(length(axis)==1)
    if(axis=="x"){
        if(viewerStyle@xlas %in% c(0, 1)){
            fontHeight <- convertHeight(stringHeight("0123456789"), 
                                        unitTo="npc", valueOnly=TRUE)
            cex <- viewerStyle@margin[1]/6/fontHeight
        }else{
            fontWidth <- convertWidth(stringWidth("012345678"), 
                                      unitTo="npc", valueOnly=TRUE)
            cex <- viewerStyle@margin[1]/6/fontWidth
        }
    }else{
        if(axis=="y"){
            fontWidth <- convertWidth(stringWidth("0123"), 
                                      unitTo="npc", valueOnly=TRUE)
            cex <- viewerStyle@margin[2]/1.5/fontWidth
        }else{
            fontHeight <- convertHeight(stringHeight("0123456789"), 
                                        unitTo="npc", valueOnly=TRUE)
            s <- min(c(height, viewerStyle@margin[2]))
            cex <- s/fontHeight
        }
    }
    
    if(cex > 1) cex <- 1
    if(cex < getMinFont()) cex <- getMinFont()
    return(cex)
}
getMinFont <- function(){
  convertHeight(stringHeight("0123456789"), 
                unitTo="picas", valueOnly=TRUE)*5/12 # 5pt
}
optFontSize1 <- function(height){
    fontHeight <- convertHeight(stringHeight("0123456789"), 
                                unitTo="npc", valueOnly=TRUE)
    cex <- height/3/fontHeight
    if(cex>2) cex <- log2(cex)
    if(cex < getMinFont()) cex <- getMinFont()
    cex
}

#' Optimize the style of plot
#' @description Automatic optimize the stlye of trackViewer
#' @param trackList An object of \code{\link{trackList}}
#' @param viewerStyle An object of \code{\link{trackViewerStyle}}
#' @param theme A character string. Could be "bw", "col" or "safe".
#' @return a list of a \code{\link{trackList}} and a \code{\link{trackViewerStyle}}
#' @seealso See Also as \code{\link{viewTracks}}
#' @export
#' @examples 
#' extdata <- system.file("extdata", package="trackViewer",
#'                        mustWork=TRUE)
#' files <- dir(extdata, ".wig")
#' tracks <- lapply(paste(extdata, files, sep="/"), 
#'                  importScore, format="WIG")
#' re <- optimizeStyle(trackList(tracks))
#' trackList <- re$tracks
#' viewerStyle <- re$style

optimizeStyle <- function(trackList, viewerStyle=trackViewerStyle(), theme=NULL){
    if(missing(trackList))
        stop("trackList must be an object of 'trackList'")
    if(!is(trackList, "trackList") && 
       !(is.list(trackList) && 
         all(vapply(trackList, function(.ele) is(.ele, "track"), 
                    FUN.VALUE = TRUE)))){
        stop("trackList must be an object of \"trackList\"
             (See ?trackList) or a list of track")
    }
    if(trackList[[length(trackList)]]@type %in% c("data", "interactionData") && 
       viewerStyle@margin[3] < .02)
        viewerStyle@margin[3] <- .02
    ##put x-axis?
    dataTracksIdx <- sapply(trackList, function(.ele) .ele@type=="data")
    dataTracksXscale <- sapply(trackList[dataTracksIdx], 
                               function(.ele) .ele@style@xscale@draw)
    if(sum(dataTracksIdx)<5 && !(any(dataTracksXscale))){
        ##put x-axis
        viewerStyle@xaxis <- TRUE
        ##increase space for x-axis
        if(viewerStyle@margin[1] < .05) viewerStyle@margin[1] <- .05
        ##adjust x fontsize
        viewerStyle@xgp <- c(viewerStyle@xgp, 
                             cex=optFontSize("x", viewerStyle))
    }
    ##put x-scale?
    if(!viewerStyle@xaxis){
        if(all(!(sapply(trackList, function(.ele) .ele@style@xscale@draw)))){
            for(i in 1:length(trackList)){
                if(trackList[[i]]@type=="data"){
                    trackList[[i]]@style@xscale@draw <- TRUE
                    break
                }
            }
        }
    }
    ##put y-axis?
    if(all(!(sapply(trackList, function(.ele) .ele@style@yaxis@label)))){
        for(i in 1:length(trackList)){
            if(trackList[[i]]@type %in% c("data", "interactionData")){
                trackList[[i]]@style@yaxis@label <- TRUE
                trackList[[i]]@style@yaxis@gp <- 
                    c(trackList[[i]]@style@yaxis@gp, 
                      cex=optFontSize("y", viewerStyle))
                trackList[[i]]@style@marginTop <- .1
            }
          if(trackList[[i]]@type %in% c("lollipopData")){
            trackList[[i]]@style@yaxis@draw <- FALSE
          }
          if(trackList[[i]]@type %in% c("interactionData")){
            trackList[[i]]@style@yaxis@main <- FALSE
            viewerStyle@margin[4] <- .05
          }
        }
    }
    ##about ylab, direction, fontsize
    viewerStyle@autolas <- TRUE
    if(!is.null(theme)){
      defaultGeneYlabPos <- "upstream"
      colors <- switch(theme,
                       "bw"  = rep("black",length(trackList)+1),
                       "col" = rep(palette(), length(trackList)),
                       "safe"= rep(c("#000000", "#D55E00", "#009E73",
                                     "#0072B2", "#56B4E9", "#CC79A7",
                                     "#E69F00", "#BEBEBE"),
                                   length(trackList)),
                       rep("black",length(trackList)+1))
      for(i in seq_along(trackList)){
        if(trackList[[i]]@type %in% c("data", "lollipopData", "interactionData")){
          trackList[[i]]@style@ylabpos="bottomleft"
          trackList[[i]]@style@marginBottom=.2
          trackList[[i]]@style@ylabgp=
            list(cex=optFontSize1(.4*trackList[[i]]@style@height), 
                 col=ifelse(theme=="col", colors[i+1], "black"))
        }else{
          trackList[[i]]@style@ylabpos=defaultGeneYlabPos
          if(trackList[[i]]@type=="gene"){
            trackList[[i]]@style@ylabgp=
              list(cex=optFontSize1(.4*trackList[[i]]@style@height), 
                   col=colors[i+1])
          }else{
            trackList[[i]]@style@ylabgp=
              list(cex=optFontSize1(trackList[[i]]@style@height), 
                   col=colors[i+1])
          }
        }
        trackList[[i]]@style@color=
          rep(colors[i+1], 2)
        trackList[[i]]@style@yaxis@main=FALSE
      }
      if(viewerStyle@margin[4] < .05) viewerStyle@margin[4] <- .05
    }
    return(list(tracks=trackList, style=viewerStyle))
    }

plotInteractionDataTrack <- function(.dat, .dat2, scale, color, yscale, breaks,
                                     NAcolor="white",
                                     style="heatmap", ysplit=.5){
  if(length(style)==1){
    style <- rep(style, 2)
  }
  ysplit <- ysplit[1]
  names(.dat) <- NULL
  ## if there is target metadata
  .dat1target <- NULL
  .dat2target <- NULL
  if(length(.dat$target)==length(.dat) && is(.dat$target, "GRanges")){
    .dat1target <- .dat$target
    if(length(.dat2)!=0){
      if(length(.dat2$target)==length(.dat2) && is(.dat2$target, "GRanges")){
        names(.dat2) <- NULL
        .dat2target <- .dat2$target
        #mcols(.dat2) <- mcols(.dat2)[, "score"]
        #colnames(mcols(.dat2)) <- "score"
      }
    }
  }else{
    .dat1target <- .dat2
  }
  #mcols(.dat) <- mcols(.dat)[, "score"]
  #colnames(mcols(.dat)) <- "score"
  if(missing(yscale)) yscale <- c(0, 1)
  if(length(.dat)<1){
    return()
  }
  ## plot rect at position
  ## x = (center1 + center2)/2
  ## y = unit((x-scale[1]+1)/((scale[2] - scale[1] + 1)/2), "npc")
  ## width = width(x1)
  ## height = uint(width(x2)/(scale[2] - scale[1] + 1), "npc")
  ## rot = 45 degree
  ## color = colorRampPalette(color)(100)
  if(length(breaks)<3){
    if(length(.dat2target)>0){
      if(style[2]==style[1]){
        rg <- list(range(c(.dat$score[!is.na(.dat$score)], 
                           .dat2$score[!is.na(.dat2$score)])))
      }else{
        rg <- list(range(.dat$score[!is.na(.dat$score)]),
                   range(.dat2$score[!is.na(.dat2$score)]))
      }
    }else{
      rg <- list(range(.dat$score[!is.na(.dat$score)]))
    }
    breaks <- lapply(rg, function(.rg){
      if(length(.rg)!=2){
        return()
      }
      .rg <- .rg + c(-1, 1)*diff(.rg)/1000
      if(.rg[1]==.rg[2] && .rg[1]==0){
        .rg[2] <- 1
      }
      seq(min(0, .rg[1]), .rg[2], length.out = 101)
    })
    userdefinedbreaks <- FALSE
  }else{
    breaks <- list(breaks, breaks)
    userdefinedbreaks <- TRUE
  }
  
  if(length(unique(color))==1){
    if(!tolower(color[1]) %in% c("white", "#ffffff", "#fff")){
      color <- c("white", color[1])
    }else{
      color <- c("black", color[1])
    }
  }
  if(length(color)==0){
    color <- c("white", "red")
  }
  crp <- lapply(breaks, function(.breaks){
    colorRampPalette(color)(length(.breaks)-1)
  })
  ym <- (scale[2]-scale[1] + 1)/2
  getMC <- function(scores, breaks, crp){
    mc <- cut(scores, breaks = breaks, labels = crp)
    mc <- as.character(mc)
    mc[is.na(mc)] <- NAcolor
    mc
  }
  plotInteractionHeatmap <- function(anchor1, anchor2, breaks, crp, f){
    xa <- (end(anchor1) + start(anchor2))/2
    xb <- (start(anchor1) + start(anchor2))/2
    xc <- (start(anchor1) + end(anchor2))/2
    xd <- (end(anchor1) + end(anchor2))/2
    ya <- (xa-end(anchor1)+1)/ym
    yb <- (xb-start(anchor1)+1)/ym
    yc <- (xc-start(anchor1)+1)/ym
    yd <- (xd-end(anchor1)+1)/ym
    irx <- inRange(xa, scale) | inRange(xb, scale) | inRange(xc, scale) | inRange(xd, scale)
    iry <- inRange(ya, yscale) | inRange(yb, yscale) | inRange(yc, yscale) | inRange(yd, yscale)
    xinr <- inRange(start(anchor1), scale) | inRange(end(anchor2), scale)
    mc <- getMC(anchor1$score, breaks, crp)
    cols <- rep(NA, length(anchor1))
    if(length(anchor1$border_color)==length(anchor1) && length(anchor1)>0){
      cols <- anchor1$border_color
    }
    tads <- rep(NA, length(anchor1))
    if(length(anchor1$tads)==length(tads) && length(anchor1)>0){
      tads <- anchor1$tads
    }
    polygon_data_x <- rbind(xa, xb, xc, xd)
    polygon_data_y <- rbind(ya, yb, yc, yd)
    keep <- irx & iry
    grid.polygon(x=polygon_data_x[, keep, drop=FALSE],
                 y=polygon_data_y[, keep, drop=FALSE],
                 default.units="native",
                 id.lengths = rep(4, sum(keep)),
                 gp = gpar(fill=mc[keep], col = cols[keep]))
    for(i in seq_along(anchor1)){
      if(xinr[i]){
        if(!is.na(tads[i])){
          tad_gp <- NA
          if(is.logical(tads[i])){
            if(tads[i]){
              tad_gp <- gpar(col = '#E69F00')
            }
          }else{
            if(is.list(tads)){
              tad_gp <- do.call(gpar, tads[[i]])
            }else{
              if(is_color(tads[i])){
                tad_gp <- gpar(col = tads[i])
              }
            }
          }
          if(!is.na(tad_gp)[1]){
            grid.lines(x=c(start(anchor1)[i], xc[i], end(anchor2)[i]),
                       y=c(0, yc[i], 0),
                       default.units = "native",
                       gp = tad_gp)
          }
        }
      }
    }
  }
  
  getBezierPoints <- function(x1, x2, y1, y2, h){
    convX <- function(x){
      (x-scale[1])/diff(scale)
    }
    reveX <- function(x){
      x*diff(scale)+scale[1]
    }
    convY <- function(y){
      h*(y-min(y, na.rm=TRUE))/diff(range(y, na.rm=TRUE))
    }
    x1 <- convX(x1)
    x2 <- convX(x2)
    y1 <- convX(y1)
    y2 <- convX(y2)
    bg1 <- bezierGrob(c(x2, x2, y1, y1),
                      c(0, h, h, 0),
                      default.units = "npc")
    trace1 <- bezierPoints(bg1)
    bg2 <- bezierGrob(c(x1, x1, y2, y2),
                      c(0, h, h, 0),
                      default.units = "npc")
    trace2 <- bezierPoints(bg2)
    return(list(x = reveX(c(convertX(trace2$x[1], unitTo = "npc", valueOnly = TRUE),
                            convertX(trace1$x, unitTo = "npc", valueOnly = TRUE),
                            convertX(rev(trace2$x), unitTo = "npc", valueOnly = TRUE))),
                y = convY(c(trace2$y[1], trace1$y, rev(trace2$y)))))
  }
  grid.link <- function(x1, x2, y1, y2, h, ...){
    pts <- getBezierPoints(x1, x2, y1, y2, h)
    grid.polygon(x=pts$x,
                 y=pts$y,
                 ...)
  }
  plotInteractioLink <- function(anchor1, anchor2, breaks, crp, f=1){
    mc <- getMC(anchor1$score, breaks, crp)
    anchor1$score[anchor1$score<0] <- 0
    hs <- sqrt(anchor1$score/
                max(anchor1$score[!is.infinite(anchor1$score)], 
                    na.rm = TRUE)) * f
    hs[is.na(hs)] <- 0
    irx <- inRange(start(anchor1), scale) | inRange(end(anchor1), scale) |
      inRange(start(anchor2), scale) | inRange(end(anchor2), scale)
    for(i in seq_along(anchor1)){
      if(irx[i]){
        grid.link(x1=start(anchor1)[i],
                  x2=end(anchor1)[i],
                  y1=start(anchor2)[i],
                  y2=end(anchor2)[i],
                  h = hs[i],
                  default.units="native",
                  gp = gpar(fill=mc[i], col = NA, alpha=hs[i]))
      }
    }
  }
  FUN = switch (tolower(style[1]),
    'heatmap' = plotInteractionHeatmap,
    'link' = plotInteractioLink,
    plotInteractionHeatmap
  )
  if(length(.dat2target)){## two interaction heatmap, back to back
    ## top triangle
    pushViewport(viewport(x=0, y=1-ysplit, 
                          height=ysplit, 
                          width=1, 
                          clip="on",
                          default.units = "npc",
                          just=c(0,0), 
                          xscale=scale, 
                          yscale=yscale))
    FUN(.dat, .dat1target, breaks[[1]], crp[[1]], f=.5)
    popViewport()
    FUN = switch (tolower(style[2]),
                  'heatmap' = plotInteractionHeatmap,
                  'link' = plotInteractioLink,
                  plotInteractionHeatmap
    )
    ## bottom triangle
    pushViewport(viewport(x=0, y=0, 
                          height=1-ysplit, 
                          width=1, 
                          clip="on", 
                          default.units = "npc",
                          just=c(0,0), 
                          xscale=scale, 
                          yscale=rev(yscale)))
    FUN(.dat2, .dat2target, breaks[[2]], crp[[2]], f=.5)
    popViewport()
  }else{
    FUN(.dat, .dat1target, breaks[[1]], crp[[1]])
  }
  #if(style[2]==style[1]){ # now we can not set two different breaks and colors
    breaks <- breaks[1]
    crp <- crp[1]
  #}
  # legend in y axis
  return(list(crp=crp, breaks=breaks, ysplit=ysplit, userdefinedbreaks=userdefinedbreaks))
  vp <- viewport(x = 1 - convertWidth(unit(5, "char"), "npc", valueOnly = TRUE), 
                 y= 1 - convertHeight(unit(1, "char"), "npc", valueOnly = TRUE), 
                 width = convertWidth(unit(5, "char"), "npc", valueOnly = TRUE),
                 height = convertHeight(unit(.5, "char"), "npc", valueOnly = TRUE),
                 default.units = "npc", xscale = range(breaks))
  pushViewport(vp)
  grid.raster(t(crp), width=1, height=1)
  grid.xaxis(at=round(range(breaks)), gp=gpar(fontsize=8))
  popViewport()
}

GRbin <- function(A, B, col, ignore.strand){
    mcols(A) <- DataFrame(source="A", id=seq_along(A), value=mcols(A)[, col])
    mcols(B) <- DataFrame(source="B", id=seq_along(B), value=mcols(B)[, col])
    D <- c(A, B)
    C <- disjoin(D, with.revmap=TRUE, 
                 ignore.strand=ignore.strand)
    revmap <- data.frame(oid=unlist(C$revmap), 
                         nid=rep(seq_along(C), lengths(C$revmap)))
    revmap$source <- D$source[revmap$oid]
    C$revmap <- NULL
    setMeta <- function(group="A"){
        X <- C
        mcols(X) <- DataFrame(source="", id=0, value=0)
        id <- revmap[revmap$source==group, ]
        mcols(X)[id$nid, ] <- mcols(D)[id$oid, ]
        X$id[X$id==0] <- NA
        X
    }
    A1 <- setMeta("A")
    B1 <- setMeta("B")
    GRangesList(A=A1, B=B1)
}

#' GRanges operator
#' @description GRanges operations (add, aubtract, multiply, divide)
#' @param A an object of GRanges
#' @param B an object of GRanges
#' @param col colname of A and B to be calculated
#' @param operator operator, "+" means A + B, and so on. 
#' User-defined function also could be used.
#' @param ignore.strand When set to TRUE, the strand information is 
#' ignored in the overlap calculations.
#' @return an object of GRanges
#' @import GenomicRanges
#' @export
#' @examples 
#' gr2 <- GRanges(seqnames=c("chr1", "chr1"),
#' ranges=IRanges(c(7,13), width=3),
#' strand=c("-", "-"), score=3:4)
#' gr3 <- GRanges(seqnames=c("chr1", "chr1"),
#'                ranges=IRanges(c(1, 4), c(3, 9)),
#'                strand=c("-", "-"), score=c(6L, 2L))
#' GRoperator(gr2, gr3, col="score", operator="+")
#' GRoperator(gr2, gr3, col="score", operator="-")
#' GRoperator(gr2, gr3, col="score", operator="*")
#' GRoperator(gr2, gr3, col="score", operator="/")
#' GRoperator(gr2, gr3, col="score", operator=mean)

GRoperator <- function(A, B, col="score", 
                       operator=c("+", "-", "*", "/", "^", "%%"),
                       ignore.strand=TRUE){
    if(!is(A, "GRanges") || !is(B, "GRanges")){
        stop("A and B must be objects of GRanges")
    }
    if(any(countOverlaps(A)>1) || any(countOverlaps(B)>1)){
        stop("A or B has overlaps.")
    }
    if(length(A)<1) return(B)
    if(length(B)<1) return(A)
    argA <- deparse(substitute(A))
    argB <- deparse(substitute(B))
    if(!is.function(operator)){
        operator <- match.arg(operator)
        operator <- .Primitive(operator)
    }
    
    if(!(col %in% colnames(mcols(A)))){
        stop("col is not in metadata of A")
    }
    if(!(col %in% colnames(mcols(B)))){
        stop("col is not in metadata of B")
    }
    C <- GRbin(A, B, col, ignore.strand=ignore.strand)
    A <- C$A
    B <- C$B
    out <- A
    mcols(out) <- NULL
    out$value <- tryCatch(operator(A$value, B$value),
                          error=function(e) mapply(operator, A$value, B$value))
    out$A <- A$source=="A"
    out$A_id <- A$id
    out$B <- B$source=="B"
    out$B_id <- B$id
    colnames(mcols(out)) <- c(col, 
                              argA, paste0(argA, "_id"),
                              argB, paste0(argB, "_id"))
    out
}

#' List the available resolutions
#' @description List the resolutions available in the file.
#' @param file character(1). File name of .hic or .cool/.mcool/.scool
#' @param format character(1). File format, "hic" or "cool".
#' @importFrom rhdf5 H5Fopen h5ls H5close
#' @importFrom strawr readHicBpResolutions
#' @export
#' @examples
#' hicfile <- system.file("extdata", "test_chr22.hic", package="trackViewer")
#' listResolutions(hicfile)
#' coolfile <- system.file("extdata", "test.mcool", package="trackViewer")
#' listResolutions(coolfile, format="cool")
listResolutions <- function(file, format=c("hic", "cool")){
  format <- match.arg(format)
  guessFormat(file, format)
  if(format=="hic"){
    readHicBpResolutions(fname=file)
  }else{
    if(isMcool(file)){
      coolfile <- checkCoolFile(file)
      coolfileRootName <- coolfileRootName(coolfile)
      h5ls(coolfile&coolfileRootName, recursive = FALSE)$name
    }else{
      return(NULL)
    }
  }
}

#' Add guide lines to the tracks
#' @description A function to add lines for emphasizing the positions 
#' @param guideLine The genomic coordinates to draw the lines
#' @param col A vector for the line color
#' @param lty A vector for the line type
#' @param lwd A vector for the line width
#' @param vp A Grid viewport object. It must be output of \code{\link{viewTracks}}
#' @import grid
#' @export
#' @return NULL
#' @seealso See Also as \code{\link{getCurTrackViewport}}, \code{\link{addArrowMark}}, 
#' \code{\link{viewTracks}}
#' @examples
#' vp <- getCurTrackViewport(trackViewerStyle(), 10000, 10200)
#' addGuideLine(c(10010, 10025, 10150), vp=vp)

addGuideLine <- function(guideLine, col="gray", lty="dashed", lwd=1, vp=NULL){
  if(missing(guideLine) | 
     !(inherits(guideLine, c("numeric", "integer"))) |
     length(guideLine) < 1)
    stop("guideLine is required as a numeric vector of coordinates of genome")
  len <- length(guideLine)
  trimLen <- function(obj, len){
    if(length(obj)<len) 
      obj <- rep(obj, len)[1:len]
    obj
  }
  vpmultiple <- FALSE
  if(length(vp)>0){
    stopifnot(is(vp, "viewport"))
    if(is(vp, "vpTree")){
      ## check the range
      vpmultiple <- TRUE
    }
  }
  selectVP <- function(x, tree){
    xscales <- sapply(tree$children, function(.ele){
      seekViewport(names(.ele$children))
      current.viewport()$xscale
    }, simplify = FALSE)
    xscales <- do.call(cbind, xscales)
    i <- which(x>=xscales[1, ] & x<=xscales[2, ])
    if(length(i)<1) {
      message(x, " out of the range.")
      return(NULL)
    }
    seekViewport(paste0("panel.", i))
    current.viewport()
  }
  col <- trimLen(col, len)
  lty <- trimLen(lty, len)
  lwd <- trimLen(lwd, len)
  for(i in seq_along(guideLine)){
    if(vpmultiple){
      currentVP <- selectVP(guideLine[i], vp)
      grid.lines(x=guideLine[i], y=c(0, 1), 
                 gp=gpar(col=col[i], lty=lty[i], lwd=lwd[i]), 
                 default.units="native")
    }else{
      currentVP <- vp
      grid.lines(x=guideLine[i], y=c(0, 1), 
                 gp=gpar(col=col[i], lty=lty[i], lwd=lwd[i]), 
                 default.units="native", vp=currentVP)
    }
  }
  return(invisible())
}

#' get gene ids by genomic location
#' @description retrieve gene ids from txdb object by genomic location.
#' @param gr GRanges object.
#' @param txdb An object of \code{\link[GenomicFeatures:TxDb-class]{TxDb}}. 
#' @return A character vector of gene ids
#' @importFrom AnnotationDbi select
#' @importFrom GenomicFeatures genes
#' @import GenomicRanges
#' @import IRanges
#' @export
#' @examples 
#' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
#' gr <- parse2GRanges("chr11:122,830,799-123,116,707")
#' ids <- getGeneIDsFromTxDb(gr, TxDb.Hsapiens.UCSC.hg19.knownGene)

getGeneIDsFromTxDb <- function(gr, txdb){
  stopifnot(is(gr, "GRanges"))
  stopifnot(length(gr)>0)
  stopifnot(is(txdb, "TxDb"))
  if(length(gr)>1){
    warning("The length of gr is greater than 1. Only first genomic location will be used.")
    gr <- gr[1]
  }
  genes <- genes(txdb, columns="gene_id")
  genes <- subsetByOverlaps(genes, gr)
  return(genes$gene_id)
}

#' plot ideogram with data for one chromosome
#' @description plot ideogram with data for one chromosome 
#' @param ideo output of \link{loadIdeogram}.
#' @param dataList a \link[GenomicRanges:GRangesList-class]{GRangesList} of data to plot.
#' @param parameterList a list of parameters for each dataset in the dataList. 
#' The elements of the parameters could be xlabs, ylabs, etc. type could be
#' barplot, line, point, heatmap.
#' @param chrom A length 1 character vector of chromosome name.
#' @param colorSheme A character vector of giemsa stain colors.
#' @param gp parameters used for \link[grid]{grid.roundrect}.
#' @param ... parameters not used.
#' @import grid
#' @examples 
#' \dontrun{
#' ideo <- loadIdeogram("hg38")
#' library(rtracklayer)
#' library(grid)
#' dataList <- ideo[seqnames(ideo) %in% "chr1"]
#' dataList$score <- as.numeric(dataList$gieStain)
#' dataList <- dataList[dataList$gieStain!="gneg"]
#' dataList <- GRangesList(dataList, dataList)
#' grid.newpage()
#' plotOneIdeo(ideo, dataList, chrom="chr1")
#' }
#' 
#' 
plotOneIdeo <- 
  function(ideo, dataList, 
           parameterList=list(vp=plotViewport(margins=c(.1, 4.1, 1.1, .1)),
                              ideoHeight=unit(1/(1+length(dataList)), "npc"), 
                              vgap=unit(1, "lines"),
                              ylabs=seqlevels(ideo)[1], ylabsRot=90, 
                              ylabsPos=unit(2.5, "lines"),
                              xaxis=FALSE, yaxis=FALSE,
                              xlab="", 
                              types="barplot", heights=NULL, 
                              dataColumn="score", 
                              gps=gpar(col="black", fill="gray")), 
           chrom=seqlevels(ideo)[1], colorSheme=gieStain(), 
           gp=gpar(fill=NA, lwd=2), ...){
  stopifnot(is(dataList, "GRangesList"))
  stopifnot(is(parameterList, "list"))
  dataList <- lapply(dataList, function(.ele){
    .ele[seqnames(.ele) %in% chrom]
  })
  if(is(parameterList$xlab, "list")){
    do.call(grid.text, args = parameterList$xlab)
  }else{
    if(parameterList$xlab[1]!=""){
      grid.text(label=parameterList$xlab, y=unit(1.5, "lines"))
    }
  }
  ideo <- ideo[seqnames(ideo) %in% chrom]
  if(is(parameterList$vp, "viewport")){
    if(identical(parameterList$vp$xscale, c(0, 1))){
      parameterList$vp$xscale <- c(1, max(end(ideo)))
    }
    pushViewport(parameterList$vp)
    
    if(length(parameterList$xaxis)){
      if(is.logical(parameterList$xaxis)){
        if(parameterList$xaxis[1]) grid.xaxis()
      }else{
        do.call(grid.xaxis, args = parameterList$xaxis)
      }
    }
  }
  ## split the plot region into ideo layer and data layer
  ideoHeight <- convertY(parameterList$ideoHeight, "npc", valueOnly = TRUE)
  ideoVP <- viewport(y=ideoHeight/2,
                     height=ideoHeight, 
                     name=paste0("ideoLayer", chrom))
  pushViewport(ideoVP)
  plotIdeo(ideo, chrom, colorSheme=colorSheme, gp=gp)
  upViewport()
  dataVP <- viewport(y=.5 + ideoHeight/2, height=1-ideoHeight, 
                     name=paste0("dataLayer", chrom))
  pushViewport(dataVP)
  if(length(parameterList$heights)){
    if(length(parameterList$heights)!=length(dataList)){
      vpH <- rep(parameterList$heights, length(dataList))[1:length(dataList)]
    }else{
      vpH <- parameterList$heights
    }
  }else{
    vpH <- rep(1/length(dataList), length(dataList))
  }
  if(length(parameterList$types)){
    if(length(parameterList$types)!=length(dataList)){
      types <- rep(parameterList$types, length(dataList))[1:length(dataList)]
    }else{
      types <- parameterList$types
    }
  }else{
    types <- rep("barplot", length(dataList))
  }
  if(length(parameterList$dataColumn)){
    if(length(parameterList$dataColumn)!=length(dataList)){ 
      dataColumn <- rep(parameterList$dataColumn, 
                        length(dataList))
      if(!is.list(parameterList$dataColumn)){
        dataColumn <- as.list(dataColumn)
      }
      dataColumn <- dataColumn[1:length(dataList)]
    }else{
      if(!is.list(parameterList$dataColumn)){
        dataColumn <- as.list(parameterList$dataColumn)
      }else{
        dataColumn <- parameterList$dataColumn
      }
    }
  }else{
    dataColumn <- as.list(rep("score", length(dataList)))
  }
  if(length(parameterList$gps)){
    if(length(parameterList$gps)!=length(dataList)){
      gps <- rep(list(parameterList$gps), length(dataList))[1:length(dataList)]
    }else{
      if(is(parameterList$gps, "gpar")){
        gps <- rep(list(parameterList$gps), length(dataList))
      }else{
        gps <- parameterList$gps
      }
    }
  }else{
    gps <- rep(list(gpar()), length(dataList))
  }
  vgap <- convertY(parameterList$vgap, "npc", valueOnly = TRUE)
  for(i in seq_along(dataList)){
    data.gr <- dataList[[i]]
    mcols(data.gr) <- mcols(data.gr)[, dataColumn[[i]]]
    if(length(mcols(data.gr))>0){
        yrg <- range(as.numeric(as.matrix(mcols(data.gr))))
        if(!any(is.infinite(yrg))){
            this.vgap <- vgap
            if(vpH[i]-vgap<=0) this.vgap <- 0
            dataVPi <- viewport(y=sum(vpH[-(i:length(vpH))])+vpH[i]*.5+this.vgap, 
                                height = vpH[i]-this.vgap,
                                xscale=parameterList$vp$xscale, 
                                yscale=c(min(0, yrg[1]), max(yrg[2], 0)),
                                name=paste0("dataLayer", chrom, "sub", i))
            pushViewport(dataVPi)
            gridPlot(data.gr, gp=gps[[i]], types[i], parameterList$vp$xscale)
            upViewport()
        }
    }
  }
  upViewport()
  if(is(parameterList$vp, "viewport")){
    upViewport()
  }
  if(length(parameterList$ylabs)){
    ylabs <- parameterList$ylabs
    if(ylabs[1]!=""){
      autofitFontSize <- 0.3 * 
          convertY(unit(vpH[1], "npc"), "inch", valueOnly=TRUE)/
            convertY(unit(1, "lines"), "inch", valueOnly = TRUE)
      grid.text(label=ylabs[1], x=convertWidth(parameterList$ylabsPos, unitTo = "npc"), 
                rot = parameterList$ylabsRot, 
                gp = gpar(cex=autofitFontSize),
                vp=viewport(y=ideoHeight/2,
                            height=ideoHeight))
      if(length(ylabs)>1){
        pushViewport(viewport(y=.5 + ideoHeight/2, height=1-ideoHeight))
        for(i in 2:length(ylabs)){
          this.vgap <- vgap
          if(vpH[i-1]-vgap<=0) this.vgap <- 0
          grid.text(label=ylabs[i], x=convertWidth(parameterList$ylabsPos, unitTo = "npc"), 
                    y=unit(.5, "npc"), rot = parameterList$ylabsRot, 
                    gp=gpar(cex=autofitFontSize),
                    vp=viewport(y=sum(vpH[-((i-1):length(vpH))])+vpH[i-1]*.5, 
                                height = vpH[i-1]-this.vgap))
        }
        popViewport()
      }
    }
  }
}

#' plot GRanges metadata 
#' @description plot GRanges metadata for different types
#' @param gr an object of \link[GenomicRanges:GRanges-class]{GRanges} with metadata. All 
#' metadata must be numeric.
#' @param gp an object of \link[grid]{gpar}
#' @param type type of the figure, could be barplot, line, point and heatmap
#' @param xscale x scale of the viewport
#' @import grid
#' 

gridPlot <- function(gr, gp, type, xscale){
  if(length(gr)==0) return()
  x0 <- start(gr)
  x1 <- end(gr)
  x <- as.numeric(rbind(x0, x1))
  xc <- (x0+x1)/2
  y0 <- mcols(gr)
  if(ncol(y0)<1){
    return()
  }
  y <- as.matrix(y0[rep(1:nrow(y0), each=2),])
  yc <- as.numeric(y0[, 1])
  w <- width(gr)
  switch(type,
         barplot=grid.rect(x=xc, y=yc/2, 
                           width=w, height=yc,
                           gp=gp, default.units = "native"),
         line=grid.polyline(x=x, y=y, gp=gp, 
                            default.units = "native"),
         point=grid.points(x=xc, y=yc, gp=gp, 
                           default.units = "native"),
         heatmap={
           div <- 10000
           if(diff(xscale)<div) div <- diff(xscale)
           ww <- ceiling(diff(xscale)/div)
           img <- matrix(NA, nrow=ncol(y0), ncol=div)
           wt <- tile(GRanges(as.character(seqnames(gr)[1]),
                              IRanges(xscale[1], xscale[2])), 
                      n=div)[[1]]
           ol <- findOverlaps(wt, gr, ignore.strand=TRUE)
           qol <- unique(queryHits(ol))
           if(length(gp$breaks)){
             cols <- gp$col
             v <- gp$breaks
           }else{
             rg <- range(as.numeric(as.matrix(y0)))
             cols <- gp$col
             if(length(cols)==0){
               cols <- colorRampPalette(c("green", "black", "red"))(100)
             }else{
               if(length(cols)==1){
                 cols <- if(cols=="white") 
                   colorRampPalette(c("black", cols))(100) else
                     colorRampPalette(c("white", cols))(100)
               }else{
                 cols <- colorRampPalette(cols)(100)
               }
             }
             v <- seq(rg[1]-1, rg[2]+1, length.out = 101)
           }
           
           group <- merge(as.data.frame(ol), data.frame(queryHits=1:div), 
                          all=TRUE)
           y00 <- as.data.frame(y0)
           y01 <- matrix(nrow=nrow(group), ncol=ncol(y00))
           y01[!is.na(group$subjectHits), ] <- 
             y00[group$subjectHits[!is.na(group$subjectHits)], ]
           y01.rowsum <- rowsum(y01, group = group$queryHits)
           y01.groupcnt <- table(group$queryHits)
           stopifnot(identical(names(y01.groupcnt), rownames(y01.rowsum)))
           y01.rowsum <- y01.rowsum/as.numeric(y01.groupcnt)
           y00 <- apply(y01.rowsum, 2, function(.ele) 
             cols[findInterval(.ele, v)])
           stopifnot(all(dim(y00)==rev(dim(img))))
           img[, qol] <- y00[qol, ]
           grid.raster(img, width=1, height=1)
           })
}

#' dandelion.plots
#' @description Plot variants and somatic mutations
#' @param SNP.gr A object of \link[GenomicRanges:GRanges-class]{GRanges} or 
#' \link[GenomicRanges:GRangesList-class]{GRangesList}. 
#' All the width of GRanges must be 1.
#' @param features A object of \link[GenomicRanges:GRanges-class]{GRanges} or
#' \link[GenomicRanges:GRangesList-class]{GRangesList}.
#' @param ranges A object of \link[GenomicRanges:GRanges-class]{GRanges} or 
#' \link[GenomicRanges:GRangesList-class]{GRangesList}.
#' @param type Character. Could be fan, circle, pie or pin.
#' @param newpage plot in the new page or not.
#' @param ylab plot ylab or not. If it is a character vector, 
#' the vector will be used as ylab.
#' @param xaxis,yaxis plot xaxis/yaxis or not. If it is a numeric vector with length 
#' greater than 1, the vector will be used as the 
#' points at which tick-marks are to be drawn. And the names of the vector will be
#' used to as labels to be placed at the tick points if it has names. 
#' @param ylab.gp,xaxis.gp,yaxis.gp An object of class gpar for ylab, xaxis or yaxis.
#' @param legend If it is a list with named color vectors, a legend will be added.
#' @param cex cex will control the size of circle.
#' @param maxgaps maxgaps between the stem of dandelions. 
#' It is calculated by the width of plot region divided by maxgaps. 
#' If a GRanges object is set, the dandelions stem will be clustered
#' in each genomic range.
#' @param heightMethod A function used to determine the height of stem of 
#' dandelion. eg. Mean. Default is length.
#' @param label_on_feature Labels of the feature directly on them. 
#' Default FALSE.
#' @param ... not used.
#' @details In SNP.gr and features, metadata of the GRanges object will be used to 
#' control the color, fill, border, height, data source of pie if the type is pie.
#' @return NULL
#' @import GenomicRanges
#' @import grid
#' @importClassesFrom grImport Picture
#' @importFrom grImport readPicture grid.picture
#' @export
#' @examples
#' SNP <- c(10, 100, 105, 108, 400, 410, 420, 600, 700, 805, 840, 1400, 1402)
#' SNP.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)), 
#'                   score=sample.int(100, length(SNP))/100)
#' features <- GRanges("chr1", IRanges(c(1, 501, 1001), 
#'                                     width=c(120, 500, 405),
#'                                     names=paste0("block", 1:3)),
#'                     color="black",
#'                     fill=c("#FF8833", "#51C6E6", "#DFA32D"),
#'                     height=c(0.1, 0.05, 0.08))
#' dandelion.plot(SNP.gr, features, type="fan")

dandelion.plot <- function(SNP.gr, features=NULL, ranges=NULL,
                      type=c("fan", "circle", "pie", "pin"),
                      newpage=TRUE, ylab=TRUE, ylab.gp=gpar(col="black"),
                      xaxis=TRUE, xaxis.gp=gpar(col="black"), 
                      yaxis=FALSE, yaxis.gp=gpar(col="black"), 
                      legend=NULL, cex=1, maxgaps=1/50, heightMethod=NULL, 
                      label_on_feature=FALSE, ...){
    stopifnot(inherits(SNP.gr, c("GRanges", "GRangesList")))
    stopifnot(inherits(features, c("GRanges", "GRangesList")))
    type <- match.arg(type)
    if(length(heightMethod)>0){
      stopifnot(is.function(heightMethod))
    }else{
      heightMethod <- length
    }
    if(is(maxgaps, "GRanges")){
      ol <- findOverlaps(maxgaps, drop.self=TRUE, drop.redundant=TRUE)
      if(length(ol)>0){
        stop("If maxgaps is an object of GRanges, maxgaps could not have overlaps.")
      }
    }
    if(type=="pin"){
        pinpath <- system.file("extdata", "map-pin-red.xml", package="trackViewer")
        pin <- readPicture(pinpath)
    }else{
        pin <- NULL
    }
    SNP.gr.name <- deparse(substitute(SNP.gr))
    if(is(SNP.gr, "GRanges")){
        SNP.gr <- list(SNP.gr)
        if(length(SNP.gr.name)==length(SNP.gr)) {
          names(SNP.gr) <- SNP.gr.name
        }
    }
    len <- length(SNP.gr)
    
    ## prepare the feature
    if(inherits(features, c("GRangesList", "list"))){
      for(i in seq_along(features)){
        stopifnot("features must be a GRanges or GRangesList object"=
                    is(features[[i]], "GRanges"))
      }
      features <- features[seq.int(len)]
    }else{
      stopifnot("features must be a GRanges or GRangesList object"=
                  is(features, "GRanges"))
      features <- list(features)[seq.int(len)]
    }
    
    ############### handle legend ####################
    ## set the legend as a list, 
    ## if all the legend for different tracks is same
    ## set draw legend for last track later
    legend <- handleLegend(legend, len)
    
    features.name <- deparse(substitute(features))
    
    ################ handle ranges #####################
    ## if !missing(ranges) set ranges as feature ranges
    ranges <- handleRanges(ranges, SNP.gr, features, len)
    
    ##cut all SNP.gr by the range
    SNP.gr <- cutSNP(SNP.gr, ranges, len)
    
    ################## plot ############################
    ## total height == 1
    height <- 1/sum(lengths(ranges))
    if(newpage) grid.newpage()
    for(i in seq.int(len)){
      if(length(ranges[[i]])>1){## split into multilayers
        args <- as.list(match.call())
        args$newpage <- FALSE
        vp <- viewport(x=.5, y=height*(i-0.5), width=1, height=height)
        pushViewport(vp)
        for(j in rev(seq_along(ranges[[i]]))){
          args$ranges <- ranges[[i]][j]
          args$SNP.gr <- SNP.gr[[i]]
          args$features <- features[[i]]
          args$legend <- legend[[i]]
          do.call(what = dandelion.plot, args = args)
        }
        popViewport()
      }else{## is GRanges with length==1
        vp <- viewport(x=.5, y=height*(i-0.5), width=1, height=height)
        pushViewport(vp)
        LINEW <- as.numeric(convertX(unit(1, "line"), "npc"))
        LINEH <- as.numeric(convertY(unit(1, "line"), "npc")) # LINEH is used for gap
        totalH <- as.numeric(unit(1, "npc"))
        if(LINEH > totalH/20){
          LINEH <- totalH/20
        }
        
        ## ylab
        if(length(yaxis)>1 && is.numeric(yaxis)){
          x <- LINEW
        }else{
          x <- unit(3, "lines")
          if(yaxis){
            x <- LINEW
          }
        }
        
        if(is.logical(ylab)){
          if(ylab && length(names(SNP.gr))>0){
            grid.text(names(SNP.gr)[i], x = x, 
                      y = .5, rot = 90, gp=ylab.gp)
          }
        }
        if(is.character(ylab)){
          if(length(ylab)==1) ylab <- rep(ylab, len)
          grid.text(ylab[i], x = x,
                    y = .5, rot = 90, gp=ylab.gp)
        }
        
        
        feature <- features[[i]]

        ## convert height to npc number
        feature$height <- convertHeight2NPCnum(feature$height)
        ## multiple transcripts in one gene could be separated by featureLayerID
        feature <- setFeatureLayerID(feature, ranges[[i]])
        feature.splited <- split(feature, feature$featureLayerID)
        
        baseline <- max(c(unlist(feature$height)/2, .0001)) + 0.2 * LINEH
        gap <- .2 * LINEH
        
        ## bottomblank, the transcripts legend height
        bottomblank <- plotFeatureLegend(feature, as.numeric(convertY(unit(1, "line"), "npc")), 
                                         ranges[[i]], xaxis, xaxis.gp, label_on_feature)
        
        pushViewport(viewport(x=LINEW + .5, y= bottomblank/2 + .5, 
                              width= 1 - 7*LINEW,
                              height= 1 - bottomblank,
                              xscale=c(start(ranges[[i]]), end(ranges[[i]])),
                              clip = "off"))
        ## axis
        plot_grid_xaxis(xaxis, gp=xaxis.gp)
        
        ## the baseline, the center of the first transcript
        baseline <- 
          max(c(feature.splited[[1]]$height/2, 
                .0001)) + 0.2 * LINEH
        baselineN <- 
          max(c(feature.splited[[length(feature.splited)]]$height/2, 
                .0001)) + 0.2 * LINEH
        
        ##plot features
        bottomHeight <- 0
        feature.height <- plotFeatures(feature.splited, LINEH, bottomHeight,
                                       label_on_feature)
        
        SNPs <- SNP.gr[[i]]
        if(length(SNPs)>0){
          strand(SNPs) <- "*"
          SNPs <- sort(SNPs)
          width <- feature.height + 2*gap
          SNPs.groups <- SNPs
          mcols(SNPs.groups) <- NULL
          SNPs.groups$w <- 0
          SNPs.groups$idx <- seq_along(SNPs)
          if(length(SNPs)==length(SNPs$score)){
            SNPs.groups$score <- SNPs$score
          }else{
            SNPs.groups$score <- 0
          }
          if(is(maxgaps, "GRanges")){
            SNPs.ol <- countOverlaps(maxgaps, SNPs)
            maxgaps <- maxgaps[SNPs.ol>0]
            if(length(maxgaps)<1){
              stop("Can not cluster the SNPs by given maxgaps")
            }
            SNPs.ol <- findOverlaps(maxgaps, SNPs)
            SNPs.groups$gps <- length(maxgaps) + seq_along(SNPs)
            SNPs.groups$gps[subjectHits(SNPs.ol)] <- queryHits(SNPs.ol)
          }else{
            if(!is.numeric(maxgaps)){
              stop("maxgaps must be a number or GRanges.")
            }
            SNPs.gap <- gaps(SNPs)
            SNPs.gap <- SNPs.gap[as.character(seqnames(SNPs.gap)) %in% as.character(seqnames(ranges[[i]])) & as.character(strand(SNPs.gap))=="*" & start(SNPs.gap)>=start(ranges[[i]]) & end(SNPs.gap)<=end(ranges[[i]])]
            SNPs.gap$w <- width(SNPs.gap)
            range.width <- floor(width(ranges[[i]])*maxgaps)
            SNPs.gap$idx <- rep(0, length(SNPs.gap))
            SNPs.gap$score <- 0
            SNPs.groups <- sort(c(SNPs.gap, SNPs.groups))
            SNPs.groups$gps <- cumsum(SNPs.groups$w >=range.width)
            SNPs.groups <- SNPs.groups[SNPs.groups$idx>0]
            SNPs.groups <- SNPs.groups[order(SNPs.groups$idx)]
          }
          if(length(names(SNPs))>0){
            maxStrHeight <- 
              max(as.numeric(
                convertX(stringWidth("W"), "npc")
              )*nchar(names(SNPs)))+LINEW/2
          }else{
            maxStrHeight <- 0
          }
          ratio.yx <- getYXratio()
          ypos <- LINEW*max(ratio.yx, 1.2) + maxStrHeight*cex + LINEH*cex 
          if(length(legend[[i]])>0){
            if(!is.null(legend[[i]]$gp$cex)){
              ypos <- ypos + 1.5*LINEH*legend[[i]]$gp$cex
            }else{
              ypos <- ypos + 1.5*LINEH
            }
          }
          SNPs.groups <- Y1pos(SNPs.groups, c(start(ranges[[i]]), end(ranges[[i]])), LINEW, width, cex, 
                               ypos, 
                               length(yaxis) > 1 || (length(yaxis)==1 && as.logical(yaxis)), heightMethod)
          
          # yaxis
          yyscale <- c(0, SNPs.groups$yyscaleMax[1])
          if(length(yaxis)==1 && as.logical(yaxis)) {
            pushViewport(viewport(y= width + (1-width-ypos-cex*LINEW*ratio.yx)/2, 
                                  height= 1 - width-ypos-cex*LINEW*ratio.yx,
                                  yscale = yyscale))
            grid.yaxis(gp=yaxis.gp)
            popViewport()
          }
          if(length(yaxis)>1 && is.numeric(yaxis)){
            yaxisLabel <- names(yaxis)
            if(length(yaxisLabel)!=length(yaxis)) yaxisLabel <- TRUE
            pushViewport(viewport(y= width + (1-width-ypos-cex*LINEW*ratio.yx)/2, 
                                  height= 1 - width-ypos-cex*LINEW*ratio.yx,
                                  yscale = yyscale))
            grid.yaxis(at=yaxis, label=yaxisLabel, gp=yaxis.gp)
            popViewport()
          }
          ## legend
          scoreMax <- max(SNPs.groups$Y2, na.rm = TRUE)
          if(length(legend[[i]])>0){
            ypos <- LINEW*max(ratio.yx, 1.2) + 
              scoreMax + maxStrHeight*cex
            if(is.list(legend[[i]])){
              thisLabels <- legend[[i]][["labels"]]
              gp <- legend[[i]][names(legend[[i]])!="labels"]
              if(is.null(gp$cex)) gp$cex <- 1
              class(gp) <- "gpar"
            }else{
              thisLabels <- names(legend[[i]])
              gp <- gpar(fill=legend[[i]], cex=1) 
            }
            if(length(thisLabels)>0){
              ncol <- getColNum(thisLabels, cex=gp$cex)
              topblank <- ceiling(length(thisLabels) / ncol) * gp$cex
              pushViewport(viewport(x=.5, y=ypos+topblank*LINEH/2, 
                                    width=1,
                                    height=topblank*LINEH,
                                    just="bottom"))
              grid.legend(label=thisLabels, ncol=ncol,
                          byrow=TRUE, vgap=unit(.1*gp$cex, "lines"),
                          hgap=unit(.5*gp$cex, "lines"),
                          pch=21,
                          gp=gp)
              popViewport()
            }
          }
          for(m in 1:length(SNPs)){
            this.dat <- SNPs[m]
            this.dat.grp <- SNPs.groups[m]
            color <- if(is.list(this.dat$color)) this.dat$color[[1]] else this.dat$color
            border <- if(is.list(this.dat$border)) this.dat$border[[1]] else this.dat$border
            fill <- if(is.list(this.dat$fill)) this.dat$fill[[1]] else this.dat$fill
            lwd <- if(is.list(this.dat$lwd)) this.dat$lwd[[1]] else this.dat$lwd
            id <- if(is.character(this.dat$label)) this.dat$label else NA
            id.col <- if(length(this.dat$label.col)>0) this.dat$label.col else "black"
            this.dat.mcols <- mcols(this.dat)
            this.dat.mcols <- cleanDataMcols(this.dat.mcols, type)
            grid.dandelion(x0=(start(this.dat)-start(ranges[[i]]))/width(ranges[[i]]), 
                           y0=baseline,
                           x1=(start(this.dat)-start(ranges[[i]]))/width(ranges[[i]]),
                           y1=this.dat.grp$Y1,
                           x2=(this.dat.grp$X2-start(ranges[[i]]))/width(ranges[[i]]), 
                           y2=this.dat.grp$Y2, 
                           radius=cex*LINEW/2,
                           col=color,
                           border=border,
                           percent=this.dat.mcols,
                           edges=100,
                           alpha=this.dat.grp$alpha,
                           type=type,
                           ratio.yx=ratio.yx,
                           pin=pin,
                           scoreMax=scoreMax,
                           id=id, id.col=id.col, 
                           name=names(this.dat), 
                           cex=cex, lwd=lwd)
          }
        }
        
        popViewport()
        popViewport()
      }
    }
}

#' Reading data from a BAM file
#' @description Read a \code{\link{track}} object from a BAM file
#' @param file The path to the BAM file to read.
#' @param file2 The path to the second BAM file to read.
#' @param ranges An object of \code{\link[GenomicRanges:GRanges-class]{GRanges}} to indicate
#' the range to be imported
#' @param pairs logical object to indicate the BAM is paired or not. See
#' \code{\link[GenomicAlignments]{readGAlignments}}
#' @return a \code{\link{track}} object 
#' @import GenomicAlignments
#' @export
#' @seealso See Also as \code{\link{importScore}}, \code{\link{track}}, 
#' \code{\link{viewTracks}}
#' @examples 
#' bamfile <- system.file("extdata", "ex1.bam", package="Rsamtools",
#' mustWork=TRUE)
#' dat <- importBam(file=bamfile, ranges=GRanges("seq1", IRanges(1, 50), strand="+"))

importBam <- function(file, file2, ranges=GRanges(), pairs=FALSE){
    if(!is(ranges, "GRanges"))
        stop("ranges must be an object of GRanges.")
    param <- ScanBamParam(which=ranges)
    stopifnot(length(pairs)==1)
    if(pairs){
        bam <- readGAlignmentPairs(file, param=param)
    }else{
        bam <- readGAlignments(file, param=param)
    }
    gr <- coverageGR(bam)
    if(!missing(file2)){
        if(pairs){
            bam2 <- readGAlignmentPairs(file2, param=param)
        }else{
            bam2 <- readGAlignments(file2, param=param)
        }
        gr2 <- coverageGR(bam2)
        return(new("track", dat=orderedGR(gr), dat2=orderedGR(gr2),
                   type="data", format="BAM"))
    }
    return(new("track", dat=orderedGR(gr), type="data", format="BAM"))
}

#' plot ideogram with data
#' @description plot ideogram with data for multiple chromosomes 
#' @param ideo output of \link{loadIdeogram}.
#' @param dataList a \link[GenomicRanges:GRangesList-class]{GRangesList} of data to plot.
#' @param layout The layout of chromosomes. Could be a list with chromosome names
#' as its elements.
#' @param horiz a logical value. If FALSE, the ideograms are drawn vertically 
#' to the left. If TRUE, the ideograms are drawn horizontally at the bottom.
#' @param parameterList a list of parameters for each dataset in the dataList. 
#' The elements of the parameters could be xlabs, ylabs, etc. type could be
#' barplot, line, point, heatmap.
#' @param colorSheme A character vector of giemsa stain colors.
#' @param gp parameters used for \link[grid]{grid.roundrect}.
#' @param ... parameters not used.
#' @import grid
#' @importFrom Seqinfo seqlevels seqlengths
#' @import S4Vectors
#' @export
#' @examples 
#' \dontrun{
#' ideo <- loadIdeogram("hg38")
#' library(rtracklayer)
#' library(grid)
#' dataList <- ideo
#' dataList$score <- as.numeric(dataList$gieStain)
#' dataList <- dataList[dataList$gieStain!="gneg"]
#' dataList <- GRangesList(dataList)
#' grid.newpage()
#' ideogramPlot(ideo, dataList, 
#'              layout=list("chr1", "chr2", c("chr3", "chr22"), 
#'                          c("chr4", "chr21"), c("chr5", "chr20"), 
#'                          c("chr6", "chr19"), c("chr7", "chr18"),
#'                          c("chr8", "chr17"), c("chr9", "chr16"),
#'                          c("chr10", "chr15"), c("chr11", "chr14"),
#'                          c("chr12", "chr13"), c("chrX", "chrY")),
#'              parameterList = list(types="heatmap", colorKeyTitle="sample1"))
#' }
#' 
#' 
ideogramPlot <- function(ideo, dataList, layout=NULL,
                         horiz=TRUE,
                         parameterList=
                           list(vp=plotViewport(margins=c(.1, 4.1, .3, .1)),
                                ideoHeight=unit(1/(1+length(dataList)), "npc"), 
                                vgap=unit(.3, "lines"),
                                ylabs="auto", 
                                ylabsRot=ifelse(horiz, 0, 90), 
                                ylabsPos=unit(2.5, "lines"), 
                                xaxis=FALSE, yaxis=FALSE,
                                xlab="", 
                                types="barplot", heights=NULL, 
                                dataColumn="score", 
                                gps=gpar(col="black", fill="gray")), 
                         colorSheme=gieStain(),
                         gp=gpar(fill=NA, lwd=2), ...){
  ## get the seqlengths
  seql <- seqlengths(ideo)
  seql <- seql[names(seql) %in% unique(seqnames(ideo))]
  stopifnot(is.list(parameterList))
  parameterList.default=list(vp=plotViewport(margins=c(.1, 4.1, .3, .1)),
                             ideoHeight=unit(1/(1+length(dataList)), "npc"), 
                             vgap=unit(.3, "lines"),
                             ylabs="auto", 
                             ylabsRot=ifelse(horiz, 0, 90), 
                             ylabsPos=unit(2.5, "lines"), 
                             xaxis=FALSE, yaxis=FALSE,
                             xlab="", 
                             types="barplot", heights=NULL, 
                             dataColumn="score", 
                             gps=gpar(col="black", fill="gray"))
  for(n in names(parameterList)){
    parameterList.default[[n]] <- parameterList[[n]]
  }
  if(parameterList.default$ylabs[1]!="auto"){
    if(!is.list(parameterList.default$ylabs)){
      stop("parameterList$ylabs must be a list with names of seqnames of ideo")
    }
    if(any(!seqlevels(ideo) %in% names(parameterList.default$ylabs))){
      stop("parameterList$ylabs must be a list with names of seqnames of ideo")
    }
  }
  if(all(parameterList.default$types=="heatmap")){
    gps <- parameterList.default$gps
    if(is(gps, "gpar")){
      cols <- list(gps$col)
      if(length(cols)==0){
        cols <- rep(list(c("green", "black", "red")), length(dataList))
      }
      if(length(cols)!=length(dataList)){
        cols <- rep(cols, length(dataList))
      }
    }else{
      if(length(gps)!=length(dataList)){
        stopifnot(all(sapply(gps, is.list)))
        gps <- rep(gps, length(dataList))[seq_along(dataList)]
      }
      cols <- lapply(gps, function(.ele) .ele$col)
    }
    
    ## get range of data points
    rg <- mapply(function(.ele, .cn) 
        range(as.numeric(as.matrix(mcols(.ele)[, .cn])), na.rm = TRUE),
        dataList, parameterList.default$dataColumn, SIMPLIFY = FALSE)
    ## break value of each color range
    v <- lapply(rg, function(.ele){
        if(diff(.ele)==0){
            c(.ele[1]-.ele[1]/10, .ele[2], .ele[2]+.ele[2]/10)
        }else{
            seq(.ele[1]-diff(.ele)/10, .ele[2]+diff(.ele)/10, length.out = 101)
        }
    })
    
    cols <- mapply(function(col, .ele){
      if(length(col)==0){
        col <- colorRampPalette(c("green", "black", "red"))(100)
      }else{
        if(length(col)==1){
          col <- if(col=="white") 
            colorRampPalette(c("black", col))(100) else
              colorRampPalette(c("white", col))(100)
        }else{
          col <- colorRampPalette(col)(100)
        }
      }
      if(diff(.ele)==0){
          col[c(1, 100)]
      }else{
          col
      }
    }, cols, rg, SIMPLIFY = FALSE)
    
    parameterList.default$gps <- mapply(function(col, breaks) 
      list(col=col, breaks=breaks), cols, v, SIMPLIFY = FALSE)
    labels <- parameterList.default$colorKeyTitle
    if(length(labels)==0){
      labels <- paste0("colorKey", 1:length(dataList))
    }
    labels.width <- convertX(stringWidth("W"), "inches", valueOnly = TRUE)*nchar(labels)
    colorKey.width <- 1
    oneChar.width <- convertX(unit(1, "lines"), "inches", valueOnly = TRUE)
    pushViewport(viewport(y=unit(1.75, "lines"), height=unit(3.5, "lines"),
                          width=unit(sum(labels.width) + 
                                       length(labels) +
                                       length(labels)*2*oneChar.width, 
                                     "inches")))
    labels.x.width <- colorKey.width + 2*oneChar.width + labels.width
    this.labels.width <- 0
    for(i in 1:length(labels)){
      pushViewport(viewport(x=unit(labels.x.width[i]/2 + this.labels.width, 
                                   'inches'),
                            width=unit(labels.x.width[i], "inches")))
      this.labels.width <- this.labels.width + labels.x.width[i]
      this.xscale <- range(v[[i]])
      vp <- viewport(x=unit(.5, "inches"), width=unit(1, "inches"),
                     y=unit(3, "lines"), height=unit(.5, "lines"),
                     xscale=this.xscale)
      pushViewport(vp)
      grid.raster(as.raster(matrix(cols[[i]], nrow=1)), 
                  width = 1, height = 1, interpolate=FALSE)
      grid.xaxis()
      upViewport()
      grid.text(label=labels[i], 
                vp = viewport(x=unit(colorKey.width+oneChar.width+
                                labels.width[i]/2, "inches"),
                              width=unit(labels.width[i], "inches"),
                              y=unit(3, "lines"), height=unit(1, "lines")))
      upViewport()
    }
    upViewport()
    
    pushViewport(plotViewport(margins = c(4.1, 0, 0, 0)))
    on.exit(upViewport())
  }
  parameterList.cp <- parameterList <- parameterList.default
  if(any(is.na(seql))){
    stop("seqlengths contain NAs")
  }
  if(length(layout)==0){
    if(length(seql)==0){
      return()
    }
    if(length(seql)==1){
      plotOneIdeo(ideo, dataList, parameterList, chrom=names(seql), 
                  colorSheme=colorSheme, gp=gp, ...)
      return()
    }
    ## 1 column
    layout <- as.list(names(seql))
  }
  stopifnot(is.list(layout))
  if(!horiz){
    canvas.w <- convertX(unit(1, "npc"), "inch", valueOnly = TRUE)
    canvas.h <- convertY(unit(1, "npc"), "inch", valueOnly = TRUE)
    tmp <- canvas.h
    canvas.h <- convertY(unit(canvas.w, "inch"), "npc")
    canvas.w <- convertX(unit(tmp, "inch"), "npc")
    pushViewport(viewport(width=canvas.w, 
                          height=canvas.h, 
                          angle=-90))
    on.exit(upViewport())
  }
  layout.df <- data.frame(seq=unlist(layout), 
                          group=rep(1:length(layout), sapply(layout, length)),
                          stringsAsFactors = FALSE)
  layout.df$seqlength <- seql[layout.df$seq]
  xscale <- max(sapply(split(layout.df$seqlength, layout.df$group), sum)) * 1.05
  vpH <- 1/length(layout)
  for(i in sort(unique(layout.df$group))){
    this.layout <- layout.df[layout.df$group==i, , drop=FALSE]
    vp <- viewport(y=1-vpH*(i-.5), height = vpH, name=paste0("layout", i))
    pushViewport(vp)
    vpW <- this.layout$seqlength/xscale
    if(length(vpW)==1){
      vpWgap <- 0
    }else{
      vpWgap <- (1/1.05-sum(vpW))/(length(vpW)-1)
    }
    for(j in seq_along(vpW)){
      vp <- viewport(x=sum(vpW[-(j:length(vpW))])+.5*vpW[j]+vpWgap*(j-1), 
                     width=vpW[j], name=paste0("layout", i, "_", j))
      pushViewport(vp)
      parameterList <- parameterList.cp
      if(parameterList$ylabs[1]=="auto"){
        parameterList$ylabs <- this.layout$seq[j]
      }else{
        parameterList$ylabs <- parameterList$ylabs[[this.layout$seq[j]]]
      }
      if(j==length(vpW) && length(vpW)>1){
        parameterList$vp$x <- 
          unit(convertX(unit(1, "npc") - parameterList$vp$x - parameterList$vp$width,
                   "lines", valueOnly = TRUE), "lines")
        parameterList$ylabsPos <- unit(1, "npc") - parameterList$ylabsPos
      }
      plotOneIdeo(ideo, dataList, parameterList, 
                  chrom=this.layout$seq[j], colorSheme=colorSheme, 
                  gp=gp, ...)
      upViewport()
    }
    upViewport()
  }
}

getHeight <- function(SNPs, ratio.yx, LINEW, GAP, cex, type, scoreMax,
                      level=c("data", "data&labels")){
    level=match.arg(level)
    stack.factors <- unique(as.character(SNPs$stack.factor))
    stack.factors <- sort(stack.factors)
    if(level=="data"){
        switch(type,
               circle={
                   labels.y <- LINEW + # add gaps for labels
                       6.5*GAP + 
                       scoreMax * LINEW * ratio.yx
               },
               pin={
                   if(length(SNPs$score)>0) {
                       this.scores <- ceiling(SNPs$score)
                   }else {
                       this.scores <- .5
                   }
                   this.scores[is.na(this.scores)] <- .5
                   labels.y <- LINEW + 
                       6.5*GAP + 
                       (this.scores-0.5*cex) * LINEW * ratio.yx
               },
               pie={
                   labels.y <- LINEW*max(ratio.yx, 1.2) + 
                       6.5*GAP + 0.5 * LINEW * ratio.yx * cex
               },
               pie.stack={
                   labels.y <- LINEW + 
                       6.5*GAP + 
                       (scoreMax-0.5*cex) * LINEW * ratio.yx
               },
               flag={
                 labels.y <- LINEW + 
                   6.5*GAP + 
                   scoreMax * LINEW * ratio.yx
               })
        labels.y
    }else{
        if(length(SNPs$label.parameter.rot)>0) {
            labels.rot <- SNPs$label.parameter.rot
        }else{
            labels.rot <- 90
        }
        labels.cex <- 1
        if(length(SNPs$label.parameter.gp)>0){
          if(length(SNPs$label.parameter.gp$cex)>0)
            labels.cex <- SNPs$label.parameter.gp$cex[[1]][1]
        }
        labels.length.rate <- labels.cex * max(cospi((labels.rot-90)/180), 0) * ratio.yx
        stringH <- as.numeric(convertY(stringHeight("W"), "npc"))
        
        switch(type,
               circle={
                   if(length(names(SNPs))>0){
                       maxStrHeight <- 
                           max(as.numeric(
                               convertX(stringWidth(names(SNPs)), "npc")
                           ))+stringH
                   }else{
                       maxStrHeight <- 0
                   }
                   maxStrHeight <- maxStrHeight * labels.length.rate
                   ypos <- LINEW + 6.5*GAP + 
                       scoreMax * LINEW * ratio.yx + maxStrHeight
               },
               pin={
                   if(length(names(SNPs))>0){
                       thisStrHeight <- max(as.numeric(
                           convertX(stringWidth(names(SNPs)), "npc")) ) +
                         stringH
                   }else{
                       thisStrHeight <- 0
                   }
                   thisStrHeight <- thisStrHeight * labels.length.rate
                   if(length(SNPs$score)>0){
                       ypos <- 
                           max(LINEW + 
                                   6.5*GAP + 
                                   (SNPs$score-0.5*cex) * LINEW * ratio.yx + 
                                   thisStrHeight)
                   }else{
                       ypos <- max(LINEW*max(ratio.yx, 1.2) + 
                                       6.5*GAP + thisStrHeight)
                   }
               },
               pie={
                   if(length(names(SNPs))>0){
                       maxStrHeight <- 
                         max(as.numeric(
                           convertX(stringWidth(names(SNPs)), "npc")
                         ))+stringH
                   }else{
                       maxStrHeight <- 0
                   }
                   maxStrHeight <- maxStrHeight * labels.length.rate
                   ypos <- LINEW + 
                       6.5*GAP + maxStrHeight
               },
               pie.stack={
                   if(length(names(SNPs))>0){
                       maxStrHeight <- 
                         max(as.numeric(
                           convertX(stringWidth(names(SNPs)), "npc")
                         ))+stringH
                   }else{
                       maxStrHeight <- 0
                   }
                   maxStrHeight <- maxStrHeight * labels.length.rate
                   ypos <- LINEW + 
                       6.5*GAP + maxStrHeight +
                       (scoreMax-0.5*cex) * LINEW * ratio.yx
               },
               flag={
                 if(length(names(SNPs))>0){
                   maxStrHeight <- 
                     max(as.numeric(
                       convertX(stringWidth(names(SNPs)), "npc")
                     ))+stringH
                 }else{
                   maxStrHeight <- 0
                 }
                 maxStrHeight <- maxStrHeight * labels.length.rate
                 ypos <- LINEW + 6.5*GAP + 
                   scoreMax * LINEW * ratio.yx + maxStrHeight
               }
        )
        ypos
    }
}

##levels of feature "CDS"   "ncRNA" "utr3"  "utr5"
plotGeneModel <- function(track, xscale, chr, yaxis.gp=gpar(), lollipop_style_switch_limit=10){
  if(length(track@dat2)>0){
    feature.height <- 
      if(is.list(track@dat2$feature.height)) track@dat2$feature.height[[1]] else track@dat2$feature.height[1]
    if(length(feature.height)==0) feature.height <- .5
    unit <- feature.height/4
    y <- feature.height/2
  }else{
    unit <- 0.25
    y <- 0.5
  }
    transcript <- track@dat
    col <- track@style@color
    strand <- as.character(strand(transcript))[1]
    unitx <- convertWidth(unit(3, "lines"), 
                          unitTo = "npc",
                          valueOnly = TRUE)
    if(unitx > 0.5) unitx <- 0.5
    unitx <- unitx * abs(xscale[2] - xscale[1]) #abs(xscale[2] - xscale[1]) / 10
    ## plot introns
    introns <- gaps(ranges(transcript))
    introns <- introns[start(introns)>min(end(transcript))
                       &end(introns)<max(start(transcript))]
    plotArrow <- function(ax0, ax1, y, col, vp){
        ay0 <- y + unit/2
        ay1 <- y
        ay2 <- y - unit/2
        grid.segments(ax0, ay0, ax1, ay1, 
                      default.units="native", 
                      gp=gpar(col=col))
        grid.segments(ax1, ay1, ax0, ay2, 
                      default.units="native", 
                      gp=gpar(col=col))
    }
    plotIntrons <- function(strand, a, b, y, xscale, col){
        if(a<xscale[1]) a <- xscale[1]
        if(b>xscale[2]) b <- xscale[2]
        w <- b - a
        times <- floor(5*w/unitx)
        for(i in seq_len(times)){
            if(b > a+i*unitx/5+unitx*3/20){
                if(strand=="+") plotArrow(a+i*unitx/5+unitx/10, 
                                          a+i*unitx/5+unitx*3/20,
                                          y, col)
                else {
                    if(strand=="-") 
                        plotArrow(a+i*unitx/5+unitx*3/20, 
                                  a+i*unitx/5+unitx/10,
                                  y, col)
                }
            }
        }
    }
    if(length(introns)>0){
        for(i in 1:length(introns)){
            if(start(introns)[i]<=xscale[2] && end(introns)[i]>=xscale[1]){
                grid.segments(start(introns)[i], y, 
                              end(introns)[i], y, default.units="native", 
                              gp=gpar(col=col))
                plotIntrons(strand, start(introns[i]), 
                            end(introns[i]),
                            y, xscale, col)
            }
        }
    }

    ## plot utr
    utr <- transcript[transcript$feature %in% c("utr3", "utr5")]
    if(length(utr)>0){
        for(i in 1:length(utr)){
            if(start(utr)[i]<=xscale[2] && end(utr)[i]>=xscale[1]){
                grid.rect(start(utr)[i], y,
                          end(utr)[i]-start(utr)[i]+1, unit, 
                          default.units="native", just=c(0, 0.5),
                          gp=gpar(col=NA, fill=col))
            }
        }
    }
    
    ## plot exons
    exons <- transcript[transcript$feature %in% c("CDS", "ncRNA", "exon")]
    if(length(exons)>0){
        for(i in 1:length(exons)){
            if(start(exons)[i]<=xscale[2] && end(exons)[i]>=xscale[1]){
                grid.rect(start(exons)[i], 
                          y, 
                          end(exons)[i]-start(exons)[i]+1, 
                          2*unit, default.units="native", just=c(0, 0.5),
                          gp=gpar(col=NA, fill=col))
            }
        }
    }
    
    if(length(track@dat2)>0){
      track@dat2 <- orderedGR(track@dat2)
      xscale.gr <- GRanges(seqnames = chr, ranges=IRanges(min(xscale), max(xscale)))
      track@dat2 <- subsetByOverlaps(track@dat2, xscale.gr, ignore.strand=TRUE)
      if(length(track@dat2)<1) return(invisible())
      width(track@dat2) <- 1
      TYPES <- c("circle", "pie", "pin", "pie.stack", "flag")
      type <- if(is.list(track@dat2$type)) track@dat2$type[[1]] else track@dat2$type[1]
      if(length(type)==0) type <- "circle"
      if(!type %in% TYPES) type <- "circle"
      if(type=="pin"){ ## read the pin shape file
        pinpath <- system.file("extdata", "map-pin-red.xml", package="trackViewer")
        pin <- readPicture(pinpath)
      }else{
        pin <- NULL
      }
      cex <- if(is.list(track@dat2$cex)) track@dat2$cex[[1]] else track@dat2$cex[1]
      if(length(cex)==0) cex <- 1
      dashline.col <- if(is.list(track@dat2$dashline.col)) track@dat2$dashline.col[[1]] else track@dat2$dashline.col[1]
      if(length(dashline.col)==0) dashline.col <- "gray80"
      jitter <- if(is.list(track@dat2$jitter)) track@dat2$jitter[[1]] else track@dat2$jitter[1]
      if(length(jitter)==0) jitter <- "node"
      if(!jitter %in% c("node", "label")) jitter <- "node"
      scoreMax0 <- scoreMax <- 
        if(length(track@dat2$score)>0) ceiling(max(c(track@dat2$score, 1), na.rm=TRUE)) else 1
      if(type=="pie.stack") scoreMax <- length(unique(track@dat2$stack.factor))
      if(!type %in% c("pie", "pie.stack")){
        if(scoreMax>lollipop_style_switch_limit) {
          track@dat2$score <- 10*track@dat2$score/scoreMax
          scoreMax <- 10*scoreMax0/scoreMax
        }else{
          scoreMax <- scoreMax0
        }
        scoreType <- 
          if(length(track@dat2$score)>0) all(floor(track@dat2$score)==track@dat2$score) else FALSE
      }else{
        scoreType <- FALSE
      }
      LINEW <- as.numeric(convertX(unit(1, "line"), "npc"))/2
      LINEH <- as.numeric(convertY(unit(1, "line"), "npc"))/2
      ## GAP the gaps between any elements
      GAP <- .2 * LINEH
      ratio.yx <- getYXratio()
      plotLollipops(track@dat2, feature.height=y+unit, bottomHeight=0, baseline=y, 
                    type=type, ranges=xscale.gr, yaxis=FALSE, yaxis.gp=yaxis.gp,
                    scoreMax=scoreMax, scoreMax0=scoreMax0, scoreType=scoreType, 
                    LINEW, cex, ratio.yx, GAP, pin, dashline.col,
                    side="top", jitter=jitter)
    }
    return(invisible())
}

#' @title Minimal designed plotting tool for genomic data
#'
#' @description A package that plot data and annotation information along genomic 
#' coordinates in an elegance style. This tool is based on Gviz but want to draw 
#' figures in minimal style for publication.
#' 
#' @examples 
#' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
#' library(org.Hs.eg.db)
#' trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
#'                          org.Hs.eg.db,
#'                          chrom="chr11", 
#'                          start=122929275, 
#'                          end=122930122)
#' extdata <- system.file("extdata", package="trackViewer",
#'                        mustWork=TRUE)
#' repA <- importScore(paste(extdata, "cpsf160.repA_+.wig", sep="/"),
#'                     paste(extdata, "cpsf160.repA_-.wig", sep="/"),
#'                     format="WIG")
#' strand(repA@dat) <- "+"
#' strand(repA@dat2) <- "-"
#' fox2 <- importScore(paste(extdata, "fox2.bed", sep="/"), format="BED")
#' dat <- coverageGR(fox2@dat)
#' fox2@dat <- dat[strand(dat)=="+"]
#' fox2@dat2 <- dat[strand(dat)=="-"]
#' gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
#' vp <- viewTracks(trackList(repA, fox2, trs), gr=gr, autoOptimizeStyle=TRUE)
#' addGuideLine(c(122929767, 122929969), vp=vp)
#' addArrowMark(list(x=unit(.5, "npc"), 
#'                   y=unit(.39, "npc")),
#'              col="blue")
"_PACKAGE"

#' plot tracks based on gene name
#' @description given a gene name, plot the tracks.
#' @param symbol Gene symbol
#' @param upstream upstream from anchor
#' @param downstream downstream from anchor
#' @param anchor TSS, or gene
#' @param filenames files used to generate tracks
#' @param format file format used to generate tracks
#' @param txdb txdb will be used to extract the genes
#' @param org org package name
#' @param plot plot the tracks or not.
#' @return an invisible list of a \code{\link{trackList}},
#'  a \code{\link{trackViewerStyle}} and a \code{\link[GenomicRanges:GRanges-class]{GRanges}}
#' @import GenomicRanges
#' @import Rsamtools
#' @importFrom BiocGenerics basename
#' @export
#' @examples 
#' library(TxDb.Hsapiens.UCSC.hg19.knownGene)
#' library(org.Hs.eg.db)
#' extdata <- system.file("extdata", package="trackViewer", mustWork=TRUE)
#' filename = file.path(extdata, "fox2.bed")
#' optSty <- viewGene("HSPA8", filenames=filename, format="BED", 
#'                    txdb=TxDb.Hsapiens.UCSC.hg19.knownGene, 
#'                    org="org.Hs.eg.db")
#'


viewGene <- function(symbol, filenames, format, txdb, org,
                     upstream=1000, downstream=1000, 
                     anchor=c("gene", "TSS"),
                     plot=FALSE){
  anchor <- match.arg(anchor)
  stopifnot(is(txdb, "TxDb"))
  gr <- getLocation(symbol, txdb, org)
  if(length(gr)>1){
    stop("Can not handle multiple positions.")
  }
  if(anchor=="TSS"){
    gr <- promoters(gr, upstream = upstream, downstream = downstream)
  }else{
    if(as.character(strand(gr))[1]=="-"){
      tmp <- upstream
      upstream <- downstream
      downstream <- upstream
    }
    start(gr) <- start(gr) - upstream
    end(gr) <- end(gr) + downstream
  }
  trs <- geneModelFromTxdb(txdb, get(org), gr=gr)
  if(format=="BAM"){
    data <- sapply(filenames, function(filename){
      importBam(filename, ranges = gr, 
                pairs = testPairedEndBam(filename))
    })
  }else{
    data <- sapply(filenames, importScore, format = format, 
                   ranges=gr, ignore.strand=TRUE)
  }
  names(data) <- basename(filenames)
  tL <- trackList(trs, data, heightDist=c(length(trs), length(data)))
  optSty <- optimizeStyle(trackList = tL)
  trackList <- optSty$tracks
  viewerStyle <- optSty$style
  optSty$range <- gr
  if(plot) viewTracks(trackList, gr=gr, viewerStyle = viewerStyle)
  return(invisible(optSty))
}

k_smooth <- function(x, y, bandwidth){
  fit <- ksmooth(x, y, kernel="normal", bandwidth = 2*bandwidth,
                 n.points=length(x), x.points = x)
  fit$y
}
xysmooth <- function(x2, y2, smooth=5){
  if(is.logical(smooth)) smooth=5
  if(smooth[1]>=length(x2) ||
     smooth[1]<1){
    return(list(x=x2, y=y2))
  }
  ord <- order(x2)
  x2 <- x2[ord]
  y2 <- y2[ord]
  list(x=x2, y=k_smooth(x2, y2, smooth))
}
resampleData_old <- function(.dat, scale, step=1000){
  .rd <- reduce(.dat)
  if(length(.rd)==length(.dat)){
    return(.dat)
  }
  .width <- which(width(.dat) > diff(scale)[1]/50)
  .idx <- sample(1:length(.dat), step)
  .idx <- unique(c(.idx, .width))
  .idx <- .idx[order(.idx)]
  .dat1 <- .dat[-.idx]
  .datM <- reduce(.dat1, with.revmap=TRUE)
  .map <- .datM$revmap
  .gp <- rep(1:length(.map), sapply(.map, length))
  .datM$score <- floor(sapply(split(.dat1$score[unlist(.map)], 
                                    .gp), mean))
  .datM$revmap <- NULL
  .dat <- c(.dat[.idx], .datM)
  .dat <- orderedGR(.dat)
  .dat
}
resampleData <- function(.dat, scale, step=1000){
  .idx <- which(width(.dat) > diff(scale)[1]/50)
  ## resample the range
  .dat2 <- .dat
  strand(.dat2) <- "*"
  .rg <- range(unname(.dat2))
  .rgs <- c(.rg, .rg, .rg)
  strand(.rgs) <- c("+", "-", "*")
  .rgs <- tile(.rgs, n = step)
  .rg$score <- 0
  .dats <- split(.dat, strand(.dat))
  .cvg <- lapply(.dats, function(.ele){
    .ele <- c(.ele, .rg)
    coverage(.ele, weight = .ele$score)
  })
  .dat2 <- mapply(.cvg, .rgs, FUN=function(x, y){
    x <- x[[as.character(seqnames(y))[1]]]
    .vw <- Views(x, start=ranges(y))
    y$score <- viewMaxs(.vw)
    y[y$score!=0]
  })
  names(.dat2) <- c("+", "-", "*")
  .dat2 <- .dat2[as.character(unique(strand(.dat)))]
  .dat2 <- unlist(GRangesList(.dat2))
  .dat3 <- c(.dat2, .dat[.idx])
  .dat1 <- disjoin(.dat3)
  .ol <- findOverlaps(.dat1, .dat2)
  .dat1$score <- 0
  .dat1[queryHits(.ol)]$score <- .dat2[subjectHits(.ol)]$score
  .ol <- findOverlaps(.dat1, .dat[.idx])
  .dat1[queryHits(.ol)]$score <- .dat[.idx][subjectHits(.ol)]$score
  .dat <- orderedGR(.dat1)
  .dat
}
plotDataTrack <- function(.dat, chr, strand, scale, color, yscale, smooth=FALSE, type='peak'){
    names(.dat) <- NULL
    mcols(.dat) <- mcols(.dat)[, "score"]
    colnames(mcols(.dat)) <- "score"
    if(missing(yscale)) yscale <- c(0, 1)
    yscale <- range(yscale)
    if(length(.dat)>0){##subset if length .dat > 1000
        trackViewerCache <- list(step=1000)
        if(length(.dat)>trackViewerCache$step){## resample the points
          .dat <- resampleData(.dat, scale, trackViewerCache$step)
        }
        .dat <- c(.dat, GRanges(seqnames=chr, 
                                IRanges(start=scale, end=scale), 
                                strand=strand, score=c(0,0)))
    }else{
        .dat <- GRanges(seqnames=chr, 
                        IRanges(start=scale, end=scale), 
                        strand=strand, score=c(0,0))
    }
    .data <- split(.dat, strand(.dat))
    if(strand!="*") .data <- .data[names(.data)==strand]
    xt <- c()
    yt <- c()
    for(i in seq_along(.data)){
        if(length(.data[[i]])>0){
            .dat <- .data[[i]]
            ## trim from back to see where is the last number that !=0
            id0 <- which(.dat$score!=0)
            if(length(id0)>0){
              .data_range <- range(.dat)
              .dat <- .dat[seq.int(id0[length(id0)]), ]
              .dat <- condenceGRs(.dat) ## can not handle +/- strands in the same track
              .dat$type <- "D"
              .gap <- gaps(.dat) ## insert 0
              .gap <- .gap[start(.gap)>=scale[1]-1 & end(.gap)<=scale[2]+1]
              if(length(.gap)>0){
                mcols(.gap)$score <- 0
                .gap$type <- "G"
                .dat <- c(.dat, .gap)
              }
              .dat <- orderedGR(.dat)
              .dat <- as.data.frame(.dat)
              ##expand the gap by 1
              .dat[.dat$type=="G", "start"] <- .dat[.dat$type=="G", "start"] - .5
              .dat[.dat$type=="G", "end"] <- .dat[.dat$type=="G", "end"] + .5
              x <- as.numeric(t(.dat[,c("start", "end")]))
              y <- as.numeric(rep(.dat[,"score"], each=2))
              x2 <- c(start(.data_range), min(x)-.5, x, max(x)+.5, end(.data_range))
              y2 <- c(0, 0, y, 0, 0)
              if(type=='line'){
                grid.lines(x2, y2, default.units="native", 
                             gp=gpar(col=color))
              }else{
                if(type=='histogram'){
                  yir <- IRanges(scale[1], scale[2])
                  tiles <- tile(yir, n = min(width(yir), 100))[[1]]
                  cvg <- coverage(ranges(.data[[i]]), weight = .data[[i]]$score)
                  cvg_views <- Views(cvg, start(tiles), end(tiles))
                  y3 <- viewMeans(cvg_views, na.rm = TRUE)
                  x3 <- start(tiles) + width(tiles)/2
                  grid.rect(x3, y3/2, default.units="native", 
                            width = width(tiles), height = y3,
                            gp=gpar(col=NA, fill=color))
                }else{
                  grid.polygon(x2, y2, default.units="native", 
                               gp=gpar(col=NA, fill=color))
                }
              }
              
              if(yscale[1]<=0 && yscale[2]>=0){ ## yscale[2] >= 0 >= yscale[1]
                ## plot baseline
                grid.lines(x=scale, y=0, default.units="native",
                           gp=gpar(col=color))
              }else{ ## both yscale>0 or <0
                dy <- abs(yscale - 0)
                dy <- yscale[order(dy)][1]
                grid.lines(x=scale, y=dy, default.units="native",
                           gp=gpar(col="red"))
              }
              if(any(y2>max(yscale, na.rm = TRUE) | y2<min(yscale, na.rm = TRUE))){
                ## out of range
                M <- max(yscale, na.rm = TRUE)
                m <- min(yscale, na.rm = TRUE)
                keep <- which(y2>M | y2<m)
                k0 <- keep-1
                k1 <- keep+1
                k0[k0<1] <- 1
                k1[k1>length(y2)] <- length(y2)
                grid.segments(x0=x2[k0], y0=ifelse(y2[keep]>M, M, m),
                              x1=x2[k1], y1=ifelse(y2[keep]>M, M, m),
                              default.units = 'native',
                              gp=gpar(col='red'))
              }
              
              xt <- c(xt, x)
              yt <- c(yt, y)
              ## do smooth curve
              if(smooth[1]){
                xy.smoothed <-xysmooth(x2, y2, smooth[1])
                x2 <- xy.smoothed$x
                y2 <- xy.smoothed$y
                grid.lines(x2, y2, default.units="native", 
                           gp=gpar(col=ifelse(length(smooth)>1, smooth[2], "red")))
              }
            }else{
              .dat <- orderedGR(.dat)
              .dat <- as.data.frame(.dat)
              x <- as.numeric(t(.dat[,c("start", "end")]))
              y <- as.numeric(rep(.dat[,"score"], each=2))
              xt <- c(xt, x)
              yt <- c(yt, y)
            }
        }
    }
    return(list(x=xt, y=yt))
}
plotAnnotationTrack <- function(.dat, chr, strand, scale, color){
  .dat <- subsetByOverlaps(.dat, GRanges(chr, IRanges(scale[1], scale[2]), strand=strand))
  if(length(.dat)){
    if(length(.dat$score)==length(.dat)){
      if(length(unique(.dat$score))>1){
        ## map color
        if(length(.dat$color)!=length(.dat)){
          color <- mapScoresToColor(.dat$score, color)$color
        }else{
          color <- unlist(lapply(.dat$color, function(.ele) .ele[1]))
        }
      }
    }
    grid.rect(x=(start(.dat) + end(.dat))/2,
              y=0.5,
              width = width(.dat),
              height=0.9,
              default.units = 'native',
              gp=gpar(fill=color, col=color))
  }
}
plotTrack <- function(name, track, curViewStyle, curYpos,
                      yscale, height, xlim, chr, strand,
                      operator, wavyLine, smooth=FALSE,
                      lollipop_style_switch_limit=10){
    style <- track@style
    yHeightBottom <- yHeightTop <- 0.01
    xscale.orderd <- sort(xlim)
    if(curViewStyle@flip){ 
        xscale <- rev(xlim)
    }else{
        xscale <- xlim
    }
    if(length(style@marginBottom)>0){
        yHeightBottom <- style@marginBottom
    }
    if(length(style@marginTop)>0){
        yHeightTop <- style@marginTop
    }
    pushViewport(viewport(x=0, y=curYpos, 
                          height=height,
                          width=1, 
                          just=c(0,0))) ## vp1
    ##put ylab
    if(style@ylabpos!='none' && !is.na(name) && name!=""){
      if(track@type %in% c("transcript", "gene") &&
         style@ylabpos %in% c("upstream", "downstream")){
        if(length(findOverlaps(ranges(range(unname(track@dat))),
                               IRanges(xscale.orderd[1], xscale.orderd[2])))>0){
          putGeneYlab(curViewStyle, style, name, height, xscale,
                      range(unname(track@dat)), length(track@dat2)>0)
        }
      }else{
        if(style@ylabpos=="upstream") style@ylabpos <- "left"
        if(style@ylabpos=="downstream") style@ylabpos <- "right"
        putYlab(curViewStyle, style, name, yHeightBottom, yHeightTop, height, yscale)
      }
    }
    
    if(any(style@tracktype=='annotation')&&track@type=='data'){
      if(style@tracktype[1]=='annotation'){
        pushViewport(viewport(x=curViewStyle@margin[2],
                              y=1 - yHeightTop,
                              width = 1-curViewStyle@margin[2]-curViewStyle@margin[4],
                              height = yHeightTop,
                              clip='on',
                              just=c(0,0),
                              xscale=xscale))
        plotAnnotationTrack(track@dat, chr, strand, xlim, style@color)
        popViewport()
      }
      if(style@tracktype[2]=='annotation'){
        pushViewport(viewport(x=curViewStyle@margin[2],
                              y=0,
                              width = 1-curViewStyle@margin[2]-curViewStyle@margin[4],
                              height = yHeightBottom,
                              clip='on',
                              just=c(0,0),
                              xscale=xscale))
        plotAnnotationTrack(track@dat2, chr, strand, xlim, style@color)
        popViewport()
      }
    }
    
    pushViewport(viewport(x=0, y=yHeightBottom, 
                          height=1 - yHeightTop - yHeightBottom,
                          width=1, 
                          just=c(0,0))) ## vp2
    xy <- list()
    if(track@type %in% c("data", "scSeq", "lollipopData", "interactionData")){
        if(track@type=="data") {
          ##plot yaxis
          drawYaxis(yscale, style@yaxis, curViewStyle)
          pushViewport(viewport(x=curViewStyle@margin[2], y=0, 
                                height=1, 
                                width=1-curViewStyle@margin[2]-curViewStyle@margin[4], 
                                clip="on",
                                just=c(0,0), 
                                xscale=xscale, 
                                yscale=yscale))
          ##grid.clip()
          ##for dat
          xy1 <- list(x=numeric(length=0L), y=numeric(length=0L))
          if(style@tracktype[1]!='annotation'){
            xy1 <- plotDataTrack(track@dat, chr, strand, xlim, style@color[1], yscale=yscale, smooth=smooth, type = style@tracktype[1])
          }
          xy2 <- list(x=numeric(length=0L), y=numeric(length=0L))
          ##for dat2
          if(length(track@dat2)>0){
            if(length(style@tracktype)<2){
              style@tracktype[2] <- style@tracktype[1]
            }
            if(style@tracktype[2]!='annotation'){
              if(is_null_na(operator)[1]){
                track@dat2$score <- -1 * track@dat2$score ##convert to negative value
              }
              xy2 <- plotDataTrack(track@dat2, chr, strand, xlim, style@color[2], yscale=yscale, smooth=smooth, type = style@tracktype[2])
            }
          }
          xy <- list(x=c(xy1$x, xy2$x), y=c(xy1$y, xy2$y))
          xy.id <- order(xy$x)
          xy <- list(x=xy$x[xy.id], y=xy$y[xy.id])
          ##plot xscale?
          if(style@xscale@draw){
            if(length(style@xscale@label)==0){
              scale <- hrScale(xlim)
              xpos <- locateScale(xy$x, abs(xy$y), max(abs(yscale)), scale)
              ypos <- yscale[abs(yscale)==max(abs(yscale))] * 0.7
              if(length(ypos)>1) ypos <- max(ypos)
              style@xscale@from <- new("pos", x=xpos[1], y=ypos, unit="native")
              style@xscale@to <- new("pos", x=xpos[2], y=ypos, unit="native")
              style@xscale@label <- paste(2*scale[[1]], scale[[2]])
            }
            drawXscale(style@xscale)
          }
          popViewport()## for data
        }else{
          if(track@type=="lollipopData"){
            pushViewport(viewport(x=curViewStyle@margin[2], y=0, 
                                  height=1, 
                                  width=1-curViewStyle@margin[2]-curViewStyle@margin[4], 
                                  clip="on",
                                  just=c(0,0), 
                                  xscale=xscale))
            ybase <- ifelse(length(track@dat2)>0, .5, 0)
            
            LINEW <- as.numeric(convertX(unit(1, "line"), "npc"))
            LINEH <- as.numeric(convertY(unit(1, "line"), "npc"))
            ## GAP the gaps between any elements
            GAP <- .2 * LINEH
            ratio.yx <- getYXratio()
            getMaxHeight <- function(lollipopData){
              if(length(lollipopData)==0) return(0)
              TYPES <- c("circle", "pie", "pin", "pie.stack", "flag")
              type <- if(is.list(lollipopData$type)) lollipopData$type[[1]] else lollipopData$type[1]
              if(length(type)==0) type <- "circle"
              if(!type %in% TYPES) type <- "circle"
              cex <- if(is.list(lollipopData$cex)) lollipopData$cex[[1]] else lollipopData$cex[1]
              if(length(cex)==0) cex <- 1
              scoreMax0 <- scoreMax <- 
                if(length(lollipopData$score)>0) ceiling(max(c(lollipopData$score, 1), na.rm=TRUE)) else 1
              if(type=="pie.stack") scoreMax <- length(unique(lollipopData$stack.factor))
              if(!type %in% c("pie", "pie.stack")){
                if(scoreMax>10) {
                  scoreMax <- 10*scoreMax0/scoreMax
                }else{
                  scoreMax <- scoreMax0
                }
              }
              getHeight(lollipopData, 
                        ratio.yx, LINEW, GAP, cex, type,
                        scoreMax=scoreMax,
                        level="data")
            }
            maxHeight <- max(c(getMaxHeight(track@dat), getMaxHeight(track@dat2)), na.rm = TRUE)
            if(length(track@dat2)>0) maxHeight + .5
            plotLollipopData(track@dat, xlim, chr, style@yaxis@draw, gpar(),
                             ybase, side="top", main=style@yaxis@main,
                             baselineCol=style@color[1], maxHeight=maxHeight)
            if(length(track@dat2)>0) {
              plotLollipopData(track@dat2, xlim, chr, style@yaxis@draw, gpar(),
                               ybase, side="bottom", main=style@yaxis@main,
                               baselineCol=style@color[2], maxHeight=maxHeight)
            }
            popViewport()## for lollipopData
          }else{
            if(track@type=="scSeq"){
              ##plot yaxis
              drawYaxis(yscale, style@yaxis, curViewStyle)
              pushViewport(viewport(x=curViewStyle@margin[2], y=0, 
                                    height=1, 
                                    width=1-curViewStyle@margin[2]-curViewStyle@margin[4], 
                                    clip="on",
                                    just=c(0,0), 
                                    xscale=xscale, 
                                    yscale=yscale))
              ##grid.clip()
              plotScSeqTrack(track@dat, track@dat2, xlim, style@color, yscale=yscale, 
                                breaks=style@breaks, NAcolor=style@NAcolor)
              popViewport()## for scSeq
            }else{##interactionData
              pushViewport(viewport(x=curViewStyle@margin[2], y=0, 
                                    height=1, 
                                    width=1-curViewStyle@margin[2]-curViewStyle@margin[4], 
                                    clip="on",
                                    just=c(0,0), 
                                    xscale=xscale, 
                                    yscale=yscale))
              ##grid.clip()
              heatlegends <- plotInteractionDataTrack(track@dat, track@dat2, xlim,
                                                      style@color, yscale=yscale, 
                                                      breaks=style@breaks, 
                                                      NAcolor=style@NAcolor,
                                                      style=style@tracktype,
                                                      ysplit=style@ysplit)
              popViewport()## for interactionData
              ##plot yaxis
              drawYaxis(yscale, style@yaxis, curViewStyle, heatlegends)
            }
          }
        }
    }else{##track@type=="transcript" or "gene"
      if(track@type %in% c("transcript", "gene")){
        pushViewport(viewport(x=curViewStyle@margin[2], y=0, 
                              height=1, 
                              width=1-curViewStyle@margin[2]-curViewStyle@margin[4], 
                              clip="on",
                              just=c(0,0), 
                              xscale=xscale))
        plotGeneTrack(track, xlim, chr)
        popViewport()## for data
      }else{##others
        pushViewport(viewport(x=curViewStyle@margin[2], y=0, 
                              height=1, 
                              width=1-curViewStyle@margin[2]-curViewStyle@margin[4], 
                              clip="on",
                              just=c(0,0), 
                              xscale=xscale))
        plotGeneModel(track, xlim, chr) ### currently do the same thing as transcript.
        popViewport()## for data
      }
    }
    
    ## plot wavy line
    if(wavyLine){
        vgap <- convertWidth(unit(0.25, "lines"), 
                             unitTo = "npc",
                             valueOnly = TRUE)
        if(track@type=="data"){
          if(any(yscale==0)){
            y0 <- abs(0-min(yscale))/diff(yscale)
          }else{
            if(all(yscale>0)){
              y0 <- 0
            }else{
              if(all(yscale<0)){
                y0 <- 1
              }else{
                y0 <- abs(0-min(yscale))/diff(yscale)
              }
            }
          }
        }else{
            y0 <- .5
        }
        grid.text(label = "~", x = 1, y=y0, rot = 60)
        grid.text(label = "~", x = 1+vgap, y=y0, rot = 60)
    }
    
    popViewport() ## vp2
    popViewport() ## vp1
    
    ##return
    return(invisible(xy))
}

#' Add annotation markers to the figure at a given position
#' @description A function to add annotation markers for emphasizing interactions 
#' @param obj A \link[InteractionSet:GInteractions-class]{GInteractions} object,
#' \link[GenomicRanges:GRanges-class]{GRanges} object or numeric vector.
#' For numeric vector, the positive value will generate a line with slope 1 and
#' negative value will generate a line at the position with slope -1.
#' @param idx The layer number of track.
#' @param FUN Function for plot. Available functions are 
#' \link[grid:grid.polygon]{grid.polygon}, \link[grid:grid.lines]{grid.lines},
#' and \link[grid:grid.text]{grid.text} for GInteractions object;
#' \link[grid:grid.lines]{grid.lines},
#' and \link[grid:grid.text]{grid.text} for GRanges object;
#' FUN is not used for numeric vector.
#' @param panel Plot regions. Available values are "top", "bottom".
#' @param ... Parameters will be passed to FUN.
#' @return invisible viewport for plot region.
#' @import grid
#' @export
#' @seealso See Also as \code{\link{addGuideLine}}, \code{\link{addArrowMark}}
#' @examples 
#' library(trackViewer)
#' library(InteractionSet)
#' gi <- readRDS(system.file("extdata", "nij.chr6.51120000.53200000.gi.rds",
#'  package="trackViewer"))
#' tads <- GInteractions(
#' GRanges("chr6", 
#'         IRanges(c(51130001, 51130001, 51450001, 52210001), width = 20000)),
#' GRanges("chr6", 
#'         IRanges(c(51530001, 52170001, 52210001, 53210001), width = 20000)))
#' range <- GRanges("chr6", IRanges(51120000, 53200000))
#' tr <- gi2track(gi)
#' viewTracks(trackList(tr), 
#'            gr=range, autoOptimizeStyle = TRUE)
## add TAD information
#' addInteractionAnnotation(tads, "tr", grid.lines,
#'  gp=gpar(col = "#E69F00", lwd=3, lty=3))
#' 
addInteractionAnnotation <- function(obj, idx, FUN=grid.polygon,
                                     panel=c("top", "bottom"), ...){
  if(!inherits(idx, c("numeric", "integer", "character")))
    stop("idx is required as a integer indexes of tracks")
  stopifnot(inherits(obj, c("GInteractions", "GRanges", "numeric", "integer")))
  if(!all(panel %in% c("top", "bottom"))){
    stop("panel must be 'top' and/or 'bottom'")
  }
  if(!is.numeric(obj)){
    stopifnot(is.function(FUN))
    FUN.NAME <- deparse(substitute(FUN))
    if(is(obj, "GInteractions")){
      stopifnot("FUN must be one of grid.polygon, grid.lines, or grid.text"=
                  FUN.NAME %in% c("grid.polygon", "grid.lines", "grid.text"))
    }else{#GRanges
      stopifnot("FUN must be one of grid.lines, or grid.text"=
                  FUN.NAME %in% c("grid.lines", "grid.text"))
    }
  }
  
  dots <- list(...)
  dots <- dots[!names(dots) %in% c("x", "y", "default.units")]
  lastTrackViewer <- getOption("LastTrackViewer")
  if(length(lastTrackViewer)<0) {
    stop("Can not locate the last trackViewer window.")
  }
  xscale <- sort(lastTrackViewer$xscale)
  yscales <- lastTrackViewer$yscales
  .dat2target <- lastTrackViewer$back2back["target2", , drop=TRUE]>0
  vp <- lastTrackViewer$vp
  if(is.character(idx)){
    if(!all(idx %in% names(lastTrackViewer$yHeights))){
      stop("Can not find the input idx from laste trackViewer window.")
    }
    idx <- which(names(lastTrackViewer$yHeights) %in% idx)
  }
  yHeightsLimits2 <- yHeights <- cumsum(lastTrackViewer$yHeights)
  yHeightsLimits1 <- c(0, yHeights)[seq_along(yHeights)]
  names(yHeightsLimits1) <- names(yHeights)
  ym <- (xscale[2]-xscale[1] + 1)/2
  plotInteractionAnno <- function(anchor1, anchor2, scale, FUN, dots, chromsome){
    xinr <- (inRange(start(anchor1), scale) &
               as.character(seqnames(anchor1)) %in% chromsome) |
      (inRange(end(anchor2), scale) &
         as.character(seqnames(anchor2)) %in% chromsome)
    anchor1 <- anchor1[xinr]
    anchor2 <- anchor2[xinr]
    xa <- (end(anchor1) + start(anchor2))/2
    xb <- (start(anchor1) + start(anchor2))/2
    xc <- (start(anchor1) + end(anchor2))/2
    xd <- (end(anchor1) + end(anchor2))/2
    ya <- (xa-end(anchor1)+1)/ym
    yb <- (xb-start(anchor1)+1)/ym
    yc <- (xc-start(anchor1)+1)/ym
    yd <- (xd-end(anchor1)+1)/ym
    if(FUN.NAME=="grid.polygon"){
      merged <- merge_shape(xa, xb, xc, xd, ya, yb, yc, yd)
      for(i in seq_along(merged)){
        args <- c(list(x=merged[[i]]$x,
                       y=merged[[i]]$y,
                       default.units="native"),
                  dots)
        do.call(FUN, args)
      }
    }else{
      for(i in seq_along(anchor1)){
        args <- switch (FUN.NAME,
                        "grid.lines" = c(list(x=c(start(anchor1)[i], xc[i], end(anchor2)[i]), 
                                              y=c(0, yc[i], 0), 
                                              default.units="native"),
                                         dots),
                        "grid.text" = c(list(x=xa[i], 
                                             y=yb[i], 
                                             default.units="native"),
                                        dots),
                        c(list(x=c(xa[i], xb[i], xc[i], xd[i]), 
                               y=c(ya[i], yb[i], yc[i], yd[i]), 
                               default.units="native"),
                          dots)
        )
        do.call(FUN, args)
      }
    }
  }
  plot1Danno <- function(posx, scale, dots){
    xinr <- inRange(abs(posx), scale)
    xc <- posx[xinr]
    for(i in seq_along(xc)){
      args <- c(list(x=c(abs(xc[i]), ifelse(xc[i]>0, scale[2], scale[1])), 
                     y=c(0, ifelse(xc[i]>0,
                                   (scale[2]-xc[i]+1)/ym,
                                   (abs(xc[i])-scale[1]+1)/ym)), 
                     default.units="native"),
                dots)
      do.call(grid.lines, args)
    }
    
  }
  plot2Danno <- function(gr, scale, FUN, dots, chromsome){
    xc <- (start(gr) + end(gr))/2
    yc <- (xc-start(gr)+1)/ym
    xinr <- (inRange(start(gr), scale) | inRange(end(gr), scale)) &
      as.character(seqnames(gr)) %in% chromsome
    for(i in seq_along(gr)){
      if(xinr[i]){
        args <- switch (FUN.NAME,
                        "grid.lines" = c(list(
                          x=c(start(gr)[i], xc[i], end(gr)[i]), 
                          y=c(0, yc[i], 0), 
                          default.units="native"),
                          dots),
                        "grid.text" = c(
                          list(x=xc[i], 
                               y=yc[i], 
                               default.units="native"),
                          dots),
                        c(list(x=xc[i], 
                               y=yc[i], 
                               default.units="native"),
                          dots)
        )
        do.call(FUN, args)
      }
    }
  }
  addAnno <- function(obj, xscale, FUN, dots, chromosome=lastTrackViewer$chromosome){
    if(is(obj, "GInteractions")){
      plotInteractionAnno(first(obj), second(obj), xscale, FUN, dots, chromosome)
    }else{
      if(is.numeric(obj)){
        plot1Danno(obj, xscale, dots)
      }else{
        if(is(obj, "GRanges")){
          plot2Danno(obj, xscale, FUN, dots, chromosome)
        }
      }
    }
  }
  pushViewport(vp)
  for(i in idx){
    currentVP <- viewport(y=mean(c(yHeightsLimits1[i], yHeightsLimits2[i])),
                          height = yHeightsLimits2[i]-yHeightsLimits1[i])
    pushViewport(currentVP)
    pushViewport(viewport(x=0, y=lastTrackViewer$yHeightBottom[i], 
                          height=1-lastTrackViewer$yHeightBottom[i]-lastTrackViewer$yHeightTop[i], 
                          width=1, 
                          clip="on",
                          just=c(0,0), 
                          xscale=xscale, 
                          yscale=sort(yscales[[i]])))
    if(.dat2target[i]){## two interaction heatmap, back to back
      ysplit <- lastTrackViewer$back2back['ysplit', i, drop=TRUE][1]
      ## top triangle
      if("top" %in% panel){
        pushViewport(viewport(x=0, y=1-ysplit, 
                              height=ysplit, 
                              width=1, 
                              clip="on",
                              default.units = "npc",
                              just=c(0,0), 
                              xscale=xscale, 
                              yscale=sort(yscales[[i]])))
        addAnno(obj, xscale, FUN, dots, chromosome=lastTrackViewer$chromosome)
        popViewport()
      }
      if("bottom" %in% panel){
        ## bottom triangle
        pushViewport(viewport(x=0, y=0, 
                              height=1-ysplit, 
                              width=1, 
                              clip="on", 
                              default.units = "npc",
                              just=c(0,0), 
                              xscale=xscale, 
                              yscale=rev(sort(yscales[[i]]))))
        addAnno(obj, xscale, FUN, dots, chromosome=lastTrackViewer$chromosome)
        popViewport()
      }
    }else{
      addAnno(obj, xscale, FUN, dots, chromosome=lastTrackViewer$chromosome)
    }
    
    popViewport()
    popViewport()
  }
  popViewport()
  return(invisible(vp))
}

#' @importFrom utils combn
merge_shape <- function(xa, xb, xc, xd, ya, yb, yc, yd){
  rects <- lapply(seq_along(xa), function(i){
    return(list(x=c(xa[i], xb[i], xc[i], xd[i]),
                y=c(ya[i], yb[i], yc[i], yd[i])))
  })
  r_tree <- list()
  others <- seq_along(rects)
  if(length(others)<2){
    return(rects)
  }
  cbn <- combn(others, 2, simplify = FALSE)
  ol <- vapply(cbn, FUN=function(.ele){
    overlap_region(rects[.ele])
  }, FUN.VALUE = logical(1L))
  cbn <- cbn[ol]
  cbn <- reduce_cbn(cbn)
  others <- setdiff(others, unlist(cbn))
  r_tree <- lapply(cbn, function(.ele){
    filter_points(rects[.ele])
  })
  ## merge points
  r_tree <- mapply(r_tree, cbn, FUN=function(keep, ids){
    pts <- rects[ids]
    x <- unlist(mapply(pts, keep, FUN=function(pt, idx){
      pt$x[idx]
    }, SIMPLIFY = FALSE))
    y <- unlist(mapply(pts, keep, FUN=function(pt, idx){
      pt$y[idx]
    }, SIMPLIFY = FALSE))
    pts <- list(x=x, y=y)
    pts <- do.call(sort_pts, pts)
  }, SIMPLIFY = FALSE)
  r_tree <- c(r_tree, rects[others])
  return(r_tree)
}
overlap_region <- function(xy){
  rx1 <- range(xy[[1]]$x)
  ry1 <- range(xy[[1]]$y)
  rx2 <- range(xy[[2]]$x)
  ry2 <- range(xy[[2]]$y)
  return((inRange(rx1[1], rx2)||inRange(rx1[2], rx2)) &&
           (inRange(ry1[1], ry2)||inRange(ry1[2], ry2)))
}
in_rect <- function(pt, polygon4){
  # calculate the sum of the areas of all possible triangles
  # if the sum is greater than the area of the polygon, the point is outside
  # else if the area of any of the triangles is 0, the point is in the edge
  # else the point is inside the polygon
  cbm <- list(c(1, 4), c(3, 4), c(2, 3), c(1, 2))
  areas <- vapply(cbm, function(.ele){
    xs <- c(polygon4$x[.ele], pt$x)
    ys <- c(polygon4$y[.ele], pt$y)
    get_area_triangle(xs, ys)
  }, FUN.VALUE = numeric(1L))
  area_rect <- get_area_4edge_polygon(polygon4)
  if(sum(areas)>area_rect){
    return(1) ## outside
  }
  if(any(areas==0)){
    return(0) ## at on edge
  }
  return(-1) ## inside
}
get_area_triangle <- function(xs, ys){
  abs(xs[1]*(ys[2]-ys[3])+xs[2]*(ys[3]-ys[1])+xs[3]*(ys[1]-ys[2]))/2
}
get_area_4edge_polygon <- function(polygon){
  #abcd, area(abc) + area(adc)
  abc_x <- polygon$x[c(1, 2, 3)]
  abc_y <- polygon$y[c(1, 2, 3)]
  adc_x <- polygon$x[c(1, 3, 4)]
  adc_y <- polygon$y[c(1, 3, 4)]
  get_area_triangle(abc_x, abc_y) + get_area_triangle(adc_x, adc_y)
}
filter_points <- function(rts){
  # if inside of a rect, remove the points
  lapply(seq_along(rts), function(id){
    .ele <- rts[[id]]
    pts <- lapply(seq_along(.ele$x), function(i){
      list(x=.ele$x[i], y=.ele$y[i])
    })
    keep <- vapply(pts, FUN = function(pt){
      vs <- vapply(rts[-id], FUN = function(pg4){
        in_rect(pt, pg4)
      }, FUN.VALUE = numeric(1L))
      all(vs>0)
    }, FUN.VALUE = logical(1L))
  })
}
reduce_cbn <- function(cbn){
  new_cbn <- list()
  for(i in seq_along(cbn)){
    id <- in_cbn(cbn[[i]], new_cbn)
    if(length(id)>0){
      if(length(id)==1){
        new_cbn[[id[1]]] <- sort(unique(c(new_cbn[[id[1]]], cbn[[i]])))
      }else{
        new_cbn[[id[1]]] <- sort(unique(c(unlist(new_cbn[id]), cbn[[i]])))
        new_cbn <- new_cbn[-id[-1]]
      }
    }else{
      new_cbn <- c(new_cbn, cbn[i])
    }
  }
  return(new_cbn)
}
in_cbn <- function(x, y){
  id <- vapply(y, FUN=function(.ele){
    any(x %in% .ele)
  }, FUN.VALUE = logical(1L))
  which(id)
}
sort_pts <- function(x, y){
  ## sort points by anti-clockwise
  x1 <- x-mean(x)
  y1 <- y-mean(y)
  ord <- order(atan2(y1, x1), sqrt(x1^2 + y1^2))
  list(x=x[ord], y=y[ord])
}

getLabelLeghtRate <- function(SNPs, ratio.yx){
  if(length(SNPs$label.parameter.rot)>0) {
    labels.rot <- SNPs$label.parameter.rot
  }else{
    labels.rot <- 90
  }
  labels.cex <- 1
  if(length(SNPs$label.parameter.gp)>0){
    if(length(SNPs$label.parameter.gp$cex)>0)
      labels.cex <- SNPs$label.parameter.gp$cex[[1]][1]
  }
  labels.length.rate <- labels.cex * 
    max(cospi((labels.rot-90)/180), 0) * ratio.yx
  return(labels.length.rate)
}
getLabelsMaxHeight <- function(SNPs, ratio.yx){
  labels.length.rate <- getLabelLeghtRate(SNPs, ratio.yx)
  stringH <- as.numeric(convertY(stringHeight("W"), "npc"))
  if(length(names(SNPs))>0){
    maxStrHeight <- 
      max(as.numeric(
        convertX(stringWidth(names(SNPs)), "npc")
      ))+stringH
  }else{
    maxStrHeight <- 0
  }
  maxStrHeight <- maxStrHeight * labels.length.rate
}

#' GInteractions operator
#' @description GInteractions operations (add, aubtract, multiply, divide)
#' @param gi_list a list of GInteractions objects
#' @param col colname of metadata to be calculated
#' @param operator operator, "+" means A + B, and so on. 
#' User-defined function also could be used.
#' @return an object of GInteractions
#' @export
#' @examples
#' library(InteractionSet)
#' gr2 <- GRanges(seqnames=c("chr1", "chr1"),
#'                ranges=IRanges(c(7,13), width=3))
#' gr3 <- GRanges(seqnames=c("chr1", "chr1"),
#'                ranges=IRanges(c(1, 4), c(3, 9)))
#' gi <- GInteractions(gr2, gr3, score=c(1, 2))
#' gi2 <- GInteractions(gr2, gr3, score=c(3, 4))
#' GIoperator(list(gi, gi2), col="score", operator="+")
#' GIoperator(list(gi, gi2), col="score", operator="-")

GIoperator <- function(gi_list, col="score",
                       operator=c("+", "-", "*", "/")){
  null <- lapply(gi_list, function(.ele){
    stopifnot(is(.ele, "GInteractions"))
    stopifnot("col in not in metadata of input" =
                col %in% colnames(mcols(.ele)))
  })
  if(!is.function(operator)){
    operator <- match.arg(operator)
    operator <- .Primitive(operator)
  }
  gi_add <- unique(Reduce(c, gi_list))
  mcols(gi_add)[, col] <- 0
  gi_add <- sort(gi_add)
  ol <- findOverlaps(gi_add, gi_list[[1]], type="equal")
  mcols(gi_add)[queryHits(ol), col] <- mcols(gi_list[[1]])[subjectHits(ol), col]
  for(i in seq_along(gi_list)[-1]){
    ol <- findOverlaps(gi_add, gi_list[[i]], type="equal")
    mcols(gi_add)[queryHits(ol), col] <- 
      do.call(operator,
              args=list(mcols(gi_add)[queryHits(ol), col], 
                        mcols(gi_list[[i]])[subjectHits(ol), col]))
  }
  gi_add
}

#' plot GRanges data
#' @description A function to plot GRanges data for given range
#' @param \dots one or more objects of \code{\link[GenomicRanges:GRanges-class]{GRanges}}
#' @param range an object of \code{\link[GenomicRanges:GRanges-class]{GRanges}}
#' @param viewerStyle an object of \code{\link{trackViewerStyle}}
#' @param autoOptimizeStyle should use \code{\link{optimizeStyle}} to optimize style
#' @param newpage should be draw on a new page?
#' @return An object of \code{\link[grid]{viewport}} for \code{\link{addGuideLine}}
#' @import GenomicRanges
#' @export
#' @seealso See Also as \code{\link{addGuideLine}}, \code{\link{addArrowMark}}
#' @examples 
#' gr1 <- GRanges("chr1", IRanges(1:50, 51:100))
#' gr2 <- GRanges("chr1", IRanges(seq(from=10, to=80, by=5),
#'                                seq(from=20, to=90, by=5)))
#' vp <- plotGRanges(gr1, gr2, range=GRanges("chr1", IRanges(1, 100)))
#' addGuideLine(guideLine=c(5, 10, 50, 90), col=2:5, vp=vp)
#' 
#' gr <- GRanges("chr1", IRanges(c(1, 11, 21, 31), width=9), 
#'               score=c(5, 10, 5, 1))
#' plotGRanges(gr, range=GRanges("chr1", IRanges(1, 50)))

plotGRanges <- function(..., range=GRanges(), 
                        viewerStyle=trackViewerStyle(),
                        autoOptimizeStyle=FALSE,
                        newpage=TRUE){
    GRList <- list(...)
    n <- length(GRList)
    if(n==1 && inherits(GRList[[1]], c("list", "GRangesList"))){
        GRList <- GRList[[1]]
        names <- names(GRList)
        if(is.null(names)) names <- paste("track", 1:length(GRList), sep="")
    }else{
        dots <- substitute(list(...))[-1]
        names <- make.names(unlist(sapply(dots, deparse)))
    }
    sta <- sapply(GRList, inherits, what="GRanges")
    if(any(!sta)) stop("All inputs must be objests of GRanges")
    if(missing(range)){
        stop("range is required!")
    }
    if(length(range)==0){
        stop("range is not given!")
    }
    cov <- lapply(GRList, coverageGR)
    tracks <- lapply(cov, function(.ele){
        new("track", dat=orderedGR(.ele), type="data", format="BED")
    })
    names(tracks) <- names
    viewTracks(tracks, gr=range, 
               viewerStyle=viewerStyle,
               autoOptimizeStyle=autoOptimizeStyle,
               newpage=newpage)
}

#' color sheme for the schema for Chromosome Band (Ideogram)
#' @description Describe the colors of giemsa stain results
#' @return A character vector of colors
#' @export
#' @examples 
#' gieStain()
#' 
gieStain <- function(){
  ### gieStain #############################
  # #FFFFFF - gneg    - Giemsa negative bands
  # #999999 - gpos25  - Giemsa positive bands
  # #666666 - gpos50  - Giemsa positive bands
  # #333333 - gpos75  - Giemsa positive bands
  # #000000 - gpos100 - Giemsa positive bands
  # #660033 - acen    - centromeric regions
  # #660099 - gvar    - variable length heterochromatic regions
  # #6600cc - stalk   - tightly constricted regions on the short arms of
  #                     the acrocentric chromosomes
  colorSheme <- c(
    "gneg"    = "#FFFFFF",
    "acen"    = "#660033",
    "gvar"    = "#660099",
    "stalk"   = "#6600CC"
  )
  gposCols <- sapply(1:100, function(i){
    i <- as.hexmode(round(256-i*2.56, digits = 0))
    i <- toupper(as.character(i))
    if(nchar(i)==1) i <- paste0("0", i)
    return(paste0("#", i, i, i))
  })
  names(gposCols) <- paste0("gpos", 1:100)
  colorSheme <- c(gposCols, colorSheme)
  colorSheme
}

#' plot ideogram
#' @description plot ideogram for one chromosome 
#' @param ideo output of \link{loadIdeogram}.
#' @param chrom A length 1 character vector of chromosome name.
#' @param colorSheme A character vector of giemsa stain colors.
#' @param gp parameters used for \link[grid]{grid.roundrect}.
#' @param ... parameters not used.
#' @import grid
#' @examples 
#' \dontrun{
#' ideo <- loadIdeogram("hg38")
#' library(grid)
#' grid.newpage()
#' plotIdeo(ideo)
#' }
#' 
#' 
plotIdeo <- function(ideo, chrom=seqlevels(ideo)[1], 
                     colorSheme=gieStain(), gp=gpar(fill=NA), ...){
  stopifnot(is(ideo, "GRanges"))
  stopifnot(length(chrom)==1)
  ideo <- ideo[seqnames(ideo) %in% chrom]
  strand(ideo) <- "*"
  ideo <- sort(ideo)
  ol <- findOverlaps(ideo, drop.self=TRUE, drop.redundant=TRUE, minoverlap = 2)
  if(length(ol)>0){
    stop("There is overlaps in ideo.")
  }
  stopifnot(!missing(colorSheme))
  stopifnot(length(names(colorSheme))>0)
  stopifnot(all(ideo$gieStain %in% names(colorSheme)))
  ### split the ideogram into p arm and q arm
  stopifnot(length(ideo$name)==length(ideo))
  ideoName <- tolower(as.character(ideo$name))
  ### first character should be p or q
  arms <- substr(ideoName, 1, 1)
  if(!all(arms %in% c("p", "q"))){
    stop("p arm or q arm only.")
  }
  ideo <- split(ideo, arms)
  ra <- lapply(ideo, function(.ele){## not use range to avoid warning
    c(min(start(.ele)) , max(end(.ele)))
  })
  if(all(c("p", 'q') %in% names(ra))){
    ## coordinates of p arm should be smaller than q arm
    if(ra$p[2]>ra$q[1]){
      stop("coordinates of p arm should be smaller than q arm")
    }
  }
  ## viewports with xscale=c(start.p.arm, end.q.arm)
  vp_pq <- viewport(xscale=range(unlist(ra)))
  pushViewport(vp_pq)
  GRanges2raster <- function(ranges, gieStain){
    wid <- width(ranges) - 1
    hcf <- function(x){
      if(length(x)>2){
        x <- x[-length(x)]
      }
      far <- range(nchar(as.character(x)))
      if(far[2]<3) return(1)
      fa <- 10^(far[2]:far[1]-2)
      fa <- fa[fa>1]
      for(i in fa){
        if(all(x %% i == 0)) return(i)
      }
      if(far[1]>2){
        return(10^(far[1]-2))
      }else{
        return(1)
      }
    }
    wid <- round(wid/hcf(wid), digits = 0)
    matrix(rep(colorSheme[as.character(gieStain)], wid), nrow=1)
  }
  
  rrpoints <- function(x) {
    left <- 0
    bottom <- 0
    right <- convertX(unit(1, "npc"), "inches", valueOnly=TRUE)
    top <- convertY(unit(1, "npc"), "inches", valueOnly=TRUE)
    rw <- convertWidth(x$r, "inches", valueOnly=TRUE)
    rh <- convertHeight(x$r, "inches", valueOnly=TRUE)
    r <- min(rw, rh)
    if(right>=top){
      rect <- c(x1=left+r, y1=bottom, x2=right-r, y2=top, r=r, horiz=1)
    }else{
      rect <- c(x1=left, y1=bottom+r, x2=right, y2=top-r, r=r, horiz=0)
    }
    return(rect)
  }
  
  inMask <- function(x, y, cx, cy, r){
    d <- sqrt((x-cx)^2+(y-cy)^2)
    return(d<=r)
  }
  imgpoints <- function(img, mask){
    img <- matrix(rep(img, 1000), nrow=1000, byrow = TRUE)
    x <- unit((1:ncol(img))/ncol(img), "npc")
    y <- unit((1000:1)/1000, "npc")
    xx <- convertX(x, "inch", valueOnly = TRUE)
    yy <- convertY(y, "inch", valueOnly = TRUE)
    if(mask["horiz"]==1){
      ## xx < mask["x1"] , leftCir
      for(i in which(xx < mask["x1"])){
        img[!inMask(xx[i], yy, cx=mask["x1"], cy=mask["r"], r=mask["r"]), i] <- NA
      }
      ## xx > mask["x2"], rightCir
      for(i in which(xx > mask["x2"])){
        img[!inMask(xx[i], yy, cx=mask["x2"], cy=mask["r"], r=mask["r"]), i] <- NA
      }
    }else{
      ## yy < mask["y1"], bottomCir
      for(i in which(yy < mask["y1"])){
        img[i, !inMask(xx, yy[i], cx=mask["r"], cy=mask["y1"], r=mask["r"])] <- NA
      }
      ## yy > mask["y2"], topCir
      for(i in which(yy > mask["y2"])){
        img[i, !inMask(xx, yy[i], cx=mask["r"], cy=mask["y2"], r=mask["r"])] <- NA
      }
    }
    
    img
  }
  # p arm
  if(length(ra$p)){
    vp_p <- viewport(x = ra$p[1], width=diff(ra$p), just="left", 
                     default.units="native", xscale=ra$p)
    pushViewport(vp_p)
    image <- GRanges2raster(ranges(ideo$p), ideo$p$gieStain)
    roundrect <- roundrectGrob(r=unit(.5, "snpc"))
    roundrect <- rrpoints(roundrect)
    image <- imgpoints(image, roundrect)
    grid.raster(image=image, height=1, width=1, interpolate=FALSE)
    grid.roundrect(r=unit(.5, "snpc"), gp=gp)
    popViewport()
  }
  # q arm
  if(length(ra$q)){
    vp_q <- viewport(x = ra$q[1], width=diff(ra$q), just="left", 
                     default.units="native", xscale=ra$q)
    pushViewport(vp_q)
    image <- GRanges2raster(ranges(ideo$q), ideo$q$gieStain)
    roundrect <- roundrectGrob(r=unit(.5, "snpc"))
    roundrect <- rrpoints(roundrect)
    image <- imgpoints(image, roundrect)
    grid.raster(image=image, height=1, width=1, interpolate=FALSE)
    grid.roundrect(r=unit(.5, "snpc"), gp=gp)
    popViewport()
  }
  popViewport()
}

#' parse text into GRanges
#' @description parse text like "chr13:99,443,451-99,848,821:-" into GRanges
#' @param text character vector like "chr13:99,443,451-99,848,821:-" or 
#' "chr13:99,443,451-99,848,821"
#' @return an object of \link[GenomicRanges:GRanges-class]{GRanges}
#' @import GenomicRanges
#' @export
#' @examples 
#' parse2GRanges("chr13:99,443,451-99,848,821:-")
#' 

parse2GRanges <- function(text){
    ## chr13:99,443,451-99,848,821:-
    x <- gsub(",", "", text)
    x <- do.call(rbind, strsplit(x, ":"))
    if(ncol(x)==1){
        stop("Can not handle the text.")
    }
    coor <- do.call(rbind, strsplit(x[,2], "\\-"))
    if(ncol(coor)!=2){
        stop("Can not handle the text.")
    }
    if(ncol(x)==2){
        x <- cbind(x, "*")
    }
    GRanges(x[,1], 
            IRanges(as.numeric(coor[,1]), as.numeric(coor[,2])),
            strand=x[,3])
}

convertHeight2NPCnum <- function(.ele){
  if(is(.ele, "unit")){
    return(convertHeight(.ele, unitTo="npc", valueOnly=TRUE))
  }else{
    if(is.list(.ele)){
      .ele <- sapply(.ele, function(.e){
        if(is(.e, "unit")){
          .e <- convertHeight(.e, unitTo="npc", valueOnly=TRUE)
        }
        .e[1]
      })
      return(unlist(.ele))
    }else{
      if(is.numeric(.ele)){
        return(.ele)
      }else{
        if(is.integer(.ele)){
          return(.ele)
        }else{
          return(.ele)
        }
      }
    }
  }
}
plotFeatures <- function(feature.splited, LINEH, bottomHeight, 
                         label_on_feature=FALSE){
    feature.height <- 0
    for(n in seq_along(feature.splited)){
        this.feature.height <- 
            max(c(feature.splited[[n]]$height/2, 
                  .0001)) + 0.2 * LINEH
        feature.height <- feature.height + this.feature.height
        ##baseline
        grid.lines(x=c(0, 1), y=c(bottomHeight+feature.height, 
                                  bottomHeight+feature.height))
        for(m in seq_along(feature.splited[[n]])){
            this.dat <- feature.splited[[n]][m]
            color <- if(is.list(this.dat$color)) this.dat$color[[1]] else 
                this.dat$color
            if(length(color)==0) color <- "black"
            fill <- if(is.list(this.dat$fill)) this.dat$fill[[1]] else 
                this.dat$fill
            if(length(fill)==0) fill <- "white"
            this.cex <- if(length(this.dat$cex)>0) this.dat$cex[[1]][1] else 1
            lwd <- if(length(this.dat$lwd)>0) this.dat$lwd[[1]][1] else 1
            this.feature.height.m <- 
                if(length(this.dat$height)>0) 
                    this.dat$height[[1]][1] else 
                        2*this.feature.height
            grid.rect(x=start(this.dat)-.1, y=bottomHeight+feature.height, 
                      width=width(this.dat)-.8, 
                      height=this.feature.height.m,
                      just="left", gp=gpar(col=color, fill=fill, lwd=lwd), 
                      default.units = "native")
            if(!is.null(names(this.dat))){
              if(label_on_feature & !is.na(names(this.dat)[1])){
                grid.text(x=(start(this.dat)+end(this.dat))/2, 
                          y=bottomHeight+feature.height,
                          just = "centre",
                          label = names(this.dat)[1],
                          gp= gpar(list(cex=this.cex * 
                                          this.feature.height.m/
                                          this.feature.height,
                                        color=color)), 
                          default.units = "native")
              }
            }
        }
        feature.height <- feature.height + this.feature.height
    }
    feature.height
}
getDrawLabelParam <- function(SNPs){
  label.parameter.draw <- rep(TRUE, length(SNPs))
  if(length(SNPs$label.parameter.draw)==length(SNPs)){
    label.parameter.draw <- vapply(SNPs$label.parameter.draw, `[`, i=1,
                                   FUN.VALUE = logical(1L))
  }
  label.parameter.draw
}
handleLabelParams <- function(SNPs, prefix="label.parameter.", cex=1,
                              ...){
  labels <- list(...)
  ## change the parameter by use definitions.
  for(label.parameter in names(labels)){
    label.para <- paste0(prefix, label.parameter)
    if(label.para %in% colnames(mcols(SNPs))){
      labels[[label.parameter]] <- mcols(SNPs)[, label.para]
    }
  }
  for(label.parameter in c("col", "fill", "alpha", "lty", "lwd", "lex",
                          "lineend", "linejoin", "linemitre",
                          "fontsize", "cex", "fontfamily", "fontface",
                          "lineheight", "font")){
    label.para <- paste0(prefix, label.parameter)
    if(label.para %in% colnames(mcols(SNPs))){
      labels$gp[[label.parameter]] <- mcols(SNPs)[, label.para]
    }
  }
  if(!"cex" %in% names(labels$gp)){
    labels$gp$cex <- cex
  }
  mergeList <- function(.ele){
    .ele <- do.call(list, .ele)
    .n <- unique(unlist(lapply(.ele, names)))
    .out <- list()
    if(length(.n)>0){
      for(.name in .n){
        .out[[.name]] <- sapply(.ele, function(.e){
          if(.name %in% names(.e)){
            .e[[.name]][1]
          }else{
            NA
          }
        })
      }
    }else{
      .n <- unique(names(.ele))
      for(.name in .n){
        .out[[.name]] <- unlist(.ele[names(.ele) %in% .name])
      }
    }
    .out
  }
  labels$gp <- mergeList(labels$gp)
  labels$gp[duplicated(names(labels$gp))] <- NULL
  labels$gp <- do.call(gpar, labels$gp)
  return(labels)
}
filterThisLabel <- function(this.label){
  na_label <- is.na(this.label$label)
  for(key in c("x", "just", "hjust", "vjust", "rot",
               "check.overlap", "default.units")){
    if(length(this.label[[key]])>1 &&
       length(this.label[[key]]==length(this.label$label))){
      this.label[[key]] <- this.label[[key]][!na_label]
    }
  }
  for(key in names(this.label$gp)){
    if(length(this.label$gp[[key]])>1 &&
       length(this.label$gp[[key]])==length(this.label$label)){
      this.label$gp[[key]] <- this.label$gp[[key]][!na_label]
    }
  }
  this.label$gp <- do.call(gpar, this.label$gp)
  this.label$label <- this.label$label[!na_label]
  this.label
}
plotLollipops <- function(SNPs, feature.height, bottomHeight, baseline, 
                          type, ranges, yaxis, yaxis.gp, scoreMax, scoreMax0, scoreType,
                          LINEW, cex, ratio.yx, GAP, pin, dashline.col,
                          side=c("top", "bottom"), jitter=c("node", "label"),
                          main=TRUE){
    side <- match.arg(side)
    jitter <- match.arg(jitter)
    if(side=="top"){
        pushViewport(viewport(y=bottomHeight,
                              height=1,
                              just="bottom",
                              xscale=c(start(ranges), 
                                       end(ranges)),
                              clip="off"))
    }else{
        pushViewport(viewport(y=bottomHeight+feature.height,
                              height=1,
                              just="top",
                              xscale=c(start(ranges), 
                                       end(ranges)),
                              yscale=c(1, 0),
                              clip="off"))
    }
    
    if(type=="pie.stack" && length(SNPs$stack.factor)>0){
        stopifnot(is.vector(SNPs$stack.factor, mode="character"))
        if(length(SNPs$stack.factor.order)>0 || 
           length(SNPs$stack.factor.first)>0){
            warning("stack.factor.order and stack.factor.first are used by this function!",
                    "The values in these column will be removed.")
        }
        ## condense the SNPs
        stack.factors <- unique(as.character(SNPs$stack.factor))
        stack.factors <- sort(stack.factors)
        stack.factors.order <- seq_along(stack.factors)
        names(stack.factors.order) <- stack.factors
        SNPs <- SNPs[order(as.character(seqnames(SNPs)), start(SNPs), 
                           as.character(SNPs$stack.factor))]
        SNPs$stack.factor.order <- stack.factors.order[SNPs$stack.factor]
        SNPs$stack.factor.first <- !duplicated(SNPs)
        SNPs.condense <- SNPs
        SNPs.condense$oid <- seq_along(SNPs)
        SNPs.condense$factor <- paste(as.character(seqnames(SNPs)), start(SNPs), end(SNPs))
        SNPs.condense <- split(SNPs.condense, SNPs.condense$factor)
        SNPs.condense <- lapply(SNPs.condense, function(.ele){
            .oid <- .ele$oid
            .gr <- .ele[1]
            mcols(.gr) <- NULL
            .gr$oid <- NumericList(.oid)
            .gr
        })
        SNPs.condense <- unlist(GRangesList(SNPs.condense), use.names = FALSE)
        SNPs.condense <- sort(SNPs.condense)
        lab.pos.condense <- start(SNPs.condense)
        label.parameter.draw <- rep(TRUE, length(SNPs.condense))
        if(length(SNPs$label.parameter.draw)==length(SNPs)){
          label.parameter.draw <- 
            SNPs$label.parameter.draw[vapply(SNPs.condense$oid, `[`, i=1,
                                             FUN.VALUE = numeric(1L))]
          label.parameter.draw <- vapply(label.parameter.draw, `[`, i=1,
                                         FUN.VALUE = logical(1L))
        }
        lab.pos.condense[label.parameter.draw] <-
          jitterLables(start(SNPs.condense)[label.parameter.draw], 
                                         xscale=c(start(ranges), end(ranges)), 
                                         lineW=LINEW*cex)
        lab.pos.condense[label.parameter.draw] <-
          reAdjustLabels(lab.pos.condense[label.parameter.draw],
                         lineW=LINEW*cex)
        condense.ids <- SNPs.condense$oid
        lab.pos <- rep(lab.pos.condense, elementNROWS(condense.ids))
        lab.pos <- lab.pos[order(unlist(condense.ids))]
    }else{
        lab.pos <- start(SNPs)
        label.parameter.draw <- getDrawLabelParam(SNPs)
        lab.pos[label.parameter.draw] <-
          jitterLables(start(SNPs)[label.parameter.draw], 
                       xscale=c(start(ranges), end(ranges)), 
                       lineW=LINEW*cex)
        lab.pos[label.parameter.draw] <-
          reAdjustLabels(lab.pos[label.parameter.draw], lineW=LINEW*cex)
    }
    if(length(SNPs)>0){
        yaxisat <- NULL
        yaxisLabel <- TRUE
        diameter <- LINEW*ratio.yx
        y2 <- feature.height
        y3 <- 4*GAP*cex
        y4 <- 2.5*GAP*cex
        if(length(yaxis)>1 && is.numeric(yaxis)){
            yaxisat <- yaxis
            if(length(names(yaxis))==length(yaxis)) yaxisLabel <- names(yaxis)
            yaxis <- TRUE
        }
        if(yaxis && scoreMax>1 && !type %in% c("pie", "pie.stack")){
            if(side=="top"){
              vp <- viewport(x=.5+ifelse(main, -1, 1) *LINEW,
                             y=feature.height + y3 + y4 + scoreMax*diameter/2,
                             width=1,
                             height=(scoreMax+1)*diameter,
                             yscale=c(0, scoreMax0+scoreMax0/scoreMax))
            }else{
              vp <- viewport(x=.5+ifelse(main, -1, 1) *LINEW,
                             y=1-(feature.height + y3 + y4 +
                                    scoreMax*diameter/2),
                             width=1,
                             height=(scoreMax+1)*diameter,
                             yscale=c(scoreMax0+scoreMax0/scoreMax, 0))
            }
          grid.yaxis(at=yaxisat,
                     label=yaxisLabel,
                     main = main,
                     gp=yaxis.gp,
                     vp=vp)
        }
        
        if(length(SNPs$alpha)==length(SNPs)){
          SNPs$alpha[is.na(SNPs$alpha)] <- 0
          if(all(is.numeric(SNPs$alpha))){
            if(any(SNPs$alpha>1)){## convert to 0-1
              SNPs$alpha <- SNPs$alpha/max(SNPs$alpha)
            }
          }else{ ## not correct format.
            SNPs$alpha <- as.numeric(factor(as.character(SNPs$alpha)))
            SNPs$alpha <- (SNPs$alpha+max(SNPs$alpha))/max(SNPs$alpha)/2
          }
        }else{
          SNPs$alpha <- NULL
        }
        if(type=="circle"){
          if(length(SNPs$shape)==length(SNPs)){
            ## shape could only be "circle", "square", "diamond", "triangle_point_up", "triangle_point_down"
            if(!all(unlist(SNPs$shape) %in% c("circle", "square", "diamond", "triangle_point_up", "triangle_point_down"))){
              message('shape must be "circle", "square", "diamond", "triangle_point_up", or "triangle_point_down"')
              if(is.list(SNPs$shape)){
                stop("The shape is a list, please confirm the format.",
                     'It must be a list with elements of "circle", "square", "diamond", "triangle_point_up", or "triangle_point_down"')
              }
              SNPs$shape <- as.numeric(factor(SNPs$shape))
              SNPs$shape <- rep(c("circle", "square", "diamond", "triangle_point_up", "triangle_point_down"), 
                                max(SNPs$shape))[SNPs$shape]
            }else{
              if(is.list(SNPs$shape)){
                if(any(lengths(SNPs$shape)[SNPs$score!=0]==0)){
                  stop("The shape is a list, but zero length of shape is detected.")
                }
              }
            }
            if(scoreType){
              if(is.list(SNPs$shape)){
                if(!all(lengths(SNPs$shape)[SNPs$score!=0]==
                        SNPs$score[SNPs$score!=0])){
                  warning('Not all the length of shape equal to score.')
                }
              }
            }
          }else{
            SNPs$shape <- NULL
          }
        }
        for(m in seq_along(SNPs)){
            this.dat <- SNPs[m]
            color <- if(is.list(this.dat$color)) this.dat$color[[1]] else this.dat$color
            border <- 
                if(is.list(this.dat$border)) this.dat$border[[1]] else this.dat$border
            fill <- if(is.list(this.dat$fill)) this.dat$fill[[1]] else this.dat$fill
            alpha <- if(length(this.dat$alpha)>0) this.dat$alpha[[1]] else 1
            lwd <- if(is.list(this.dat$lwd)) this.dat$lwd[[1]] else this.dat$lwd
            shape <- if(length(this.dat$shape)>0) this.dat$shape[[1]] else "circle"
            rot <- if(length(this.dat$label.rot)>0) this.dat$label.rot else 15
            this.cex <- if(length(this.dat$cex)>0) this.dat$cex[[1]][1] else 1
            this.dashline.col <- 
              if(length(this.dat$dashline.col)>0) this.dat$dashline.col[[1]][1] else dashline.col
            ## control plot the dash line or not
            if(length(names(this.dat))<1) this.dashline.col <- NA
            if(length(SNPs$label.parameter.label)==length(SNPs) && length(SNPs)>0){
              if(this.dat$label.parameter.label[[1]]=="" ||
                 is.na(this.dat$label.parameter.label[[1]])) this.dashline.col <- NA
            }
            if(length(SNPs$label.parameter.pfm)==length(SNPs) && length(SNPs)>0){
              if(is.null(SNPs$label.parameter.pfm[[m]])){
                this.dashline.col <- NA
              }
            }
            if(length(SNPs$label.parameter.draw)==length(SNPs) && length(SNPs)>0){
              if(!(SNPs$label.parameter.draw[[m]])){
                this.dashline.col <- NA
              }
            }
            id <- 
              handleLabelParams(this.dat, cex = this.cex, prefix = "node.label.",
                                label = if(is.character(this.dat$label)) this.dat$label else
                                  if(is.character(this.dat$node.label)) this.dat$node.label else NA,
                                rot = if(length(this.dat$label.rot)>0) this.dat$label.rot else ifelse(type=="flag", 15, 0),
                                gp = gpar(cex=this.cex,
                                          col = if(length(this.dat$label.col)>0) this.dat$label.col else "black"),
                                just = "centre",
                                hjust = .5,
                                vjust = .5)
            this.dat.mcols <- mcols(this.dat)
            this.dat.mcols <- cleanDataMcols(this.dat.mcols, type)

            grid.lollipop(x1=convertX(unit(start(this.dat), "native"), "npc", 
                                      valueOnly=TRUE),  
                          y1=baseline,
                          x2=convertX(unit(ifelse(jitter=="node", 
                                                  lab.pos[m], 
                                                  start(this.dat)), 
                                           "native"), "npc", valueOnly=TRUE), 
                          y2=y2,
                          y3=y3, y4=y4, 
                          radius=LINEW/2,
                          col=color,
                          border=border,
                          percent=this.dat.mcols,
                          edges=100,
                          type=type,
                          ratio.yx=ratio.yx,
                          pin=pin,
                          scoreMax=scoreMax * LINEW,
                          scoreType=scoreType,
                          id=id,
                          cex=this.cex, lwd=lwd, dashline.col=this.dashline.col,
                          side=side, alpha=alpha, shape=shape)

        }
        this.height <- getHeight(SNPs, 
                                 ratio.yx, LINEW, GAP, cex, type,
                                 scoreMax=scoreMax,
                                 level="data")
        labels.keep <- getDrawLabelParam(SNPs)
        SNPs <- SNPs[labels.keep]
        lab.pos <- lab.pos[labels.keep]
        if(length(names(SNPs))>0){
            if(type=="pie.stack"){
                ## unique lab.pos and SNPs
                idx <- !duplicated(names(SNPs))
                lab.pos <- lab.pos[idx]
                SNPs <- SNPs[idx]
            }
            this.label <- 
              handleLabelParams(SNPs, cex = cex,
                                prefix = "label.parameter.",
                                x = lab.pos,
                                label = names(SNPs),
                                just = ifelse(side=="top", "left", "right"),
                                hjust = NULL,
                                vjust = NULL,
                                rot = 90,
                                check.overlap = FALSE,
                                default.units = "native",
                                gp = gpar(cex=cex),
                                pfm = NULL,
                                font = "Helvetica-Bold",
                                fontface = "bold",
                                ic.scale = TRUE)
            if(jitter=="label"){
              ## add guide lines
              rased.height <- 4*GAP*cex
              guide.height <- 2.5*GAP*cex
              for(i in seq_along(SNPs)){
                this.dashline.col <- 
                  if(length(SNPs[i]$dashline.col)>0) 
                    SNPs[i]$dashline.col[[1]][1] else 
                      dashline.col
                if(length(this.label$label)<1 && length(this.label$pfm)<1){
                  next
                }
                if(length(this.label$label)==length(SNPs) && length(SNPs)>0){
                  if(is.na(this.label$label[i])){
                    next
                  }
                  if(this.label$label[i]==""){
                    next
                  }
                }
                if(length(this.label$pfm)==length(SNPs) && length(SNPs)>0){
                  if(is.null(this.label$pfm[[i]])){
                    next
                  }
                }
                grid.lines(x=c(start(SNPs[i]), this.label$x[i]), 
                           y=c(this.height+feature.height-cex*LINEW, 
                               this.height+feature.height+rased.height),
                           default.units = this.label$default.units,
                           gp=gpar(col=this.dashline.col, lty=3))
                grid.lines(x=c(this.label$x[i], this.label$x[i]),
                           y=c(this.height+rased.height+feature.height,
                               this.height+rased.height+
                                 guide.height+feature.height),
                           default.units = this.label$default.units,
                           gp=gpar(col=this.dashline.col, lty=3))
              }
              ## add this height
              this.height <- this.height + rased.height + guide.height
            }
            if(length(this.label$pfm)>0){
              if(!requireNamespace("motifStack", quietly = TRUE)){
                stop("When plot motifs as labels,",
                     " the Bioconductor package 'motifStack' is required!")
              }
              for(idx in seq_along(this.label$pfm)){
                if(!is.null(this.label$pfm[[idx]])){
                  this_cex <- ifelse(length(cex)==length(this.label$pfm),
                                     cex[[idx]], cex[1])
                  this_just <- ifelse(length(this.label$just)==length(this.label$pfm),
                                      this.label$just[[idx]], this.label$just[1])
                  this_rot <- ifelse(length(this.label$rot)==length(this.label$pfm),
                                     this.label$rot[[idx]], this.label$rot[1])
                  this_font <- ifelse(length(this.label$font)==length(this.label$pfm),
                                      this.label$font[[idx]], this.label$font[1])
                  this_fontface <- ifelse(length(this.label$fontface)==length(this.label$pfm),
                                          this.label$fontface[[idx]], this.label$fontface[1])
                  this_ic.scale <- ifelse(length(this.label$ic.scale)==length(this.label$pfm),
                                          this.label$ic.scale[[idx]], this.label$ic.scale[1])
                  pushViewport(viewport(x=this.label$x[[idx]],
                                        y=this.height + feature.height,  
                                        just = this_just,
                                        width =  convertWidth(
                                          stringWidth(paste(rep("A",
                                                                ncol(this.label$pfm[[idx]]@mat)),
                                                            collapse = "")), 
                                                     unitTo="npc",
                                                     valueOnly=FALSE),
                                        height = LINEW * this_cex,
                                        angle = this_rot,
                                        default.units = this.label$default.units))
                  motifStack::plotMotifLogoA(pfm = this.label$pfm[[idx]],
                                 font=this_font,
                                 fontface = this_fontface,
                                 ic.scale = this_ic.scale)
                  popViewport()
                }
              }
            }else{
              if(any(is.na(this.label$label))){
                ## grid.text will not plot if first element is empty
                this.label <- filterThisLabel(this.label)
              }
              if(length(this.label$label)>0){
                grid.text(x=this.label$x, y=this.height + feature.height, 
                          label = this.label$label,  
                          just = this.label$just, 
                          hjust = this.label$hjust,
                          vjust = this.label$vjust,
                          rot=this.label$rot,
                          check.overlap = this.label$check.overlap,
                          default.units = this.label$default.units,
                          gp=this.label$gp)
              }
            }
        }
    }
    popViewport()
}

plotLegend <- function(legend, this.height, LINEH){
    ypos <- this.height
    pch <- 21
    if(length(legend)>0){
        if(is.list(legend)){
            thisLabels <- legend[["labels"]]
            if(is.null(legend$gp)){
              gp <- legend[!names(legend) %in% formalArgs(legendGrob)]
              class(gp) <- "gpar"
              legend$gp <- gp
            }else{
              gp <- legend$gp
            }
            if(is.null(gp$cex)) gp$cex <- 1
        }else{
          thisLabels <- names(legend)
          gp <- gpar(fill=legend, cex=1)
          legend <- list(
            labels = thisLabels,
            gp = gp
          )
        }
        if(length(thisLabels)>0){ 
          if(is.null(legend$byrow)) legend$byrow <- TRUE
          if(is.null(legend$vgap)) legend$vgap <- unit(.1*gp$cex[1], "lines")
          if(is.null(legend$hgap)) legend$hgap <- unit(.5*gp$cex[1], "lines")
          if(is.null(legend$ncol)){
            legend$ncol <- getColNum(thisLabels, cex=gp$cex)
          }
          ncol <- legend$ncol
          if(is.null(legend$pch)) legend$pch <- pch
          legend <- legend[names(legend) %in% formalArgs(legendGrob)]
          topblank <- ceiling(length(thisLabels) / ncol) * gp$cex[1]
          pushViewport(viewport(x=.5, 
                                y=ypos+(topblank+.2*gp$cex[1])*LINEH/2, 
                                width=1,
                                height=topblank*LINEH,
                                just="bottom"))
          this.height <- ypos + (topblank+.2*gp$cex[1])*LINEH
          do.call(grid.legend, legend)
          popViewport()
        }
    }
    this.height + LINEH
}

---
title: "trackViewer Vignette: lollipopPlot"
author: "Jianhong Ou, Lihua Julie Zhu"
date: "`r BiocStyle::doc_date()`"
package: "`r BiocStyle::pkg_ver('trackViewer')`"
abstract: >
  Visualize methylation or mutation sites along with annotation as track layers.
vignette: >
  %\VignetteIndexEntry{trackViewer Vignette: lollipopPlot}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  html_document:
    theme: simplex
    toc: true
    toc_float: true
    toc_depth: 4
    fig_caption: true
---

```{r, echo=FALSE, results="hide", warning=FALSE}
suppressPackageStartupMessages({
    library(trackViewer)
    library(rtracklayer)
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    library(org.Hs.eg.db)
    library(VariantAnnotation)
  library(httr)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)
```

# Lolliplot

Lolliplot is for the visualization of the methylation/variant/mutation data. 

```{r lolliplot1, fig.width=6, fig.height=3}
library(trackViewer)
SNP <- c(10, 12, 1400, 1402)
sample.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)))
features <- GRanges("chr1", IRanges(c(1, 501, 1001), 
                                    width=c(120, 400, 405),
                                    names=paste0("block", 1:3)))
lolliplot(sample.gr, features)
## More SNPs
SNP <- c(10, 100, 105, 108, 400, 410, 420, 600, 700, 805, 840, 1400, 1402)
sample.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)))
lolliplot(sample.gr, features)
## Define the range
lolliplot(sample.gr, unname(features), ranges = GRanges("chr1", IRanges(104, 109)))
```

## Change the lolliplot color
### Change the color of the features.
```{r lolliplot2, fig.width=6, fig.height=3}
features$fill <- c("#FF8833", "#51C6E6", "#DFA32D")
lolliplot(sample.gr, features)
```

### Change the color and opacity of the lollipop.
```{r lolliplot3, fig.width=6, fig.height=3}
sample.gr$color <- sample.int(6, length(SNP), replace=TRUE)
sample.gr$border <- sample(c("gray80", "gray30"), length(SNP), replace=TRUE)
sample.gr$alpha <- sample(100:255, length(SNP), replace = TRUE)/255
lolliplot(sample.gr, features)
```


## Add the index labels in the node
Users can control the node labels one by one by setting metadata start with 'node.label.' 
Please note that for each node label, the node.label.gp must be a list to control the style of node labels.
You can also simply use node.label.col, node.label.cex, node.label.fontsize, node.label.fontfamily, node.label.fontface, node.label.font to assign the node labels attributes.
```{r lolliplot.index, fig.width=6, fig.height=3}
sample.gr$node.label <- as.character(seq_along(sample.gr))
sample.gr$node.label.col <- 
  ifelse(sample.gr$alpha>0.5 | sample.gr$color==1, "white", "black")
sample.gr$node.label.cex <- sample.int(3, length(sample.gr), replace = TRUE)/2
lolliplot(sample.gr, features)
sample.gr$node.label.cex <- 1 ## change it back for pretty showcase.
```

## Change the height of the features
```{r lolliplot4, fig.width=6, fig.height=3}
features$height <- c(0.02, 0.05, 0.08)
lolliplot(sample.gr, features)
## Specifying the height and its unit
features$height <- list(unit(1/16, "inches"),
                        unit(3, "mm"),
                        unit(12, "points"))
lolliplot(sample.gr, features)
```

## Plot multiple transcripts in the features
The metadata 'featureLayerID' are used for drawing features in different layers.
```{r lolliplot.mul.features, fig.width=6, fig.height=3}
features.mul <- rep(features, 2)
features.mul$height[4:6] <- list(unit(1/8, "inches"),
                                 unit(0.5, "lines"),
                                 unit(.2, "char"))
features.mul$fill <- c("#FF8833", "#F9712A", "#DFA32D", 
                       "#51C6E6", "#009DDA", "#4B9CDF")
end(features.mul)[5] <- end(features.mul[5])+50
features.mul$featureLayerID <- 
    paste("tx", rep(1:2, each=length(features)), sep="_")
names(features.mul) <- 
    paste(features.mul$featureLayerID, 
          rep(1:length(features), 2), sep="_")
lolliplot(sample.gr, features.mul)
## One name per transcript
names(features.mul) <- features.mul$featureLayerID
lolliplot(sample.gr, features.mul)
```

## Change the height of a lollipop plot
```{r lolliplot4.1, fig.width=6, fig.height=3.5}
#Note: the score value is an integer less than 10
sample.gr$score <- sample.int(5, length(sample.gr), replace = TRUE)
lolliplot(sample.gr, features)
##Remove y-axis
lolliplot(sample.gr, features, yaxis=FALSE)
```
```{r lolliplot4.20, fig.width=6, fig.height=4.5}
#Try a score value greater than 10
sample.gr$score <- sample.int(15, length(sample.gr), replace=TRUE)
sample.gr$node.label <- as.character(sample.gr$score)
lolliplot(sample.gr, features)
```
```{r lolliplot4.21, fig.width=6, fig.height=5}
#increase the cutoff value of style switch.
lolliplot(sample.gr, features, lollipop_style_switch_limit=15)
```
```{r lolliplot4.22, fig.width=6, fig.height=4.5}
#Try a float numeric score
sample.gr$score <- runif(length(sample.gr))*10
sample.gr$node.label <- as.character(round(sample.gr$score, digits = 1))
lolliplot(sample.gr, features)
# Score should not be smaller than 1
# remove the alpha for following samples
sample.gr$alpha <- NULL
```

## Customize the x-axis label position
```{r lolliplot.xaxis, fig.width=6, fig.height=4.5}
xaxis <- c(1, 200, 400, 701, 1000, 1200, 1402) ## define the position
lolliplot(sample.gr, features, xaxis=xaxis)
names(xaxis) <- xaxis # define the labels
names(xaxis)[4] <- "center" 
lolliplot(sample.gr, features, xaxis=xaxis)
```

## Customize the y-axis label position
```{r lolliplot.yaxis, fig.width=6, fig.height=4.5}
#yaxis <- c(0, 5) ## define the position
#lolliplot(sample.gr, features, yaxis=yaxis)
yaxis <- c(0, 5, 10, 15) ## define the position
names(yaxis) <- yaxis # define the labels
names(yaxis)[3] <- "y-axis" 
lolliplot(sample.gr, features, yaxis=yaxis)
```

## Jitter the label
```{r lolliplot.jitter, fig.width=6, fig.height=4.5}
sample.gr$dashline.col <- sample.gr$color
lolliplot(sample.gr, features, jitter="label")
```

## Add a legend
```{r lolliplot.legend, fig.width=6, fig.height=4.5}
legend <- 1:6 ## legend fill color
names(legend) <- paste0("legend", letters[1:6]) ## legend labels
lolliplot(sample.gr, features, legend=legend)
## use list to define more attributes. see ?grid::gpar to get more details.
legend <- list(labels=paste0("legend", LETTERS[1:6]), 
               col=palette()[6:1], 
               fill=palette()[legend])
lolliplot(sample.gr, features, legend=legend)
## if you have multiple tracks, please try to set the legend by list.
## see more examples in the section [Plot multiple samples](#plot-multiple-samples)
legendList <- list(legend)
lolliplot(sample.gr, features, legend=legendList)

# from version 1.21.8, users can also try to set legend 
# as a column name in the metadata of GRanges.
sample.gr.newlegend <- sample.gr
sample.gr.newlegend$legend <- LETTERS[sample.gr$color]
lolliplot(sample.gr.newlegend, features, legend="legend")
# from version 1.41.6, users can set the legend position to right
lolliplot(sample.gr, features, legend=legend,
          legendPosition = list(position='right',
                                width=unit(1, 'inch')))
## use ncol or nrow to control the legend layout.
legendList[[1]]$ncol <- 2 # if legend is not a list of list, use legend$ncol <- 2
lolliplot(sample.gr, features, legend=legendList, legendPosition = 'right')
```

## Control the labels

Users can control the parameters of labels by naming the metadata start as 
label.parameter  such as label.parameter.rot or label.parameter.gp.
Note: label.parameter.label can be used to plot snp labels other than the names of inputs.
The parameter is used for [grid.text](https://www.rdocumentation.org/packages/grid/topics/grid.text).

```{r lolliplot.labels.control, fig.width=6, fig.height=4}
sample.gr.rot <- sample.gr
sample.gr.rot$label.parameter.rot <- 45
lolliplot(sample.gr.rot, features, legend=legend)
sample.gr.rot$label.parameter.rot <- 60
sample.gr.rot$label.parameter.col <- "brown"
## change the label text into user-defined names other than names of the sample.gr
sample.gr.rot$label.parameter.label <- names(sample.gr)
random_ids <- sample(seq_along(sample.gr), 5)
sample.gr.rot$label.parameter.label[random_ids] <- 
  paste("new label", random_ids)
random_ids <- sample(seq_along(sample.gr), 2)
sample.gr.rot$label.parameter.label[random_ids] <- NA ## remove some labels
lolliplot(sample.gr.rot, features, legend=legend)
## try different colors
sample.gr.rot$label.parameter.col <- sample.int(7,
                                                length(sample.gr),
                                                replace = TRUE)
sample.gr.rot$label.parameter.draw <- TRUE
sample.gr.rot$label.parameter.draw[[1]] <- FALSE ## another method to remove the first label
lolliplot(sample.gr.rot, features, legend=legend)
```

Users can also control the labels one by one by setting label.parameter.gp. 
Please note that for each label, the label.parameter.gp must be a list.

```{r lolliplot.labels.ctl.one.by.one, fig.width=6, fig.height=4}
label.parameter.gp.brown <- gpar(col="brown")
label.parameter.gp.blue <- gpar(col="blue")
label.parameter.gp.red <- gpar(col="red")
sample.gr$label.parameter.gp <- sample(list(label.parameter.gp.blue,
                                            label.parameter.gp.brown,
                                            label.parameter.gp.red),
                                       length(sample.gr), replace = TRUE)
lolliplot(sample.gr, features)
```

User can write the labels of the features directly on them and not in the 
legend by set the parameter label_on_feature to `TRUE`.
```{r lolliplot.labels.on.feature, fig.width=6, fig.height=4}
lolliplot(sample.gr, features, label_on_feature=TRUE)
```

Please note that **lolliplot** does not support any parameters to set the title and xlab. If you want to 
add the title and xlab, please try to add them by [grid.text](https://www.rdocumentation.org/packages/grid/topics/grid.text).
```{r lolliplot.xlab.ylab.title, fig.width=6, fig.height=5.2}
lolliplot(sample.gr.rot, features, legend=legend, ylab="y label here")
grid.text("label of x-axis here", x=.5, y=.01, just="bottom")
grid.text("title here", x=.5, y=.98, just="top", 
          gp=gpar(cex=1.5, fontface="bold"))
```

Start from version `1.33.3`, **lolliplot** can also plot motifs as labels.
The parameters are controlled by the parameters of labels by naming
the metadata start as label.parameter such as label.parameter.pfm or 
label.parameter.font. The parameter is used for [plotMotifLogoA](https://www.rdocumentation.org/packages/motifStack/topics/plotMotifLogoA).

```{r lolliplot.motiflogo, fig.width=6, fig.height=5.2}
library(motifStack)
pcms<-readPCM(file.path(find.package("motifStack"), "extdata"),"pcm$")
sample.gr.rot$label.parameter.pfm <- pcms[sample(seq_along(pcms),
                                                 length(sample.gr.rot),
                                                 replace = TRUE)]
lolliplot(sample.gr.rot, features, legend=legend)
```

## Change the lolliplot type

### Change the shape for "circle" plot

```{r lolliplotShape, fig.width=6, fig.height=4.5}
## shape must be "circle", "square", "diamond", "triangle_point_up", or "triangle_point_down"
available.shapes <- c("circle", "square", "diamond", 
                      "triangle_point_up", "triangle_point_down")
sample.gr$shape <- sample(available.shapes, size = length(sample.gr), replace = TRUE)
sample.gr$legend <- paste0("legend", as.numeric(factor(sample.gr$shape)))
lolliplot(sample.gr, features, type="circle", legend = "legend")
sample.gr.mul.shape <- sample.gr
sample.gr.mul.shape$score <- ceiling(sample.gr.mul.shape$score)
sample.gr.mul.shape$shape <- lapply(sample.gr.mul.shape$score, function(s){
  sample(available.shapes, size = s, replace = TRUE)
})
sample.gr.mul.shape$color <- lapply(sample.gr.mul.shape$score, function(s){
  sample.int(7, size = s, replace = TRUE)
})
lolliplot(sample.gr.mul.shape, features, type="circle",
          lollipop_style_switch_limit = max(sample.gr.mul.shape$score))
```

### Google pin
```{r lolliplot5, fig.width=6, fig.height=4.5}
lolliplot(sample.gr, features, type="pin")
sample.gr$color <- lapply(sample.gr$color, function(.ele) c(.ele, sample.int(6, 1)))
sample.gr$border <- sample.int(6, length(SNP), replace=TRUE)
lolliplot(sample.gr, features, type="pin")
```

### Flag

```{r lolliplotFlag, fig.width=6, fig.height=4}
sample.gr.flag <- sample.gr
sample.gr.flag$label <- names(sample.gr) ## move the names to metadata:label
names(sample.gr.flag) <- NULL
#lolliplot(sample.gr.flag, features, 
#          ranges=GRanges("chr1", IRanges(0, 1600)), ## use ranges to leave more space on the right margin.
#          type="flag")
## change the flag rotation angle
sample.gr.flag$node.label.rot <- 15
sample.gr.flag$node.label.rot[c(2, 5)] <- c(60, -15)
sample.gr.flag$label[7] <- "I have a long name"
sample.gr.flag$node.label.cex <- 1
sample.gr.flag$node.label.cex[4] <- 2
lolliplot(sample.gr.flag, features, 
          ranges=GRanges("chr1", IRanges(0, 1600)),## use ranges to leave more space on the right margin. 
          type="flag")
```

### Pie plot
```{r lolliplot6, fig.width=6, fig.height=3}
sample.gr$score <- NULL ## must be removed, because pie will consider all the numeric columns except column "color", "fill", "alpha", "shape", "lwd", "id" and "id.col".
sample.gr$label <- NULL
sample.gr$node.label.col <- NULL
x <- sample.int(100, length(SNP))
sample.gr$value1 <- x
sample.gr$value2 <- 100 - x # for pie plot, 2 value columns are required.
## the length of the color should be no less than that of value1 or value2
sample.gr$color <- rep(list(c("#87CEFA", '#98CE31')), length(SNP))
sample.gr$border <- "gray30"
lolliplot(sample.gr, features, type="pie")
```


## Plot multiple samples
### Multiple layers
```{r lolliplot7, fig.width=6, fig.height=5.5}
SNP2 <- sample(4000:8000, 30)
x2 <- sample.int(100, length(SNP2), replace=TRUE)
sample2.gr <- GRanges("chr3", IRanges(SNP2, width=1, names=paste0("snp", SNP2)), 
             value1=x2, value2=100-x2)
sample2.gr$color <- rep(list(c('#DB7575', '#FFD700')), length(SNP2))
sample2.gr$border <- "gray30"

features2 <- GRanges("chr3", IRanges(c(5001, 5801, 7001), 
                                    width=c(500, 500, 405),
                                    names=paste0("block", 4:6)),
                    fill=c("orange", "gray30", "lightblue"),
                    height=unit(c(0.5, 0.3, 0.8), "cm"))
legends <- list(list(labels=c("WT", "MUT"), fill=c("#87CEFA", '#98CE31')), 
                list(labels=c("WT", "MUT"), fill=c('#DB7575', '#FFD700')))
lolliplot(list(A=sample.gr, B=sample2.gr), 
          list(x=features, y=features2), 
          type="pie", legend=legends)
```

Different layouts are also possible.
```{r lolliplot.multiple.type, fig.width=6, fig.height=7.5}
sample2.gr$score <- sample2.gr$value1 ## The circle layout needs the score column 
lolliplot(list(A=sample.gr, B=sample2.gr), 
          list(x=features, y=features2), 
          type=c("pie", "circle"), legend=legends)
```

### pie.stack layout
```{r lolliplot.pie.stack, fig.width=6, fig.height=5}
rand.id <- sample.int(length(sample.gr), 3*length(sample.gr), replace=TRUE)
rand.id <- sort(rand.id)
sample.gr.mul.patient <- sample.gr[rand.id]
## pie.stack require metadata "stack.factor", and the metadata can not be 
## stack.factor.order or stack.factor.first
len.max <- max(table(rand.id))
stack.factors <- paste0("patient", formatC(1:len.max, 
                                           width=nchar(as.character(len.max)), 
                                           flag="0"))
sample.gr.mul.patient$stack.factor <- 
    unlist(lapply(table(rand.id), sample, x=stack.factors))
sample.gr.mul.patient$value1 <- 
    sample.int(100, length(sample.gr.mul.patient), replace=TRUE)
sample.gr.mul.patient$value2 <- 100 - sample.gr.mul.patient$value1
patient.color.set <- as.list(as.data.frame(rbind(rainbow(length(stack.factors)), 
                                                 "#FFFFFFFF"), 
                                           stringsAsFactors=FALSE))
names(patient.color.set) <- stack.factors
sample.gr.mul.patient$color <- 
    patient.color.set[sample.gr.mul.patient$stack.factor]
legend <- list(labels=stack.factors, col="gray80", 
               fill=sapply(patient.color.set, `[`, 1))
## remove one mutation label
sample.gr.mul.patient$label.parameter.draw <- TRUE
sample.gr.mul.patient$label.parameter.draw[
  names(sample.gr.mul.patient)==
    sample(unique(names(sample.gr.mul.patient)), 1)] <-
  FALSE
lolliplot(sample.gr.mul.patient, features, type="pie.stack", 
          legend=legend, dashline.col="gray")
```

### Caterpillar layout

Metadata SNPsideID is used to trigger caterpillar layout.
SNPsideID must be 'top' or 'bottom'.

```{r lolliplot.caterpillar, fig.width=6, fig.height=4}
sample.gr$SNPsideID <- sample(c("top", "bottom"), 
                               length(sample.gr),
                               replace=TRUE)
lolliplot(sample.gr, features, type="pie", 
          legend=legends[[1]])
```
```{r lolliplot.caterpillar2, fig.width=6, fig.height=12}
## Two layers
sample2.gr$SNPsideID <- "top"
idx <- sample.int(length(sample2.gr), 15)
sample2.gr$SNPsideID[idx] <- "bottom"
sample2.gr$color[idx] <- '#FFD700'
lolliplot(list(A=sample.gr, B=sample2.gr), 
          list(x=features.mul, y=features2), 
          type=c("pie", "circle"), legend=legends)
```

## EMBL-EBI Proteins API

Following code will show how to use [EBI Proteins REST API](https://www.ebi.ac.uk/proteins/api/doc/) to get annotations of protein domains.

```{r ProteinsAPI, fig.width=6, fig.height=3, eval=FALSE, echo=TRUE}
library(httr) # load library to get data from REST API
APIurl <- "https://www.ebi.ac.uk/proteins/api/" # base URL of the API
taxid <- "9606" # human tax ID
gene <- "TP53" # target gene
orgDB <- "org.Hs.eg.db" # org database to get the uniprot accession id
eid <- mget("TP53", get(sub(".db", "SYMBOL2EG", orgDB)))[[1]]
chr <- mget(eid, get(sub(".db", "CHR", orgDB)))[[1]]
accession <- unlist(lapply(eid, function(.ele){
  mget(.ele, get(sub(".db", "UNIPROT", orgDB)))
}))
stopifnot(length(accession)<=20) # max number of accession is 20

tryCatch({ ## in case the internet connection does not work
  featureURL <- paste0(APIurl, 
                       "features?offset=0&size=-1&reviewed=true",
                       "&types=DNA_BIND%2CMOTIF%2CDOMAIN",
                       "&taxid=", taxid,
                       "&accession=", paste(accession, collapse = "%2C")
  )
  response <- GET(featureURL)
  if(!http_error(response)){
    content <- httr::content(response)
    content <- content[[1]]
    acc <- content$accession
    sequence <- content$sequence
    gr <- GRanges(chr, IRanges(1, nchar(sequence)))
    domains <- do.call(rbind, content$features)
    domains <- GRanges(chr, IRanges(as.numeric(domains[, "begin"]),
                                     as.numeric(domains[, "end"]),
                                     names = domains[, "description"]))
    names(domains)[1] <- "DNA_BIND" ## this is hard coding.
    domains$fill <- 1+seq_along(domains)
    domains$height <- 0.04
    ## GET variations. This part can be replaced by user-defined data.
    variationURL <- paste0(APIurl,
                           "variation?offset=0&size=-1",
                           "&sourcetype=uniprot&dbtype=dbSNP",
                           "&taxid=", taxid,
                           "&accession=", acc)
    response <- GET(variationURL)
    if(!http_error(response)){
      content <- httr::content(response)
      content <- content[[1]]
      keep <- sapply(content$features, function(.ele) length(.ele$evidences)>2 && # filter the data by at least 2 evidences
                       !grepl("Unclassified", .ele$clinicalSignificances)) # filter the data by classified clinical significances.
      nkeep <- c("wildType", "alternativeSequence", "begin", "end",
                 "somaticStatus", "consequenceType", "score")
      content$features <- lapply(content$features[keep], function(.ele){
        .ele$score <- length(.ele$evidences)
        unlist(.ele[nkeep]) 
      })
      variation <- do.call(rbind, content$features)
      variation <- 
        GRanges(chr, 
                IRanges(as.numeric(variation[, "begin"]),
                        width = 1,
                        names = paste0(variation[, "wildType"],
                                       variation[, "begin"],
                                       variation[, "alternativeSequence"])),
                score = as.numeric(variation[, "score"]),
                color = as.numeric(factor(variation[, "consequenceType"]))+1)
      variation$label.parameter.gp <- gpar(cex=.5)
      lolliplot(variation, domains, ranges = gr, ylab = "# evidences", yaxis = FALSE)
    }else{
      message("Can not get variations. http error")
    }
  }else{
    message("Can not get features. http error")
  }
},error=function(e){
  message(e)
},warning=function(w){
  message(w)
},interrupt=function(i){
  message(i)
})
```


## Variant Call Format (VCF) data
```{r VCF, fig.width=6, fig.height=5}
library(VariantAnnotation)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
fl <- system.file("extdata", "chr22.vcf.gz", package="VariantAnnotation")
gr <- GRanges("22", IRanges(50968014, 50970514, names="TYMP"))
if(.Platform$OS.type!="windows"){# This line is for avoiding error from VariantAnnotation in the windows platform, which will be removed when VariantAnnotation's issue gets fixed.
tab <- TabixFile(fl)
vcf <- readVcf(fl, "hg19", param=gr)
mutation.frequency <- rowRanges(vcf)
mcols(mutation.frequency) <- cbind(mcols(mutation.frequency), 
                                   VariantAnnotation::info(vcf))
mutation.frequency$border <- "gray30"
mutation.frequency$color <- 
    ifelse(grepl("^rs", names(mutation.frequency)), "lightcyan", "lavender")
## Plot Global Allele Frequency based on AC/AN
mutation.frequency$score <- mutation.frequency$AF*100
seqlevelsStyle(mutation.frequency) <- "UCSC"
if(!grepl("chr", seqlevels(mutation.frequency)[1])){
  seqlevels(mutation.frequency) <- 
    paste0("chr", seqlevels(mutation.frequency))
}
}
seqlevelsStyle(gr) <- "UCSC" 
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)
features <- c(range(trs[[1]]$dat), range(trs[[5]]$dat))
names(features) <- c(trs[[1]]$name, trs[[5]]$name)
features$fill <- c("lightblue", "mistyrose")
features$height <- c(.02, .04)
if(.Platform$OS.type!="windows"){
lolliplot(mutation.frequency, features, ranges=gr)
}
```


## Methylation data
```{r methylation, eval=FALSE, echo=TRUE}
library(rtracklayer)
session <- browserSession()
query <- ucscTableQuery(session, 
                        table="wgEncodeHaibMethylRrbs", 
                        range=GRangesForUCSCGenome("hg19", 
                                                   seqnames(gr),
                                                   ranges(gr)))
tableName(query) <- tableNames(query)[1]
methy <- track(query)
methy <- GRanges(methy)
```
```{r methylation.hide, echo=FALSE}
methy <- import(system.file("extdata", "methy.bed", package="trackViewer"), "BED")
```
```{r fig.width=6,fig.height=4}
lolliplot(methy, features, ranges=gr, type="pin")
```

## Change the node size
In the above example, some of the nodes overlap each other. To change the node size, cex prameter could be used.
```{r fig.width=6,fig.height=2.5}
methy$lwd <- .5
lolliplot(methy, features, ranges=gr, type="pin", cex=.5)
#lolliplot(methy, features, ranges=gr, type="circle", cex=.5)
methy$score2 <- max(methy$score) - methy$score
lolliplot(methy, features, ranges=gr, type="pie", cex=.5)
## We can change it one by one
methy$cex <- runif(length(methy))
lolliplot(methy, features, ranges=gr, type="pin")
#lolliplot(methy, features, ranges=gr, type="circle")
```

## Change the scale of the x-axis (xscale)
In the above examples, some of the nodes are moved too far from the original coordinates.
To rescale, the x-axis could be reset as below.
```{r fig.width=6,fig.height=2.5}
methy$cex <- 1
lolliplot(methy, features, ranges=gr, rescale = TRUE)
## by set percentage for features and non-features segments
xaxis <- c(50968014, 50968514, 50968710, 50968838, 50970514)
rescale <- c(.3, .4, .3)
lolliplot(methy, features, ranges=gr, type="pin",
          rescale = rescale, xaxis = xaxis)
## by set data.frame to rescale
rescale <- data.frame(
  from.start = c(50968014, 50968515, 50968838), 
  from.end   = c(50968514, 50968837, 50970514),
  to.start   = c(50968014, 50968838, 50969501), 
  to.end     = c(50968837, 50969500, 50970514)
)
lolliplot(methy, features, ranges=gr, type="pin",
          rescale = rescale, xaxis = xaxis)
```

Rescale the region to emphasize exons region only or introns region only.
Here "exon" indicates all regions in features.

```{r fig.width=6,fig.height=2.5}
lolliplot(methy, features, ranges=gr, rescale = "exon")
# exon region occupy 99% of the plot region.
lolliplot(methy, features, ranges=gr, rescale = "exon_99")
lolliplot(methy, features, ranges=gr, rescale = "intron")
```


## Split the lollipop plot into multiLayers
In the above examples, people may be misled when the x-axis is ignored.
It will be better to plot the data into multiple layers. This can be done by
setting parameter `ranges` into a `GRangesList` object.

```{r fig.width=9,fig.height=8}
grSplited <- tile(gr, n=2)
lolliplot(methy, features, ranges=grSplited, type="pin")
```

# Plot the lollipop plot with the coverage and annotation tracks
```{r fig.width=8,fig.height=4}
gene <- geneTrack(get("HSPA8", org.Hs.egSYMBOL2EG), TxDb.Hsapiens.UCSC.hg19.knownGene)[[1]]
SNPs <- GRanges("chr11", IRanges(sample(122929275:122930122, size = 20), width = 1), strand="-")
SNPs$score <- sample.int(5, length(SNPs), replace = TRUE)
SNPs$color <- sample.int(6, length(SNPs), replace=TRUE)
SNPs$border <- "gray80"
SNPs$feature.height = .1
gene$dat2 <- SNPs
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
repA <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                    file.path(extdata, "cpsf160.repA_+.wig"),
                    format="WIG")
fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED",
                    ranges=GRanges("chr11", IRanges(122830799, 123116707)))
optSty <- optimizeStyle(trackList(repA, fox2, gene), theme="col")
trackList <- optSty$tracks
viewerStyle <- optSty$style
gr <- GRanges("chr11", IRanges(122929275, 122930122))
setTrackStyleParam(trackList[[3]], "ylabgp", list(cex=.8))
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
## lollipopData track
SNPs2 <- GRanges("chr11", IRanges(sample(122929275:122930122, size = 30), width = 1), strand="-")
SNPs2 <- c(SNPs2, promoters(gene$dat, upstream = 0, downstream = 1))
SNPs2$score <- sample.int(3, length(SNPs2), replace = TRUE)
SNPs2$color <- sample.int(6, length(SNPs2), replace=TRUE)
SNPs2$border <- "gray30"
SNPs2$feature.height = .1
lollipopData <- new("track", dat=SNPs, dat2=SNPs2, type="lollipopData")
gene <- geneTrack(get("HSPA8", org.Hs.egSYMBOL2EG), TxDb.Hsapiens.UCSC.hg19.knownGene)[[1]]
optSty <- optimizeStyle(trackList(repA, lollipopData, gene, heightDist = c(3, 3, 1)), theme="col")
trackList <- optSty$tracks
viewerStyle <- optSty$style
gr <- GRanges("chr11", IRanges(122929275, 122930122))
setTrackStyleParam(trackList[[2]], "ylabgp", list(cex=.8))
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
addGuideLine(122929538, vp=vp)
## plot with customized geneTrack
dat <- gene$dat
mcols(dat) <- NULL
dat <- subsetByOverlaps(dat, gr)
dat$feature <- 'exon' # feature is required
dat$featureID <- paste0('name', seq_along(dat)) # treat each as single exon gene
dat$color <- sample(seq.int(7), length(dat), replace = TRUE) # set the color
dat$height <- sample(c(0.5, 1, 2), length(dat), replace = TRUE) # set the height
dat$hide_label <- TRUE # do not add labels to the block
gene <- new('track', dat=dat, dat2=SNPs, type='gene', name='a name')
optSty <- optimizeStyle(trackList(fox2, gene), theme="col")
trackList <- optSty$tracks
viewerStyle <- optSty$style
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```


# Session Info
```{r sessionInfo, results='asis'}
sessionInfo()
```

---
title: "trackViewer Vignette: overview"
author: "Jianhong Ou, Lihua Julie Zhu"
date: "`r BiocStyle::doc_date()`"
package: "`r BiocStyle::pkg_ver('trackViewer')`"
abstract: >
  Visualize mapped reads along with annotation as track layers for NGS dataset 
  such as ChIP-seq, RNA-seq, miRNA-seq, DNA-seq, SNPs and methylation data.
vignette: >
  %\VignetteIndexEntry{trackViewer Vignette: overview}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  html_document:
    theme: simplex
    toc: true
    toc_float: true
    toc_depth: 4
    fig_caption: true
---

```{r, echo=FALSE, results="hide", warning=FALSE}
suppressPackageStartupMessages({
    library(trackViewer)
    library(rtracklayer)
    library(Gviz)
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    library(org.Hs.eg.db)
    library(VariantAnnotation)
  library(httr)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)
```

# Introduction

There are two packages available in Bioconductor for visualizing genomic data: **rtracklayer** and **Gviz**. **rtracklayer** provides an interface to genome browsers and associated annotation tracks. **Gviz** can be used to plot coverage and annotation tracks. **TrackViewer** is a lightweight visualization tool for generating interactive figures for publication. Not only can **trackViewer** be used to visualize coverage and annotation tracks, but it can also be employed to generate lollipop/dandelion plots that depict dense methylation/mutation/variant data to facilitate an integrative analysis of these multi-omics data. It leverages **Gviz** and **rtracklayer**, is easy to use, and has a low memory and cpu consumption. In addition, we implemented a web application of **trackViewer** leveraging **Shiny** package. The web application of **trackViewer** is available at https://github.com/jianhong/trackViewer.documentation/tree/master/trackViewerShinyApp.

* [Change the styles of the track plot](changeTracksStyles.html)

* [Plot methylation/mutation/variant data by lollipop plot](lollipopPlot.html)

* [Plot high dense methylation/mutation/variant data by dandelion plot](dandelionPlot.html)

* [Plot genomic interaction data (HiC, cooler, etc) by viewTracks](plotInteractionData.html)

```{r plotComp,echo=TRUE,fig.keep='none'}
library(Gviz)
library(rtracklayer)
library(trackViewer)
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED",
                    ranges=gr)
fox2$dat <- coverageGR(fox2$dat)

viewTracks(trackList(fox2), gr=gr, autoOptimizeStyle=TRUE, newpage=FALSE)

dt <- DataTrack(range=fox2$dat[strand(fox2$dat)=="-"] , 
                genome="hg19", type="hist", name="fox2", 
                window=-1, chromosome="chr11", 
                fill.histogram="black", col.histogram="NA",
                background.title="white",
                col.frame="white", col.axis="black",
                col="black", col.title="black")
plotTracks(dt, from=122929275, to=122930122, strand="-")
```
```{r Gviz,echo=FALSE,fig.cap='Plot data with **Gviz** and **trackViewer**. Please note that **trackViewer** can generate similar figure as **Gviz** with several lines of simple codes.',fig.width=8,fig.height=3}
viewerStyle <- trackViewerStyle()
setTrackViewerStyleParam(viewerStyle, "margin", c(.01, .13, .02, .02))
empty <- DataTrack(showAxis=FALSE, showTitle=FALSE, background.title="white")
plotTracks(list(empty, dt), from=122929275, to=122930122, strand="-")
pushViewport(viewport(0, .5, 1, .5, just=c(0, 0)))
viewTracks(trackList(fox2), viewerStyle=viewerStyle, 
                 gr=gr, autoOptimizeStyle=TRUE, newpage=FALSE)
popViewport()
grid.text(label="Gviz track", x=.3, y=.4)
grid.text(label="trackViewer track", x=.3, y=.9)
```

 **trackViewer** not only has the functionalities to produce the figures generated by **Gviz**, as shown in the Figure above, but also provides additional plotting styles as shown in the Figure below. The mimimalist design requires minimum input from the users while retaining the flexibility to change the output style easily. 


```{r lostcode,echo=TRUE,fig.keep='none'}
gr <- GRanges("chr1", IRanges(c(1, 6, 10), c(3, 6, 12)), score=c(3, 4, 1))
dt <- DataTrack(range=gr, data="score", type="hist")
plotTracks(dt, from=2, to=11)
tr <- new("track", dat=gr, type="data", format="BED")
viewTracks(trackList(tr), chromosome="chr1", start=2, end=11)
```
```{r GvizLost,echo=FALSE,fig.cap='Plot data with **Gviz** and **trackViewer**. Note that **trackViewer** is not only including more details but also showing all the data involved in the given range.',fig.width=8,fig.height=3}
plotTracks(list(empty, dt), from=2, to=11)
pushViewport(viewport(0, .5, 1, .5, just=c(0, 0)))
viewTracks(trackList(tr), viewerStyle=viewerStyle, 
           chromosome="chr1", start=2, end=11, 
           autoOptimizeStyle=TRUE, newpage=FALSE)
popViewport()
grid.text(label="Gviz track", x=.3, y=.4)
grid.text(label="trackViewer track", x=.3, y=.9)
```


**Gviz** requires huge memory space to handle big wig files. To solve this problem, we rewrote the import function in **trackViewer** by importing the entire file first and parsing it later when plot. As a result, **trackViewer** decreases the import time from 180 min to 21 min and the memory cost from 10G to 5.32G for a half giga wig file (GSM917672). 

# Browse 
## Steps of using trackViewer \Biocpkg{trackViewer}
### Step 1. Import data
The function **importScore** is used to import BED, WIG, bedGraph or BigWig files. 
The function **importBam** is employed to import the bam files. Here is an example.

```{r importData}
library(trackViewer)
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
repA <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                    file.path(extdata, "cpsf160.repA_+.wig"),
                    format="WIG")
## Because the wig file does not contain any strand info, 
## we need to set it manually.
strand(repA$dat) <- "-"
strand(repA$dat2) <- "+"
```

The function **coverageGR** could be used to calculate the coverage after the data is imported.

```{r coverage}
fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED",
                    ranges=GRanges("chr11", IRanges(122830799, 123116707)))
dat <- coverageGR(fox2$dat)
## We can split the data by strand into two different track channels
## Here, we set the dat2 slot to save the negative strand info. 
 
fox2$dat <- dat[strand(dat)=="+"]
fox2$dat2 <- dat[strand(dat)=="-"]
```

### Step 2. Build the gene model
The gene model can be built for a given genomic range using **geneModelFromTxdb** function which uses the **TranscriptDb** object as the input.

```{r geneModel}
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)
```
Users can generate a track object with the geneTrack function by inputting a TxDb and a list of gene Entrez IDs. Entrez IDs can be obtained from other types of gene IDs such as gene symbol by using the ID mapping function. For example, to generate a track object given gene FMR1 and human TxDb, refer to the code below.

```{r geneTrack}
entrezIDforFMR1 <- get("FMR1", org.Hs.egSYMBOL2EG)
theTrack <- geneTrack(entrezIDforFMR1,TxDb.Hsapiens.UCSC.hg19.knownGene)[[1]]
```

### Step 3. View the tracks
Use **viewTracks** function to plot data and annotation information along genomic coordinates.
**addGuideLine** or **addArrowMark** can be used to highlight a specific region.
    
```{r viewTracks,fig.cap='plot data and annotation information along genomic coordinates',fig.width=8,fig.height=3}
viewerStyle <- trackViewerStyle()
setTrackViewerStyleParam(viewerStyle, "margin", c(.1, .05, .02, .02))
trackList <- trackList(repA, fox2, trs)
vp <- viewTracks(trackList, 
                 gr=gr, viewerStyle=viewerStyle, 
                 autoOptimizeStyle=TRUE)
addGuideLine(c(122929767, 122929969), vp=vp)
addArrowMark(list(x=122929650, 
                  y=2), # 2 means track 2 from the bottom.
             label="label",
             col="blue",
             vp=vp)
```

For more details, please refer [vignette changeTracksStyles](changeTracksStyles.html).

## Generate and edit an interactive plot using the browseTracks function
As shown above, figures produced by trackViewer are highly customizable, allowing users to alter the label, symbol, color, and size with various functions.

For users who prefer to modify the look and feel of a figure interactively, they can use the function `browseTracks` to draw interactive tracks, leveraging the htmlwidgets package. 

```{r browseTrack,fig.cap='interactive tracks',fig.width=6,fig.height=4}
browseTracks(trackList, gr=gr)
```
\
\
The videos at https://youtu.be/lSmeTu4WMlc and  https://youtu.be/lvF0tnJiHQI illustrate how to generate and modify an interactive plot. Please note that the interactive feature is only fully implemented in version 1.19.14 or later.  

## Plot multiple genes in one track

```{r plotgeneTrack,fig.cap='Plot multiple genes in one track',fig.width=6,fig.height=4}
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
grW <- parse2GRanges("chr11:122,830,799-123,116,707")
ids <- getGeneIDsFromTxDb(grW, TxDb.Hsapiens.UCSC.hg19.knownGene)
symbols <- mget(ids, org.Hs.egSYMBOL)
genes <- geneTrack(ids, TxDb.Hsapiens.UCSC.hg19.knownGene, 
                   symbols, asList=FALSE)
# set color to the gene
genes$dat$color <- as.numeric(factor(genes$dat$featureID))
optSty <- optimizeStyle(trackList(repA, fox2, genes), theme="safe")
trackListW <- optSty$tracks
viewerStyleW <- optSty$style
viewTracks(trackListW, gr=grW, viewerStyle=viewerStyleW)
```


# Operators
If you are interested in drawing a combined track from two input tracks, e.g, adding or substractiong one from the other, then you can try one of the operators such as + and - as showing below.

```{r viewTracksOperator1,fig.cap='show data with operator "+"',fig.width=8,fig.height=4}
newtrack <- repA
## Must keep the same format for dat and dat2
newtrack <- parseWIG(newtrack, "chr11", 122929275, 122930122)
newtrack$dat2 <- newtrack$dat
newtrack$dat <- fox2$dat2
setTrackStyleParam(newtrack, "color", c("#4A95E0", "#CF5D6D"))
viewTracks(trackList(newtrack, trs), 
           gr=gr, viewerStyle=viewerStyle, operator="+")
```

```{r viewTracksOperator2,fig.cap='show data with operator "-"',fig.width=8,fig.height=4}
viewTracks(trackList(newtrack, trs), gr=gr, viewerStyle=viewerStyle, operator="-")
```

Multiple operators are acceptable.

```{r viewTracksOperator3,fig.cap='show data with multiple operators', fig.width=8,fig.height=5}
viewTracks(trackList(newtrack, newtrack, newtrack, trs), gr=gr, viewerStyle=viewerStyle, operator=c(NA, "-", "+"))
```

Alternatively, you can try **GRoperator** before viewing tracks.

```{r viewTracksOperator4,fig.cap='show data with operator "-"',fig.width=8,fig.height=4}
newtrack$dat <- GRoperator(newtrack$dat, newtrack$dat2, col="score", operator="-")
newtrack$dat2 <- GRanges()
viewTracks(trackList(newtrack, trs), gr=gr, viewerStyle=viewerStyle)
```

# Lolliplot
The `lolliplot` function is for the visualization of the methylation/variant/mutation data. 
For more details, please refer [vignette lollipopPlot](lollipopPlot.html).

```{r}
library(trackViewer)
features <- GRanges("chr1", IRanges(c(1, 501, 1001), 
                                    width=c(120, 400, 405),
                                    names=paste0("block", 1:3)),
                    fill = c("#FF8833", "#51C6E6", "#DFA32D"),
                    height = c(0.02, 0.05, 0.08))
SNP <- c(10, 100, 105, 108, 400, 410, 420, 600, 700, 805, 840, 1400, 1402)
sample.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)),
                     color = sample.int(6, length(SNP), replace=TRUE),
                     score = sample.int(5, length(SNP), replace = TRUE))
lolliplot(sample.gr, features)

```

# Dandelion Plot

Dandelion Plot is used to plot high dense of mutation/methylation data.
For more details, please refer [vignette dandelionPlot](dandelionPlot.html).

```{r fig.width=4.5,fig.height=3}
library(trackViewer)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
methy <- import(system.file("extdata", "methy.bed", package="trackViewer"), "BED")
gr <- GRanges("chr22", IRanges(50968014, 50970514, names="TYMP"))
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)
features <- c(range(trs[[1]]$dat), range(trs[[5]]$dat))
names(features) <- c(trs[[1]]$name, trs[[5]]$name)
features$fill <- c("lightblue", "mistyrose")
features$height <- c(.02, .04)
dandelion.plot(methy, features, ranges=gr, type="pin")
```

# Plot chromatin interactions data

Plot chromatin interactions heatmap as tracks.
For more details, please refer [vignette plotInteractionData](plotInteractionData.html).

```{r}
library(InteractionSet)
gi <- readRDS(system.file("extdata", "nij.chr6.51120000.53200000.gi.rds", package="trackViewer"))
head(gi)
range <- GRanges("chr6", IRanges(51120000, 53200000))
tr <- gi2track(gi)
ctcf <- readRDS(system.file("extdata", "ctcf.sample.rds",
                            package="trackViewer"))
## change the color
setTrackStyleParam(tr, "breaks", 
                   c(seq(from=0, to=50, by=10), 200))
setTrackStyleParam(tr, "color",
                   c("lightblue", "yellow", "red"))
viewTracks(trackList(ctcf, tr, heightDist=c(1, 3)), gr=range,
           autoOptimizeStyle = TRUE)
```

# Ideogram Plot

Plot ideograms with a list of chromosomes and a genome.
```{r eval=FALSE}
ideo <- loadIdeogram("hg38", chrom=c("chr1", "chr3", "chr4", 'chr21', "chr22"))
```
```{r echo=FALSE, results="hide"}
path <- system.file("extdata", "ideo.hg38.rds", package = "trackViewer")
ideo <- readRDS(path)
```
```{r, eval=FALSE}
dataList <- trim(ideo)
dataList$score <- as.numeric(as.factor(dataList$gieStain))
dataList <- dataList[dataList$gieStain!="gneg"]
dataList <- GRangesList(dataList)
ideogramPlot(ideo, dataList, 
            layout=list("chr1", c("chr3", "chr22"), 
                        c("chr4", "chr21")))
```

# Web application of **trackViewer**
We created a web application of **trackViewer** (available in 1.19.14 or later) by leveraging the R package **Shiny**. The web application of trackViewer and sample data are available at  https://github.com/jianhong/trackViewer.documentation/tree/master/trackViewerShinyApp. Here is a demo on how to to use the web application at https://www.nature.com/articles/s41592-019-0430-y#Sec2 Supplementary Video 5.

# Session Info
```{r sessionInfo, results='asis'}
sessionInfo()
```

---
title: "trackViewer Vignette: change the track styles"
author: "Jianhong Ou, Lihua Julie Zhu"
date: "`r BiocStyle::doc_date()`"
package: "`r BiocStyle::pkg_ver('trackViewer')`"
abstract: >
  Visualize mapped reads along with annotation as track layers for NGS dataset 
  such as ChIP-seq, RNA-seq, miRNA-seq, DNA-seq, SNPs and methylation data.
vignette: >
  %\VignetteIndexEntry{trackViewer Vignette: change the track styles}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  html_document:
    theme: simplex
    toc: true
    toc_float: true
    toc_depth: 4
    fig_caption: true
---

```{r, echo=FALSE, results="hide", warning=FALSE}
suppressPackageStartupMessages({
    library(trackViewer)
    library(rtracklayer)
    library(Gviz)
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    library(org.Hs.eg.db)
    library(VariantAnnotation)
  library(httr)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)
```

* [Plot methylation/mutation/variant data by lollipop plot](lollipopPlot.html)

* [Plot high dense methylation/mutation/variant data by dandelion plot](dandelionPlot.html)

* [Plot genomic interaction data (HiC, cooler, etc) by viewTracks](dandelionPlot.html)

## Prepare toy data

First, we import the sample data.

```{r}
library(trackViewer)
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
repA <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                    file.path(extdata, "cpsf160.repA_+.wig"),
                    format="WIG")
## Because the wig file does not contain any strand info, 
## we need to set it manually.
strand(repA$dat) <- "-"
strand(repA$dat2) <- "+"

fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED",
                    ranges=GRanges("chr11", IRanges(122830799, 123116707)))
dat <- coverageGR(fox2$dat)
## We can split the data by strand into two different track channels
## Here, we set the dat2 slot to save the negative strand info. 
 
fox2$dat <- dat[strand(dat)=="+"]
fox2$dat2 <- dat[strand(dat)=="-"]

library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)

```

```{r optSty}
optSty <- optimizeStyle(trackList(repA, fox2, trs))
trackList <- optSty$tracks
viewerStyle <- optSty$style
```

## Using the `browseTracks` function as a helper

Since version 1.33.6, the interactive plot generated by `browseTracks` function
will print the psuedocode about how to change the track styles at the bottom,
which can be used as a quick reference.

```{r browseTrack, fig.width=6,fig.height=4}
browseTracks(trackList, gr=gr)
```
\
\

## Adjust the x-axis or the X scale
In most cases, researchers are interested in the relative position of the peaks in the gene. 
Sometimes, margin needs to be adjusted to be able to show the entire  gene model. 
The Figure below shows how to add an X scale (x-scale) and 
remove the x-axis using the **setTrackXscaleParam** and **setTrackViewerStyleParam** functions. 

```{r viewTracksXaxis,fig.cap='plot data with x-scale',fig.width=8,fig.height=3}
setTrackViewerStyleParam(viewerStyle, "xaxis", FALSE)
setTrackViewerStyleParam(viewerStyle, "margin", c(.01, .05, .01, .01))
setTrackXscaleParam(trackList[[1]], "draw", TRUE)
setTrackXscaleParam(trackList[[1]], "gp", list(cex=0.8))
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
setTrackXscaleParam(trackList[[1]], attr="position", 
                    value=list(x=122929700, y=3, label=200))
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```


## Adjust the y-axis
The y-axis can be put to the right side of the track by setting the main slot to FALSE in the y-axis slot of each track. 
In addition, the limit of y-axis (ylim) can be set by **setTrackStyleParam**. 
```{r viewTracksYaxis,fig.cap='plot data with y-axis in right side',fig.width=8,fig.height=3}
setTrackViewerStyleParam(viewerStyle, "margin", c(.01, .05, .01, .05))
for(i in 1:2){
    setTrackYaxisParam(trackList[[i]], "main", FALSE)
}
## Adjust the limit of y-axis
setTrackStyleParam(trackList[[1]], "ylim", c(0, 25))
setTrackStyleParam(trackList[[2]], "ylim", c(-25, 0))
## The y-axis tick labels can be turn off/on by setting the 'label' to FALSE/TRUE
#setTrackYaxisParam(trackList[[1]], 'label', TRUE)
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

The function `setTrackYaxisParam` can be sued to adjust the track color/size/etc of y-axis and the y-axis labels.

```{r viewTracksYaxisCol,fig.cap='plot data with adjusted track color',fig.width=8,fig.height=3}
## change the 
setTrackYaxisParam(trackList[[1]], "gp", list(cex=.8, col="green"))
setTrackYaxisParam(trackList[[2]], "gp", list(cex=.8, col="blue"))
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

## Adjust the label of y-axis
The style of y-axis can be changed by setting the ylabgp slot in the style of each track.
```{r viewTracksYlab,fig.cap='plot data with adjusted color and size of y-axis label',fig.width=8,fig.height=3}
setTrackStyleParam(trackList[[1]], "ylabgp", list(cex=.8, col="green"))
## set cex to avoid automatic adjust
setTrackStyleParam(trackList[[2]], "ylabgp", list(cex=.8, col="blue"))
setTrackStyleParam(trackList[[2]], "marginBottom", .2)
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

The y-axis label can be put at the top or the bottom of each track.
```{r viewTracksYlabTopBottom,fig.cap='plot data with adjusted y-axis label position',fig.width=8,fig.height=3}
setTrackStyleParam(trackList[[1]], "ylabpos", "bottomleft")
setTrackStyleParam(trackList[[2]], "ylabpos", "topright")
setTrackStyleParam(trackList[[2]], "marginTop", .2)
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

For each transcript, the transcript name can be put on the upstream or downstream of the transcript. 
The Y-lab can also be removed by set 'ylabpos' to 'none' or set the name as ''.

```{r viewTracksYlabUpsDown,fig.cap='plot data with adjusted transcripts name position',fig.width=8,fig.height=3}
trackListN <- trackList
setTrackStyleParam(trackListN[[3]], "ylabpos", "upstream")
setTrackStyleParam(trackListN[[4]], "ylabpos", "downstream")
## set cex to avoid automatic adjust
setTrackStyleParam(trackListN[[3]], "ylabgp", list(cex=.6))
setTrackStyleParam(trackListN[[4]], "ylabgp", list(cex=.6))
## remove ylab
setTrackStyleParam(trackListN[[5]], "ylabpos", "none")
names(trackListN)[6] <- ""
gr1 <- range(unname(unlist(GRangesList(sapply(trs, function(.ele) .ele$dat)))))
start(gr1) <- start(gr1) - 2000
end(gr1) <- end(gr1) + 2000
viewTracks(trackListN, gr=gr1, viewerStyle=viewerStyle)
```

## Adjust the track color
The track color can be changed by setting the color slot in the style of each track.
The first color is for the dat slot of 
**track** and the second color is for the dat2 slot.
```{r viewTracksCol,fig.cap='plot data with adjusted track color',fig.width=8,fig.height=3}
setTrackStyleParam(trackList[[1]], "color", c("green", "black"))
setTrackStyleParam(trackList[[2]], "color", c("black", "blue"))
for(i in 3:length(trackList)) 
    setTrackStyleParam(trackList[[i]], "color", "black")
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

## Adjust the track height
The track height can be changed by setting the height slot in the style of each track. However, the total height for all 
the tracks should be 1.
```{r viewTracksHeight,fig.cap='plot data with adjusted track height',fig.width=8,fig.height=3}
trackListH <- trackList
setTrackStyleParam(trackListH[[1]], "height", .1)
setTrackStyleParam(trackListH[[2]], "height", .44)
for(i in 3:length(trackListH)){
    setTrackStyleParam(trackListH[[i]], "height", 
                       (1-(0.1+0.44))/(length(trackListH)-2))
}
viewTracks(trackListH, gr=gr, viewerStyle=viewerStyle)
```

## Change the track names
The track names such as gene model names can be edited easily by changing the names of **trackList**.

```{r viewTracksNames,fig.cap='change the track names',fig.width=8,fig.height=3}
names(trackList) <- c("cpsf160", "fox2", rep("HSPA8", 5))
viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

## Show paired data in the same track
**trackViewer** can be used to show to-be-compared data in the same track side by side.

```{r viewTracksPaired,fig.cap='show two data in the same track',fig.width=8,fig.height=2.4}
cpsf160 <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                       file.path(extdata, "cpsf160.repB_-.wig"),
                       format="WIG")
strand(cpsf160$dat) <- strand(cpsf160$dat2) <- "-"
setTrackStyleParam(cpsf160, "color", c("black", "red"))
viewTracks(trackList(trs, cpsf160), gr=gr, viewerStyle=viewerStyle)
```
## Show signals and the called peaks
The called peaks can be plotted as annotation.

```{r viewTracksAnnotation,fig.cap='show called peaks in track', fig.width=8, fig.height=2.4}
cpsf160_cp <- cpsf160
cpsf160$dat2 <- reduce(cpsf160$dat)
cpsf160$dat2$score <- sample.int(length(cpsf160$dat2))
setTrackStyleParam(cpsf160, "tracktype", c("peak", "annotation"))
cpsf160_cp$dat <- reduce(cpsf160_cp$dat2)
cpsf160_cp$dat$score <- sample.int(length(cpsf160_cp$dat))
setTrackStyleParam(cpsf160_cp, "tracktype", c("annotation", "peak"))
setTrackStyleParam(cpsf160_cp, "marginTop", 0.05)
viewTracks(trackList(trs, cpsf160, cpsf160_cp), gr=gr, viewerStyle=viewerStyle)
```


## Flip the x-axis
The x-axis can be horizotally flipped for the genes in the negative strand.

```{r viewTracksFlipped,fig.cap='show data in the flipped track',fig.width=8,fig.height=4}
viewerStyleF <- viewerStyle
setTrackViewerStyleParam(viewerStyleF, "flip", TRUE)
setTrackViewerStyleParam(viewerStyleF, "xaxis", TRUE)
setTrackViewerStyleParam(viewerStyleF, "margin", c(.1, .05, .01, .01))
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyleF)
addGuideLine(c(122929767, 122929969), vp=vp)
addArrowMark(list(x=122929650,
                  y=2),
             label="label",
             col="blue",
             vp=vp)
```

## Optimize the theme

Currently, we support two themes: bw (black and white) and col (colored).
```{r themeBW,fig.cap='balck & white theme',fig.width=8,fig.height=4}
optSty <- optimizeStyle(trackList(repA, fox2, trs), theme="bw")
trackList <- optSty$tracks
viewerStyle <- optSty$style
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

```{r themeCol,fig.cap='colorful theme',fig.width=8,fig.height=4}
optSty <- optimizeStyle(trackList(repA, fox2, trs), theme="col")
trackList <- optSty$tracks
viewerStyle <- optSty$style
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

```{r themeSafe,fig.cap='safe theme',fig.width=8,fig.height=4}
optSty <- optimizeStyle(trackList(repA, fox2, trs), theme="safe")
trackList <- optSty$tracks
viewerStyle <- optSty$style
vp <- viewTracks(trackList, gr=gr, viewerStyle=viewerStyle)
```

## Plot with breaks

We could plot the tracks with breaks by setting multiple genomic ranges.
```{r axisBreak,fig.cap='axis with breaks',fig.width=8,fig.height=4}
gr.breaks <- GRanges("chr11", 
                     IRanges(c(122929275, 122929575, 122929775), 
                             c(122929555, 122929725, 122930122)), 
                     strand="-", percentage=c(.4, .2, .4))
vp <- viewTracks(trackList, gr=gr.breaks, viewerStyle=viewerStyle)
```

## Session Info
```{r sessionInfo, results='asis'}
sessionInfo()
```

---
title: "trackViewer Vignette: dandelionPlot"
author: "Jianhong Ou, Lihua Julie Zhu"
date: "`r BiocStyle::doc_date()`"
package: "`r BiocStyle::pkg_ver('trackViewer')`"
abstract: >
  Visualize high dense methylation or mutation data
  along with annotation as track layers via Dandelion Plot.
vignette: >
  %\VignetteIndexEntry{trackViewer Vignette: dandelionPlot}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  html_document:
    theme: simplex
    toc: true
    toc_float: true
    toc_depth: 4
    fig_caption: true
---

```{r, echo=FALSE, results="hide", warning=FALSE}
suppressPackageStartupMessages({
    library(trackViewer)
    library(rtracklayer)
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    library(org.Hs.eg.db)
    library(VariantAnnotation)
  library(httr)
})
knitr::opts_chunk$set(warning=FALSE, message=FALSE)
```


# Dandelion Plot
Sometimes, there are as many as hundreds of SNPs invoved in one gene. Dandelion plot can be used to depict such dense SNPs. Please note that the height of the dandelion indicates the desity of the SNPs.
```{r fig.width=4.5,fig.height=3}
library(trackViewer)
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
library(rtracklayer)
methy <- import(system.file("extdata", "methy.bed", package="trackViewer"), "BED")
gr <- GRanges("chr22", IRanges(50968014, 50970514, names="TYMP"))
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         gr=gr)
features <- c(range(trs[[1]]$dat), range(trs[[5]]$dat))
names(features) <- c(trs[[1]]$name, trs[[5]]$name)
features$fill <- c("lightblue", "mistyrose")
features$height <- c(.02, .04)
dandelion.plot(methy, features, ranges=gr, type="pin")
```

## Change the type of Dandelion plot
There are one more type for dandelion plot, i.e., type "fan". The area of the fan indicates the percentage of methylation or rate of mutation.
```{r fig.width=4.5,fig.height=3}
methy$color <- 3
methy$border <- "gray"
## Score info is required and the score must be a number in [0, 1]
m <- max(methy$score)
methy$score <- methy$score/m
dandelion.plot(methy, features, ranges=gr, type="fan")
```
```{r fig.width=4.5,fig.height=4}
methy$color <- rep(list(c(3, 5)), length(methy))
methy$score2 <- (max(methy$score) - methy$score)/m
legends <- list(list(labels=c("s1", "s2"), fill=c(3, 5)))
dandelion.plot(methy, features, ranges=gr, type="pie", legend=legends)
```

## Change the number of dandelions
```{r fig.width=4.5,fig.height=3.5}
## Less dandelions
dandelion.plot(methy, features, ranges=gr, type="circle", maxgaps=1/10)
```
```{r fig.width=4.5,fig.height=4}
## More dandelions
dandelion.plot(methy, features, ranges=gr, type="circle", maxgaps=1/100)
```

Users can also specify the maximum distance between neighboring dandelions by setting the maxgaps as a GRanges object.

```{r fig.width=4,fig.height=4}
maxgaps <- tile(gr, n = 10)[[1]]
dandelion.plot(methy, features, ranges=gr, type="circle", maxgaps=maxgaps)
```

## Add y-axis (yaxis)
Set yaxis to TRUE to add y-axis, and set `heightMethod`=`mean` to use the mean score as the height.
```{r fig.width=4.5,fig.height=3}
dandelion.plot(methy, features, ranges=gr, type="pie", 
               maxgaps=1/100, yaxis = TRUE, heightMethod = mean,
               ylab='mean of methy scores')
```
```{r fig.width=4.5,fig.height=3}
yaxis = c(0, 0.5, 1)
dandelion.plot(methy, features, ranges=gr, type="pie", 
               maxgaps=1/100, yaxis = yaxis, heightMethod = mean,
               ylab='mean of methy scores')
```

# Session Info
```{r sessionInfo, results='asis'}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browseTracks.R
\name{browseTracks}
\alias{browseTracks}
\title{browse tracks}
\usage{
browseTracks(
  trackList,
  gr = GRanges(),
  ignore.strand = TRUE,
  width = NULL,
  height = NULL,
  ...
)
}
\arguments{
\item{trackList}{an object of \code{\link{trackList}}}

\item{gr}{an object of \code{\link[GenomicRanges:GRanges-class]{GRanges}}}

\item{ignore.strand}{ignore the strand or not when do filter. default TRUE}

\item{width}{width of the figure}

\item{height}{height of the figure}

\item{...}{parameters not used}
}
\value{
An object of class htmlwidget that will intelligently print itself 
into HTML in a variety of contexts including the R console, 
within R Markdown documents, and within Shiny output bindings.
}
\description{
browse tracks by a web browser.
}
\examples{
extdata <- system.file("extdata", package="trackViewer", mustWork=TRUE)
files <- dir(extdata, "-.wig")
tracks <- lapply(paste(extdata, files, sep="/"), 
                 importScore, format="WIG")
tracks <- lapply(tracks, function(.ele) {strand(.ele@dat) <- "-"; .ele})
names(tracks) <- c("trackA", "trackB")
fox2 <- importScore(paste(extdata, "fox2.bed", sep="/"), format="BED")
dat <- coverageGR(fox2@dat)
fox2@dat <- dat[strand(dat)=="+"]
fox2@dat2 <- dat[strand(dat)=="-"]
gr <- GRanges("chr11", IRanges(122929275, 122930122))
browseTracks(trackList(tracks, fox2), gr=gr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importScSeqScore.R
\name{importScSeqScore}
\alias{importScSeqScore}
\title{plot tracks for single cell RNAseq}
\usage{
importScSeqScore(
  object,
  files,
  samplenames,
  ...,
  txdb,
  gene,
  id,
  idents,
  gr,
  color,
  withCoverageTrack = TRUE,
  flag = scanBamFlag(isSecondaryAlignment = FALSE, isUnmappedQuery = FALSE,
    isNotPassingQualityControls = FALSE, isSupplementaryAlignment = FALSE)
)
}
\arguments{
\item{object}{Seurat object.}

\item{files}{bam file to be scanned.}

\item{samplenames}{sample names for files.}

\item{...}{parameters used by \link[GenomicAlignments:readGAlignments]{readGAlignmentsList} 
or \link[GenomicAlignments]{readGAlignments}}

\item{txdb}{TxDb object for gene model.}

\item{gene}{Gene name to plot. (row value)}

\item{id}{The id of gene used in txdb.}

\item{idents}{indentity class to define the groups to plot. (column value)}

\item{gr}{GRanges object to define the ploting region.}

\item{color}{vector of colors used in heatmap.}

\item{withCoverageTrack}{plot coverage track or not.}

\item{flag}{An integer(2) vector used to filter reads based on their 
'flag' entry.}
}
\description{
Plot single cell RNAseq data as heatmap track for Seurat object.
}
\examples{
\dontrun{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
test_file <- "https://github.com/10XGenomics/subset-bam/raw/master/test/test.bam"
trs <- importScSeqScore(files=test_file, 
                        txdb=TxDb.Hsapiens.UCSC.hg19.knownGene,
                        id="653635", gene = "WASH7P")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reduce_GInteractions.R
\name{reduce,GInteractions-method}
\alias{reduce,GInteractions-method}
\alias{reduce,GInteractions}
\title{Reduce method for 'GInteractions'}
\usage{
\S4method{reduce}{GInteractions}(x, min.gapwidth = 1L, ignore.strand = TRUE, ...)
}
\arguments{
\item{x}{GInteractions object.}

\item{min.gapwidth}{Ranges separated by a gap of at least min.gapwidth
positions are not merged.}

\item{ignore.strand}{TRUE or FALSE. Whether the strand of the input ranges
should be ignored or not.}

\item{...}{Not used.}
}
\description{
Reduce returns an object of the same type as x containing reduced ranges for
each distinct (seqname, strand) pairing.
}
\examples{
\dontrun{
library(InteractionSet) 
gi <- readRDS(system.file("extdata", "gi.rds", package="trackViewer"))
reduce(head(gi, n=20))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importGInteractions.R
\name{importGInteractions}
\alias{importGInteractions}
\title{Reading data from a ginteractions, hic, cool, or validPairs file}
\usage{
importGInteractions(
  file,
  format = c("ginteractions", "hic", "cool", "validPairs"),
  ranges = GRanges(),
  ignore.strand = TRUE,
  out = c("track", "GInteractions"),
  resolution = 1e+05,
  unit = c("BP", "FRAG"),
  normalization = c("NONE", "VC", "VC_SQRT", "KR", "SCALE", "GW_KR", "GW_SCALE", "GW_VC",
    "INTER_KR", "INTER_SCALE", "INTER_VC", "balanced"),
  matrixType = c("observed", "oe", "expected"),
  ...
)
}
\arguments{
\item{file}{The path to the file to read.}

\item{format}{The format of import file. Could be ginteractions, hic, cool or validPairs}

\item{ranges}{An object of \code{\link[GenomicRanges:GRanges-class]{GRanges}} to indicate
the range to be imported. For .hic file, if the length of ranges is 2, 
the first range will be used as anchor 1 and the second range will be used
as anchor 2.}

\item{ignore.strand}{ignore the strand or not when do filter. default TRUE}

\item{out}{output format. Default is track. Possible values: track, GInteractions.}

\item{resolution}{Resolutions for the interaction data.}

\item{unit}{BP (base pair) or FRAG (fragment) (.hic file only).}

\item{normalization}{Type of normalization, NONE, VC, VC_SORT or KR for .hic and
NONE, balanced for .cool.}

\item{matrixType}{Type of matrix for .hic file. Available choices are "observed", "oe", and "expected".
default is "observed".}

\item{...}{NOT used.}
}
\value{
a \code{\link{track}} object
}
\description{
Read a \code{\link{track}} object from a ginteractions, hic, mcool, or validPairs file
}
\examples{
#import a ginteractions file
#gi <- system.file("extdata", "test.ginteractions.tsv", package="trackViewer",
#                       mustWork=TRUE)
#dat <- importGInteractions(file=gi, format="ginteractions",
#                   ranges=GRanges("chr7", IRanges(127471197, 127474697)))

##import a hic file
if(.Platform$OS.type!="windows"){
hic <- system.file("extdata", "test_chr22.hic", package = "trackViewer",
                       mustWork=TRUE)
dat <- importGInteractions(file=hic, format="hic", 
                           ranges=GRanges("22", IRanges(1500000, 100000000)))
}

##import a cool file
cool <- system.file("extdata", "test.mcool", package = "trackViewer",
                     mustWork=TRUE)
dat <- importGInteractions(file=cool, format="cool",
                           resolution = 2,
                           ranges=GRanges("chr1", IRanges(10, 28)))

##import a validPairs file
#validPairs <- system.file("extdata", "test.validPairs", package = "trackViewer",
#                       mustWork=TRUE)
#dat <- importGInteractions(file=validPairs, format="validPairs")

}
\seealso{
See Also as \code{\link{listResolutions}}, \code{\link{listChromosomes}}, 
\code{\link[strawr]{readHicNormTypes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCurTrackViewport.R
\name{getCurTrackViewport}
\alias{getCurTrackViewport}
\title{Get current track viewport}
\usage{
getCurTrackViewport(curViewerStyle, start, end)
}
\arguments{
\item{curViewerStyle}{an object of \code{\link{trackViewerStyle}}}

\item{start}{start position of current track}

\item{end}{end position of current track}
}
\value{
an object of \code{\link[grid]{viewport}}
}
\description{
Get current track viewport for addGuideLine
}
\examples{
vp <- getCurTrackViewport(trackViewerStyle(), 10000, 10200)
addGuideLine(c(10010, 10025, 10150), vp=vp)
}
\seealso{
See Also as \code{\link{addGuideLine}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/viewTracks.R
\name{viewTracks}
\alias{viewTracks}
\title{plot the tracks}
\usage{
viewTracks(
  trackList,
  chromosome,
  start,
  end,
  strand,
  gr = GRanges(),
  ignore.strand = TRUE,
  viewerStyle = trackViewerStyle(),
  autoOptimizeStyle = FALSE,
  newpage = TRUE,
  operator = NULL,
  smooth = FALSE,
  lollipop_style_switch_limit = 10
)
}
\arguments{
\item{trackList}{an object of \code{\link{trackList}}}

\item{chromosome}{chromosome}

\item{start}{start position}

\item{end}{end position}

\item{strand}{strand}

\item{gr}{an object of \code{\link[GenomicRanges:GRanges-class]{GRanges}}}

\item{ignore.strand}{ignore the strand or not when do filter. default TRUE}

\item{viewerStyle}{an object of \code{\link{trackViewerStyle}}}

\item{autoOptimizeStyle}{should use \code{\link{optimizeStyle}} to optimize style}

\item{newpage}{should be draw on a new page?}

\item{operator}{operator, could be +, -, *, /, ^, \%\%, and NA. "-" means dat - dat2, 
and so on. NA means do not apply any operator.
Note: if multiple operator is supplied, please make sure the length of operator keep
same as the length of trackList.}

\item{smooth}{logical(1) or numeric(). Plot smooth curve or not. If it is numeric, eg n,
mean of nearby n points will be used for plot.
If it is numeric, the second number will be the color. Default coloer is 2 (red).}

\item{lollipop_style_switch_limit}{The cutoff value for lollipop style for the 'circle' type.
If the max score is greater than this cutoff value, trackViewer will only plot one shape at
the highest score. Otherwise trackViewer will draw the shapes like `Tanghulu`.}
}
\value{
An object of \code{\link[grid]{viewport}} for \code{\link{addGuideLine}}
}
\description{
A function to plot the data for given range
}
\examples{
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
files <- dir(extdata, "-.wig")
tracks <- lapply(paste(extdata, files, sep="/"), 
                 importScore, format="WIG")
tracks <- lapply(tracks, function(.ele) {strand(.ele@dat) <- "-"; .ele})
fox2 <- importScore(paste(extdata, "fox2.bed", sep="/"), format="BED")
dat <- coverageGR(fox2@dat)
fox2@dat <- dat[strand(dat)=="+"]
fox2@dat2 <- dat[strand(dat)=="-"]
gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
viewTracks(trackList(track=tracks, fox2=fox2), gr=gr, autoOptimizeStyle=TRUE)
}
\seealso{
See Also as \code{\link{addGuideLine}}, \code{\link{addArrowMark}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/viewGene.R
\name{viewGene}
\alias{viewGene}
\title{plot tracks based on gene name}
\usage{
viewGene(
  symbol,
  filenames,
  format,
  txdb,
  org,
  upstream = 1000,
  downstream = 1000,
  anchor = c("gene", "TSS"),
  plot = FALSE
)
}
\arguments{
\item{symbol}{Gene symbol}

\item{filenames}{files used to generate tracks}

\item{format}{file format used to generate tracks}

\item{txdb}{txdb will be used to extract the genes}

\item{org}{org package name}

\item{upstream}{upstream from anchor}

\item{downstream}{downstream from anchor}

\item{anchor}{TSS, or gene}

\item{plot}{plot the tracks or not.}
}
\value{
an invisible list of a \code{\link{trackList}},
 a \code{\link{trackViewerStyle}} and a \code{\link[GenomicRanges:GRanges-class]{GRanges}}
}
\description{
given a gene name, plot the tracks.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
extdata <- system.file("extdata", package="trackViewer", mustWork=TRUE)
filename = file.path(extdata, "fox2.bed")
optSty <- viewGene("HSPA8", filenames=filename, format="BED", 
                   txdb=TxDb.Hsapiens.UCSC.hg19.knownGene, 
                   org="org.Hs.eg.db")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GRoperator.R
\name{GRoperator}
\alias{GRoperator}
\title{GRanges operator}
\usage{
GRoperator(
  A,
  B,
  col = "score",
  operator = c("+", "-", "*", "/", "^", "\%\%"),
  ignore.strand = TRUE
)
}
\arguments{
\item{A}{an object of GRanges}

\item{B}{an object of GRanges}

\item{col}{colname of A and B to be calculated}

\item{operator}{operator, "+" means A + B, and so on. 
User-defined function also could be used.}

\item{ignore.strand}{When set to TRUE, the strand information is 
ignored in the overlap calculations.}
}
\value{
an object of GRanges
}
\description{
GRanges operations (add, aubtract, multiply, divide)
}
\examples{
gr2 <- GRanges(seqnames=c("chr1", "chr1"),
ranges=IRanges(c(7,13), width=3),
strand=c("-", "-"), score=3:4)
gr3 <- GRanges(seqnames=c("chr1", "chr1"),
               ranges=IRanges(c(1, 4), c(3, 9)),
               strand=c("-", "-"), score=c(6L, 2L))
GRoperator(gr2, gr3, col="score", operator="+")
GRoperator(gr2, gr3, col="score", operator="-")
GRoperator(gr2, gr3, col="score", operator="*")
GRoperator(gr2, gr3, col="score", operator="/")
GRoperator(gr2, gr3, col="score", operator=mean)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addArrowMark.R
\name{addArrowMark}
\alias{addArrowMark}
\title{Add arrow mark to the figure at a given position}
\usage{
addArrowMark(
  pos = grid.locator(),
  label = NULL,
  angle = 15,
  length = unit(0.25, "inches"),
  col = "red",
  cex = 1,
  quadrant = 4,
  type = "closed",
  vp = NULL
)
}
\arguments{
\item{pos}{A unit object representing the location of arrow mark to be placed 
at current viewport. Default is the value of grid.locator, which will get the 
location of the mouse click.}

\item{label}{A character or expression vector.}

\item{angle}{A parameter passed into grid::arrow function. The angle of arrow 
head in degrees (smaller numbers produce narrower, pointier arrows). Essentially 
describes the width of the arrow head.}

\item{length}{A parameter passed into grid::arrow function. Aunit specifying
the length of the arrow head.}

\item{col}{color of the arrow}

\item{cex}{Multiplier applied to fontsize}

\item{quadrant}{the direction of arrow, 1: to bottomleft, 2: to bottomright,
3: to topright, 4: to topleft}

\item{type}{A parameter passed into grid::arrow function. One of "open" or "closed"
indicating whether the arrow head should be a closed triangle.}

\item{vp}{A Grid viewport object. It must be output of \code{\link{viewTracks}}}
}
\value{
invisible x, y position value.
}
\description{
A function to add arrow mark for emphasizing peaks
}
\examples{
grid.newpage()
addArrowMark(list(x=unit(.5, "npc"), 
               y=unit(.5, "npc")), 
               label="label1",
               col="blue")
##  how to get the position by mouse click
if(interactive()){
pos <- addArrowMark(label="byClick")
addArrowMark(pos, label="samePosAsAbove")
}

}
\seealso{
See Also as \code{\link{addGuideLine}}, \code{\link[grid:arrow]{arrow}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dandelion.plot.R
\name{dandelion.plot}
\alias{dandelion.plot}
\title{dandelion.plots}
\usage{
dandelion.plot(
  SNP.gr,
  features = NULL,
  ranges = NULL,
  type = c("fan", "circle", "pie", "pin"),
  newpage = TRUE,
  ylab = TRUE,
  ylab.gp = gpar(col = "black"),
  xaxis = TRUE,
  xaxis.gp = gpar(col = "black"),
  yaxis = FALSE,
  yaxis.gp = gpar(col = "black"),
  legend = NULL,
  cex = 1,
  maxgaps = 1/50,
  heightMethod = NULL,
  label_on_feature = FALSE,
  ...
)
}
\arguments{
\item{SNP.gr}{A object of \link[GenomicRanges:GRanges-class]{GRanges} or 
\link[GenomicRanges:GRangesList-class]{GRangesList}. 
All the width of GRanges must be 1.}

\item{features}{A object of \link[GenomicRanges:GRanges-class]{GRanges} or
\link[GenomicRanges:GRangesList-class]{GRangesList}.}

\item{ranges}{A object of \link[GenomicRanges:GRanges-class]{GRanges} or 
\link[GenomicRanges:GRangesList-class]{GRangesList}.}

\item{type}{Character. Could be fan, circle, pie or pin.}

\item{newpage}{plot in the new page or not.}

\item{ylab}{plot ylab or not. If it is a character vector, 
the vector will be used as ylab.}

\item{ylab.gp, xaxis.gp, yaxis.gp}{An object of class gpar for ylab, xaxis or yaxis.}

\item{xaxis, yaxis}{plot xaxis/yaxis or not. If it is a numeric vector with length 
greater than 1, the vector will be used as the 
points at which tick-marks are to be drawn. And the names of the vector will be
used to as labels to be placed at the tick points if it has names.}

\item{legend}{If it is a list with named color vectors, a legend will be added.}

\item{cex}{cex will control the size of circle.}

\item{maxgaps}{maxgaps between the stem of dandelions. 
It is calculated by the width of plot region divided by maxgaps. 
If a GRanges object is set, the dandelions stem will be clustered
in each genomic range.}

\item{heightMethod}{A function used to determine the height of stem of 
dandelion. eg. Mean. Default is length.}

\item{label_on_feature}{Labels of the feature directly on them. 
Default FALSE.}

\item{...}{not used.}
}
\description{
Plot variants and somatic mutations
}
\details{
In SNP.gr and features, metadata of the GRanges object will be used to 
control the color, fill, border, height, data source of pie if the type is pie.
}
\examples{
SNP <- c(10, 100, 105, 108, 400, 410, 420, 600, 700, 805, 840, 1400, 1402)
SNP.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)), 
                  score=sample.int(100, length(SNP))/100)
features <- GRanges("chr1", IRanges(c(1, 501, 1001), 
                                    width=c(120, 500, 405),
                                    names=paste0("block", 1:3)),
                    color="black",
                    fill=c("#FF8833", "#51C6E6", "#DFA32D"),
                    height=c(0.1, 0.05, 0.08))
dandelion.plot(SNP.gr, features, type="fan")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ARA.R
\name{ARA}
\alias{ARA}
\title{Aggregate Region Analysis}
\usage{
ARA(gr, upstream = 2e+05, downstream = upstream, resolution = 10000, ...)
}
\arguments{
\item{gr}{A `GRanges` object. The center of the object will be used for
alignment for all the given regions.}

\item{upstream, downstream}{numeric(1L). Upstream and downstream from the 
center of given `gr` input will be used to extract the signals.}

\item{resolution}{numeric(1L). The resolution will be passed to
\link{importGInteractions} function.}

\item{...}{The parameters used by \link{importGInteractions} function.
Please note that the ranges resolution and out parameter should not be
involved.}
}
\value{
A \link[InteractionSet:GInteractions-class]{GInteractions} object
with scores which represent the mean values of the interactions.
}
\description{
Extract the interaction signal means from given coordinates.
}
\examples{
hic <- system.file("extdata", "test_chr22.hic", package = "trackViewer",
                   mustWork=TRUE)
gr <- GRanges("22", c(seq(20000001, 50000001, by=10000000), width=1))
gi <- ARA(gr, file=hic, format="hic")
rg <- GRanges("22", IRanges(1, 400000))
op <- optimizeStyle(trackList(gi2track(gi)))
heatmap <- op$tracks
sty <- op$style
setTrackViewerStyleParam(sty, "xat", c(1, 200000, 400000))
setTrackViewerStyleParam(sty, "xlabel",c("-20K", "center", "20K"))
viewTracks(heatmap, viewerStyle=sty, gr=rg)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGeneIDsFromTxDb.R
\name{getGeneIDsFromTxDb}
\alias{getGeneIDsFromTxDb}
\title{get gene ids by genomic location}
\usage{
getGeneIDsFromTxDb(gr, txdb)
}
\arguments{
\item{gr}{GRanges object.}

\item{txdb}{An object of \code{\link[GenomicFeatures:TxDb-class]{TxDb}}.}
}
\value{
A character vector of gene ids
}
\description{
retrieve gene ids from txdb object by genomic location.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
gr <- parse2GRanges("chr11:122,830,799-123,116,707")
ids <- getGeneIDsFromTxDb(gr, TxDb.Hsapiens.UCSC.hg19.knownGene)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizeStyle.R
\name{optimizeStyle}
\alias{optimizeStyle}
\title{Optimize the style of plot}
\usage{
optimizeStyle(trackList, viewerStyle = trackViewerStyle(), theme = NULL)
}
\arguments{
\item{trackList}{An object of \code{\link{trackList}}}

\item{viewerStyle}{An object of \code{\link{trackViewerStyle}}}

\item{theme}{A character string. Could be "bw", "col" or "safe".}
}
\value{
a list of a \code{\link{trackList}} and a \code{\link{trackViewerStyle}}
}
\description{
Automatic optimize the stlye of trackViewer
}
\examples{
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
files <- dir(extdata, ".wig")
tracks <- lapply(paste(extdata, files, sep="/"), 
                 importScore, format="WIG")
re <- optimizeStyle(trackList(tracks))
trackList <- re$tracks
viewerStyle <- re$style
}
\seealso{
See Also as \code{\link{viewTracks}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allclasses.R
\docType{class}
\name{trackStyle-class}
\alias{trackStyle-class}
\alias{trackStyle}
\alias{track-class}
\alias{track}
\alias{seqlevels,track-method}
\alias{seqlevelsStyle,track-method}
\alias{seqlevelsStyle<-,track-method}
\alias{show,track-method}
\alias{$,track-method}
\alias{$,trackStyle-method}
\alias{$<-,track-method}
\alias{$<-,trackStyle-method}
\alias{setTrackStyleParam}
\alias{setTrackStyleParam,track,character-method}
\alias{setTrackStyleParam,track,character,ANY-method}
\alias{setTrackXscaleParam}
\alias{setTrackXscaleParam,track,character-method}
\alias{setTrackXscaleParam,track,character,ANY-method}
\alias{setTrackYaxisParam}
\alias{setTrackYaxisParam,track,character-method}
\alias{setTrackYaxisParam,track,character,ANY-method}
\title{Class \code{"trackStyle"}}
\usage{
\S4method{seqlevels}{track}(x)

\S4method{seqlevelsStyle}{track}(x)

\S4method{seqlevelsStyle}{track}(x) <- value

\S4method{show}{track}(object)

\S4method{$}{track}(x, name)

\S4method{$}{track}(x, name) <- value

setTrackStyleParam(ts, attr, value)

\S4method{setTrackStyleParam}{track,character}(ts, attr, value)

setTrackXscaleParam(ts, attr, value)

\S4method{setTrackXscaleParam}{track,character}(ts, attr, value)

setTrackYaxisParam(ts, attr, value)

\S4method{setTrackYaxisParam}{track,character}(ts, attr, value)
}
\arguments{
\item{x}{an object of trackStyle or track}

\item{value}{values to be assigned.}

\item{object}{an object of trackStyle.}

\item{name}{slot name of trackStyle or track}

\item{ts}{An object of \code{track}.}

\item{attr}{the name of slot of \code{\link{trackStyle}} object to be changed.}
}
\description{
An object of class \code{"trackStyle"} represents track style.

An object of class \code{"track"} represents scores of a given track.
}
\details{
The attr of \code{setTrackXscaleParam} could not only be a slot of xscale, but also be position.
If the attr is set to position, value must be a list of x, y and label. For example
setTrackXscaleParam(track, attr="position", value=list(x=122929675, y=4, label=500))
}
\section{Slots}{

\describe{
\item{\code{tracktype}}{\code{"character"} track type, could be peak, 
line, histogram, or annotation. 
Default is "peak". "annotation" is used to mark the peak regions.
For interaction data,
it could be "heatmap" or "link".}

\item{\code{color}}{\code{"character"} track color. If the track has dat and dat2 slot,
it should have two values.}

\item{\code{NAcolor}}{\code{"character"} NA color for interactionData.}

\item{\code{breaks}}{\code{"numeric"} breaks for color keys of interactionData.}

\item{\code{height}}{\code{"numeric"} track height. It should be a value between 0 and 1}

\item{\code{marginTop}}{\code{"numeric"} track top margin}

\item{\code{marginBottom}}{\code{"numeric"} track bottom margin}

\item{\code{xscale}}{object of \code{\link{xscale}}, describe the details of x-scale}

\item{\code{yaxis}}{object of \code{\link{yaxisStyle}}, describe the details of y-axis}

\item{\code{ylim}}{\code{"numeric"} y-axis range}

\item{\code{ylabpos}}{\code{"character"}, ylable postion, ylabpos should 
be 'left', 'right', 'topleft', 'bottomleft', 'topright', 'bottomright',
'abovebaseline', 'underbaseline', or 'none'.
For gene type track, it also could be 'upstream' or 'downstream'}

\item{\code{ylablas}}{\code{"numeric"} y lable direction. It should 
be a integer 0-3. See \code{\link[graphics]{par}:las}}

\item{\code{ylabgp}}{A \code{"list"} object, It will convert to an object of 
class \code{\link[grid]{gpar}}. This is basically a list of graphical 
parameter settings of y-label.}

\item{\code{ysplit}}{A \code{"numeric"} to split y plot region for interaction data.
Default is 0.5, which will split the region half to half. It will only work
for back to back plot.}

\item{\code{dat}}{Object of class \code{\link[GenomicRanges:GRanges-class]{GRanges}}
the scores of a given track. It should contain score metadata.}

\item{\code{dat2}}{Object of class \code{\link[GenomicRanges:GRanges-class]{GRanges}}
the scores of a given track. It should contain score metadata. When dat2
and dat is paired, dat will be drawn as positive value where dat2 will be 
drawn as negative value (-1 * score)}

\item{\code{type}}{The type of track. It could be 'data', 'gene', 'transcript', 'scSeq', 'lollipopData' or 'interactionData'.}

\item{\code{format}}{The format of the input. It could be "BED", "bedGraph",
"WIG", "BigWig" or "BAM"}

\item{\code{style}}{Object of class \code{\link{trackStyle}}}

\item{\code{name}}{unused yet}
}}

\examples{
extdata <- system.file("extdata", package="trackViewer",
mustWork=TRUE)
fox2 <- importScore(file.path(extdata, "fox2.bed"), format="BED")
setTrackStyleParam(fox2, "color", c("red","green"))
setTrackXscaleParam(fox2, "gp", list(cex=.5))
setTrackYaxisParam(fox2, "gp", list(col="blue"))
fox2$dat <- GRanges(score=numeric(0))
}
\seealso{
Please try to use \code{\link{importScore}} and \code{\link{importBam}} to 
generate the object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importBam.R
\name{importBam}
\alias{importBam}
\title{Reading data from a BAM file}
\usage{
importBam(file, file2, ranges = GRanges(), pairs = FALSE)
}
\arguments{
\item{file}{The path to the BAM file to read.}

\item{file2}{The path to the second BAM file to read.}

\item{ranges}{An object of \code{\link[GenomicRanges:GRanges-class]{GRanges}} to indicate
the range to be imported}

\item{pairs}{logical object to indicate the BAM is paired or not. See
\code{\link[GenomicAlignments]{readGAlignments}}}
}
\value{
a \code{\link{track}} object
}
\description{
Read a \code{\link{track}} object from a BAM file
}
\examples{
bamfile <- system.file("extdata", "ex1.bam", package="Rsamtools",
mustWork=TRUE)
dat <- importBam(file=bamfile, ranges=GRanges("seq1", IRanges(1, 50), strand="+"))
}
\seealso{
See Also as \code{\link{importScore}}, \code{\link{track}}, 
\code{\link{viewTracks}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGRanges.R
\name{plotGRanges}
\alias{plotGRanges}
\title{plot GRanges data}
\usage{
plotGRanges(
  ...,
  range = GRanges(),
  viewerStyle = trackViewerStyle(),
  autoOptimizeStyle = FALSE,
  newpage = TRUE
)
}
\arguments{
\item{\dots}{one or more objects of \code{\link[GenomicRanges:GRanges-class]{GRanges}}}

\item{range}{an object of \code{\link[GenomicRanges:GRanges-class]{GRanges}}}

\item{viewerStyle}{an object of \code{\link{trackViewerStyle}}}

\item{autoOptimizeStyle}{should use \code{\link{optimizeStyle}} to optimize style}

\item{newpage}{should be draw on a new page?}
}
\value{
An object of \code{\link[grid]{viewport}} for \code{\link{addGuideLine}}
}
\description{
A function to plot GRanges data for given range
}
\examples{
gr1 <- GRanges("chr1", IRanges(1:50, 51:100))
gr2 <- GRanges("chr1", IRanges(seq(from=10, to=80, by=5),
                               seq(from=20, to=90, by=5)))
vp <- plotGRanges(gr1, gr2, range=GRanges("chr1", IRanges(1, 100)))
addGuideLine(guideLine=c(5, 10, 50, 90), col=2:5, vp=vp)

gr <- GRanges("chr1", IRanges(c(1, 11, 21, 31), width=9), 
              score=c(5, 10, 5, 1))
plotGRanges(gr, range=GRanges("chr1", IRanges(1, 50)))
}
\seealso{
See Also as \code{\link{addGuideLine}}, \code{\link{addArrowMark}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadIdeogram.R
\name{loadIdeogram}
\alias{loadIdeogram}
\title{load ideogram from UCSC}
\usage{
loadIdeogram(genome, chrom = NULL, ranges = NULL, ...)
}
\arguments{
\item{genome}{Assembly name  assigned by UCSC, such as hg38, mm10.}

\item{chrom}{A character vector of chromosome names, or NULL.}

\item{ranges}{A \link[IRanges:IRanges-class]{Ranges} object with the intervals.}

\item{...}{Additional arguments to pass to the 
\link[GenomicRanges:GRanges-class]{GRanges} constructor.}
}
\value{
A \link[GenomicRanges:GRanges-class]{GRanges} object.
}
\description{
Download ideogram table from UCSC
}
\examples{
\dontrun{
head(loadIdeogram("hg38", chrom = "chr1"))
}


}
\seealso{
See Also as \code{\link{ideogramPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getLocation.R
\name{getLocation}
\alias{getLocation}
\title{get genomic location by gene symbol}
\usage{
getLocation(symbol, txdb, org)
}
\arguments{
\item{symbol}{Gene symbol}

\item{txdb}{txdb will be used to extract the genes}

\item{org}{org package name}
}
\description{
given a gene name, get the genomic coordinates.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
getLocation("HSPA8", TxDb.Hsapiens.UCSC.hg19.knownGene, "org.Hs.eg.db")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ideogramPlot.R
\name{ideogramPlot}
\alias{ideogramPlot}
\title{plot ideogram with data}
\usage{
ideogramPlot(
  ideo,
  dataList,
  layout = NULL,
  horiz = TRUE,
  parameterList = list(vp = plotViewport(margins = c(0.1, 4.1, 0.3, 0.1)), ideoHeight =
    unit(1/(1 + length(dataList)), "npc"), vgap = unit(0.3, "lines"), ylabs = "auto",
    ylabsRot = ifelse(horiz, 0, 90), ylabsPos = unit(2.5, "lines"), xaxis = FALSE, yaxis
    = FALSE, xlab = "", types = "barplot", heights = NULL, dataColumn = "score", gps =
    gpar(col = "black", fill = "gray")),
  colorSheme = gieStain(),
  gp = gpar(fill = NA, lwd = 2),
  ...
)
}
\arguments{
\item{ideo}{output of \link{loadIdeogram}.}

\item{dataList}{a \link[GenomicRanges:GRangesList-class]{GRangesList} of data to plot.}

\item{layout}{The layout of chromosomes. Could be a list with chromosome names
as its elements.}

\item{horiz}{a logical value. If FALSE, the ideograms are drawn vertically 
to the left. If TRUE, the ideograms are drawn horizontally at the bottom.}

\item{parameterList}{a list of parameters for each dataset in the dataList. 
The elements of the parameters could be xlabs, ylabs, etc. type could be
barplot, line, point, heatmap.}

\item{colorSheme}{A character vector of giemsa stain colors.}

\item{gp}{parameters used for \link[grid]{grid.roundrect}.}

\item{...}{parameters not used.}
}
\description{
plot ideogram with data for multiple chromosomes
}
\examples{
\dontrun{
ideo <- loadIdeogram("hg38")
library(rtracklayer)
library(grid)
dataList <- ideo
dataList$score <- as.numeric(dataList$gieStain)
dataList <- dataList[dataList$gieStain!="gneg"]
dataList <- GRangesList(dataList)
grid.newpage()
ideogramPlot(ideo, dataList, 
             layout=list("chr1", "chr2", c("chr3", "chr22"), 
                         c("chr4", "chr21"), c("chr5", "chr20"), 
                         c("chr6", "chr19"), c("chr7", "chr18"),
                         c("chr8", "chr17"), c("chr9", "chr16"),
                         c("chr10", "chr15"), c("chr11", "chr14"),
                         c("chr12", "chr13"), c("chrX", "chrY")),
             parameterList = list(types="heatmap", colorKeyTitle="sample1"))
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotOneIdeo.R
\name{gridPlot}
\alias{gridPlot}
\title{plot GRanges metadata}
\usage{
gridPlot(gr, gp, type, xscale)
}
\arguments{
\item{gr}{an object of \link[GenomicRanges:GRanges-class]{GRanges} with metadata. All 
metadata must be numeric.}

\item{gp}{an object of \link[grid]{gpar}}

\item{type}{type of the figure, could be barplot, line, point and heatmap}

\item{xscale}{x scale of the viewport}
}
\description{
plot GRanges metadata for different types
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lollipop.plot.R
\name{lolliplot}
\alias{lolliplot}
\title{Lolliplots}
\usage{
lolliplot(
  SNP.gr,
  features = NULL,
  ranges = NULL,
  type = "circle",
  newpage = TRUE,
  ylab = TRUE,
  ylab.gp = gpar(col = "black"),
  yaxis = TRUE,
  yaxis.gp = gpar(col = "black"),
  xaxis = TRUE,
  xaxis.gp = gpar(col = "black"),
  legend = NULL,
  legendPosition = "top",
  cex = 1,
  dashline.col = "gray80",
  jitter = c("node", "label"),
  rescale = FALSE,
  label_on_feature = FALSE,
  lollipop_style_switch_limit = 10,
  ...
)
}
\arguments{
\item{SNP.gr}{A object of \link[GenomicRanges:GRanges-class]{GRanges}, 
\link[GenomicRanges:GRangesList-class]{GRangesList}
or a list of \link[GenomicRanges:GRanges-class]{GRanges}.
All the width of GRanges must be 1.}

\item{features}{A object of \link[GenomicRanges:GRanges-class]{GRanges}, 
\link[GenomicRanges:GRangesList-class]{GRangesList}
or a list of \link[GenomicRanges:GRanges-class]{GRanges}. 
The metadata 'featureLayerID' are used for drawing features in different layers.
 See details in vignette.}

\item{ranges}{A object of \link[GenomicRanges:GRanges-class]{GRanges} or 
\link[GenomicRanges:GRangesList-class]{GRangesList}.}

\item{type}{character. Could be circle, pie, pin, pie.stack or flag.}

\item{newpage}{Plot in the new page or not.}

\item{ylab}{Plot ylab or not. If it is a character vector, 
the vector will be used as ylab.}

\item{ylab.gp, xaxis.gp, yaxis.gp}{An object of class gpar for ylab, xaxis or yaxis.}

\item{yaxis}{Plot yaxis or not.}

\item{xaxis}{Plot xaxis or not. If it is a numeric vector with length greater than 1, 
the vector will be used as the points at which tick-marks are to be drawn. 
And the names of the vector will be used to as labels to be placed at the tick 
points if it has names.}

\item{legend}{If it is a list with named color vectors, a legend will be added.}

\item{legendPosition}{The position of legend. Possible positions are 'top',
'right', and 'left'.}

\item{cex}{cex will control the size of circle.}

\item{dashline.col}{color for the dashed line.}

\item{jitter}{jitter the position of nodes or labels.}

\item{rescale}{logical(1), character(1), numeric vector,
  or a dataframe with rescale from and to. 
Rescalse the x-axis or not.
if dataframe is used, colnames must be from.start, from.end,
 to.start, to.end. And the from scale must cover the whole plot region.
The rescale parameter can be set as "exon" or "intron" to 
emphasize "exon" or "intron" region. The "exon" or "intron"
can be followed with an integer e.g. "exon_80", or "intron_99".
The integer indicates the total percentage of "exon" or "intron" region.
Here "exon" indicates all regions in features. 
And "intron" indicates all flank regions of the features.}

\item{label_on_feature}{Labels of the feature directly on them. 
Default FALSE.}

\item{lollipop_style_switch_limit}{The cutoff value for lollipop style for the 'circle' type.
If the max score is greater than this cutoff value, trackViewer will only plot one shape at
the highest score. Otherwise trackViewer will draw the shapes like `Tanghulu`.}

\item{...}{not used.}
}
\description{
Plot variants and somatic mutations
}
\details{
In SNP.gr and features, metadata of the GRanges object will be used to control the 
color, fill, border, alpha, shape, height, cex, dashline.col, data source of pie if the type is pie. 
And also the controls for labels by name the metadata start as 
label.parameter.<properties>, and for node labels by name the metadata start as
node.label.<properties>,
such as label.parameter.rot, label.parameter.gp. The parameter is used for 
\link[grid]{grid.text}.or \link[motifStack]{plotMotifLogoA}.
The metadata 'featureLayerID' for features are used 
for drawing features in different layers. The metadata 'SNPsideID' for SNP.gr
are used for determining the side of lollipops. And the 'SNPsideID' could only
be 'top' or 'bottom'.
}
\examples{
SNP <- c(10, 100, 105, 108, 400, 410, 420, 600, 700, 805, 840, 1400, 1402)
x <- sample.int(100, length(SNP))
SNP.gr <- GRanges("chr1", IRanges(SNP, width=1, names=paste0("snp", SNP)), 
                  value1=x, value2=100-x)
SNP.gr$color <- rep(list(c("red", 'blue')), length(SNP))
SNP.gr$border <- sample.int(7, length(SNP), replace=TRUE)
features <- GRanges("chr1", IRanges(c(1, 501, 1001), 
                                    width=c(120, 500, 405),
                                    names=paste0("block", 1:3)),
                    color="black",
                    fill=c("#FF8833", "#51C6E6", "#DFA32D"),
                    height=c(0.1, 0.05, 0.08),
                    label.parameter.rot=45)
lolliplot(SNP.gr, features, type="pie") 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotOneIdeo.R
\name{plotOneIdeo}
\alias{plotOneIdeo}
\title{plot ideogram with data for one chromosome}
\usage{
plotOneIdeo(
  ideo,
  dataList,
  parameterList = list(vp = plotViewport(margins = c(0.1, 4.1, 1.1, 0.1)), ideoHeight =
    unit(1/(1 + length(dataList)), "npc"), vgap = unit(1, "lines"), ylabs =
    seqlevels(ideo)[1], ylabsRot = 90, ylabsPos = unit(2.5, "lines"), xaxis = FALSE,
    yaxis = FALSE, xlab = "", types = "barplot", heights = NULL, dataColumn = "score",
    gps = gpar(col = "black", fill = "gray")),
  chrom = seqlevels(ideo)[1],
  colorSheme = gieStain(),
  gp = gpar(fill = NA, lwd = 2),
  ...
)
}
\arguments{
\item{ideo}{output of \link{loadIdeogram}.}

\item{dataList}{a \link[GenomicRanges:GRangesList-class]{GRangesList} of data to plot.}

\item{parameterList}{a list of parameters for each dataset in the dataList. 
The elements of the parameters could be xlabs, ylabs, etc. type could be
barplot, line, point, heatmap.}

\item{chrom}{A length 1 character vector of chromosome name.}

\item{colorSheme}{A character vector of giemsa stain colors.}

\item{gp}{parameters used for \link[grid]{grid.roundrect}.}

\item{...}{parameters not used.}
}
\description{
plot ideogram with data for one chromosome
}
\examples{
\dontrun{
ideo <- loadIdeogram("hg38")
library(rtracklayer)
library(grid)
dataList <- ideo[seqnames(ideo) \%in\% "chr1"]
dataList$score <- as.numeric(dataList$gieStain)
dataList <- dataList[dataList$gieStain!="gneg"]
dataList <- GRangesList(dataList, dataList)
grid.newpage()
plotOneIdeo(ideo, dataList, chrom="chr1")
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GIoperator.R
\name{GIoperator}
\alias{GIoperator}
\title{GInteractions operator}
\usage{
GIoperator(gi_list, col = "score", operator = c("+", "-", "*", "/"))
}
\arguments{
\item{gi_list}{a list of GInteractions objects}

\item{col}{colname of metadata to be calculated}

\item{operator}{operator, "+" means A + B, and so on. 
User-defined function also could be used.}
}
\value{
an object of GInteractions
}
\description{
GInteractions operations (add, aubtract, multiply, divide)
}
\examples{
library(InteractionSet)
gr2 <- GRanges(seqnames=c("chr1", "chr1"),
               ranges=IRanges(c(7,13), width=3))
gr3 <- GRanges(seqnames=c("chr1", "chr1"),
               ranges=IRanges(c(1, 4), c(3, 9)))
gi <- GInteractions(gr2, gr3, score=c(1, 2))
gi2 <- GInteractions(gr2, gr3, score=c(3, 4))
GIoperator(list(gi, gi2), col="score", operator="+")
GIoperator(list(gi, gi2), col="score", operator="-")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse2GRanges.R
\name{parse2GRanges}
\alias{parse2GRanges}
\title{parse text into GRanges}
\usage{
parse2GRanges(text)
}
\arguments{
\item{text}{character vector like "chr13:99,443,451-99,848,821:-" or 
"chr13:99,443,451-99,848,821"}
}
\value{
an object of \link[GenomicRanges:GRanges-class]{GRanges}
}
\description{
parse text like "chr13:99,443,451-99,848,821:-" into GRanges
}
\examples{
parse2GRanges("chr13:99,443,451-99,848,821:-")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverageGR.R
\name{coverageGR}
\alias{coverageGR}
\title{calculate coverage}
\usage{
coverageGR(gr)
}
\arguments{
\item{gr}{an object of RGanges, GAlignments or GAlignmentPairs}
}
\value{
an object of GRanges
}
\description{
calculate coverage for \code{\link[GenomicRanges:GRanges-class]{GRanges}},
 \code{\link[GenomicAlignments:GAlignments-class]{GAlignments}} or 
 \code{\link[GenomicAlignments:GAlignmentPairs-class]{GAlignmentPairs}}
}
\examples{
bed <- system.file("extdata", "fox2.bed", package="trackViewer",
                   mustWork=TRUE)
fox2 <- importScore(bed)
fox2$dat <- coverageGR(fox2$dat)

}
\seealso{
See Also as \code{\link[IRanges:coverage-methods]{coverage}}, 
\code{\link[GenomicRanges]{coverage-methods}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allclasses.R
\docType{class}
\name{yaxisStyle-class}
\alias{yaxisStyle-class}
\alias{yaxisStyle}
\title{Class \code{"yaxisStyle"}}
\description{
An object of class \code{"yaxisStyle"} represents y-axis style.
}
\section{Slots}{

\describe{
\item{\code{at}}{\code{"numeric"} vector of y-value locations for the tick marks}

\item{\code{label}}{\code{"logical"} value indicating whether to draw the 
labels on the tick marks.}

\item{\code{gp}}{A \code{"list"} object, It will convert to an object of 
class \code{\link[grid]{gpar}}. This is basically a list of graphical 
parameter settings of y-axis.}

\item{\code{draw}}{A \code{"logical"} value indicating whether the y-axis
should be draw.}

\item{\code{main}}{A \code{"logical"} value indicating whether the y-axis
should be draw in left (TRUE) or right (FALSE).}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addInteractionAnnotation.R
\name{addInteractionAnnotation}
\alias{addInteractionAnnotation}
\title{Add annotation markers to the figure at a given position}
\usage{
addInteractionAnnotation(
  obj,
  idx,
  FUN = grid.polygon,
  panel = c("top", "bottom"),
  ...
)
}
\arguments{
\item{obj}{A \link[InteractionSet:GInteractions-class]{GInteractions} object,
\link[GenomicRanges:GRanges-class]{GRanges} object or numeric vector.
For numeric vector, the positive value will generate a line with slope 1 and
negative value will generate a line at the position with slope -1.}

\item{idx}{The layer number of track.}

\item{FUN}{Function for plot. Available functions are 
\link[grid:grid.polygon]{grid.polygon}, \link[grid:grid.lines]{grid.lines},
and \link[grid:grid.text]{grid.text} for GInteractions object;
\link[grid:grid.lines]{grid.lines},
and \link[grid:grid.text]{grid.text} for GRanges object;
FUN is not used for numeric vector.}

\item{panel}{Plot regions. Available values are "top", "bottom".}

\item{...}{Parameters will be passed to FUN.}
}
\value{
invisible viewport for plot region.
}
\description{
A function to add annotation markers for emphasizing interactions
}
\examples{
library(trackViewer)
library(InteractionSet)
gi <- readRDS(system.file("extdata", "nij.chr6.51120000.53200000.gi.rds",
 package="trackViewer"))
tads <- GInteractions(
GRanges("chr6", 
        IRanges(c(51130001, 51130001, 51450001, 52210001), width = 20000)),
GRanges("chr6", 
        IRanges(c(51530001, 52170001, 52210001, 53210001), width = 20000)))
range <- GRanges("chr6", IRanges(51120000, 53200000))
tr <- gi2track(gi)
viewTracks(trackList(tr), 
           gr=range, autoOptimizeStyle = TRUE)
addInteractionAnnotation(tads, "tr", grid.lines,
 gp=gpar(col = "#E69F00", lwd=3, lty=3))

}
\seealso{
See Also as \code{\link{addGuideLine}}, \code{\link{addArrowMark}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addGuideLine.R
\name{addGuideLine}
\alias{addGuideLine}
\title{Add guide lines to the tracks}
\usage{
addGuideLine(guideLine, col = "gray", lty = "dashed", lwd = 1, vp = NULL)
}
\arguments{
\item{guideLine}{The genomic coordinates to draw the lines}

\item{col}{A vector for the line color}

\item{lty}{A vector for the line type}

\item{lwd}{A vector for the line width}

\item{vp}{A Grid viewport object. It must be output of \code{\link{viewTracks}}}
}
\description{
A function to add lines for emphasizing the positions
}
\examples{
vp <- getCurTrackViewport(trackViewerStyle(), 10000, 10200)
addGuideLine(c(10010, 10025, 10150), vp=vp)
}
\seealso{
See Also as \code{\link{getCurTrackViewport}}, \code{\link{addArrowMark}}, 
\code{\link{viewTracks}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allclasses.R
\docType{class}
\name{pos-class}
\alias{pos-class}
\alias{pos}
\title{Class \code{"pos"}}
\description{
An object of class \code{"pos"} represents a point location
}
\section{Slots}{

\describe{
\item{\code{x}}{A \code{\link{numeric}} value, indicates the x position}

\item{\code{y}}{A \code{\link{numeric}} value, indicates the y position}

\item{\code{unit}}{\code{"character"} apecifying the units for the corresponding
numeric values. See \code{\link[grid]{unit}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trackViewerPackage.R
\docType{package}
\name{trackViewer-package}
\alias{trackViewer}
\alias{trackViewer-package}
\title{Minimal designed plotting tool for genomic data}
\description{
A package that plot data and annotation information along genomic 
coordinates in an elegance style. This tool is based on Gviz but want to draw 
figures in minimal style for publication.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         chrom="chr11", 
                         start=122929275, 
                         end=122930122)
extdata <- system.file("extdata", package="trackViewer",
                       mustWork=TRUE)
repA <- importScore(paste(extdata, "cpsf160.repA_+.wig", sep="/"),
                    paste(extdata, "cpsf160.repA_-.wig", sep="/"),
                    format="WIG")
strand(repA@dat) <- "+"
strand(repA@dat2) <- "-"
fox2 <- importScore(paste(extdata, "fox2.bed", sep="/"), format="BED")
dat <- coverageGR(fox2@dat)
fox2@dat <- dat[strand(dat)=="+"]
fox2@dat2 <- dat[strand(dat)=="-"]
gr <- GRanges("chr11", IRanges(122929275, 122930122), strand="-")
vp <- viewTracks(trackList(repA, fox2, trs), gr=gr, autoOptimizeStyle=TRUE)
addGuideLine(c(122929767, 122929969), vp=vp)
addArrowMark(list(x=unit(.5, "npc"), 
                  y=unit(.39, "npc")),
             col="blue")
}
\author{
\strong{Maintainer}: Jianhong Ou \email{jou@morgridge.org} (\href{https://orcid.org/0000-0002-8652-2488}{ORCID})

Authors:
\itemize{
  \item Julie Lihua Zhu \email{Julie.Zhu@umassmed.edu}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotIdeo.R
\name{gieStain}
\alias{gieStain}
\title{color sheme for the schema for Chromosome Band (Ideogram)}
\usage{
gieStain()
}
\value{
A character vector of colors
}
\description{
Describe the colors of giemsa stain results
}
\examples{
gieStain()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allclasses.R
\docType{class}
\name{xscale-class}
\alias{xscale-class}
\alias{xscale}
\title{Class \code{"xscale"}}
\description{
An object of class \code{"xscale"} represents x-scale style.
}
\section{Slots}{

\describe{
\item{\code{from}}{A \code{\link{pos}} class, indicates the start point 
postion of x-scale.}

\item{\code{to}}{A \code{\link{pos}} class, indicates the end point 
postion of x-scale.}

\item{\code{label}}{\code{"character"} the label of x-scale}

\item{\code{gp}}{A \code{"list"} object, It will convert to an object of 
class \code{\link[grid]{gpar}}. This is basically a list of graphical 
parameter settings of x-scale.}

\item{\code{draw}}{A \code{"logical"} value indicating whether the x-scale
should be draw.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importData.R
\name{importData}
\alias{importData}
\title{Reading data from a BED or WIG file to RleList}
\usage{
importData(files, format = NA, ranges = GRanges())
}
\arguments{
\item{files}{The path to the files to read.}

\item{format}{The format of import file. Could be BAM, BED, bedGraph, WIG or BigWig}

\item{ranges}{An object of \code{\link[GenomicRanges:GRanges-class]{GRanges}} to indicate
the range to be imported}
}
\value{
a list of \code{\link[IRanges:AtomicList-class]{RleList}}.
}
\description{
Read a \code{\link{track}} object from a BED, bedGraph, 
WIG or BigWig file to RleList
}
\examples{
#import a BED file
bedfile <- system.file("tests", "test.bed", package="rtracklayer",
                       mustWork=TRUE)
dat <- importData(files=bedfile, format="BED",
                  ranges=GRanges("chr7", IRanges(127471197, 127474697)))

##import a WIG file
wigfile <- system.file("tests", "step.wig", package = "rtracklayer",
                       mustWork=TRUE)
dat <- importData(files=wigfile, format="WIG", 
                  ranges=GRanges("chr19", 
                                 IRanges(59104701, 59110920)))

##import a BigWig file
if(.Platform$OS.type!="windows"){
  ##this is because we are using rtracklayer::import
  bwfile <- system.file("tests", "test.bw", package = "rtracklayer",
                        mustWork=TRUE)
  dat <- importData(files=bwfile, format="BigWig", 
                    ranges=GRanges("chr19", IRanges(1500, 2700)))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneTrack.R
\name{geneTrack}
\alias{geneTrack}
\title{track from TxDb}
\usage{
geneTrack(ids, txdb, symbols, type = c("gene", "transcript"), asList = TRUE)
}
\arguments{
\item{ids}{Gene IDs. A vector of character. It should be keys in txdb.}

\item{txdb}{An object of \code{\link[GenomicFeatures:TxDb-class]{TxDb}}.}

\item{symbols}{symbol of genes.}

\item{type}{Output type of track, "gene" or "transcript".}

\item{asList}{Output a list of tracks or not. Default TRUE.}
}
\value{
An object of \link{track}
}
\description{
Generate a track object from TxDb by given gene ids
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
ids <- c("3312", "390259", "341056", "79827")
symbols <- mget(ids, org.Hs.egSYMBOL)
geneTrack(ids, TxDb.Hsapiens.UCSC.hg19.knownGene, symbols)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allclasses.R
\docType{class}
\name{trackViewerStyle-class}
\alias{trackViewerStyle-class}
\alias{trackViewerStyle}
\alias{setTrackViewerStyleParam}
\alias{setTrackViewerStyleParam,trackViewerStyle,character-method}
\alias{setTrackViewerStyleParam,trackViewerStyle,character,ANY-method}
\title{Class \code{"trackViewerStyle"}}
\usage{
trackViewerStyle(...)

setTrackViewerStyleParam(tvs, attr, value)

\S4method{setTrackViewerStyleParam}{trackViewerStyle,character}(tvs, attr, value)
}
\arguments{
\item{\dots}{Each argument in \dots becomes an slot in the new trackViewerStyle.}

\item{tvs}{An object of \code{trackViewerStyle}.}

\item{attr}{the name of slot to be changed.}

\item{value}{values to be assigned.}
}
\description{
An object of class \code{"trackViewerStyle"} 
             represents track viewer style.
}
\section{Slots}{

\describe{
\item{\code{margin}}{\code{"numeric"}, specify the bottom, left, top and right margin.}

\item{\code{xlas}}{\code{"numeric"}, label direction of x-axis mark. It should 
be a integer 0-3. See \code{\link[graphics]{par}:las}}

\item{\code{xgp}}{A \code{"list"}, object, It will convert to an object of 
class \code{\link[grid]{gpar}}. This is basically a list of graphical 
parameter settings of x-axis. For y-axis, see \code{\link{yaxisStyle}}}

\item{\code{xaxis}}{\code{"logical"}, draw x-axis or not}

\item{\code{xat}}{\code{"numeric"}, the values will be passed to grid.xaxis as
'at' parameter.}

\item{\code{xlabel}}{\code{"character"}, the values will be passed to grid.xaxis as
'label' parameter.}

\item{\code{autolas}}{\code{"logical"} automatic determine y label direction}

\item{\code{flip}}{\code{"logical"} flip the x-axis or not, default FALSE}
}}

\examples{
tvs <- trackViewerStyle()
setTrackViewerStyleParam(tvs, "xaxis", TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseWIG.R
\name{parseWIG}
\alias{parseWIG}
\title{convert WIG format track to BED format track}
\usage{
parseWIG(trackScore, chrom, from, to)
}
\arguments{
\item{trackScore}{an object of track with WIG format}

\item{chrom}{sequence name of the chromosome}

\item{from}{start coordinate}

\item{to}{end coordinate}
}
\value{
an object of \code{\link{track}}
}
\description{
convert WIG format track to BED format track for a given range
}
\examples{
extdata <- system.file("extdata", package="trackViewer", mustWork=TRUE)
repA <- importScore(file.path(extdata, "cpsf160.repA_-.wig"),
                    file.path(extdata, "cpsf160.repA_+.wig"),
                    format="WIG")
strand(repA$dat) <- "-"
strand(repA$dat2) <- "+"
parseWIG(repA, chrom="chr11", from=122929275, to=122930122)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotIdeo.R
\name{plotIdeo}
\alias{plotIdeo}
\title{plot ideogram}
\usage{
plotIdeo(
  ideo,
  chrom = seqlevels(ideo)[1],
  colorSheme = gieStain(),
  gp = gpar(fill = NA),
  ...
)
}
\arguments{
\item{ideo}{output of \link{loadIdeogram}.}

\item{chrom}{A length 1 character vector of chromosome name.}

\item{colorSheme}{A character vector of giemsa stain colors.}

\item{gp}{parameters used for \link[grid]{grid.roundrect}.}

\item{...}{parameters not used.}
}
\description{
plot ideogram for one chromosome
}
\examples{
\dontrun{
ideo <- loadIdeogram("hg38")
library(grid)
grid.newpage()
plotIdeo(ideo)
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browseTracks.R
\name{browseTracks-shiny}
\alias{browseTracks-shiny}
\alias{browseTracksOutput}
\alias{renderbrowseTracks}
\title{Shiny bindings for browseTracks}
\usage{
browseTracksOutput(outputId, width = "100\%", height = "600px")

renderbrowseTracks(expr, env = parent.frame(), quoted = FALSE)
}
\arguments{
\item{outputId}{output variable to read from}

\item{width, height}{Must be a valid CSS unit (like \code{'100\%'},
\code{'400px'}, \code{'auto'}) or a number, which will be coerced to a
string and have \code{'px'} appended.}

\item{expr}{An expression that generates a browseTracks}

\item{env}{The environment in which to evaluate \code{expr}.}

\item{quoted}{Is \code{expr} a quoted expression (with \code{quote()})? This
is useful if you want to save an expression in a variable.}
}
\description{
Output and render functions for using browseTracks within Shiny
applications and interactive Rmd documents.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importScore.R
\name{importScore}
\alias{importScore}
\title{Reading data from a BED or WIG file}
\usage{
importScore(
  file,
  file2,
  format = c("BED", "bedGraph", "WIG", "BigWig"),
  ranges = GRanges(),
  ignore.strand = TRUE
)
}
\arguments{
\item{file}{The path to the file to read.}

\item{file2}{The path to the second file to read.}

\item{format}{The format of import file. Could be BED, bedGraph, WIG or BigWig}

\item{ranges}{An object of \code{\link[GenomicRanges:GRanges-class]{GRanges}} to indicate
the range to be imported}

\item{ignore.strand}{ignore the strand or not when do filter. default TRUE}
}
\value{
a \code{\link{track}} object
}
\description{
Read a \code{\link{track}} object from a BED, bedGraph, WIG or BigWig file
}
\examples{
#import a BED file
bedfile <- system.file("tests", "test.bed", package="rtracklayer",
                       mustWork=TRUE)
dat <- importScore(file=bedfile, format="BED",
                   ranges=GRanges("chr7", IRanges(127471197, 127474697)))

##import a WIG file
wigfile <- system.file("tests", "step.wig", package = "rtracklayer",
                       mustWork=TRUE)
dat <- importScore(file=wigfile, format="WIG")

##import a BigWig file
if(.Platform$OS.type!="windows"){##this is because we are using rtracklayer::import
  bwfile <- system.file("tests", "test.bw", package = "rtracklayer",
                        mustWork=TRUE)
  dat <- importScore(file=bwfile, format="BigWig")
}

##import 2 file
wigfile1 <- system.file("extdata", "cpsf160.repA_+.wig", package="trackViewer",
                        mustWork=TRUE)
wigfile2 <- system.file("extdata", "cpsf160.repA_-.wig", package="trackViewer",
                        mustWork=TRUE)
dat <- importScore(wigfile1, wigfile2, format="WIG", 
                   ranges=GRanges("chr11", IRanges(122817703, 122889073)))
}
\seealso{
See Also as \code{\link{importBam}}, \code{\link{track}}, 
\code{\link{viewTracks}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allclasses.R
\docType{class}
\name{trackList-class}
\alias{trackList-class}
\alias{trackList}
\alias{seqlevelsStyle<-,trackList-method}
\title{List of tracks}
\usage{
\S4method{seqlevelsStyle}{trackList}(x) <- value

trackList(..., heightDist = NA)
}
\arguments{
\item{x}{trackList object.}

\item{value}{values to be assigned.}

\item{\dots}{Each tracks in ... becomes an element in the new 
trackList, in the same order. This is analogous to the list constructor, except
every argument in ... must be derived from \code{\link{track}}.}

\item{heightDist}{A vector or NA to define the height of each track.}
}
\description{
An extension of List that holds only \code{\link{track}} objects.
}
\seealso{
\code{\link{track}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneModelFromTxdb.R
\name{geneModelFromTxdb}
\alias{geneModelFromTxdb}
\title{Prepare gene model from an object of TxDb}
\usage{
geneModelFromTxdb(
  txdb,
  orgDb,
  gr,
  chrom,
  start,
  end,
  strand = c("*", "+", "-"),
  txdump = NULL
)
}
\arguments{
\item{txdb}{An object of \code{\link[GenomicFeatures:TxDb-class]{TxDb}}}

\item{orgDb}{An object of "OrgDb"}

\item{gr}{An object of GRanges.}

\item{chrom}{chromosome name, must be a seqname of txdb}

\item{start}{start position}

\item{end}{end position}

\item{strand}{strand}

\item{txdump}{output of as.list(txdb), a list of data frames that can be used 
to make the db again with no loss of information.}
}
\value{
Generate a list of \code{\link{track}} from a TxDb object.
}
\description{
Generate an object of \code{\link{track}} for 
\code{\link{viewTracks}} by given parameters.
}
\examples{
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
library(org.Hs.eg.db)
trs <- geneModelFromTxdb(TxDb.Hsapiens.UCSC.hg19.knownGene,
                         org.Hs.eg.db,
                         chrom="chr20", 
                         start=22560000, 
                         end=22565000, 
                         strand="-")
}
\seealso{
See Also as \code{\link{importScore}}, \code{\link{importBam}}, 
\code{\link{viewTracks}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gi2track.R
\name{gi2track}
\alias{gi2track}
\title{convert GInteractions to track object}
\usage{
gi2track(gi, gi2)
}
\arguments{
\item{gi}{an object of GInteractions}

\item{gi2}{an object of GInteractions}
}
\value{
an track object
}
\description{
Convert GInteractions object to track object
}
\examples{
gi <- readRDS(system.file("extdata", "nij.chr6.51120000.53200000.gi.rds", package="trackViewer"))
gi2track(gi)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listChromosomes.R
\name{listChromosomes}
\alias{listChromosomes}
\title{List the available chromosome}
\usage{
listChromosomes(file, format = c("hic", "cool"))
}
\arguments{
\item{file}{character(1). File name of .hic or .cool/.mcool/.scool}

\item{format}{character(1). File format, "hic" or "cool".}
}
\description{
List the chromosomes available in the file.
}
\examples{
hicfile <- system.file("extdata", "test_chr22.hic", package="trackViewer")
listChromosomes(hicfile)
coolfile <- system.file("extdata", "test.mcool", package="trackViewer")
listChromosomes(coolfile, format="cool")
}
